<link rel="stylesheet" href="slipbox.css">
<script type="module" src="slipbox.js"></script>
<script src="es5/tex-chtml-full.js" id="MathJax-script" async></script>
<link rel="apple-touch-icon" sizes="180x180" href="apple-touch-icon.png">
<link rel="icon" type="image/png" sizes="32x32" href="favicon-32x32.png">
<link rel="icon" type="image/png" sizes="16x16" href="favicon-16x16.png">
<link rel="manifest" href="site.webmanifest">
