<nav>
  <sb-icon-button title="Home" icon="boxicons/bx-home" href="#home"></sb-icon-button>
  <sb-icon-button title="Tags" icon="boxicons/bx-tag" href="#tags"></sb-icon-button>
  <sb-icon-button title="References" icon="boxicons/bx-library" href="#references"></sb-icon-button>
  <sb-icon-button title="Graph" icon="boxicons/bx-network-chart"></sb-icon-button>
  <sb-icon-button title="Random note" icon="boxicons/bx-shuffle" href="#random"></sb-icon-button>
  <sb-icon-button title="Search" icon="boxicons/bx-search" href="#search"></sb-icon-button>
</nav>
