{
"de/si/prefixes.js": 
[{"Y":"Yotta","Z":"Zetta","E":"Exa","P":"Peta","T":"Tera","G":"Giga","M":"Mega","k":"Kilo","h":"Hekto","da":"Deka","d":"Dezi","c":"Zenti","m":"Milli","µ":"Mikro","μ":"Mikro","n":"Nano","p":"Piko","f":"Femto","a":"Atto","z":"Zepto","y":"Yokto"}]
,
"de/functions/algebra.js": 
[{"locale":"de"},{"category":"Algebra","mappings":{"default":{"default":"Grad"}},"key":"deg","names":["deg"]},{"key":"det","mappings":{"default":{"default":"Determinante"}},"category":"Algebra","names":["det"]},{"category":"Algebra","mappings":{"default":{"default":"Dimension"}},"key":"dim","names":["dim"]},{"category":"Algebra","mappings":{"default":{"default":"Homomorphismus"},"mathspeak":{"default":"hom"},"clearspeak":{"default":"hom"}},"key":"hom","names":["hom","Hom"]},{"category":"Algebra","mappings":{"default":{"default":"Kern"}},"key":"ker","names":["ker"]},{"category":"Algebra","mappings":{"default":{"default":"Spur"}},"key":"Tr","names":["Tr","tr"]}]
,
"de/functions/elementary.js": 
[{"locale":"de"},{"key":"log","mappings":{"default":{"default":"Logarithmus"}},"category":"Logarithm","names":["log"]},{"key":"ln","mappings":{"default":{"default":"natürlicher Logarithmus"},"clearspeak":{"default":"l n","Log_LnAsNaturalLog":"natürlicher Logarithmus"}},"category":"Logarithm","names":["ln"]},{"category":"Logarithm","mappings":{"default":{"default":"Logarithmus zur Basis 10"}},"key":"lg","names":["lg"]},{"category":"Elementary","mappings":{"default":{"default":"Exponent"},"mathspeak":{"default":"exp"},"clearspeak":{"default":"exp"}},"key":"exp","names":["exp","expt"]},{"category":"Elementary","mappings":{"default":{"default":"größter gemeinsamer Teiler"},"mathspeak":{"default":"ggt"},"clearspeak":{"default":"ggt"}},"key":"gcd","names":["gcd","ggt","ggT"]},{"category":"Elementary","mappings":{"default":{"default":"kleinstes gemeinsames Vielfaches"},"mathspeak":{"default":"kgv"},"clearspeak":{"default":"kgv"}},"key":"lcm","names":["lcm","kgv","kgV"]},{"category":"Complex","mappings":{"default":{"default":"Argument"}},"key":"arg","names":["arg"]},{"key":"im","mappings":{"default":{"default":"der Imaginärteil der komplexen Zahl"}},"category":"Complex","names":["im"]},{"key":"re","mappings":{"default":{"default":"der Realteil der komplexen Zahl"}},"category":"Complex","names":["re"]},{"key":"mod","mappings":{"default":{"default":"modulo"}},"category":"Elementary","names":["mod"]},{"category":"Limits","mappings":{"default":{"default":"Infimum"}},"key":"inf","names":["inf"]},{"category":"Limits","mappings":{"default":{"default":"Limes"}},"key":"lim","names":["lim"]},{"category":"Limits","mappings":{"default":{"default":"Limes inferior"}},"key":"liminf","names":["lim inf","liminf"]},{"category":"Limits","mappings":{"default":{"default":"Limes superior"}},"key":"limsup","names":["lim sup","limsup"]},{"category":"Limits","mappings":{"default":{"default":"Maximum"}},"key":"max","names":["max"]},{"category":"Limits","mappings":{"default":{"default":"Minimum"}},"key":"min","names":["min"]},{"category":"Limits","mappings":{"default":{"default":"Supremum"}},"key":"sup","names":["sup"]},{"category":"Limits","mappings":{"default":{"default":"Colimes"}},"key":"injlim","names":["injlim","inj lim"]},{"category":"Limits","mappings":{"default":{"default":"projektiver Limes"}},"key":"projlim","names":["projlim","proj lim"]},{"category":"Probability","mappings":{"default":{"default":"Wahrscheinlichkeit"}},"key":"Pr","names":["Pr"]}]
,
"de/functions/hyperbolic.js": 
[{"locale":"de"},{"key":"cosh","mappings":{"default":{"default":"Kosinus hyperbolicus"}},"category":"Hyperbolic","names":["cosh"]},{"key":"coth","mappings":{"default":{"default":"Kotangens hyperbolicus"}},"category":"Hyperbolic","names":["coth"]},{"key":"csch","mappings":{"default":{"default":"Kosekans hyperbolicus"}},"category":"Hyperbolic","names":["csch"]},{"key":"sech","mappings":{"default":{"default":"Sekans hyperbolicus"}},"category":"Hyperbolic","names":["sech"]},{"key":"sinh","mappings":{"default":{"default":"Sinus hyperbolicus"}},"category":"Hyperbolic","names":["sinh"]},{"key":"tanh","mappings":{"default":{"default":"Tangens hyperbolicus"}},"category":"Hyperbolic","names":["tanh"]},{"key":"arcosh","mappings":{"default":{"default":"Areakosinus hyperbolicus"}},"category":"Area","names":["arcosh","arccosh"]},{"key":"arcoth","mappings":{"default":{"default":"Areakotangens hyperbolicus"}},"category":"Area","names":["arcoth","arccoth"]},{"key":"arcsch","mappings":{"default":{"default":"Areakosekans hyperbolicus"}},"category":"Area","names":["arcsch","arccsch"]},{"key":"arsech","mappings":{"default":{"default":"Areasekans hyperbolicus"}},"category":"Area","names":["arsech","arcsech"]},{"key":"arsinh","mappings":{"default":{"default":"Areasinus hyperbolicus"}},"category":"Area","names":["arsinh","arcsinh"]},{"key":"artanh","mappings":{"default":{"default":"Areatangens hyperbolicus "}},"category":"Area","names":["artanh","arctanh"]}]
,
"de/functions/trigonometry.js": 
[{"locale":"de"},{"key":"cos","mappings":{"default":{"default":"Kosinus"}},"category":"Trigonometric","names":["cos","cosine"]},{"key":"cot","mappings":{"default":{"default":"Kotangens"}},"category":"Trigonometric","names":["cot"]},{"key":"csc","mappings":{"default":{"default":"Kosekans"}},"category":"Trigonometric","names":["csc"]},{"key":"sec","mappings":{"default":{"default":"Sekans"}},"category":"Trigonometric","names":["sec"]},{"key":"sin","mappings":{"default":{"default":"Sinus"}},"category":"Trigonometric","names":["sin","sine"]},{"key":"tan","mappings":{"default":{"default":"Tangens"}},"category":"Trigonometric","names":["tan"]},{"key":"arccos","mappings":{"default":{"default":"Arkuskosinus"},"clearspeak":{"Trig_TrigInverse":"inverser Kosinus"}},"category":"Cyclometric","names":["arccos"]},{"key":"arccot","mappings":{"default":{"default":"Arkuskotangens"},"clearspeak":{"Trig_TrigInverse":"inverser Kotangens"}},"category":"Cyclometric","names":["arccot"]},{"key":"arccsc","mappings":{"default":{"default":"Arkuskosekans"},"clearspeak":{"Trig_TrigInverse":"inverser Kosekans"}},"category":"Cyclometric","names":["arccsc"]},{"key":"arcsec","mappings":{"default":{"default":"Arkussekans"},"clearspeak":{"Trig_TrigInverse":"inverser Sekans"}},"category":"Cyclometric","names":["arcsec"]},{"key":"arcsin","mappings":{"default":{"default":"Arkussinus"},"clearspeak":{"Trig_TrigInverse":"inverser Sinus"}},"category":"Cyclometric","names":["arcsin"]},{"key":"arctan","mappings":{"default":{"default":"Arkustangens"},"clearspeak":{"Trig_TrigInverse":"inverser Tangens"}},"category":"Cyclometric","names":["arctan"]}]
,
"de/symbols/digits_rest.js": 
[{"locale":"de"},{"key":"00B2","mappings":{"default":{"default":"Quadrat"},"clearspeak":{"Exponent_Exponent":"mit Exponent 2","Exponent_OrdinalPower":"zur zweiten Potenz"}},"category":"No"},{"key":"00B3","mappings":{"default":{"default":"Kubik"},"clearspeak":{"Exponent_Exponent":"mit Exponent 3","Exponent_OrdinalPower":"zur dritten Potenz"}},"category":"No"},{"key":"00BC","mappings":{"default":{"default":"ein Viertel"}},"category":"No"},{"key":"00BD","mappings":{"default":{"default":"ein Halb"}},"category":"No"},{"key":"00BE","mappings":{"default":{"default":"drei Viertel"}},"category":"No"},{"category":"No","key":"2150","mappings":{"default":{"default":"ein siebtel"}}},{"category":"No","key":"2151","mappings":{"default":{"default":"ein neuntel"}}},{"category":"No","key":"2152","mappings":{"default":{"default":"ein zehntel"}}},{"category":"No","key":"2153","mappings":{"default":{"default":"ein drittel"}}},{"category":"No","key":"2154","mappings":{"default":{"default":"zwei drittel"}}},{"category":"No","key":"2155","mappings":{"default":{"default":"ein fünftel"}}},{"category":"No","key":"2156","mappings":{"default":{"default":"zwei fünftel"}}},{"category":"No","key":"2157","mappings":{"default":{"default":"drei fünftel"}}},{"category":"No","key":"2158","mappings":{"default":{"default":"vier fünftel"}}},{"category":"No","key":"2159","mappings":{"default":{"default":"ein sechstel"}}},{"category":"No","key":"215A","mappings":{"default":{"default":"fünf sechstel"}}},{"category":"No","key":"215B","mappings":{"default":{"default":"ein achtel"}}},{"category":"No","key":"215C","mappings":{"default":{"default":"drei achtel"}}},{"category":"No","key":"215D","mappings":{"default":{"default":"fünf achtel"}}},{"category":"No","key":"215E","mappings":{"default":{"default":"sieben achtel"}}},{"category":"No","key":"215F","mappings":{"default":{"default":"Bruchzähler eins"}}},{"category":"No","key":"2189","mappings":{"default":{"default":"null drittel"}}},{"category":"No","key":"3248","mappings":{"default":{"default":"eingekreiste Zehn auf schwarzem Quadrat"}}},{"category":"No","key":"3249","mappings":{"default":{"default":"eingekreiste Zwanzig auf schwarzem Quadrat"}}},{"category":"No","key":"324A","mappings":{"default":{"default":"eingekreiste Dreißig auf schwarzem Quadrat"}}},{"category":"No","key":"324B","mappings":{"default":{"default":"eingekreiste Vierzig auf schwarzem Quadrat"}}},{"category":"No","key":"324C","mappings":{"default":{"default":"eingekreiste Fünfzig auf schwarzem Quadrat"}}},{"category":"No","key":"324D","mappings":{"default":{"default":"eingekreiste Sechzig auf schwarzem Quadrat"}}},{"category":"No","key":"324E","mappings":{"default":{"default":"eingekreiste Siebzig auf schwarzem Quadrat"}}},{"category":"No","key":"324F","mappings":{"default":{"default":"eingekreiste Achtzig auf schwarzem Quadrat"}}}]
,
"de/symbols/greek-rest.js": 
[{"locale":"de"},{"key":"0394","mappings":{"clearspeak":{"default":"Dreieck","TriangleSymbol_Delta":"großes Delta"}},"category":"Lu"}]
,
"de/symbols/greek-scripts.js": 
[{"locale":"de"},{"category":"Ll","key":"1D26","mappings":{"default":{"default":"Kapitälchen Gamma"}}},{"category":"Ll","key":"1D27","mappings":{"default":{"default":"Kapitälchen Lamda"}}},{"category":"Ll","key":"1D28","mappings":{"default":{"default":"Kapitälchen Pi"}}},{"category":"Ll","key":"1D29","mappings":{"default":{"default":"Kapitälchen Rho"}}},{"category":"Ll","key":"1D2A","mappings":{"default":{"default":"Kapitälchen Psi"}}},{"category":"Lm","key":"1D5E","mappings":{"default":{"default":"hochgestelltes gamma"}}},{"category":"Lm","key":"1D60","mappings":{"default":{"default":"hochgestelltes phi"}}},{"category":"Lm","key":"1D66","mappings":{"default":{"default":"tiefgestelltes beta"}}},{"category":"Lm","key":"1D67","mappings":{"default":{"default":"tiefgestelltes gamma"}}},{"category":"Lm","key":"1D68","mappings":{"default":{"default":"tiefgestelltes rho"}}},{"category":"Lm","key":"1D69","mappings":{"default":{"default":"tiefgestelltes phi"}}},{"category":"Lm","key":"1D6A","mappings":{"default":{"default":"tiefgestelltes chi"}}}]
,
"de/symbols/greek-symbols.js": 
[{"locale":"de"},{"category":"Ll","key":"03D0","mappings":{"default":{"default":"beta"}}},{"category":"Ll","key":"03D7","mappings":{"default":{"default":"kai"}}},{"category":"Sm","key":"03F6","mappings":{"default":{"default":"umgekehrtes epsilon"}}},{"category":"Lu","key":"1D7CA","mappings":{"default":{"default":"fettes großes Digamma"}}},{"category":"Ll","key":"1D7CB","mappings":{"default":{"default":"fettes digamma"}}}]
,
"de/symbols/hebrew_letters.js": 
[{"locale":"de"},{"category":"Lo","key":"2135","mappings":{"default":{"default":"Alef-Symbol"}}},{"category":"Lo","key":"2136","mappings":{"default":{"default":"Bet-Symbol"}}},{"category":"Lo","key":"2137","mappings":{"default":{"default":"Gimel-Symbol"}}},{"category":"Lo","key":"2138","mappings":{"default":{"default":"Dalet-Symbol"}}}]
,
"de/symbols/latin-lower-double-accent.js": 
[{"locale":"de"},{"category":"Ll","key":"01D6","mappings":{"default":{"default":"u mit Diärese und Makron"}}},{"category":"Ll","key":"01D8","mappings":{"default":{"default":"u mit Diärese und Akut"}}},{"category":"Ll","key":"01DA","mappings":{"default":{"default":"u mit Diärese und Caron"}}},{"category":"Ll","key":"01DC","mappings":{"default":{"default":"u mit Diärese und Grab"}}},{"category":"Ll","key":"01DF","mappings":{"default":{"default":"a mit Diärese und Makron"}}},{"category":"Ll","key":"01E1","mappings":{"default":{"default":"a mit Dot Above und Makron"}}},{"category":"Ll","key":"01ED","mappings":{"default":{"default":"o mit Ogonek und Makron"}}},{"category":"Ll","key":"01FB","mappings":{"default":{"default":"a mit Ring oben und akut"}}},{"category":"Ll","key":"022B","mappings":{"default":{"default":"o mit Diärese und Makron"}}},{"category":"Ll","key":"022D","mappings":{"default":{"default":"o mit Tilde und Makron"}}},{"category":"Ll","key":"0231","mappings":{"default":{"default":"o mit Punkt oben und Makron"}}},{"category":"Ll","key":"1E09","mappings":{"default":{"default":"c mit Cedilla und Akut"}}},{"category":"Ll","key":"1E15","mappings":{"default":{"default":"e mit Makron und Grab"}}},{"category":"Ll","key":"1E17","mappings":{"default":{"default":"e mit Makron und Akut"}}},{"category":"Ll","key":"1E1D","mappings":{"default":{"default":"e mit Cedilla und Breve"}}},{"category":"Ll","key":"1E2F","mappings":{"default":{"default":"i mit Diärese und Akut"}}},{"category":"Ll","key":"1E39","mappings":{"default":{"default":"l mit Punkt unten und Makron"}}},{"category":"Ll","key":"1E4D","mappings":{"default":{"default":"o mit Tilde und Akut"}}},{"category":"Ll","key":"1E4F","mappings":{"default":{"default":"o mit Tilde und Diärese"}}},{"category":"Ll","key":"1E51","mappings":{"default":{"default":"o mit Makron und Grab"}}},{"category":"Ll","key":"1E53","mappings":{"default":{"default":"o mit Makron und Akut"}}},{"category":"Ll","key":"1E5D","mappings":{"default":{"default":"r mit Punkt unten und Makron"}}},{"category":"Ll","key":"1E65","mappings":{"default":{"default":"s mit Akute und Punkt oben"}}},{"category":"Ll","key":"1E67","mappings":{"default":{"default":"s mit Caron und Punkt oben"}}},{"category":"Ll","key":"1E69","mappings":{"default":{"default":"s mit Punkt unten und Punkt oben"}}},{"category":"Ll","key":"1E79","mappings":{"default":{"default":"u mit Tilde und Akut"}}},{"category":"Ll","key":"1E7B","mappings":{"default":{"default":"u mit Makron und Diärese"}}},{"category":"Ll","key":"1EA5","mappings":{"default":{"default":"a mit Zirkumflex und Akut"}}},{"category":"Ll","key":"1EA7","mappings":{"default":{"default":"a mit Zirkumflex und Grab"}}},{"category":"Ll","key":"1EA9","mappings":{"default":{"default":"a mit Zirkumflex und Haken oben"}}},{"category":"Ll","key":"1EAB","mappings":{"default":{"default":"a mit Zirkumflex und Tilde"}}},{"category":"Ll","key":"1EAD","mappings":{"default":{"default":"a mit Zirkumflex und Punkt darunter"}}},{"category":"Ll","key":"1EAF","mappings":{"default":{"default":"a mit Breve und Akut"}}},{"category":"Ll","key":"1EB1","mappings":{"default":{"default":"a mit Breve und Grab"}}},{"category":"Ll","key":"1EB3","mappings":{"default":{"default":"a mit Breve und Haken oben"}}},{"category":"Ll","key":"1EB5","mappings":{"default":{"default":"a mit Breve und Tilde"}}},{"category":"Ll","key":"1EB7","mappings":{"default":{"default":"a mit Breve und Dot Below"}}},{"category":"Ll","key":"1EBF","mappings":{"default":{"default":"e mit Zirkumflex und Akut"}}},{"category":"Ll","key":"1EC1","mappings":{"default":{"default":"e mit Zirkumflex und Grab"}}},{"category":"Ll","key":"1EC3","mappings":{"default":{"default":"e mit Zirkumflex und Haken oben"}}},{"category":"Ll","key":"1EC5","mappings":{"default":{"default":"e mit Zirkumflex und Tilde"}}},{"category":"Ll","key":"1EC7","mappings":{"default":{"default":"e mit Zirkumflex und Punkt darunter"}}},{"category":"Ll","key":"1ED1","mappings":{"default":{"default":"o mit Zirkumflex und Akut"}}},{"category":"Ll","key":"1ED3","mappings":{"default":{"default":"o mit Zirkumflex und Grab"}}},{"category":"Ll","key":"1ED5","mappings":{"default":{"default":"o mit Zirkumflex und Haken oben"}}},{"category":"Ll","key":"1ED7","mappings":{"default":{"default":"o mit Zirkumflex und Tilde"}}},{"category":"Ll","key":"1ED9","mappings":{"default":{"default":"o mit Zirkumflex und Punkt darunter"}}},{"category":"Ll","key":"1EDB","mappings":{"default":{"default":"o mit Horn und Akut"}}},{"category":"Ll","key":"1EDD","mappings":{"default":{"default":"o mit Horn und Grab"}}},{"category":"Ll","key":"1EDF","mappings":{"default":{"default":"o mit Horn und Haken oben"}}},{"category":"Ll","key":"1EE1","mappings":{"default":{"default":"o mit Horn und Tilde"}}},{"category":"Ll","key":"1EE3","mappings":{"default":{"default":"o mit Horn und Punkt"}}},{"category":"Ll","key":"1EE9","mappings":{"default":{"default":"u mit Horn und Akut"}}},{"category":"Ll","key":"1EEB","mappings":{"default":{"default":"u mit Horn und Grab"}}},{"category":"Ll","key":"1EED","mappings":{"default":{"default":"u mit Horn und Haken oben"}}},{"category":"Ll","key":"1EEF","mappings":{"default":{"default":"u mit Horn und Tilde"}}},{"category":"Ll","key":"1EF1","mappings":{"default":{"default":"u mit Horn und Punkt"}}}]
,
"de/symbols/latin-lower-phonetic.js": 
[{"locale":"de"},{"category":"Ll","key":"00F8","mappings":{"default":{"default":"o mit Strich"}}},{"category":"Ll","key":"0111","mappings":{"default":{"default":"d mit Strich"}}},{"category":"Ll","key":"0127","mappings":{"default":{"default":"h mit Strich"}}},{"category":"Ll","key":"0142","mappings":{"default":{"default":"l mit Strich"}}},{"category":"Ll","key":"0167","mappings":{"default":{"default":"t mit Strich"}}},{"category":"Ll","key":"0180","mappings":{"default":{"default":"b mit Strich"}}},{"category":"Ll","key":"019B","mappings":{"default":{"default":"lambda mit Strich"}}},{"category":"Ll","key":"01B6","mappings":{"default":{"default":"z mit Strich"}}},{"category":"Ll","key":"01BE","mappings":{"default":{"default":"umgekehrter Glottalanschlag mit Strich"}}},{"category":"Ll","key":"01E5","mappings":{"default":{"default":"g mit Strich"}}},{"category":"Ll","key":"01FF","mappings":{"default":{"default":"o mit Strich und Akut"}}},{"category":"Ll","key":"023C","mappings":{"default":{"default":"c mit Strich"}}},{"category":"Ll","key":"0247","mappings":{"default":{"default":"e mit Strich"}}},{"category":"Ll","key":"0249","mappings":{"default":{"default":"j mit Strich"}}},{"category":"Ll","key":"024D","mappings":{"default":{"default":"r mit Strich"}}},{"category":"Ll","key":"024F","mappings":{"default":{"default":"y mit Strich"}}},{"category":"Ll","key":"025F","mappings":{"default":{"default":"punktloses j mit Strich"}}},{"category":"Ll","key":"0268","mappings":{"default":{"default":"i mit Strich"}}},{"category":"Ll","key":"0284","mappings":{"default":{"default":"punktloses j mit Strich und Haken"}}},{"category":"Ll","key":"02A1","mappings":{"default":{"default":"Glottalanschlag mit Strich"}}},{"category":"Ll","key":"02A2","mappings":{"default":{"default":"umgekehrter Glottalanschlag mit Strich"}}},{"category":"Ll","key":"1D13","mappings":{"default":{"default":"liegendes O mit Strich"}}},{"category":"Ll","key":"1D7C","mappings":{"default":{"default":"iota mit Strich"}}},{"category":"Ll","key":"1D7D","mappings":{"default":{"default":"p mit Strich"}}},{"category":"Ll","key":"1D7F","mappings":{"default":{"default":"upsilon mit Strich"}}},{"category":"Ll","key":"1E9C","mappings":{"default":{"default":"lang mit diagonalem Anschlag"}}},{"category":"Ll","key":"1E9D","mappings":{"default":{"default":"lang S mit hohem Hub"}}},{"category":"Ll","key":"018D","mappings":{"default":{"default":"wurde Delta gedreht"}}},{"category":"Ll","key":"1E9B","mappings":{"default":{"default":"lang mit Punkt oben"}}},{"category":"Ll","key":"1E9F","mappings":{"default":{"default":"delta"}}},{"key":"0138","mappings":{"default":{"default":"kra"}},"category":"Ll"},{"category":"Ll","key":"017F","mappings":{"default":{"default":"lang S"}}},{"category":"Ll","key":"0183","mappings":{"default":{"default":"b mit Oberleiste"}}},{"category":"Ll","key":"0185","mappings":{"default":{"default":"ton sechs"}}},{"category":"Ll","key":"0188","mappings":{"default":{"default":"c mit Haken"}}},{"category":"Ll","key":"018C","mappings":{"default":{"default":"d mit Oberleiste"}}},{"key":"0192","mappings":{"default":{"default":"f mit Haken"}},"category":"Ll"},{"category":"Ll","key":"0195","mappings":{"default":{"default":"hv"}}},{"category":"Ll","key":"0199","mappings":{"default":{"default":"k mit Haken"}}},{"key":"019A","mappings":{"default":{"default":"l mit Strich"}},"category":"Ll"},{"category":"Ll","key":"019E","mappings":{"default":{"default":"n mit langem rechtem Bein"}}},{"category":"Ll","key":"01A1","mappings":{"default":{"default":"o mit Horn"}}},{"category":"Ll","key":"01A3","mappings":{"default":{"default":"oi"}}},{"category":"Ll","key":"01A5","mappings":{"default":{"default":"p mit Haken"}}},{"category":"Ll","key":"01A8","mappings":{"default":{"default":"ton zwei"}}},{"category":"Ll","key":"01AA","mappings":{"default":{"default":"umgedrehte esh Schleife"}}},{"category":"Ll","key":"01AB","mappings":{"default":{"default":"t mit palatal Haken"}}},{"category":"Ll","key":"01AD","mappings":{"default":{"default":"t mit Haken"}}},{"category":"Ll","key":"01B0","mappings":{"default":{"default":"u mit Horn"}}},{"category":"Ll","key":"01B4","mappings":{"default":{"default":"y mit Haken"}}},{"category":"Ll","key":"01B9","mappings":{"default":{"default":"umgekehrtes ezh"}}},{"category":"Ll","key":"01BA","mappings":{"default":{"default":"ezh mit Schwanz"}}},{"category":"Ll","key":"01BD","mappings":{"default":{"default":"ton fünf"}}},{"category":"Ll","key":"01BF","mappings":{"default":{"default":"wynn"}}},{"category":"Ll","key":"01C6","mappings":{"default":{"default":"dz mit Caron"}}},{"category":"Ll","key":"01C9","mappings":{"default":{"default":"lj"}}},{"category":"Ll","key":"01CC","mappings":{"default":{"default":"nj"}}},{"category":"Ll","key":"01E3","mappings":{"default":{"default":"ae mit Makron"}}},{"category":"Ll","key":"01EF","mappings":{"default":{"default":"ezh mit Caron"}}},{"category":"Ll","key":"01F3","mappings":{"default":{"default":"dz"}}},{"category":"Ll","key":"021D","mappings":{"default":{"default":"yogh"}}},{"category":"Ll","key":"026E","mappings":{"default":{"default":"lezh"}}},{"category":"Ll","key":"0292","mappings":{"default":{"default":"ezh"}}},{"category":"Ll","key":"0293","mappings":{"default":{"default":"ezh mit Kringel"}}},{"category":"Ll","key":"02A4","mappings":{"default":{"default":"dezh Digraph"}}},{"category":"Ll","key":"01DD","mappings":{"default":{"default":"gedreht E"}}},{"category":"Ll","key":"01FD","mappings":{"default":{"default":"ae mit Akut"}}},{"category":"Ll","key":"0221","mappings":{"default":{"default":"d mit Kringel"}}},{"category":"Ll","key":"0223","mappings":{"default":{"default":"ou"}}},{"category":"Ll","key":"0225","mappings":{"default":{"default":"z mit Haken"}}},{"category":"Ll","key":"0234","mappings":{"default":{"default":"l mit Kringel"}}},{"category":"Ll","key":"0235","mappings":{"default":{"default":"n mit Kringel"}}},{"category":"Ll","key":"0236","mappings":{"default":{"default":"t mit Kringel"}}},{"category":"Ll","key":"0238","mappings":{"default":{"default":"db"}}},{"category":"Ll","key":"0239","mappings":{"default":{"default":"qp Digraph"}}},{"category":"Ll","key":"023F","mappings":{"default":{"default":"s mit Swash Tail"}}},{"category":"Ll","key":"0240","mappings":{"default":{"default":"z mit Swash Tail"}}},{"category":"Ll","key":"0242","mappings":{"default":{"default":"Glottalanschlag"}}},{"category":"Ll","key":"024B","mappings":{"default":{"default":"q mit Hakenschwanz"}}},{"category":"Ll","key":"0250","mappings":{"default":{"default":"gedreht A"}}},{"category":"Ll","key":"0251","mappings":{"default":{"default":"alpha"}}},{"category":"Ll","key":"0252","mappings":{"default":{"default":"gedrehtes Alpha"}}},{"category":"Ll","key":"0253","mappings":{"default":{"default":"b mit Haken"}}},{"category":"Ll","key":"0254","mappings":{"default":{"default":"open O"}}},{"category":"Ll","key":"0255","mappings":{"default":{"default":"c mit Kringel"}}},{"category":"Ll","key":"0256","mappings":{"default":{"default":"d mit Schwanz"}}},{"category":"Ll","key":"0257","mappings":{"default":{"default":"d mit Haken"}}},{"category":"Ll","key":"0258","mappings":{"default":{"default":"umgedreht E"}}},{"category":"Ll","key":"0259","mappings":{"default":{"default":"schwa"}}},{"category":"Ll","key":"025A","mappings":{"default":{"default":"schwa mit Haken"}}},{"key":"025B","mappings":{"default":{"default":"offenes e"}},"category":"Ll"},{"category":"Ll","key":"025C","mappings":{"default":{"default":"umgedrehtes offenes e"}}},{"category":"Ll","key":"025D","mappings":{"default":{"default":"umgekehrtes offenes e mit Haken"}}},{"category":"Ll","key":"025E","mappings":{"default":{"default":"umgekehrtes geschlossenes epsilon"}}},{"category":"Ll","key":"0260","mappings":{"default":{"default":"g mit Haken"}}},{"category":"Ll","key":"0261","mappings":{"default":{"default":"g"}}},{"category":"Ll","key":"0263","mappings":{"default":{"default":"gamma"}}},{"category":"Ll","key":"0264","mappings":{"default":{"default":"mini gamma"}}},{"category":"Ll","key":"0265","mappings":{"default":{"default":"gedrehtes h"}}},{"category":"Ll","key":"0266","mappings":{"default":{"default":"h mit Haken"}}},{"category":"Ll","key":"0267","mappings":{"default":{"default":"h mit Haken"}}},{"category":"Ll","key":"0269","mappings":{"default":{"default":"iota"}}},{"category":"Ll","key":"026B","mappings":{"default":{"default":"l mit mittlerer Tilde"}}},{"category":"Ll","key":"026C","mappings":{"default":{"default":"l mit Gurt"}}},{"category":"Ll","key":"026D","mappings":{"default":{"default":"l mit Retroflexhaken"}}},{"category":"Ll","key":"026F","mappings":{"default":{"default":"gedreht M"}}},{"category":"Ll","key":"0270","mappings":{"default":{"default":"gedrehtes M mit langem Bein"}}},{"category":"Ll","key":"0271","mappings":{"default":{"default":"m mit Haken"}}},{"category":"Ll","key":"0272","mappings":{"default":{"default":"n mit linkem Haken"}}},{"category":"Ll","key":"0273","mappings":{"default":{"default":"n mit Retroflexhaken"}}},{"category":"Ll","key":"0275","mappings":{"default":{"default":"o mit balken"}}},{"category":"Ll","key":"0277","mappings":{"default":{"default":"geschlossenes omega"}}},{"category":"Ll","key":"0278","mappings":{"default":{"default":"phi"}}},{"category":"Ll","key":"0279","mappings":{"default":{"default":"gedrehtes r"}}},{"category":"Ll","key":"027A","mappings":{"default":{"default":"gedrehtes r mit langem Bein"}}},{"category":"Ll","key":"027B","mappings":{"default":{"default":"gedrehtes r mit langem Bein und Haken"}}},{"category":"Ll","key":"027C","mappings":{"default":{"default":"r mit langem Bein"}}},{"category":"Ll","key":"027D","mappings":{"default":{"default":"r mit Endstück"}}},{"category":"Ll","key":"027E","mappings":{"default":{"default":"r mit Angelhaken"}}},{"category":"Ll","key":"027F","mappings":{"default":{"default":"umgedrehtes R mit Angelhaken"}}},{"category":"Ll","key":"0282","mappings":{"default":{"default":"s mit Haken"}}},{"category":"Ll","key":"0283","mappings":{"default":{"default":"esh"}}},{"category":"Ll","key":"0285","mappings":{"default":{"default":"squat umgekehrt"}}},{"category":"Ll","key":"0286","mappings":{"default":{"default":"esh mit Kringel"}}},{"category":"Ll","key":"0287","mappings":{"default":{"default":"gedreht T"}}},{"category":"Ll","key":"0288","mappings":{"default":{"default":"t mit Retroflexhaken"}}},{"category":"Ll","key":"0289","mappings":{"default":{"default":"u Bar"}}},{"category":"Ll","key":"028A","mappings":{"default":{"default":"upsilon"}}},{"category":"Ll","key":"028B","mappings":{"default":{"default":"v mit Haken"}}},{"category":"Ll","key":"028C","mappings":{"default":{"default":"gedrehtes v"}}},{"category":"Ll","key":"028D","mappings":{"default":{"default":"gedrehtes w"}}},{"category":"Ll","key":"028E","mappings":{"default":{"default":"gedrehtes y"}}},{"category":"Ll","key":"0290","mappings":{"default":{"default":"z mit Retroflexhaken"}}},{"category":"Ll","key":"0291","mappings":{"default":{"default":"z mit Kringel"}}},{"category":"Ll","key":"0295","mappings":{"default":{"default":"pharyngales stimmhaft Frikativ"}}},{"category":"Ll","key":"0296","mappings":{"default":{"default":"invertierter Glottalanschlag"}}},{"category":"Ll","key":"0297","mappings":{"default":{"default":"gestrecktes C"}}},{"category":"Ll","key":"0298","mappings":{"default":{"default":"bilabialer Klick"}}},{"category":"Ll","key":"029A","mappings":{"default":{"default":"geschlossenes offenes e"}}},{"category":"Ll","key":"029E","mappings":{"default":{"default":"gedrehtes k"}}},{"category":"Ll","key":"02A0","mappings":{"default":{"default":"q mit Haken"}}},{"category":"Ll","key":"02A3","mappings":{"default":{"default":"dz Digraph"}}},{"category":"Ll","key":"02A5","mappings":{"default":{"default":"dz Digraph mit Kringel"}}},{"category":"Ll","key":"02A6","mappings":{"default":{"default":"ts Digraph"}}},{"category":"Ll","key":"02A7","mappings":{"default":{"default":"tesh Digraph"}}},{"category":"Ll","key":"02A8","mappings":{"default":{"default":"tc Digraph mit Kringel"}}},{"category":"Ll","key":"02A9","mappings":{"default":{"default":"feng Digraph"}}},{"category":"Ll","key":"02AA","mappings":{"default":{"default":"ls Digraph"}}},{"category":"Ll","key":"02AB","mappings":{"default":{"default":"lz Digraph"}}},{"category":"Ll","key":"02AC","mappings":{"default":{"default":"bilabiales Perkussiv"}}},{"category":"Ll","key":"02AD","mappings":{"default":{"default":"bidentales Perkussiv"}}},{"category":"Ll","key":"02AE","mappings":{"default":{"default":"gedrehtes h mit Angelhaken"}}},{"category":"Ll","key":"02AF","mappings":{"default":{"default":"gedrehtes h mit Angelhaken und Schwanz"}}},{"category":"Ll","key":"1D02","mappings":{"default":{"default":"gedrehtes ae"}}},{"category":"Ll","key":"1D08","mappings":{"default":{"default":"gedrehtes e"}}},{"category":"Ll","key":"1D09","mappings":{"default":{"default":"gedrehtes i"}}},{"category":"Ll","key":"1D11","mappings":{"default":{"default":"liegendes O"}}},{"category":"Ll","key":"1D12","mappings":{"default":{"default":"liegendes geöffnet O"}}},{"category":"Ll","key":"1D14","mappings":{"default":{"default":"gedreht Oe"}}},{"category":"Ll","key":"1D16","mappings":{"default":{"default":"obere Hälfte O"}}},{"category":"Ll","key":"1D17","mappings":{"default":{"default":"untere Hälfte O"}}},{"category":"Ll","key":"1D1D","mappings":{"default":{"default":"liegendes U"}}},{"category":"Ll","key":"1D1E","mappings":{"default":{"default":"liegendes u Umlaut"}}},{"category":"Ll","key":"1D1F","mappings":{"default":{"default":"liegendes m"}}},{"category":"Ll","key":"1D24","mappings":{"default":{"default":"stimmhaft laryngaler Frikativ"}}},{"category":"Ll","key":"1D25","mappings":{"default":{"default":"Ain"}}},{"category":"Ll","key":"1D6B","mappings":{"default":{"default":"ue"}}},{"category":"Ll","key":"1D6C","mappings":{"default":{"default":"b mit mittlerer Tilde"}}},{"category":"Ll","key":"1D6D","mappings":{"default":{"default":"d mit mittlerer Tilde"}}},{"category":"Ll","key":"1D6E","mappings":{"default":{"default":"f mit mittlerer Tilde"}}},{"category":"Ll","key":"1D6F","mappings":{"default":{"default":"m mit mittlerer Tilde"}}},{"category":"Ll","key":"1D70","mappings":{"default":{"default":"n mit mittlerer Tilde"}}},{"category":"Ll","key":"1D71","mappings":{"default":{"default":"p mit mittlerer Tilde"}}},{"category":"Ll","key":"1D72","mappings":{"default":{"default":"r mit mittlerer Tilde"}}},{"category":"Ll","key":"1D73","mappings":{"default":{"default":"r mit Angelhaken und mittlerer Tilde"}}},{"category":"Ll","key":"1D74","mappings":{"default":{"default":"s mit mittlerer Tilde"}}},{"category":"Ll","key":"1D75","mappings":{"default":{"default":"t mit mittlerer Tilde"}}},{"category":"Ll","key":"1D76","mappings":{"default":{"default":"z mit mittlerer Tilde"}}},{"category":"Ll","key":"1D77","mappings":{"default":{"default":"gedreht G"}}},{"category":"Ll","key":"1D79","mappings":{"default":{"default":"insular G"}}},{"category":"Ll","key":"1D7A","mappings":{"default":{"default":"th mit Durchgestrichen"}}},{"category":"Ll","key":"1D80","mappings":{"default":{"default":"b mit Palatalhaken"}}},{"category":"Ll","key":"1D81","mappings":{"default":{"default":"d mit Palatalhaken"}}},{"category":"Ll","key":"1D82","mappings":{"default":{"default":"f mit Palatalhaken"}}},{"category":"Ll","key":"1D83","mappings":{"default":{"default":"g mit Palatalhaken"}}},{"category":"Ll","key":"1D84","mappings":{"default":{"default":"k mit Palatalhaken"}}},{"category":"Ll","key":"1D85","mappings":{"default":{"default":"l mit Palatalhaken"}}},{"category":"Ll","key":"1D86","mappings":{"default":{"default":"m mit Palatalhaken"}}},{"category":"Ll","key":"1D87","mappings":{"default":{"default":"n mit Palatalhaken"}}},{"category":"Ll","key":"1D88","mappings":{"default":{"default":"p mit Palatalhaken"}}},{"category":"Ll","key":"1D89","mappings":{"default":{"default":"r mit Palatalhaken"}}},{"category":"Ll","key":"1D8A","mappings":{"default":{"default":"s mit Palatalhaken"}}},{"category":"Ll","key":"1D8B","mappings":{"default":{"default":"esh mit Palatalhaken"}}},{"category":"Ll","key":"1D8C","mappings":{"default":{"default":"v mit Palatalhaken"}}},{"category":"Ll","key":"1D8D","mappings":{"default":{"default":"x mit Palatalhaken"}}},{"category":"Ll","key":"1D8E","mappings":{"default":{"default":"z mit Palatalhaken"}}},{"category":"Ll","key":"1D8F","mappings":{"default":{"default":"a mit Retroflexhaken"}}},{"category":"Ll","key":"1D90","mappings":{"default":{"default":"alpha mit Retroflexhaken"}}},{"category":"Ll","key":"1D91","mappings":{"default":{"default":"d mit Haken und Schwanz"}}},{"category":"Ll","key":"1D92","mappings":{"default":{"default":"e mit Retroflexhaken"}}},{"category":"Ll","key":"1D93","mappings":{"default":{"default":"offenes e mit Retroflexhaken"}}},{"category":"Ll","key":"1D94","mappings":{"default":{"default":"umgedrehtes offenes e mit Retroflexhaken"}}},{"category":"Ll","key":"1D95","mappings":{"default":{"default":"schwa mit Retroflexhaken"}}},{"category":"Ll","key":"1D96","mappings":{"default":{"default":"i mit Retroflexhaken"}}},{"category":"Ll","key":"1D97","mappings":{"default":{"default":"offenes o mit Retroflexhaken"}}},{"category":"Ll","key":"1D98","mappings":{"default":{"default":"esh mit Retroflexhaken"}}},{"category":"Ll","key":"1D99","mappings":{"default":{"default":"u mit Retroflexhaken"}}},{"category":"Ll","key":"1D9A","mappings":{"default":{"default":"ezh mit Retroflexhaken"}}},{"key":"0149","mappings":{"default":{"default":"n mit Apostroph davor"}},"category":"Ll"},{"key":"014B","mappings":{"default":{"default":"eng"}},"category":"Ll"}]
,
"de/symbols/latin-lower-single-accent.js": 
[{"locale":"de"},{"key":"00E0","mappings":{"default":{"default":"a mit Grave"}},"category":"Ll"},{"key":"00E1","mappings":{"default":{"default":"a mit Aigu"}},"category":"Ll"},{"key":"00E2","mappings":{"default":{"default":"a mit Zirkumflex"}},"category":"Ll"},{"key":"00E3","mappings":{"default":{"default":"a mit Tilde"}},"category":"Ll"},{"key":"00E4","mappings":{"default":{"default":"a mit Diärese"}},"category":"Ll"},{"key":"00E5","mappings":{"default":{"default":"a mit Ring darüber"}},"category":"Ll"},{"key":"00E7","mappings":{"default":{"default":"c mit Cedille"}},"category":"Ll"},{"key":"00E8","mappings":{"default":{"default":"e mit Grave"}},"category":"Ll"},{"key":"00E9","mappings":{"default":{"default":"e mit Aigu"}},"category":"Ll"},{"key":"00EA","mappings":{"default":{"default":"e mit Zirkumflex"}},"category":"Ll"},{"key":"00EB","mappings":{"default":{"default":"e mit Diärese"}},"category":"Ll"},{"key":"00EC","mappings":{"default":{"default":"i mit Grave"}},"category":"Ll"},{"key":"00ED","mappings":{"default":{"default":"i mit Aigu"}},"category":"Ll"},{"key":"00EE","mappings":{"default":{"default":"i mit Zirkumflex"}},"category":"Ll"},{"key":"00EF","mappings":{"default":{"default":"i mit Diärese"}},"category":"Ll"},{"key":"00F1","mappings":{"default":{"default":"n mit Tilde"}},"category":"Ll"},{"key":"00F2","mappings":{"default":{"default":"o mit Grave"}},"category":"Ll"},{"key":"00F3","mappings":{"default":{"default":"o mit Aigu"}},"category":"Ll"},{"key":"00F4","mappings":{"default":{"default":"o mit Zirkumflex"}},"category":"Ll"},{"key":"00F5","mappings":{"default":{"default":"o mit Tilde"}},"category":"Ll"},{"key":"00F6","mappings":{"default":{"default":"o mit Diärese"}},"category":"Ll"},{"key":"00F9","mappings":{"default":{"default":"u mit Grave"}},"category":"Ll"},{"key":"00FA","mappings":{"default":{"default":"u mit Aigu"}},"category":"Ll"},{"key":"00FB","mappings":{"default":{"default":"u mit Zirkumflex"}},"category":"Ll"},{"key":"00FC","mappings":{"default":{"default":"u mit Diärese"}},"category":"Ll"},{"key":"00FD","mappings":{"default":{"default":"y mit Aigu"}},"category":"Ll"},{"key":"00FF","mappings":{"default":{"default":"y mit Diärese"}},"category":"Ll"},{"category":"Ll","key":"0101","mappings":{"default":{"default":"a mit Makron"}}},{"category":"Ll","key":"0103","mappings":{"default":{"default":"a mit Breve"}}},{"category":"Ll","key":"0105","mappings":{"default":{"default":"a mit Ogonek"}}},{"key":"0107","mappings":{"default":{"default":"c mit Aigu"}},"category":"Ll"},{"key":"0109","mappings":{"default":{"default":"c mit Zirkumflex"}},"category":"Ll"},{"key":"010B","mappings":{"default":{"default":"c mit Punkt darüber"}},"category":"Ll"},{"category":"Ll","key":"010D","mappings":{"default":{"default":"c mit Caron"}}},{"category":"Ll","key":"010F","mappings":{"default":{"default":"d mit Caron"}}},{"category":"Ll","key":"0113","mappings":{"default":{"default":"e mit Makron"}}},{"category":"Ll","key":"0115","mappings":{"default":{"default":"e mit Breve"}}},{"key":"0117","mappings":{"default":{"default":"e mit Punkt darüber"}},"category":"Ll"},{"category":"Ll","key":"0119","mappings":{"default":{"default":"e mit Ogonek"}}},{"category":"Ll","key":"011B","mappings":{"default":{"default":"e mit Caron"}}},{"key":"011D","mappings":{"default":{"default":"g mit Zirkumflex"}},"category":"Ll"},{"category":"Ll","key":"011F","mappings":{"default":{"default":"g mit Breve"}}},{"key":"0121","mappings":{"default":{"default":"g mit Punkt darüber"}},"category":"Ll"},{"category":"Ll","key":"0123","mappings":{"default":{"default":"g mit Cedilla"}}},{"key":"0125","mappings":{"default":{"default":"h mit Zirkumflex"}},"category":"Ll"},{"key":"0129","mappings":{"default":{"default":"i mit Tilde"}},"category":"Ll"},{"category":"Ll","key":"012B","mappings":{"default":{"default":"i mit Makron"}}},{"category":"Ll","key":"012D","mappings":{"default":{"default":"i mit Breve"}}},{"category":"Ll","key":"012F","mappings":{"default":{"default":"i mit Ogonek"}}},{"key":"0131","mappings":{"default":{"default":"i ohne punkt"}},"category":"Ll"},{"key":"0135","mappings":{"default":{"default":"j mit Zirkumflex"}},"category":"Ll"},{"key":"0137","mappings":{"default":{"default":"k mit Cedille"}},"category":"Ll"},{"key":"013A","mappings":{"default":{"default":"l mit Aigu"}},"category":"Ll"},{"key":"013C","mappings":{"default":{"default":"l mit Cedille"}},"category":"Ll"},{"category":"Ll","key":"013E","mappings":{"default":{"default":"l mit Caron"}}},{"key":"0140","mappings":{"default":{"default":"l mit Punkt in der Mitte"}},"category":"Ll"},{"key":"0144","mappings":{"default":{"default":"n mit Aigu"}},"category":"Ll"},{"key":"0146","mappings":{"default":{"default":"n mit Cedille"}},"category":"Ll"},{"category":"Ll","key":"0148","mappings":{"default":{"default":"n mit Caron"}}},{"category":"Ll","key":"014D","mappings":{"default":{"default":"o mit Makron"}}},{"category":"Ll","key":"014F","mappings":{"default":{"default":"o mit Breve"}}},{"key":"0151","mappings":{"default":{"default":"o mit doppeltem Aigu"}},"category":"Ll"},{"key":"0155","mappings":{"default":{"default":"r mit Aigu"}},"category":"Ll"},{"key":"0157","mappings":{"default":{"default":"r mit Cedille"}},"category":"Ll"},{"category":"Ll","key":"0159","mappings":{"default":{"default":"r mit Caron"}}},{"key":"015B","mappings":{"default":{"default":"s mit Aigu"}},"category":"Ll"},{"key":"015D","mappings":{"default":{"default":"s mit Zirkumflex"}},"category":"Ll"},{"key":"015F","mappings":{"default":{"default":"s mit Cedille"}},"category":"Ll"},{"category":"Ll","key":"0161","mappings":{"default":{"default":"s mit Caron"}}},{"key":"0163","mappings":{"default":{"default":"t mit Cedille"}},"category":"Ll"},{"category":"Ll","key":"0165","mappings":{"default":{"default":"t mit Caron"}}},{"key":"0169","mappings":{"default":{"default":"u mit Tilde"}},"category":"Ll"},{"category":"Ll","key":"016B","mappings":{"default":{"default":"u mit Makron"}}},{"category":"Ll","key":"016D","mappings":{"default":{"default":"u mit Breve"}}},{"key":"016F","mappings":{"default":{"default":"u mit Ring darüber"}},"category":"Ll"},{"key":"0171","mappings":{"default":{"default":"u mit doppeltem Aigu"}},"category":"Ll"},{"category":"Ll","key":"0173","mappings":{"default":{"default":"u mit Ogonek"}}},{"key":"0175","mappings":{"default":{"default":"w mit Zirkumflex"}},"category":"Ll"},{"key":"0177","mappings":{"default":{"default":"y mit Zirkumflex"}},"category":"Ll"},{"key":"017A","mappings":{"default":{"default":"z mit Aigu"}},"category":"Ll"},{"key":"017C","mappings":{"default":{"default":"z mit Punkt darüber"}},"category":"Ll"},{"category":"Ll","key":"017E","mappings":{"default":{"default":"z mit Caron"}}},{"category":"Ll","key":"01CE","mappings":{"default":{"default":"a mit Caron"}}},{"category":"Ll","key":"01D0","mappings":{"default":{"default":"i mit Caron"}}},{"category":"Ll","key":"01D2","mappings":{"default":{"default":"o mit Caron"}}},{"category":"Ll","key":"01D4","mappings":{"default":{"default":"u mit Caron"}}},{"category":"Ll","key":"01E7","mappings":{"default":{"default":"g mit Caron"}}},{"category":"Ll","key":"01E9","mappings":{"default":{"default":"k mit Caron"}}},{"category":"Ll","key":"01EB","mappings":{"default":{"default":"o mit Ogonek"}}},{"category":"Ll","key":"01F0","mappings":{"default":{"default":"j mit Caron"}}},{"key":"01F5","mappings":{"default":{"default":"g mit Aigu"}},"category":"Ll"},{"category":"Ll","key":"01F9","mappings":{"default":{"default":"n mit Grab"}}},{"category":"Ll","key":"0201","mappings":{"default":{"default":"a mit Doppelgrab"}}},{"category":"Ll","key":"0203","mappings":{"default":{"default":"a mit umgekehrter Breve"}}},{"category":"Ll","key":"0205","mappings":{"default":{"default":"e mit Doppelgrab"}}},{"category":"Ll","key":"0207","mappings":{"default":{"default":"e mit umgekehrter Breve"}}},{"category":"Ll","key":"0209","mappings":{"default":{"default":"i mit Doppelgrab"}}},{"category":"Ll","key":"020B","mappings":{"default":{"default":"i mit umgekehrter Breve"}}},{"category":"Ll","key":"020D","mappings":{"default":{"default":"o mit Doppelgrab"}}},{"category":"Ll","key":"020F","mappings":{"default":{"default":"o mit umgekehrter Breve"}}},{"category":"Ll","key":"0211","mappings":{"default":{"default":"r mit Doppelgrab"}}},{"category":"Ll","key":"0213","mappings":{"default":{"default":"r mit umgekehrter Breve"}}},{"category":"Ll","key":"0215","mappings":{"default":{"default":"u mit Doppelgrab"}}},{"category":"Ll","key":"0217","mappings":{"default":{"default":"u mit umgekehrter Breve"}}},{"category":"Ll","key":"0219","mappings":{"default":{"default":"s mit untenstehendem Komma"}}},{"category":"Ll","key":"021B","mappings":{"default":{"default":"t mit untenstehendem Komma"}}},{"category":"Ll","key":"021F","mappings":{"default":{"default":"h mit Caron"}}},{"category":"Ll","key":"0227","mappings":{"default":{"default":"a mit Punkt oben"}}},{"category":"Ll","key":"0229","mappings":{"default":{"default":"e mit Cedilla"}}},{"category":"Ll","key":"022F","mappings":{"default":{"default":"o mit Punkt oben"}}},{"category":"Ll","key":"0233","mappings":{"default":{"default":"y mit Makron"}}},{"category":"Ll","key":"0237","mappings":{"default":{"default":"dotless J"}}},{"category":"Ll","key":"1E01","mappings":{"default":{"default":"a mit Ring unten"}}},{"category":"Ll","key":"1E03","mappings":{"default":{"default":"b mit Punkt oben"}}},{"category":"Ll","key":"1E05","mappings":{"default":{"default":"b mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E07","mappings":{"default":{"default":"b mit Zeile darunter"}}},{"category":"Ll","key":"1E0B","mappings":{"default":{"default":"d mit Punkt oben"}}},{"category":"Ll","key":"1E0D","mappings":{"default":{"default":"d mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E0F","mappings":{"default":{"default":"d mit Zeile darunter"}}},{"category":"Ll","key":"1E11","mappings":{"default":{"default":"d mit Cedilla"}}},{"category":"Ll","key":"1E13","mappings":{"default":{"default":"d mit darunterem Zirkumflex"}}},{"category":"Ll","key":"1E19","mappings":{"default":{"default":"e mit darunterem Zirkumflex"}}},{"category":"Ll","key":"1E1B","mappings":{"default":{"default":"e mit darunterliegender Tilde"}}},{"category":"Ll","key":"1E1F","mappings":{"default":{"default":"f mit Punkt oben"}}},{"category":"Ll","key":"1E21","mappings":{"default":{"default":"g mit Makron"}}},{"category":"Ll","key":"1E23","mappings":{"default":{"default":"h mit Punkt oben"}}},{"category":"Ll","key":"1E25","mappings":{"default":{"default":"h mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E27","mappings":{"default":{"default":"h mit Diärese"}}},{"category":"Ll","key":"1E29","mappings":{"default":{"default":"h mit Cedilla"}}},{"category":"Ll","key":"1E2B","mappings":{"default":{"default":"h mit Breve unten"}}},{"category":"Ll","key":"1E2D","mappings":{"default":{"default":"i mit darunterliegender Tilde"}}},{"category":"Ll","key":"1E31","mappings":{"default":{"default":"k mit Akut"}}},{"category":"Ll","key":"1E33","mappings":{"default":{"default":"k mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E35","mappings":{"default":{"default":"k mit Zeile darunter"}}},{"category":"Ll","key":"1E37","mappings":{"default":{"default":"l mit Punkt unten"}}},{"category":"Ll","key":"1E3B","mappings":{"default":{"default":"l mit Zeile darunter"}}},{"category":"Ll","key":"1E3D","mappings":{"default":{"default":"l mit darunterem Zirkumflex"}}},{"category":"Ll","key":"1E3F","mappings":{"default":{"default":"m mit Akut"}}},{"category":"Ll","key":"1E41","mappings":{"default":{"default":"m mit Punkt oben"}}},{"category":"Ll","key":"1E43","mappings":{"default":{"default":"m mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E45","mappings":{"default":{"default":"n mit Punkt oben"}}},{"category":"Ll","key":"1E47","mappings":{"default":{"default":"n mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E49","mappings":{"default":{"default":"n mit Zeile darunter"}}},{"category":"Ll","key":"1E4B","mappings":{"default":{"default":"n mit darunterem Zirkumflex"}}},{"category":"Ll","key":"1E55","mappings":{"default":{"default":"p mit Akut"}}},{"category":"Ll","key":"1E57","mappings":{"default":{"default":"p mit Punkt oben"}}},{"category":"Ll","key":"1E59","mappings":{"default":{"default":"r mit Punkt oben"}}},{"category":"Ll","key":"1E5B","mappings":{"default":{"default":"r mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E5F","mappings":{"default":{"default":"r mit Zeile darunter"}}},{"category":"Ll","key":"1E61","mappings":{"default":{"default":"s mit Punkt oben"}}},{"category":"Ll","key":"1E63","mappings":{"default":{"default":"s mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E6B","mappings":{"default":{"default":"t mit Punkt oben"}}},{"category":"Ll","key":"1E6D","mappings":{"default":{"default":"t mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E6F","mappings":{"default":{"default":"t mit Zeile darunter"}}},{"category":"Ll","key":"1E71","mappings":{"default":{"default":"t mit darunterem Zirkumflex"}}},{"category":"Ll","key":"1E73","mappings":{"default":{"default":"u mit untenstehender Diärese"}}},{"category":"Ll","key":"1E75","mappings":{"default":{"default":"u mit darunterliegender Tilde"}}},{"category":"Ll","key":"1E77","mappings":{"default":{"default":"u mit darunterem Zirkumflex"}}},{"category":"Ll","key":"1E7D","mappings":{"default":{"default":"v mit Tilde"}}},{"category":"Ll","key":"1E7F","mappings":{"default":{"default":"v mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E81","mappings":{"default":{"default":"w mit Grab"}}},{"category":"Ll","key":"1E83","mappings":{"default":{"default":"w mit Akut"}}},{"category":"Ll","key":"1E85","mappings":{"default":{"default":"w mit Diärese"}}},{"category":"Ll","key":"1E87","mappings":{"default":{"default":"w mit Punkt oben"}}},{"category":"Ll","key":"1E89","mappings":{"default":{"default":"w mit Punkt unten"}}},{"category":"Ll","key":"1E8B","mappings":{"default":{"default":"x mit Punkt oben"}}},{"category":"Ll","key":"1E8D","mappings":{"default":{"default":"x mit Diärese"}}},{"category":"Ll","key":"1E8F","mappings":{"default":{"default":"y mit Punkt oben"}}},{"category":"Ll","key":"1E91","mappings":{"default":{"default":"z mit Zirkumflex"}}},{"category":"Ll","key":"1E93","mappings":{"default":{"default":"z mit untenstehendem Punkt"}}},{"category":"Ll","key":"1E95","mappings":{"default":{"default":"z mit Zeile darunter"}}},{"category":"Ll","key":"1E96","mappings":{"default":{"default":"h mit Zeile darunter"}}},{"category":"Ll","key":"1E97","mappings":{"default":{"default":"t mit Diärese"}}},{"category":"Ll","key":"1E98","mappings":{"default":{"default":"w mit Ring oben"}}},{"category":"Ll","key":"1E99","mappings":{"default":{"default":"y mit Ring oben"}}},{"category":"Ll","key":"1E9A","mappings":{"default":{"default":"a mit rechtem Halbring"}}},{"category":"Ll","key":"1EA1","mappings":{"default":{"default":"a mit Punkt unten"}}},{"category":"Ll","key":"1EA3","mappings":{"default":{"default":"a mit Haken oben"}}},{"category":"Ll","key":"1EB9","mappings":{"default":{"default":"e mit untenstehendem Punkt"}}},{"category":"Ll","key":"1EBB","mappings":{"default":{"default":"e mit Haken oben"}}},{"category":"Ll","key":"1EBD","mappings":{"default":{"default":"e mit Tilde"}}},{"category":"Ll","key":"1EC9","mappings":{"default":{"default":"i mit Haken oben"}}},{"category":"Ll","key":"1ECB","mappings":{"default":{"default":"i mit untenstehendem Punkt"}}},{"category":"Ll","key":"1ECD","mappings":{"default":{"default":"o mit untenstehendem Punkt"}}},{"category":"Ll","key":"1ECF","mappings":{"default":{"default":"o mit Haken oben"}}},{"category":"Ll","key":"1EE5","mappings":{"default":{"default":"u mit untenstehendem Punkt"}}},{"category":"Ll","key":"1EE7","mappings":{"default":{"default":"u mit Haken oben"}}},{"category":"Ll","key":"1EF3","mappings":{"default":{"default":"y mit Grab"}}},{"category":"Ll","key":"1EF5","mappings":{"default":{"default":"y mit untenstehendem Punkt"}}},{"category":"Ll","key":"1EF7","mappings":{"default":{"default":"y mit Haken oben"}}},{"category":"Ll","key":"1EF9","mappings":{"default":{"default":"y mit Tilde"}}}]
,
"de/symbols/latin-rest.js": 
[{"locale":"de"},{"category":"Ll","key":"210E","mappings":{"default":{"default":"Planck-Konstante"}}},{"category":"Mn","key":"0363","mappings":{"default":{"default":"kombinierendes a"}}},{"category":"Mn","key":"0364","mappings":{"default":{"default":"kombinierendes e"}}},{"category":"Mn","key":"0365","mappings":{"default":{"default":"kombinierendes i"}}},{"category":"Mn","key":"0366","mappings":{"default":{"default":"kombinierendes o"}}},{"category":"Mn","key":"0367","mappings":{"default":{"default":"kombinierendes u"}}},{"category":"Mn","key":"0368","mappings":{"default":{"default":"kombinierendes c"}}},{"category":"Mn","key":"0369","mappings":{"default":{"default":"kombinierendes d"}}},{"category":"Mn","key":"036A","mappings":{"default":{"default":"kombinierendes h"}}},{"category":"Mn","key":"036B","mappings":{"default":{"default":"kombinierendes m"}}},{"category":"Mn","key":"036C","mappings":{"default":{"default":"kombinierendes r"}}},{"category":"Mn","key":"036D","mappings":{"default":{"default":"kombinierendes t"}}},{"category":"Mn","key":"036E","mappings":{"default":{"default":"kombinierendes v"}}},{"category":"Mn","key":"036F","mappings":{"default":{"default":"kombinierendes x"}}},{"category":"Lm","key":"1D62","mappings":{"default":{"default":"tiefgestelltes i"}}},{"category":"Lm","key":"1D63","mappings":{"default":{"default":"tiefgestelltes r"}}},{"category":"Lm","key":"1D64","mappings":{"default":{"default":"tiefgestelltes u"}}},{"category":"Lm","key":"1D65","mappings":{"default":{"default":"tiefgestelltes v"}}},{"category":"Mn","key":"1DCA","mappings":{"default":{"default":"tiefgestelltes r"}}},{"category":"Mn","key":"1DD3","mappings":{"default":{"default":"kombinierendes oberes w"}}},{"category":"Mn","key":"1DD4","mappings":{"default":{"default":"kombinierendes ae"}}},{"category":"Mn","key":"1DD5","mappings":{"default":{"default":"kombinierendes ao"}}},{"category":"Mn","key":"1DD6","mappings":{"default":{"default":"kombinierendes av"}}},{"category":"Mn","key":"1DD7","mappings":{"default":{"default":"kombinierendes c mit Cedilla"}}},{"category":"Mn","key":"1DD8","mappings":{"default":{"default":"kombinierendes insulares d"}}},{"category":"Mn","key":"1DD9","mappings":{"default":{"default":"kombinierendes eth"}}},{"category":"Mn","key":"1DDA","mappings":{"default":{"default":"kombinierendes g"}}},{"category":"Mn","key":"1DDB","mappings":{"default":{"default":"kombinierendes Kapitälchen g"}}},{"category":"Mn","key":"1DDC","mappings":{"default":{"default":"kombinierendes k"}}},{"category":"Mn","key":"1DDD","mappings":{"default":{"default":"kombinierendes l"}}},{"category":"Mn","key":"1DDE","mappings":{"default":{"default":"kombinierendes Kapitälchen L"}}},{"category":"Mn","key":"1DDF","mappings":{"default":{"default":"kombinierendes Kapitälchen M"}}},{"category":"Mn","key":"1DE0","mappings":{"default":{"default":"kombinierendes n"}}},{"category":"Mn","key":"1DE1","mappings":{"default":{"default":"kombinierendes Kapitälchen N"}}},{"category":"Mn","key":"1DE2","mappings":{"default":{"default":"kombinierendes lateinisches Kapitälchen R"}}},{"category":"Mn","key":"1DE3","mappings":{"default":{"default":"kombinierendes r rotunda"}}},{"category":"Mn","key":"1DE4","mappings":{"default":{"default":"kombinierendes s"}}},{"category":"Mn","key":"1DE5","mappings":{"default":{"default":"kombinierendes langes s"}}},{"category":"Mn","key":"1DE6","mappings":{"default":{"default":"kombinierendes z"}}},{"category":"Lm","key":"2071","mappings":{"default":{"default":"hochgestelltes i"}}},{"category":"Lm","key":"207F","mappings":{"default":{"default":"hochgestelltes n"}}},{"category":"Lm","key":"2090","mappings":{"default":{"default":"tiefgestelltes a"}}},{"category":"Lm","key":"2091","mappings":{"default":{"default":"tiefgestelltes e"}}},{"category":"Lm","key":"2092","mappings":{"default":{"default":"tiefgestelltes o"}}},{"category":"Lm","key":"2093","mappings":{"default":{"default":"tiefgestelltes x"}}},{"category":"Lm","key":"2094","mappings":{"default":{"default":"tiefgestelltes schwa"}}},{"category":"Lm","key":"2095","mappings":{"default":{"default":"tiefgestelltes h"}}},{"category":"Lm","key":"2096","mappings":{"default":{"default":"tiefgestelltes k"}}},{"category":"Lm","key":"2097","mappings":{"default":{"default":"tiefgestelltes l"}}},{"category":"Lm","key":"2098","mappings":{"default":{"default":"tiefgestelltes m"}}},{"category":"Lm","key":"2099","mappings":{"default":{"default":"tiefgestelltes n"}}},{"category":"Lm","key":"209A","mappings":{"default":{"default":"tiefgestelltes p"}}},{"category":"Lm","key":"209B","mappings":{"default":{"default":"tiefgestelltes s"}}},{"category":"Lm","key":"209C","mappings":{"default":{"default":"tiefgestelltes t"}}},{"category":"Lm","key":"2C7C","mappings":{"default":{"default":"tiefgestelltes j"}}},{"category":"So","key":"1F12A","mappings":{"default":{"default":"groß S in stumpfen Klammern"}}},{"category":"So","key":"1F12B","mappings":{"default":{"default":"eingekreistes kursives groß C"}}},{"category":"So","key":"1F12C","mappings":{"default":{"default":"eingekreistes kursives groß R"}}},{"category":"So","key":"1F18A","mappings":{"default":{"default":"durchgestrichenes P auf schwarzen Quadrat"}}}]
,
"de/symbols/latin-upper-double-accent.js": 
[{"locale":"de"},{"category":"Lu","key":"01D5","mappings":{"default":{"default":"groß U mit Diärese und Makron"}}},{"category":"Lu","key":"01D7","mappings":{"default":{"default":"groß U mit Diärese und Akut"}}},{"category":"Lu","key":"01D9","mappings":{"default":{"default":"groß U mit Diärese und Caron"}}},{"category":"Lu","key":"01DB","mappings":{"default":{"default":"groß U mit Diärese und Grab"}}},{"category":"Lu","key":"01DE","mappings":{"default":{"default":"groß A mit Diärese und Makron"}}},{"category":"Lu","key":"01E0","mappings":{"default":{"default":"groß A mit Dot Above und Makron"}}},{"category":"Lu","key":"01EC","mappings":{"default":{"default":"groß O mit Ogonek und Makron"}}},{"category":"Lu","key":"01FA","mappings":{"default":{"default":"groß A mit Ring oben und akut"}}},{"category":"Lu","key":"022A","mappings":{"default":{"default":"groß O mit Diärese und Makron"}}},{"category":"Lu","key":"022C","mappings":{"default":{"default":"groß O mit Tilde und Makron"}}},{"category":"Lu","key":"0230","mappings":{"default":{"default":"groß O mit Punkt oben und Makron"}}},{"category":"Lu","key":"1E08","mappings":{"default":{"default":"groß C mit Cedilla und Akut"}}},{"category":"Lu","key":"1E14","mappings":{"default":{"default":"groß E mit Makron und Grab"}}},{"category":"Lu","key":"1E16","mappings":{"default":{"default":"groß E mit Makron und Akut"}}},{"category":"Lu","key":"1E1C","mappings":{"default":{"default":"groß E mit Cedilla und Breve"}}},{"category":"Lu","key":"1E2E","mappings":{"default":{"default":"groß I mit Diärese und Akut"}}},{"category":"Lu","key":"1E38","mappings":{"default":{"default":"groß L mit Dot Below und Makron"}}},{"category":"Lu","key":"1E4C","mappings":{"default":{"default":"groß O mit Tilde und Akut"}}},{"category":"Lu","key":"1E4E","mappings":{"default":{"default":"groß O mit Tilde und Diärese"}}},{"category":"Lu","key":"1E50","mappings":{"default":{"default":"groß O mit Makron und Grab"}}},{"category":"Lu","key":"1E52","mappings":{"default":{"default":"groß O mit Makron und Akut"}}},{"category":"Lu","key":"1E5C","mappings":{"default":{"default":"groß R mit Punkt unten und Makron"}}},{"category":"Lu","key":"1E64","mappings":{"default":{"default":"groß S mit Akute und Punkt oben"}}},{"category":"Lu","key":"1E66","mappings":{"default":{"default":"groß S mit Caron und Punkt oben"}}},{"category":"Lu","key":"1E68","mappings":{"default":{"default":"groß S mit untenstehendem Punkt und oben stehendem Punkt"}}},{"category":"Lu","key":"1E78","mappings":{"default":{"default":"groß U mit Tilde und Akut"}}},{"category":"Lu","key":"1E7A","mappings":{"default":{"default":"groß U mit Makron und Diärese"}}},{"category":"Lu","key":"1EA4","mappings":{"default":{"default":"groß A mit Zirkumflex und Akut"}}},{"category":"Lu","key":"1EA6","mappings":{"default":{"default":"groß A mit Zirkumflex und Grab"}}},{"category":"Lu","key":"1EA8","mappings":{"default":{"default":"groß A mit Zirkumflex und Haken oben"}}},{"category":"Lu","key":"1EAA","mappings":{"default":{"default":"groß A mit Circumflex und Tilde"}}},{"category":"Lu","key":"1EAC","mappings":{"default":{"default":"groß A mit Zirkumflex und Punkt darunter"}}},{"category":"Lu","key":"1EAE","mappings":{"default":{"default":"groß A mit Breve und Akut"}}},{"category":"Lu","key":"1EB0","mappings":{"default":{"default":"groß A mit Breve und Grab"}}},{"category":"Lu","key":"1EB2","mappings":{"default":{"default":"groß A mit Breve und Hook oben"}}},{"category":"Lu","key":"1EB4","mappings":{"default":{"default":"groß A mit Breve und Tilde"}}},{"category":"Lu","key":"1EB6","mappings":{"default":{"default":"groß A mit Breve und Dot Below"}}},{"category":"Lu","key":"1EBE","mappings":{"default":{"default":"groß E mit Zirkumflex und Akut"}}},{"category":"Lu","key":"1EC0","mappings":{"default":{"default":"groß E mit Zirkumflex und Grab"}}},{"category":"Lu","key":"1EC2","mappings":{"default":{"default":"groß E mit Zirkumflex und Haken oben"}}},{"category":"Lu","key":"1EC4","mappings":{"default":{"default":"groß E mit Zirkumflex und Tilde"}}},{"category":"Lu","key":"1EC6","mappings":{"default":{"default":"groß E mit Zirkumflex und Punkt darunter"}}},{"category":"Lu","key":"1ED0","mappings":{"default":{"default":"groß O mit Circumflex und Akut"}}},{"category":"Lu","key":"1ED2","mappings":{"default":{"default":"groß O mit Zirkumflex und Grab"}}},{"category":"Lu","key":"1ED4","mappings":{"default":{"default":"groß O mit Zirkumflex und Haken oben"}}},{"category":"Lu","key":"1ED6","mappings":{"default":{"default":"groß O mit Zirkumflex und Tilde"}}},{"category":"Lu","key":"1ED8","mappings":{"default":{"default":"groß O mit Zirkumflex und Punkt darunter"}}},{"category":"Lu","key":"1EDA","mappings":{"default":{"default":"groß O mit Horn und Akut"}}},{"category":"Lu","key":"1EDC","mappings":{"default":{"default":"groß O mit Horn und Grab"}}},{"category":"Lu","key":"1EDE","mappings":{"default":{"default":"groß O mit Horn und Haken oben"}}},{"category":"Lu","key":"1EE0","mappings":{"default":{"default":"groß O mit Horn und Tilde"}}},{"category":"Lu","key":"1EE2","mappings":{"default":{"default":"groß O mit Horn und Punkt"}}},{"category":"Lu","key":"1EE8","mappings":{"default":{"default":"groß U mit Horn und Akut"}}},{"category":"Lu","key":"1EEA","mappings":{"default":{"default":"groß U mit Horn und Grab"}}},{"category":"Lu","key":"1EEC","mappings":{"default":{"default":"groß U mit Horn und Haken oben"}}},{"category":"Lu","key":"1EEE","mappings":{"default":{"default":"groß U mit Horn und Tilde"}}},{"category":"Lu","key":"1EF0","mappings":{"default":{"default":"groß U mit Horn und Punkt"}}}]
,
"de/symbols/latin-upper-single-accent.js": 
[{"locale":"de"},{"key":"00C0","mappings":{"default":{"default":"groß A mit Gravis"}},"category":"Lu"},{"key":"00C1","mappings":{"default":{"default":"groß A mit Aigu"}},"category":"Lu"},{"key":"00C2","mappings":{"default":{"default":"groß A mit Zirkumflex"}},"category":"Lu"},{"key":"00C3","mappings":{"default":{"default":"groß A mit Tilde"}},"category":"Lu"},{"key":"00C4","mappings":{"default":{"default":"groß A mit Diärese"}},"category":"Lu"},{"key":"00C5","mappings":{"default":{"default":"groß A mit Ring darüber"}},"category":"Lu"},{"key":"00C7","mappings":{"default":{"default":"groß C mit Cedille"}},"category":"Lu"},{"key":"00C8","mappings":{"default":{"default":"groß E mit Gravis"}},"category":"Lu"},{"key":"00C9","mappings":{"default":{"default":"groß E mit Aigu"}},"category":"Lu"},{"key":"00CA","mappings":{"default":{"default":"groß E mit Zirkumflex"}},"category":"Lu"},{"key":"00CB","mappings":{"default":{"default":"groß E mit Diärese"}},"category":"Lu"},{"key":"00CC","mappings":{"default":{"default":"groß I mit Gravis"}},"category":"Lu"},{"key":"00CD","mappings":{"default":{"default":"groß I mit Aigu"}},"category":"Lu"},{"key":"00CE","mappings":{"default":{"default":"groß I mit Zirkumflex"}},"category":"Lu"},{"key":"00CF","mappings":{"default":{"default":"groß I mit Diärese"}},"category":"Lu"},{"key":"00D1","mappings":{"default":{"default":"großes n mit Tilde"}},"category":"Lu"},{"key":"00D2","mappings":{"default":{"default":"großes o mit Gravis"}},"category":"Lu"},{"key":"00D3","mappings":{"default":{"default":"groß O mit Aigu"}},"category":"Lu"},{"key":"00D4","mappings":{"default":{"default":"groß O mit Zirkumflex"}},"category":"Lu"},{"key":"00D5","mappings":{"default":{"default":"groß O mit Tilde"}},"category":"Lu"},{"key":"00D6","mappings":{"default":{"default":"groß O mit Diärese"}},"category":"Lu"},{"key":"00D9","mappings":{"default":{"default":"groß U mit Gravis"}},"category":"Lu"},{"key":"00DA","mappings":{"default":{"default":"groß U mit Aigu"}},"category":"Lu"},{"key":"00DB","mappings":{"default":{"default":"groß U mit Zirkumflex"}},"category":"Lu"},{"key":"00DC","mappings":{"default":{"default":"groß U mit Diärese"}},"category":"Lu"},{"key":"00DD","mappings":{"default":{"default":"groß Y mit Aigu"}},"category":"Lu"},{"category":"Lu","key":"0100","mappings":{"default":{"default":"groß A mit Makron"}}},{"category":"Lu","key":"0102","mappings":{"default":{"default":"groß A mit Breve"}}},{"category":"Lu","key":"0104","mappings":{"default":{"default":"groß A mit Ogonek"}}},{"key":"0106","mappings":{"default":{"default":"groß C mit Aigu"}},"category":"Lu"},{"key":"0108","mappings":{"default":{"default":"groß C mit Zirkumflex"}},"category":"Lu"},{"key":"010A","mappings":{"default":{"default":"groß C mit Punkt darüber"}},"category":"Lu"},{"category":"Lu","key":"010C","mappings":{"default":{"default":"groß C mit Caron"}}},{"category":"Lu","key":"010E","mappings":{"default":{"default":"groß D mit Caron"}}},{"category":"Lu","key":"0112","mappings":{"default":{"default":"groß E mit Makron"}}},{"category":"Lu","key":"0114","mappings":{"default":{"default":"groß E mit Breve"}}},{"key":"0116","mappings":{"default":{"default":"groß E mit Punkt darüber"}},"category":"Lu"},{"category":"Lu","key":"0118","mappings":{"default":{"default":"groß E mit Ogonek"}}},{"category":"Lu","key":"011A","mappings":{"default":{"default":"groß E mit Caron"}}},{"key":"011C","mappings":{"default":{"default":"groß G mit Zirkumflex"}},"category":"Lu"},{"category":"Lu","key":"011E","mappings":{"default":{"default":"groß G mit Breve"}}},{"key":"0120","mappings":{"default":{"default":"groß G mit Punkt darüber"}},"category":"Lu"},{"key":"0122","mappings":{"default":{"default":"groß G mit Cedille"}},"category":"Lu"},{"key":"0124","mappings":{"default":{"default":"groß H mit Zirkumflex"}},"category":"Lu"},{"key":"0128","mappings":{"default":{"default":"groß I mit Tilde"}},"category":"Lu"},{"category":"Lu","key":"012A","mappings":{"default":{"default":"groß I mit Makron"}}},{"category":"Lu","key":"012C","mappings":{"default":{"default":"groß I mit Breve"}}},{"category":"Lu","key":"012E","mappings":{"default":{"default":"groß I mit Ogonek"}}},{"key":"0130","mappings":{"default":{"default":"groß I mit Punkt darüber"}},"category":"Lu"},{"key":"0134","mappings":{"default":{"default":"groß J mit Zirkumflex"}},"category":"Lu"},{"key":"0136","mappings":{"default":{"default":"groß K mit Cedille"}},"category":"Lu"},{"key":"0139","mappings":{"default":{"default":"groß L mit Aigu"}},"category":"Lu"},{"key":"013B","mappings":{"default":{"default":"groß L mit Cedille"}},"category":"Lu"},{"category":"Lu","key":"013D","mappings":{"default":{"default":"groß L mit Caron"}}},{"key":"013F","mappings":{"default":{"default":"groß L mit Punkt in der Mitte"}},"category":"Lu"},{"key":"0143","mappings":{"default":{"default":"groß N mit Aigu"}},"category":"Lu"},{"key":"0145","mappings":{"default":{"default":"groß N mit Cedille"}},"category":"Lu"},{"category":"Lu","key":"0147","mappings":{"default":{"default":"groß N mit Caron"}}},{"category":"Lu","key":"014C","mappings":{"default":{"default":"groß O mit Makron"}}},{"category":"Lu","key":"014E","mappings":{"default":{"default":"groß O mit Breve"}}},{"key":"0150","mappings":{"default":{"default":"groß O mit doppeltem Aigu"}},"category":"Lu"},{"key":"0154","mappings":{"default":{"default":"groß R mit Aigu"}},"category":"Lu"},{"key":"0156","mappings":{"default":{"default":"groß R mit Cedille"}},"category":"Lu"},{"category":"Lu","key":"0158","mappings":{"default":{"default":"groß R mit Caron"}}},{"key":"015A","mappings":{"default":{"default":"groß S mit Aigu"}},"category":"Lu"},{"key":"015C","mappings":{"default":{"default":"groß S mit Zirkumflex"}},"category":"Lu"},{"key":"015E","mappings":{"default":{"default":"groß S mit Cedille"}},"category":"Lu"},{"category":"Lu","key":"0160","mappings":{"default":{"default":"groß S mit Caron"}}},{"key":"0162","mappings":{"default":{"default":"groß T mit Cedille"}},"category":"Lu"},{"category":"Lu","key":"0164","mappings":{"default":{"default":"groß T mit Caron"}}},{"key":"0168","mappings":{"default":{"default":"groß U mit Tilde"}},"category":"Lu"},{"category":"Lu","key":"016A","mappings":{"default":{"default":"groß U mit Makron"}}},{"category":"Lu","key":"016C","mappings":{"default":{"default":"groß U mit Breve"}}},{"key":"016E","mappings":{"default":{"default":"groß U mit Ring darüber"}},"category":"Lu"},{"key":"0170","mappings":{"default":{"default":"groß U mit dopppeltem Aigu"}},"category":"Lu"},{"category":"Lu","key":"0172","mappings":{"default":{"default":"groß U mit Ogonek"}}},{"key":"0174","mappings":{"default":{"default":"groß W mit Zirkumflex"}},"category":"Lu"},{"key":"0176","mappings":{"default":{"default":"groß Y mit Zirkumflex"}},"category":"Lu"},{"key":"0178","mappings":{"default":{"default":"groß Y mit Diärese"}},"category":"Lu"},{"key":"0179","mappings":{"default":{"default":"groß Z mit Aigu"}},"category":"Lu"},{"key":"017B","mappings":{"default":{"default":"groß Z mit Punkt darüber"}},"category":"Lu"},{"category":"Lu","key":"017D","mappings":{"default":{"default":"groß Z mit Caron"}}},{"category":"Lu","key":"01CD","mappings":{"default":{"default":"groß A mit Caron"}}},{"category":"Lu","key":"01CF","mappings":{"default":{"default":"groß I mit Caron"}}},{"category":"Lu","key":"01D1","mappings":{"default":{"default":"groß O mit Caron"}}},{"category":"Lu","key":"01D3","mappings":{"default":{"default":"groß U mit Caron"}}},{"category":"Lu","key":"01E6","mappings":{"default":{"default":"groß G mit Caron"}}},{"category":"Lu","key":"01E8","mappings":{"default":{"default":"groß K mit Caron"}}},{"category":"Lu","key":"01EA","mappings":{"default":{"default":"groß O mit Ogonek"}}},{"category":"Lu","key":"01F4","mappings":{"default":{"default":"groß G mit Akut"}}},{"category":"Lu","key":"01F8","mappings":{"default":{"default":"groß N mit Gravis"}}},{"category":"Lu","key":"0200","mappings":{"default":{"default":"groß A mit Doppelgravis"}}},{"category":"Lu","key":"0202","mappings":{"default":{"default":"groß A mit umgekehrter Breve"}}},{"category":"Lu","key":"0204","mappings":{"default":{"default":"groß E mit Doppelgravis"}}},{"category":"Lu","key":"0206","mappings":{"default":{"default":"groß E mit umgekehrter Breve"}}},{"category":"Lu","key":"0208","mappings":{"default":{"default":"groß I mit Doppelgravis"}}},{"category":"Lu","key":"020A","mappings":{"default":{"default":"groß I mit umgekehrter Breve"}}},{"category":"Lu","key":"020C","mappings":{"default":{"default":"groß O mit Doppelgravis"}}},{"category":"Lu","key":"020E","mappings":{"default":{"default":"groß O mit umgekehrter Breve"}}},{"category":"Lu","key":"0210","mappings":{"default":{"default":"groß R mit Doppelgravis"}}},{"category":"Lu","key":"0212","mappings":{"default":{"default":"groß R mit umgekehrter Breve"}}},{"category":"Lu","key":"0214","mappings":{"default":{"default":"groß U mit Doppelgravis"}}},{"category":"Lu","key":"0216","mappings":{"default":{"default":"groß U mit umgekehrter Breve"}}},{"category":"Lu","key":"0218","mappings":{"default":{"default":"groß S mit untenstehendem Komma"}}},{"category":"Lu","key":"021A","mappings":{"default":{"default":"groß T mit untenstehendem Komma"}}},{"category":"Lu","key":"021E","mappings":{"default":{"default":"groß H mit Caron"}}},{"category":"Lu","key":"0226","mappings":{"default":{"default":"groß A mit Punkt oben"}}},{"category":"Lu","key":"0228","mappings":{"default":{"default":"groß E mit Cedilla"}}},{"category":"Lu","key":"022E","mappings":{"default":{"default":"groß O mit Punkt oben"}}},{"category":"Lu","key":"0232","mappings":{"default":{"default":"groß Y mit Makron"}}},{"category":"Lu","key":"1E00","mappings":{"default":{"default":"groß A mit Ring unten"}}},{"category":"Lu","key":"1E02","mappings":{"default":{"default":"groß B mit Punkt oben"}}},{"category":"Lu","key":"1E04","mappings":{"default":{"default":"groß B mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E06","mappings":{"default":{"default":"groß B mit Zeile darunter"}}},{"category":"Lu","key":"1E0A","mappings":{"default":{"default":"groß D mit Punkt oben"}}},{"category":"Lu","key":"1E0C","mappings":{"default":{"default":"groß D mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E0E","mappings":{"default":{"default":"groß D mit Zeile darunter"}}},{"category":"Lu","key":"1E10","mappings":{"default":{"default":"groß D mit Cedilla"}}},{"category":"Lu","key":"1E12","mappings":{"default":{"default":"groß D mit darunterem Zirkumflex"}}},{"category":"Lu","key":"1E18","mappings":{"default":{"default":"groß E mit darunterem Zirkumflex"}}},{"category":"Lu","key":"1E1A","mappings":{"default":{"default":"groß E mit darunterliegender Tilde"}}},{"category":"Lu","key":"1E1E","mappings":{"default":{"default":"groß F mit Punkt oben"}}},{"category":"Lu","key":"1E20","mappings":{"default":{"default":"groß G mit Makron"}}},{"category":"Lu","key":"1E22","mappings":{"default":{"default":"groß H mit Punkt oben"}}},{"category":"Lu","key":"1E24","mappings":{"default":{"default":"groß H mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E26","mappings":{"default":{"default":"groß H mit Diärese"}}},{"category":"Lu","key":"1E28","mappings":{"default":{"default":"groß H mit Cedilla"}}},{"category":"Lu","key":"1E2A","mappings":{"default":{"default":"groß H mit Breve unten"}}},{"category":"Lu","key":"1E2C","mappings":{"default":{"default":"groß I mit unten stehender Tilde"}}},{"category":"Lu","key":"1E30","mappings":{"default":{"default":"groß K mit Akut"}}},{"category":"Lu","key":"1E32","mappings":{"default":{"default":"groß K mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E34","mappings":{"default":{"default":"groß K mit Zeile"}}},{"category":"Lu","key":"1E36","mappings":{"default":{"default":"groß L mit Punkt unten"}}},{"category":"Lu","key":"1E3A","mappings":{"default":{"default":"groß L mit Zeile darunter"}}},{"category":"Lu","key":"1E3C","mappings":{"default":{"default":"groß L mit darunterem Zirkumflex"}}},{"category":"Lu","key":"1E3E","mappings":{"default":{"default":"groß M mit Akut"}}},{"category":"Lu","key":"1E40","mappings":{"default":{"default":"groß M mit Punkt oben"}}},{"category":"Lu","key":"1E42","mappings":{"default":{"default":"groß M mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E44","mappings":{"default":{"default":"groß N mit Punkt oben"}}},{"category":"Lu","key":"1E46","mappings":{"default":{"default":"groß N mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E48","mappings":{"default":{"default":"groß N mit Zeile darunter"}}},{"category":"Lu","key":"1E4A","mappings":{"default":{"default":"groß N mit darunterem Zirkumflex"}}},{"category":"Lu","key":"1E54","mappings":{"default":{"default":"groß P mit Akut"}}},{"category":"Lu","key":"1E56","mappings":{"default":{"default":"groß P mit Punkt oben"}}},{"category":"Lu","key":"1E58","mappings":{"default":{"default":"groß R mit Punkt oben"}}},{"category":"Lu","key":"1E5A","mappings":{"default":{"default":"groß R mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E5E","mappings":{"default":{"default":"groß R mit Zeile darunter"}}},{"category":"Lu","key":"1E60","mappings":{"default":{"default":"groß S mit Punkt oben"}}},{"category":"Lu","key":"1E62","mappings":{"default":{"default":"groß S mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E6A","mappings":{"default":{"default":"groß T mit Punkt oben"}}},{"category":"Lu","key":"1E6C","mappings":{"default":{"default":"groß T mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E6E","mappings":{"default":{"default":"groß T mit Zeile darunter"}}},{"category":"Lu","key":"1E70","mappings":{"default":{"default":"groß T mit darunterem Zirkumflex"}}},{"category":"Lu","key":"1E72","mappings":{"default":{"default":"groß U mit untenstehender Diärese"}}},{"category":"Lu","key":"1E74","mappings":{"default":{"default":"groß U mit darunterliegender Tilde"}}},{"category":"Lu","key":"1E76","mappings":{"default":{"default":"groß U mit darunterem Zirkumflex"}}},{"category":"Lu","key":"1E7C","mappings":{"default":{"default":"groß V mit Tilde"}}},{"category":"Lu","key":"1E7E","mappings":{"default":{"default":"groß V mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E80","mappings":{"default":{"default":"groß W mit Gravis"}}},{"category":"Lu","key":"1E82","mappings":{"default":{"default":"groß W mit Akut"}}},{"category":"Lu","key":"1E84","mappings":{"default":{"default":"groß W mit Diärese"}}},{"category":"Lu","key":"1E86","mappings":{"default":{"default":"groß W mit Punkt oben"}}},{"category":"Lu","key":"1E88","mappings":{"default":{"default":"groß W mit Punkt unten"}}},{"category":"Lu","key":"1E8A","mappings":{"default":{"default":"groß X mit Punkt oben"}}},{"category":"Lu","key":"1E8C","mappings":{"default":{"default":"groß X mit Diärese"}}},{"category":"Lu","key":"1E8E","mappings":{"default":{"default":"groß Y mit Punkt oben"}}},{"category":"Lu","key":"1E90","mappings":{"default":{"default":"groß Z mit Zirkumflex"}}},{"category":"Lu","key":"1E92","mappings":{"default":{"default":"groß Z mit untenstehendem Punkt"}}},{"category":"Lu","key":"1E94","mappings":{"default":{"default":"groß Z mit Zeile darunter"}}},{"category":"Lu","key":"1EA0","mappings":{"default":{"default":"groß A mit Punkt unten"}}},{"category":"Lu","key":"1EA2","mappings":{"default":{"default":"groß A mit Haken oben"}}},{"category":"Lu","key":"1EB8","mappings":{"default":{"default":"groß E mit untenstehendem Punkt"}}},{"category":"Lu","key":"1EBA","mappings":{"default":{"default":"groß E mit Haken oben"}}},{"category":"Lu","key":"1EBC","mappings":{"default":{"default":"groß E mit Tilde"}}},{"category":"Lu","key":"1EC8","mappings":{"default":{"default":"groß I mit Haken oben"}}},{"category":"Lu","key":"1ECA","mappings":{"default":{"default":"groß I mit untenstehendem Punkt"}}},{"category":"Lu","key":"1ECC","mappings":{"default":{"default":"groß O mit untenstehendem Punkt"}}},{"category":"Lu","key":"1ECE","mappings":{"default":{"default":"groß O mit Haken oben"}}},{"category":"Lu","key":"1EE4","mappings":{"default":{"default":"groß U mit untenstehendem Punkt"}}},{"category":"Lu","key":"1EE6","mappings":{"default":{"default":"groß U mit Haken oben"}}},{"category":"Lu","key":"1EF2","mappings":{"default":{"default":"groß Y mit Gravis"}}},{"category":"Lu","key":"1EF4","mappings":{"default":{"default":"groß Y mit untenstehendem Punkt"}}},{"category":"Lu","key":"1EF6","mappings":{"default":{"default":"groß Y mit Haken oben"}}},{"category":"Lu","key":"1EF8","mappings":{"default":{"default":"groß Y mit Tilde"}}}]
,
"de/symbols/math_angles.js": 
[{"locale":"de"},{"category":"Sm","key":"22BE","mappings":{"default":{"default":"rechter Winkel mit Bogen"}}},{"category":"Sm","key":"237C","mappings":{"default":{"default":"rechter Winkel mit nach unten gerichtetem Zickzackpfeil"}}},{"category":"Sm","key":"27C0","mappings":{"default":{"default":"Dreidimensionaler Winkel"}}},{"category":"Sm","key":"299B","mappings":{"default":{"default":"Gemessene Winkelöffnung links"}}},{"category":"Sm","key":"299C","mappings":{"default":{"default":"Rechtwinklige Variante mit Quadrat"}}},{"category":"Sm","key":"299D","mappings":{"default":{"default":"rechter Winkel mit Punkt gemessen"}}},{"category":"Sm","key":"299E","mappings":{"default":{"default":"Winkel mit S nach innen"}}},{"category":"Sm","key":"299F","mappings":{"default":{"default":"Spitzer Winkel"}}},{"category":"Sm","key":"29A0","mappings":{"default":{"default":"Sphärischer Öffnungswinkel nach links"}}},{"category":"Sm","key":"29A1","mappings":{"default":{"default":"Sphärischer Winkel, der sich öffnet"}}},{"category":"Sm","key":"29A2","mappings":{"default":{"default":"Gedrehter Winkel"}}},{"category":"Sm","key":"29A3","mappings":{"default":{"default":"Umgekehrter Winkel"}}},{"category":"Sm","key":"29A4","mappings":{"default":{"default":"Winkel mit Underbar"}}},{"category":"Sm","key":"29A5","mappings":{"default":{"default":"Umgekehrter Winkel mit Unterleiste"}}},{"category":"Sm","key":"29A6","mappings":{"default":{"default":"Schräger Winkel, der sich öffnet"}}},{"category":"Sm","key":"29A7","mappings":{"default":{"default":"Schräger Winkel, der sich nach unten öffnet"}}},{"category":"Sm","key":"29A8","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm endet im Pfeil nach oben und rechts"}}},{"category":"Sm","key":"29A9","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm und Pfeil nach oben und links"}}},{"category":"Sm","key":"29AA","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm endet im nach unten zeigenden Pfeil und rechts"}}},{"category":"Sm","key":"29AB","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm endet im nach unten zeigenden Pfeil"}}},{"category":"Sm","key":"29AC","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm endet im Pfeil nach rechts und oben"}}},{"category":"Sm","key":"29AD","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm endet im Pfeil nach links und oben"}}},{"category":"Sm","key":"29AE","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm endet im nach rechts und unten zeigenden Pfeil"}}},{"category":"Sm","key":"29AF","mappings":{"default":{"default":"Gemessener Winkel mit offenem Arm und Pfeil nach links und unten"}}}]
,
"de/symbols/math_arrows.js": 
[{"locale":"de"},{"category":"Sm","key":"2190","mappings":{"default":{"default":"Pfeil nach links"}}},{"category":"Sm","key":"2191","mappings":{"default":{"default":"Aufwärtspfeil"}}},{"category":"Sm","key":"2192","mappings":{"default":{"default":"Pfeil nach rechts"}}},{"category":"Sm","key":"2193","mappings":{"default":{"default":"Pfeil nach unten"}}},{"category":"Sm","key":"2194","mappings":{"default":{"default":"Linker rechter Pfeil"}}},{"category":"So","key":"2195","mappings":{"default":{"default":"Pfeil nach oben"}}},{"category":"So","key":"2196","mappings":{"default":{"default":"Nordwestpfeil"}}},{"category":"So","key":"2197","mappings":{"default":{"default":"Nordostpfeil"}}},{"category":"So","key":"2198","mappings":{"default":{"default":"Südostpfeil"}}},{"category":"So","key":"2199","mappings":{"default":{"default":"Südwestpfeil"}}},{"category":"Sm","key":"219A","mappings":{"default":{"default":"Pfeil nach links mit Strich"}}},{"category":"Sm","key":"219B","mappings":{"default":{"default":"Pfeil nach rechts mit Strich"}}},{"category":"So","key":"219C","mappings":{"default":{"default":"Wellenpfeil nach links"}}},{"category":"So","key":"219D","mappings":{"default":{"default":"Wellenpfeil nach rechts"}}},{"category":"So","key":"219E","mappings":{"default":{"default":"Pfeil nach links mit zwei Spitzen"}}},{"category":"So","key":"219F","mappings":{"default":{"default":"Aufwärts zwei Pfeil nach unten"}}},{"category":"Sm","key":"21A0","mappings":{"default":{"default":"Pfeil nach rechts mit zwei Köpfen"}}},{"category":"So","key":"21A1","mappings":{"default":{"default":"Abwärts zwei Pfeil nach unten"}}},{"category":"So","key":"21A2","mappings":{"default":{"default":"Pfeil nach links mit Schwanz"}}},{"category":"Sm","key":"21A3","mappings":{"default":{"default":"Pfeil nach rechts mit Schwanz"}}},{"category":"So","key":"21A4","mappings":{"default":{"default":"Pfeil nach links von der Bar"}}},{"category":"So","key":"21A5","mappings":{"default":{"default":"Pfeil von der Bar nach oben"}}},{"category":"Sm","key":"21A6","mappings":{"default":{"default":"Pfeil nach rechts von der Bar"}}},{"category":"So","key":"21A7","mappings":{"default":{"default":"Pfeil von der Bar nach unten"}}},{"category":"So","key":"21A8","mappings":{"default":{"default":"Pfeil nach unten mit Basis"}}},{"category":"So","key":"21A9","mappings":{"default":{"default":"Pfeil nach links mit Haken"}}},{"category":"So","key":"21AA","mappings":{"default":{"default":"Pfeil nach rechts mit Haken"}}},{"category":"So","key":"21AB","mappings":{"default":{"default":"Pfeil nach links mit Schleife"}}},{"category":"So","key":"21AC","mappings":{"default":{"default":"Pfeil nach rechts mit Schleife"}}},{"category":"So","key":"21AD","mappings":{"default":{"default":"Linker rechter Wellenpfeil"}}},{"category":"Sm","key":"21AE","mappings":{"default":{"default":"Linker rechter Pfeil mit Strich"}}},{"category":"So","key":"21AF","mappings":{"default":{"default":"Zickzack-Pfeil nach unten"}}},{"category":"So","key":"21B0","mappings":{"default":{"default":"Aufwärtspfeil mit Spitze nach links"}}},{"category":"So","key":"21B1","mappings":{"default":{"default":"Aufwärtspfeil mit Spitze nach rechts"}}},{"category":"So","key":"21B2","mappings":{"default":{"default":"Pfeil nach unten mit Spitze nach links"}}},{"category":"So","key":"21B3","mappings":{"default":{"default":"Abwärtspfeil mit Spitze nach rechts"}}},{"category":"So","key":"21B4","mappings":{"default":{"default":"Pfeil nach rechts mit Ecke nach unten"}}},{"category":"So","key":"21B5","mappings":{"default":{"default":"Pfeil nach unten mit Ecke nach links"}}},{"category":"So","key":"21B6","mappings":{"default":{"default":"Gegen den Uhrzeigersinn oberer halbkreisförmiger Pfeil"}}},{"category":"So","key":"21B7","mappings":{"default":{"default":"Halbkreis im Uhrzeigersinn"}}},{"category":"So","key":"21B8","mappings":{"default":{"default":"Nordwestpfeil zur langen Bar"}}},{"category":"So","key":"21B9","mappings":{"default":{"default":"Pfeil nach links zum Überfahren Pfeil nach rechts zum Überfahren"}}},{"category":"So","key":"21BA","mappings":{"default":{"default":"Offener Kreispfeil gegen den Uhrzeigersinn"}}},{"category":"So","key":"21BB","mappings":{"default":{"default":"Kreispfeil im Uhrzeigersinn öffnen"}}},{"category":"So","key":"21C4","mappings":{"default":{"default":"Pfeil nach rechts über Pfeil nach links"}}},{"category":"So","key":"21C5","mappings":{"default":{"default":"Aufwärtspfeil nach links von Abwärtspfeil"}}},{"category":"So","key":"21C6","mappings":{"default":{"default":"Pfeil nach links über Pfeil nach rechts"}}},{"category":"So","key":"21C7","mappings":{"default":{"default":"Links gepaarte Pfeile"}}},{"category":"So","key":"21C8","mappings":{"default":{"default":"Aufwärts gepaarte Pfeile"}}},{"category":"So","key":"21C9","mappings":{"default":{"default":"Rechts gepaarte Pfeile"}}},{"category":"So","key":"21CA","mappings":{"default":{"default":"Abwärts gepaarte Pfeile"}}},{"category":"So","key":"21CD","mappings":{"default":{"default":"Doppelpfeil nach links mit Strich"}}},{"category":"Sm","key":"21CE","mappings":{"default":{"default":"Links Rechts Doppelpfeil mit Strich"}}},{"category":"Sm","key":"21CF","mappings":{"default":{"default":"Doppelpfeil nach rechts mit Strich"}}},{"category":"So","key":"21D0","mappings":{"default":{"default":"Doppelpfeil nach links"}}},{"category":"So","key":"21D1","mappings":{"default":{"default":"Doppelpfeil nach oben"}}},{"category":"Sm","key":"21D2","mappings":{"default":{"default":"Doppelpfeil nach rechts"}}},{"category":"So","key":"21D3","mappings":{"default":{"default":"Doppelpfeil nach unten"}}},{"category":"Sm","key":"21D4","mappings":{"default":{"default":"Linker rechter Doppelpfeil"}}},{"category":"So","key":"21D5","mappings":{"default":{"default":"Doppelpfeil nach unten"}}},{"category":"So","key":"21D6","mappings":{"default":{"default":"Nordwest-Doppelpfeil"}}},{"category":"So","key":"21D7","mappings":{"default":{"default":"Nordost-Doppelpfeil"}}},{"category":"So","key":"21D8","mappings":{"default":{"default":"Südost Doppelpfeil"}}},{"category":"So","key":"21D9","mappings":{"default":{"default":"Südwest-Doppelpfeil"}}},{"category":"So","key":"21DA","mappings":{"default":{"default":"Dreifacher Pfeil nach links"}}},{"category":"So","key":"21DB","mappings":{"default":{"default":"Dreifacher Pfeil nach rechts"}}},{"category":"So","key":"21DC","mappings":{"default":{"default":"Linker Squiggle-Pfeil"}}},{"category":"So","key":"21DD","mappings":{"default":{"default":"Nach rechts gekrümmter Pfeil"}}},{"category":"So","key":"21DE","mappings":{"default":{"default":"Aufwärtspfeil mit Doppelstrich"}}},{"category":"So","key":"21DF","mappings":{"default":{"default":"Abwärtspfeil mit Doppelstrich"}}},{"category":"So","key":"21E0","mappings":{"default":{"default":"Pfeil nach links nach links"}}},{"category":"So","key":"21E1","mappings":{"default":{"default":"Pfeil nach oben gestrichen"}}},{"category":"So","key":"21E2","mappings":{"default":{"default":"Rechts gestrichener Pfeil"}}},{"category":"So","key":"21E3","mappings":{"default":{"default":"Abwärts gestrichelter Pfeil"}}},{"category":"So","key":"21E4","mappings":{"default":{"default":"Pfeil nach links zur Bar"}}},{"category":"So","key":"21E5","mappings":{"default":{"default":"Pfeil nach rechts zur Bar"}}},{"category":"So","key":"21E6","mappings":{"default":{"default":"Weißer Pfeil nach links"}}},{"category":"So","key":"21E7","mappings":{"default":{"default":"Aufwärts weißer Pfeil"}}},{"category":"So","key":"21E8","mappings":{"default":{"default":"Rechts weißer Pfeil"}}},{"category":"So","key":"21E9","mappings":{"default":{"default":"Abwärts weißer Pfeil"}}},{"category":"So","key":"21EA","mappings":{"default":{"default":"Aufwärts weißer Pfeil von der Stange"}}},{"category":"So","key":"21EB","mappings":{"default":{"default":"Aufwärts weißer Pfeil auf Sockel"}}},{"category":"So","key":"21EC","mappings":{"default":{"default":"Aufwärts weißer Pfeil auf Sockel mit horizontaler Stange"}}},{"category":"So","key":"21ED","mappings":{"default":{"default":"Aufwärts weißer Pfeil auf Sockel mit vertikaler Stange"}}},{"category":"So","key":"21EE","mappings":{"default":{"default":"Aufwärts weißer Doppelpfeil"}}},{"category":"So","key":"21EF","mappings":{"default":{"default":"Aufwärts weißer doppelter Pfeil auf Sockel"}}},{"category":"So","key":"21F0","mappings":{"default":{"default":"Rechts weißer Pfeil von der Wand"}}},{"category":"So","key":"21F1","mappings":{"default":{"default":"Nordwestpfeil zur Ecke"}}},{"category":"So","key":"21F2","mappings":{"default":{"default":"Südostpfeil zur Ecke"}}},{"category":"So","key":"21F3","mappings":{"default":{"default":"Weißer Pfeil nach unten"}}},{"category":"Sm","key":"21F4","mappings":{"default":{"default":"rechter Pfeil mit kleinem Kreis"}}},{"category":"Sm","key":"21F5","mappings":{"default":{"default":"Abwärtspfeil nach links von Aufwärtspfeil"}}},{"category":"Sm","key":"21F6","mappings":{"default":{"default":"Drei nach rechts gerichtete Pfeile"}}},{"category":"Sm","key":"21F7","mappings":{"default":{"default":"Pfeil nach links mit vertikalem Strich"}}},{"category":"Sm","key":"21F8","mappings":{"default":{"default":"Pfeil nach rechts mit vertikalem Strich"}}},{"category":"Sm","key":"21F9","mappings":{"default":{"default":"Linker rechter Pfeil mit vertikalem Strich"}}},{"category":"Sm","key":"21FA","mappings":{"default":{"default":"Pfeil nach links mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"21FB","mappings":{"default":{"default":"Pfeil nach rechts mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"21FC","mappings":{"default":{"default":"Linker rechter Pfeil mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"21FD","mappings":{"default":{"default":"Pfeil nach links mit offenem Kopf"}}},{"category":"Sm","key":"21FE","mappings":{"default":{"default":"Pfeil nach rechts mit offenem Kopf"}}},{"category":"Sm","key":"21FF","mappings":{"default":{"default":"Linker rechter Pfeil mit offenem Kopf"}}},{"category":"So","key":"2301","mappings":{"default":{"default":"Elektrischer Pfeil"}}},{"category":"So","key":"2303","mappings":{"default":{"default":"Pfeil nach oben"}}},{"category":"So","key":"2304","mappings":{"default":{"default":"Pfeil nach unten"}}},{"category":"So","key":"2324","mappings":{"default":{"default":"Pfeilspitze zwischen zwei horizontalen Balken"}}},{"category":"So","key":"238B","mappings":{"default":{"default":"Gebrochener Kreis mit Nordwestpfeil"}}},{"category":"So","key":"2794","mappings":{"default":{"default":"Schwerer, nach rechts gerichteter Pfeil nach rechts"}}},{"category":"So","key":"2798","mappings":{"default":{"default":"Schwerer Südostpfeil"}}},{"category":"So","key":"2799","mappings":{"default":{"default":"Schwerer Pfeil nach rechts"}}},{"category":"So","key":"279A","mappings":{"default":{"default":"Schwerer Nordostpfeil"}}},{"category":"So","key":"279B","mappings":{"default":{"default":"Zeichnungspunkt Pfeil nach rechts"}}},{"category":"So","key":"279C","mappings":{"default":{"default":"Schwerer runder Pfeil nach rechts"}}},{"category":"So","key":"279D","mappings":{"default":{"default":"Dreieckköpfiger Pfeil nach rechts"}}},{"category":"So","key":"279E","mappings":{"default":{"default":"Schwerer Dreieckspfeil nach rechts"}}},{"category":"So","key":"279F","mappings":{"default":{"default":"Dreieckköpfiger Pfeil nach rechts"}}},{"category":"So","key":"27A0","mappings":{"default":{"default":"Schwerer, gestrichelter Dreieckspfeil nach rechts"}}},{"category":"So","key":"27A1","mappings":{"default":{"default":"Schwarzer Pfeil nach rechts"}}},{"category":"So","key":"27A2","mappings":{"default":{"default":"Dreidimensionale, nach oben beleuchtete Pfeilspitze"}}},{"category":"So","key":"27A3","mappings":{"default":{"default":"Dreidimensionale, von unten beleuchtete Pfeilspitze nach rechts"}}},{"category":"So","key":"27A4","mappings":{"default":{"default":"Schwarze Pfeilspitze nach rechts"}}},{"category":"So","key":"27A5","mappings":{"default":{"default":"Schwerer schwarzer Pfeil nach unten und rechts gebogen"}}},{"category":"So","key":"27A6","mappings":{"default":{"default":"Schwerer schwarzer Pfeil nach oben und rechts gebogen"}}},{"category":"So","key":"27A7","mappings":{"default":{"default":"Schwarzer nach rechts gerichteter Pfeil"}}},{"category":"So","key":"27A8","mappings":{"default":{"default":"Schwerer konkaver Spitze-Pfeil nach rechts"}}},{"category":"So","key":"27A9","mappings":{"default":{"default":"Rechts geschatteter weißer Pfeil nach rechts"}}},{"category":"So","key":"27AA","mappings":{"default":{"default":"Links schattierter weißer Pfeil nach rechts"}}},{"category":"So","key":"27AB","mappings":{"default":{"default":"Zurückgekippter, beschatteter weißer Pfeil nach rechts"}}},{"category":"So","key":"27AC","mappings":{"default":{"default":"Vorne geneigter, beschatteter weißer Pfeil nach rechts"}}},{"category":"So","key":"27AD","mappings":{"default":{"default":"Schwerer unterer rechter Schatten mit weißem Pfeil nach rechts"}}},{"category":"So","key":"27AE","mappings":{"default":{"default":"Schwerer oberer rechter Schatten mit weißem Pfeil nach rechts"}}},{"category":"So","key":"27AF","mappings":{"default":{"default":"Gekerbter Pfeil nach rechts unten rechts"}}},{"category":"So","key":"27B1","mappings":{"default":{"default":"Weißer Pfeil nach rechts oben gekerbt"}}},{"category":"So","key":"27B2","mappings":{"default":{"default":"Eingekreister schwerer weißer Pfeil nach rechts"}}},{"category":"So","key":"27B3","mappings":{"default":{"default":"Pfeil mit weißer Feder nach rechts"}}},{"category":"So","key":"27B4","mappings":{"default":{"default":"Schwarz-gefiederter Südostpfeil"}}},{"category":"So","key":"27B5","mappings":{"default":{"default":"Pfeil mit schwarzer Feder nach rechts"}}},{"category":"So","key":"27B6","mappings":{"default":{"default":"Schwarzgefiederter Nordostpfeil"}}},{"category":"So","key":"27B7","mappings":{"default":{"default":"Schwerer schwarzgefiederter Südostpfeil"}}},{"category":"So","key":"27B8","mappings":{"default":{"default":"Schwerer schwarzgefiederter Pfeil nach rechts"}}},{"category":"So","key":"27B9","mappings":{"default":{"default":"Schwerer schwarzgefiederter Nordostpfeil"}}},{"category":"So","key":"27BA","mappings":{"default":{"default":"Teardrop-Barbed-Pfeil nach rechts"}}},{"category":"So","key":"27BB","mappings":{"default":{"default":"Schwerer Tropfen-Pfeil nach rechts"}}},{"category":"So","key":"27BC","mappings":{"default":{"default":"Keil-Schwanz nach rechts Pfeil"}}},{"category":"So","key":"27BD","mappings":{"default":{"default":"Schwerer, nach rechts gekeilter Pfeil"}}},{"category":"So","key":"27BE","mappings":{"default":{"default":"Offener Pfeil nach rechts"}}},{"category":"Sm","key":"27F0","mappings":{"default":{"default":"Vierfachpfeil nach oben"}}},{"category":"Sm","key":"27F1","mappings":{"default":{"default":"Vierfacher Pfeil nach unten"}}},{"category":"Sm","key":"27F2","mappings":{"default":{"default":"Kreispfeil gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"27F3","mappings":{"default":{"default":"Kreispfeil im Uhrzeigersinn"}}},{"category":"Sm","key":"27F4","mappings":{"default":{"default":"rechter Pfeil mit Circled Plus"}}},{"category":"Sm","key":"27F5","mappings":{"default":{"default":"Langer Pfeil nach links"}}},{"category":"Sm","key":"27F6","mappings":{"default":{"default":"Langer Pfeil nach rechts"}}},{"category":"Sm","key":"27F7","mappings":{"default":{"default":"Langer linker rechter Pfeil"}}},{"category":"Sm","key":"27F8","mappings":{"default":{"default":"Langer Doppelpfeil nach links"}}},{"category":"Sm","key":"27F9","mappings":{"default":{"default":"Langer Doppelpfeil nach rechts"}}},{"category":"Sm","key":"27FA","mappings":{"default":{"default":"Langer linker rechter Doppelpfeil"}}},{"category":"Sm","key":"27FB","mappings":{"default":{"default":"Langer Pfeil nach links von der Bar"}}},{"category":"Sm","key":"27FC","mappings":{"default":{"default":"Langer Rechtspfeil von der Bar"}}},{"category":"Sm","key":"27FD","mappings":{"default":{"default":"Langer Doppelpfeil nach links von der Bar"}}},{"category":"Sm","key":"27FE","mappings":{"default":{"default":"Langer Doppelpfeil nach rechts von der Bar"}}},{"category":"Sm","key":"27FF","mappings":{"default":{"default":"Langer nach rechts gerichteter Squiggle-Pfeil"}}},{"category":"Sm","key":"2900","mappings":{"default":{"default":"Pfeil nach rechts mit senkrechtem Strich"}}},{"category":"Sm","key":"2901","mappings":{"default":{"default":"Zwei Pfeil nach rechts mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"2902","mappings":{"default":{"default":"Doppelpfeil nach links mit vertikalem Strich"}}},{"category":"Sm","key":"2903","mappings":{"default":{"default":"Doppelpfeil nach rechts mit vertikalem Strich"}}},{"category":"Sm","key":"2904","mappings":{"default":{"default":"Linker rechter Doppelpfeil mit vertikalem Strich"}}},{"category":"Sm","key":"2905","mappings":{"default":{"default":"Zwei Pfeil nach rechts von der Bar"}}},{"category":"Sm","key":"2906","mappings":{"default":{"default":"Doppelpfeil nach links von der Bar"}}},{"category":"Sm","key":"2907","mappings":{"default":{"default":"Doppelter Pfeil nach rechts von der Bar"}}},{"category":"Sm","key":"2908","mappings":{"default":{"default":"Abwärtspfeil mit horizontalem Strich"}}},{"category":"Sm","key":"2909","mappings":{"default":{"default":"Aufwärtspfeil mit horizontalem Anschlag"}}},{"category":"Sm","key":"290A","mappings":{"default":{"default":"Dreifachpfeil nach oben"}}},{"category":"Sm","key":"290B","mappings":{"default":{"default":"Dreifacher Pfeil nach unten"}}},{"category":"Sm","key":"290C","mappings":{"default":{"default":"Doppelpfeil nach links"}}},{"category":"Sm","key":"290D","mappings":{"default":{"default":"Doppelpfeil nach rechts"}}},{"category":"Sm","key":"290E","mappings":{"default":{"default":"Dreifacher Strichpfeil nach links"}}},{"category":"Sm","key":"290F","mappings":{"default":{"default":"Dreifacher Bindestrich nach rechts"}}},{"category":"Sm","key":"2910","mappings":{"default":{"default":"Dreipoliger Pfeil nach rechts"}}},{"category":"Sm","key":"2911","mappings":{"default":{"default":"Pfeil nach rechts mit gepunktetem Stamm"}}},{"category":"Sm","key":"2912","mappings":{"default":{"default":"Aufwärtspfeil zur Bar"}}},{"category":"Sm","key":"2913","mappings":{"default":{"default":"Abwärtspfeil zur Bar"}}},{"category":"Sm","key":"2914","mappings":{"default":{"default":"Pfeil nach rechts mit Schwanz mit senkrechtem Anschlag"}}},{"category":"Sm","key":"2915","mappings":{"default":{"default":"Pfeil nach rechts mit Schwanz mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"2916","mappings":{"default":{"default":"Nach rechts gerichteter zweiköpfiger Pfeil mit Schwanz"}}},{"category":"Sm","key":"2917","mappings":{"default":{"default":"Nach rechts gerichteter Doppelpfeil mit Schwanz mit senkrechtem Anschlag"}}},{"category":"Sm","key":"2918","mappings":{"default":{"default":"Nach rechts gerichteter Doppelpfeil mit Schwanz mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"2919","mappings":{"default":{"default":"Pfeil nach links nach links"}}},{"category":"Sm","key":"291A","mappings":{"default":{"default":"Pfeil nach rechts nach rechts"}}},{"category":"Sm","key":"291B","mappings":{"default":{"default":"Doppelter Pfeilschwanz nach links"}}},{"category":"Sm","key":"291C","mappings":{"default":{"default":"Doppelter Pfeilschwanz nach rechts"}}},{"category":"Sm","key":"291D","mappings":{"default":{"default":"Pfeil nach links zu Black Diamond"}}},{"category":"Sm","key":"291E","mappings":{"default":{"default":"Pfeil nach rechts zu Black Diamond"}}},{"category":"Sm","key":"291F","mappings":{"default":{"default":"Pfeil nach links von Bar zu Black Diamond"}}},{"category":"Sm","key":"2920","mappings":{"default":{"default":"Pfeil nach rechts von Bar zu Black Diamond"}}},{"category":"Sm","key":"2921","mappings":{"default":{"default":"Nordwest- und Südostpfeil"}}},{"category":"Sm","key":"2922","mappings":{"default":{"default":"Nordost- und Südwestpfeil"}}},{"category":"Sm","key":"2923","mappings":{"default":{"default":"Nordwestpfeil mit Haken"}}},{"category":"Sm","key":"2924","mappings":{"default":{"default":"Nordostpfeil mit Haken"}}},{"category":"Sm","key":"2925","mappings":{"default":{"default":"Südostpfeil mit Haken"}}},{"category":"Sm","key":"2926","mappings":{"default":{"default":"Südwestpfeil mit Haken"}}},{"category":"Sm","key":"2927","mappings":{"default":{"default":"Nordwestpfeil und Nordostpfeil"}}},{"category":"Sm","key":"2928","mappings":{"default":{"default":"Nordostpfeil und Südostpfeil"}}},{"category":"Sm","key":"2929","mappings":{"default":{"default":"Südostpfeil und Südwestpfeil"}}},{"category":"Sm","key":"292A","mappings":{"default":{"default":"Südwestpfeil und Nordwestpfeil"}}},{"category":"Sm","key":"292D","mappings":{"default":{"default":"Südostpfeil Kreuzung Nordostpfeil"}}},{"category":"Sm","key":"292E","mappings":{"default":{"default":"Nordostpfeil Kreuzung Südostpfeil"}}},{"category":"Sm","key":"292F","mappings":{"default":{"default":"Fallende Diagonale, die Nordostpfeil kreuzt"}}},{"category":"Sm","key":"2930","mappings":{"default":{"default":"Steigende Diagonale, die Südostpfeil kreuzt"}}},{"category":"Sm","key":"2931","mappings":{"default":{"default":"Nordostpfeilüberquerung Nordwestpfeil"}}},{"category":"Sm","key":"2932","mappings":{"default":{"default":"Nordwestpfeilüberquerung Nordostpfeil"}}},{"category":"Sm","key":"2933","mappings":{"default":{"default":"Wellenpfeil, der direkt nach rechts zeigt"}}},{"category":"Sm","key":"2934","mappings":{"default":{"default":"Pfeil nach rechts, dann nach oben gekrümmt"}}},{"category":"Sm","key":"2935","mappings":{"default":{"default":"Pfeil nach rechts, dann nach unten gekrümmt"}}},{"category":"Sm","key":"2936","mappings":{"default":{"default":"Pfeil nach unten, dann nach links gekrümmt"}}},{"category":"Sm","key":"2937","mappings":{"default":{"default":"Pfeil nach unten, dann nach rechts gekrümmt"}}},{"category":"Sm","key":"2938","mappings":{"default":{"default":"Rechtsseitiger Bogen im Uhrzeigersinn"}}},{"category":"Sm","key":"2939","mappings":{"default":{"default":"Bogen der linken Seite gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"293A","mappings":{"default":{"default":"Pfeil oben gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"293B","mappings":{"default":{"default":"Pfeil unten gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"293C","mappings":{"default":{"default":"Pfeil oben im Uhrzeigersinn mit Minus"}}},{"category":"Sm","key":"293D","mappings":{"default":{"default":"Pfeil oben gegen den Uhrzeigersinn mit Plus"}}},{"category":"Sm","key":"293E","mappings":{"default":{"default":"Rechts unten halbkreisförmig im Uhrzeigersinn"}}},{"category":"Sm","key":"293F","mappings":{"default":{"default":"Linker halbkreisförmiger Pfeil gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"2940","mappings":{"default":{"default":"Geschlossener Kreispfeil gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"2941","mappings":{"default":{"default":"Geschlossener Kreispfeil im Uhrzeigersinn"}}},{"category":"Sm","key":"2942","mappings":{"default":{"default":"Pfeil nach rechts über Kurzer Pfeil nach links"}}},{"category":"Sm","key":"2943","mappings":{"default":{"default":"Pfeil nach links über Kurzer Pfeil nach rechts"}}},{"category":"Sm","key":"2944","mappings":{"default":{"default":"Kurzer Pfeil nach rechts Über Pfeil nach links"}}},{"category":"Sm","key":"2945","mappings":{"default":{"default":"Pfeil nach rechts mit Plus unten"}}},{"category":"Sm","key":"2946","mappings":{"default":{"default":"Pfeil nach links mit Plus unten"}}},{"category":"Sm","key":"2947","mappings":{"default":{"default":"Pfeil nach rechts durch X"}}},{"category":"Sm","key":"2948","mappings":{"default":{"default":"Linker rechter Pfeil durch kleinen Kreis"}}},{"category":"Sm","key":"2949","mappings":{"default":{"default":"Aufwärts zweiköpfiger Pfeil vom kleinen Kreis"}}},{"key":"2970","mappings":{"default":{"default":"round implies"}},"category":"Sm"},{"category":"Sm","key":"2971","mappings":{"default":{"default":"Gleichheitszeichen über dem Pfeil nach rechts"}}},{"category":"Sm","key":"2972","mappings":{"default":{"default":"Tilde-Operator über dem Pfeil nach rechts"}}},{"category":"Sm","key":"2973","mappings":{"default":{"default":"Pfeil nach links über Tilde-Operator"}}},{"category":"Sm","key":"2974","mappings":{"default":{"default":"Pfeil nach rechts über Tilde-Operator"}}},{"category":"Sm","key":"2975","mappings":{"default":{"default":"Pfeil nach rechts fast gleich"}}},{"category":"Sm","key":"2976","mappings":{"default":{"default":"Weniger als oben Pfeil nach links"}}},{"category":"Sm","key":"2977","mappings":{"default":{"default":"Pfeil nach links durch weniger als"}}},{"category":"Sm","key":"2978","mappings":{"default":{"default":"Größer als über dem Pfeil nach rechts"}}},{"category":"Sm","key":"2979","mappings":{"default":{"default":"Untersatz über dem Pfeil nach rechts"}}},{"category":"Sm","key":"297A","mappings":{"default":{"default":"Pfeil nach links durch Teilmenge"}}},{"category":"Sm","key":"297B","mappings":{"default":{"default":"Superset über dem Pfeil nach links"}}},{"category":"Sm","key":"29B3","mappings":{"default":{"default":"Leeres Set mit Pfeil rechts oben"}}},{"category":"Sm","key":"29B4","mappings":{"default":{"default":"Leeres Set mit linkem Pfeil oben"}}},{"category":"Sm","key":"29BD","mappings":{"default":{"default":"Pfeil nach oben durch Kreis"}}},{"category":"Sm","key":"29EA","mappings":{"default":{"default":"Schwarzer Diamant mit Pfeil nach unten"}}},{"category":"Sm","key":"29EC","mappings":{"default":{"default":"Weißer Kreis mit Pfeil nach unten"}}},{"category":"Sm","key":"29ED","mappings":{"default":{"default":"Schwarzer Kreis mit Pfeil nach unten"}}},{"category":"Sm","key":"2A17","mappings":{"default":{"default":"Integral mit Pfeil nach links mit Haken"}}},{"category":"So","key":"2B00","mappings":{"default":{"default":"North East White Arrow"}}},{"category":"So","key":"2B01","mappings":{"default":{"default":"Nordwestlicher weißer Pfeil"}}},{"category":"So","key":"2B02","mappings":{"default":{"default":"Südostweißer Pfeil"}}},{"category":"So","key":"2B03","mappings":{"default":{"default":"Südwestweißer Pfeil"}}},{"category":"So","key":"2B04","mappings":{"default":{"default":"Linker rechter weißer Pfeil"}}},{"category":"So","key":"2B05","mappings":{"default":{"default":"Schwarzer Pfeil nach links"}}},{"category":"So","key":"2B06","mappings":{"default":{"default":"Aufwärts schwarzer Pfeil"}}},{"category":"So","key":"2B07","mappings":{"default":{"default":"Abwärts schwarzer Pfeil"}}},{"category":"So","key":"2B08","mappings":{"default":{"default":"Schwarzer Nordostpfeil"}}},{"category":"So","key":"2B09","mappings":{"default":{"default":"Nordwestlicher schwarzer Pfeil"}}},{"category":"So","key":"2B0A","mappings":{"default":{"default":"Schwarzer Südostpfeil"}}},{"category":"So","key":"2B0B","mappings":{"default":{"default":"Südwestlicher schwarzer Pfeil"}}},{"category":"So","key":"2B0C","mappings":{"default":{"default":"Linker rechter schwarzer Pfeil"}}},{"category":"So","key":"2B0D","mappings":{"default":{"default":"Schwarzer Pfeil nach unten"}}},{"category":"So","key":"2B0E","mappings":{"default":{"default":"Pfeil nach rechts mit Spitze nach unten"}}},{"category":"So","key":"2B0F","mappings":{"default":{"default":"Pfeil nach rechts mit Spitze nach oben"}}},{"category":"So","key":"2B10","mappings":{"default":{"default":"Pfeil nach links mit Spitze nach unten"}}},{"category":"So","key":"2B11","mappings":{"default":{"default":"Pfeil nach links mit Spitze nach oben"}}},{"category":"Sm","key":"2B30","mappings":{"default":{"default":"Linker Pfeil mit kleinem Kreis"}}},{"category":"Sm","key":"2B31","mappings":{"default":{"default":"Drei nach links gerichtete Pfeile"}}},{"category":"Sm","key":"2B32","mappings":{"default":{"default":"Linker Pfeil mit Circled Plus"}}},{"category":"Sm","key":"2B33","mappings":{"default":{"default":"Langer nach links gerichteter Squiggle-Pfeil"}}},{"category":"Sm","key":"2B34","mappings":{"default":{"default":"Pfeil nach links mit senkrechtem Strich"}}},{"category":"Sm","key":"2B35","mappings":{"default":{"default":"Doppelpfeil nach links mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"2B36","mappings":{"default":{"default":"Doppelpfeil nach links von der Bar"}}},{"category":"Sm","key":"2B37","mappings":{"default":{"default":"Pfeil nach links mit zwei Köpfen"}}},{"category":"Sm","key":"2B38","mappings":{"default":{"default":"Pfeil nach links mit gepunktetem Stamm"}}},{"category":"Sm","key":"2B39","mappings":{"default":{"default":"Pfeil nach links mit Schwanz mit vertikalem Strich"}}},{"category":"Sm","key":"2B3A","mappings":{"default":{"default":"Pfeil nach links mit Schwanz mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"2B3B","mappings":{"default":{"default":"Nach links gerichteter zweiköpfiger Pfeil mit Schwanz"}}},{"category":"Sm","key":"2B3C","mappings":{"default":{"default":"Nach links gerichteter zweiköpfiger Pfeil mit Schwanz mit vertikalem Strich"}}},{"category":"Sm","key":"2B3D","mappings":{"default":{"default":"Nach links gerichteter zweiköpfiger Pfeil mit Schwanz mit doppeltem vertikalem Strich"}}},{"category":"Sm","key":"2B3E","mappings":{"default":{"default":"Pfeil nach links durch X"}}},{"category":"Sm","key":"2B3F","mappings":{"default":{"default":"Wellenpfeil, der direkt nach links zeigt"}}},{"category":"Sm","key":"2B40","mappings":{"default":{"default":"Gleichheitszeichen über dem Pfeil nach links"}}},{"category":"Sm","key":"2B41","mappings":{"default":{"default":"Reverse-Tilde-Operator Über Linkspfeil"}}},{"category":"Sm","key":"2B42","mappings":{"default":{"default":"Pfeil nach links über der Rückseite fast gleich"}}},{"category":"Sm","key":"2B43","mappings":{"default":{"default":"Pfeil nach rechts durch mehr als"}}},{"category":"Sm","key":"2B44","mappings":{"default":{"default":"Pfeil nach rechts durch Superset"}}},{"category":"So","key":"2B45","mappings":{"default":{"default":"Vierfachpfeil nach links"}}},{"category":"So","key":"2B46","mappings":{"default":{"default":"Vierfachpfeil nach rechts"}}},{"category":"Sm","key":"2B47","mappings":{"default":{"default":"Reverse-Tilde-Operator über dem Pfeil nach rechts"}}},{"category":"Sm","key":"2B48","mappings":{"default":{"default":"Pfeil nach rechts oben fast gleich"}}},{"category":"Sm","key":"2B49","mappings":{"default":{"default":"Tilde-Operator über dem Pfeil nach links"}}},{"category":"Sm","key":"2B4A","mappings":{"default":{"default":"Pfeil nach links oben fast gleich"}}},{"category":"Sm","key":"2B4B","mappings":{"default":{"default":"Pfeil nach links über Reverse-Tilde-Operator"}}},{"category":"Sm","key":"2B4C","mappings":{"default":{"default":"Pfeil nach rechts über Reverse-Tilde-Operator"}}},{"category":"Sm","key":"FFE9","mappings":{"default":{"default":"Halbbreiter Pfeil nach links"}}},{"category":"Sm","key":"FFEA","mappings":{"default":{"default":"Halbbreiter Pfeil nach oben"}}},{"category":"Sm","key":"FFEB","mappings":{"default":{"default":"Halbbreiter Pfeil nach rechts"}}},{"category":"Sm","key":"FFEC","mappings":{"default":{"default":"Halbbreiter Pfeil nach unten"}}}]
,
"de/symbols/math_characters.js": 
[{"locale":"de"},{"category":"Ll","key":"2113","mappings":{"default":{"default":"Skript klein L"}}},{"category":"Sm","key":"2118","mappings":{"default":{"default":"Skript groß P"}}},{"category":"Ll","key":"213C","mappings":{"default":{"default":"pi mit Doppelstrich"}}},{"category":"Ll","key":"213D","mappings":{"default":{"default":"gamma mit Doppelstrich"}}},{"category":"Lu","key":"213E","mappings":{"default":{"default":"groß Gamma mit Doppelstrich"}}},{"category":"Lu","key":"213F","mappings":{"default":{"default":"groß Pi mit Doppelstrich"}}},{"category":"Sm","key":"2140","mappings":{"default":{"default":"Summenzeichen mit Doppelstrich"}}},{"category":"Lu","key":"2145","mappings":{"default":{"default":"groß D kursiv mit Doppelstrich"}}},{"category":"Ll","mappings":{"default":{"default":"d kursiv mit Doppelstrich"}},"key":"2146"},{"category":"Ll","mappings":{"default":{"default":"e kursiv mit Doppelstrich"}},"key":"2147"},{"category":"Ll","mappings":{"default":{"default":"i kursiv mit Doppelstrich"}},"key":"2148"},{"category":"Ll","key":"2149","mappings":{"default":{"default":"j kursiv mit Doppelstrich"}}},{"category":"Ll","key":"1D6A4","mappings":{"default":{"default":"punktloses i kursiv"}}},{"category":"Ll","key":"1D6A5","mappings":{"default":{"default":"punktloses j kursiv"}}}]
,
"de/symbols/math_delimiters.js": 
[{"locale":"de"},{"key":"0028","mappings":{"default":{"default":"Klammer auf"}},"category":"Ps"},{"key":"0029","mappings":{"default":{"default":"Klammer zu"}},"category":"Pe"},{"key":"005B","mappings":{"default":{"default":"eckige Klammer auf"}},"category":"Ps"},{"key":"005D","mappings":{"default":{"default":"eckige Klammer zu"}},"category":"Pe"},{"key":"007B","mappings":{"default":{"default":"geschwungene Klammer auf"}},"category":"Ps"},{"key":"007D","mappings":{"default":{"default":"geschwungene Klammer zu"}},"category":"Pe"},{"category":"Ps","key":"2045","mappings":{"default":{"default":"Linke eckige Klammer mit Stachel"}}},{"category":"Pe","key":"2046","mappings":{"default":{"default":"Rechte eckige Klammer mit Stachel"}}},{"category":"Sm","key":"2308","mappings":{"default":{"default":"linke Aufrundungsklammer"}}},{"category":"Sm","key":"2309","mappings":{"default":{"default":"rechte Aufrundungsklammer"}}},{"category":"Sm","key":"230A","mappings":{"default":{"default":"linke Abrundungsklammer"}}},{"category":"Sm","key":"230B","mappings":{"default":{"default":"rechter Abrundungsklammer"}}},{"category":"So","key":"230C","mappings":{"default":{"default":"unterer rechter Abschnitt"}}},{"category":"So","key":"230D","mappings":{"default":{"default":"Unterer linker Abschnitt"}}},{"category":"So","key":"230E","mappings":{"default":{"default":"Oberer rechter Abschnitt"}}},{"category":"So","key":"230F","mappings":{"default":{"default":"Oberer linker Abschnitt"}}},{"category":"So","key":"231C","mappings":{"default":{"default":"Obere linke Ecke"}}},{"category":"So","key":"231D","mappings":{"default":{"default":"Obere rechte Ecke"}}},{"category":"So","key":"231E","mappings":{"default":{"default":"Untere linke Ecke"}}},{"category":"So","key":"231F","mappings":{"default":{"default":"Rechte untere Ecke"}}},{"category":"Sm","key":"2320","mappings":{"default":{"default":"Obere Hälfte eines Integrals"}}},{"category":"Sm","key":"2321","mappings":{"default":{"default":"Untere Hälfte eines Integrals"}}},{"category":"Ps","key":"2329","mappings":{"default":{"default":"Linke spitze Klammer"}}},{"category":"Pe","key":"232A","mappings":{"default":{"default":"Rechte spitze Klammer"}}},{"category":"Sm","key":"239B","mappings":{"default":{"default":"Oberer Teil der linken Klammer"}}},{"category":"Sm","key":"239C","mappings":{"default":{"default":"Linke Klammerverlängerung"}}},{"category":"Sm","key":"239D","mappings":{"default":{"default":"Unterer Teil der linken Klammer"}}},{"category":"Sm","key":"239E","mappings":{"default":{"default":"Oberer Teil der rechten Klammer"}}},{"category":"Sm","key":"239F","mappings":{"default":{"default":"Rechte Klammerverlängerung"}}},{"category":"Sm","key":"23A0","mappings":{"default":{"default":"Unterer Teil der rechten Klammer"}}},{"category":"Sm","key":"23A1","mappings":{"default":{"default":"Oberer Teil der linken eckigen Klammer"}}},{"category":"Sm","key":"23A2","mappings":{"default":{"default":"Linke eckige Klammerverlängerung"}}},{"category":"Sm","key":"23A3","mappings":{"default":{"default":"unterer Teil der linken eckigen Klammer"}}},{"category":"Sm","key":"23A4","mappings":{"default":{"default":"Oberer Teil der rechten eckigen Klammer"}}},{"category":"Sm","key":"23A5","mappings":{"default":{"default":"Rechte eckige Klammerverlängerung"}}},{"category":"Sm","key":"23A6","mappings":{"default":{"default":"unterer Teil der rechten eckigen Klammer"}}},{"category":"Sm","key":"23A7","mappings":{"default":{"default":"oberer Teil der linken geschweiften Klammer"}}},{"category":"Sm","key":"23A8","mappings":{"default":{"default":"Mittelstück der linken geschweiften Klammer"}}},{"category":"Sm","key":"23A9","mappings":{"default":{"default":"unterer Teil der linken geschweiften Klammer"}}},{"category":"Sm","key":"23AA","mappings":{"default":{"default":"Zwischenteil einer geschweiften Klammer"}}},{"category":"Sm","key":"23AB","mappings":{"default":{"default":"oberer Teil der rechten geschweiften Klammer"}}},{"category":"Sm","key":"23AC","mappings":{"default":{"default":"Mittelstück der rechten geschweiften Klammer"}}},{"category":"Sm","key":"23AD","mappings":{"default":{"default":"unterer Teil der rechten geschweiften Klammer"}}},{"category":"Sm","key":"23AE","mappings":{"default":{"default":"Mittelteil eines Integrals"}}},{"category":"Sm","key":"23AF","mappings":{"default":{"default":"Erweiterung einer Querlinie"}}},{"category":"Sm","key":"23B0","mappings":{"default":{"default":"Geschweiftes Klammerteil oben links oder rechts unten"}}},{"category":"Sm","key":"23B1","mappings":{"default":{"default":"Geschweiftes Klammerteil oben rechts oder unten links"}}},{"category":"Sm","key":"23B2","mappings":{"default":{"default":"Oberer Teil einer Summe"}}},{"category":"Sm","key":"23B3","mappings":{"default":{"default":"Unterer Teil einer Summe"}}},{"key":"23B4","mappings":{"default":{"default":"Obere eckige Klammer"}},"category":"So"},{"key":"23B5","mappings":{"default":{"default":"Untere eckige Klammer"}},"category":"So"},{"category":"So","key":"23B6","mappings":{"default":{"default":"Untere eckige Klammer auf oberer eckiger Klammer"}}},{"category":"So","key":"23B7","mappings":{"default":{"default":"Unterer Teil des Radikalsymbols"}}},{"category":"So","key":"23B8","mappings":{"default":{"default":"Linke vertikale Boxlinie"}}},{"category":"So","key":"23B9","mappings":{"default":{"default":"Rechte vertikale Boxlinie"}}},{"category":"Sm","key":"23DC","mappings":{"default":{"default":"Obere Klammer"}}},{"category":"Sm","key":"23DD","mappings":{"default":{"default":"Untere Klammer"}}},{"category":"Sm","key":"23DE","mappings":{"default":{"default":"Obere geschweifte Klammer"}}},{"category":"Sm","key":"23DF","mappings":{"default":{"default":"Untere geschweifte Klammer"}}},{"category":"Sm","key":"23E0","mappings":{"default":{"default":"Obere eckige Klammer"}}},{"category":"Sm","key":"23E1","mappings":{"default":{"default":"Untere eckige Klammer"}}},{"category":"Ps","key":"2768","mappings":{"default":{"default":"Mittlere linke Klammerverzierung"}}},{"category":"Pe","key":"2769","mappings":{"default":{"default":"Mittlere rechte Klammerverzierung"}}},{"category":"Ps","key":"276A","mappings":{"default":{"default":"Mittlere abgeflachte linke Klammerverzierung"}}},{"category":"Pe","key":"276B","mappings":{"default":{"default":"Mittlere abgeflachte rechte Klammerverzierung"}}},{"category":"Ps","key":"276C","mappings":{"default":{"default":"Mittlere nach links weisende Winkelhalterung"}}},{"category":"Pe","key":"276D","mappings":{"default":{"default":"Mittlere, rechtwinklige Winkelhalterung"}}},{"category":"Ps","key":"276E","mappings":{"default":{"default":"Schwere nach links weisende Anführungszeichen-Verzierung"}}},{"category":"Pe","key":"276F","mappings":{"default":{"default":"Schwere nach rechts zeigenden Winkel-Anführungszeichen-Verzierung"}}},{"category":"Ps","key":"2770","mappings":{"default":{"default":"Schwere nach links weisende Winkelhalterung"}}},{"category":"Pe","key":"2771","mappings":{"default":{"default":"Schwere, nach rechts weisende Winkelhalterung"}}},{"category":"Ps","key":"2772","mappings":{"default":{"default":"Leichte linke Schildpatt Bracket Ornament"}}},{"category":"Pe","key":"2773","mappings":{"default":{"default":"Leichte rechte Schildpatt-Klammerverzierung"}}},{"category":"Ps","key":"2774","mappings":{"default":{"default":"Mittlere linke geschweifte Klammerverzierung"}}},{"category":"Pe","key":"2775","mappings":{"default":{"default":"Mittlere rechte geschweifte Klammerverzierung"}}},{"category":"Ps","key":"27C5","mappings":{"default":{"default":"Linker S-förmiger Taschenbegrenzer"}}},{"category":"Pe","key":"27C6","mappings":{"default":{"default":"rechter S-förmiger Taschenbegrenzer"}}},{"category":"Ps","key":"27E6","mappings":{"default":{"default":"Mathematische linke weiße eckige Klammer"}}},{"category":"Pe","key":"27E7","mappings":{"default":{"default":"Mathematische rechte weiße eckige Klammer"}}},{"category":"Ps","key":"27E8","mappings":{"default":{"default":"Mathematische linke Winkelklammer"}}},{"category":"Pe","key":"27E9","mappings":{"default":{"default":"Mathematische rechtwinklige Klammer"}}},{"category":"Ps","key":"27EA","mappings":{"default":{"default":"Mathematische linke doppelte Winkelklammer"}}},{"category":"Pe","key":"27EB","mappings":{"default":{"default":"Mathematische rechtwinklige eckige Klammer"}}},{"category":"Ps","key":"27EC","mappings":{"default":{"default":"Mathematische linke Schildkrötenpanzerhalterung"}}},{"category":"Pe","key":"27ED","mappings":{"default":{"default":"Mathematische rechte weiße Schildpatt-Klammer"}}},{"category":"Ps","key":"27EE","mappings":{"default":{"default":"Mathematische linke abgeflachte Klammer"}}},{"category":"Pe","key":"27EF","mappings":{"default":{"default":"Mathematische rechte abgeflachte Klammer"}}},{"category":"Ps","key":"2983","mappings":{"default":{"default":"Linke weiße geschweifte Klammer"}}},{"category":"Pe","key":"2984","mappings":{"default":{"default":"Rechte weiße geschweifte Klammer"}}},{"category":"Ps","key":"2985","mappings":{"default":{"default":"Linke weiße Klammer"}}},{"category":"Pe","key":"2986","mappings":{"default":{"default":"Rechte weiße Klammer"}}},{"category":"Ps","key":"2987","mappings":{"default":{"default":"Z-Notation Linke Bildklammer"}}},{"category":"Pe","key":"2988","mappings":{"default":{"default":"Z-Notation rechte Bildklammer"}}},{"category":"Ps","key":"2989","mappings":{"default":{"default":"Z-Notation Linke Klammer"}}},{"category":"Pe","key":"298A","mappings":{"default":{"default":"Z Notation Right Binding Bracket"}}},{"category":"Ps","key":"298B","mappings":{"default":{"default":"Linke eckige Klammer mit Unterlenker"}}},{"category":"Pe","key":"298C","mappings":{"default":{"default":"Rechte eckige Klammer mit Unterlenker"}}},{"category":"Ps","key":"298D","mappings":{"default":{"default":"Linke eckige Klammer mit Häkchen in der oberen Ecke"}}},{"category":"Pe","key":"298E","mappings":{"default":{"default":"Rechte eckige Klammer mit Tick in der unteren Ecke"}}},{"category":"Ps","key":"298F","mappings":{"default":{"default":"Linke eckige Klammer mit Tick in der unteren Ecke"}}},{"category":"Pe","key":"2990","mappings":{"default":{"default":"Rechte eckige Klammer mit Tick in der oberen Ecke"}}},{"category":"Ps","key":"2991","mappings":{"default":{"default":"Linke Winkelklammer mit Punkt"}}},{"category":"Pe","key":"2992","mappings":{"default":{"default":"Rechtwinklige Klammer mit Punkt"}}},{"category":"Ps","key":"2993","mappings":{"default":{"default":"Linker Bogen weniger als Halterung"}}},{"category":"Pe","key":"2994","mappings":{"default":{"default":"rechter Bogen größer als Halterung"}}},{"category":"Ps","key":"2995","mappings":{"default":{"default":"Doppelter linker Bogen größer als Halterung"}}},{"category":"Pe","key":"2996","mappings":{"default":{"default":"Doppelter rechter Bogen weniger als Halterung"}}},{"category":"Ps","key":"2997","mappings":{"default":{"default":"Linke schwarze Schildkrötenhalterung"}}},{"category":"Pe","key":"2998","mappings":{"default":{"default":"Rechte schwarze Schildpatt-Klammer"}}},{"category":"Ps","key":"29D8","mappings":{"default":{"default":"Linker wackeliger Zaun"}}},{"category":"Pe","key":"29D9","mappings":{"default":{"default":"rechter wackeliger Zaun"}}},{"category":"Ps","key":"29DA","mappings":{"default":{"default":"Linker doppelter wackeliger Zaun"}}},{"category":"Pe","key":"29DB","mappings":{"default":{"default":"rechter doppelter wackeliger Zaun"}}},{"category":"Ps","key":"29FC","mappings":{"default":{"default":"Nach links zeigender gebogener Winkelhalter"}}},{"category":"Pe","key":"29FD","mappings":{"default":{"default":"Nach rechts zeigender gebogener Winkelhalter"}}},{"category":"Ps","key":"2E22","mappings":{"default":{"default":"Obere linke halbe Halterung"}}},{"category":"Pe","key":"2E23","mappings":{"default":{"default":"Obere rechte halbe Klammer"}}},{"category":"Ps","key":"2E24","mappings":{"default":{"default":"Linke halbe halterung unten"}}},{"category":"Pe","key":"2E25","mappings":{"default":{"default":"Untere rechte halbe Halterung"}}},{"category":"Ps","key":"2E26","mappings":{"default":{"default":"U-Halterung links seitlich"}}},{"category":"Pe","key":"2E27","mappings":{"default":{"default":"U-Halterung rechts seitlich"}}},{"category":"Ps","key":"2E28","mappings":{"default":{"default":"Linke doppelte Klammer"}}},{"category":"Pe","key":"2E29","mappings":{"default":{"default":"Rechte doppelte Klammer"}}},{"category":"Ps","key":"3008","mappings":{"default":{"default":"Linke Winkelklammer"}}},{"category":"Pe","key":"3009","mappings":{"default":{"default":"Rechtwinklige Halterung"}}},{"category":"Ps","key":"300A","mappings":{"default":{"default":"Linke Doppelwinkelhalterung"}}},{"category":"Pe","key":"300B","mappings":{"default":{"default":"Rechte Doppelwinkelhalterung"}}},{"category":"Ps","key":"300C","mappings":{"default":{"default":"Linke Eckklammer"}}},{"category":"Pe","key":"300D","mappings":{"default":{"default":"Rechte Eckhalterung"}}},{"category":"Ps","key":"300E","mappings":{"default":{"default":"Linke weiße Eckhalterung"}}},{"category":"Pe","key":"300F","mappings":{"default":{"default":"Rechte weiße Eckhalterung"}}},{"category":"Ps","key":"3010","mappings":{"default":{"default":"Linke schwarze linsenförmige Halterung"}}},{"category":"Pe","key":"3011","mappings":{"default":{"default":"Rechte schwarze linsenförmige Halterung"}}},{"category":"Ps","key":"3014","mappings":{"default":{"default":"Linke Schildpatt-Klammer"}}},{"category":"Pe","key":"3015","mappings":{"default":{"default":"Rechte Schildpattlasche"}}},{"category":"Ps","key":"3016","mappings":{"default":{"default":"Linke weiße linsenförmige Halterung"}}},{"category":"Pe","key":"3017","mappings":{"default":{"default":"Rechte weiße linsenförmige Halterung"}}},{"category":"Ps","key":"3018","mappings":{"default":{"default":"Linke weiße Schildpatt-Klammer"}}},{"category":"Pe","key":"3019","mappings":{"default":{"default":"Rechte weiße Schildpatt-Klammer"}}},{"category":"Ps","key":"301A","mappings":{"default":{"default":"Linke weiße eckige Klammer"}}},{"category":"Pe","key":"301B","mappings":{"default":{"default":"Rechte weiße eckige Klammer"}}},{"category":"Ps","key":"301D","mappings":{"default":{"default":"Doppelte Anführungszeichen in umgekehrter Reihenfolge"}}},{"category":"Pe","key":"301E","mappings":{"default":{"default":"Doppelte Anführungszeichen"}}},{"category":"Pe","key":"301F","mappings":{"default":{"default":"Niedriges doppeltes Prime-Anführungszeichen"}}},{"category":"Ps","key":"FD3E","mappings":{"default":{"default":"Verzierte linke Klammer"}}},{"category":"Pe","key":"FD3F","mappings":{"default":{"default":"Verzierte rechte Klammer"}}},{"category":"Ps","key":"FE17","mappings":{"default":{"default":"Präsentationsformular für vertikale linke weiße linsenförmige Halterung"}}},{"category":"Pe","key":"FE18","mappings":{"default":{"default":"Präsentationsformular für vertikale rechtwinklige linsenförmige Halterung"}}},{"category":"Ps","key":"FE35","mappings":{"default":{"default":"Präsentationsformular für vertikale linke Klammer"}}},{"category":"Pe","key":"FE36","mappings":{"default":{"default":"Präsentationsformular für vertikale rechte Klammer"}}},{"category":"Ps","key":"FE37","mappings":{"default":{"default":"Präsentationsformular für vertikale linke geschweifte Klammer"}}},{"category":"Pe","key":"FE38","mappings":{"default":{"default":"Präsentationsformular für vertikale, recht geschweifte Klammer"}}},{"category":"Ps","key":"FE39","mappings":{"default":{"default":"Präsentationsformular für vertikale linke Schildpattklammer"}}},{"category":"Pe","key":"FE3A","mappings":{"default":{"default":"Präsentationsformular für vertikale rechte Schildpatt-Klammer"}}},{"category":"Ps","key":"FE3B","mappings":{"default":{"default":"Präsentationsformular für vertikale linke schwarze linsenförmige Halterung"}}},{"category":"Pe","key":"FE3C","mappings":{"default":{"default":"Präsentationsformular für vertikale rechtwinklige schwarze linsenförmige Halterung"}}},{"category":"Ps","key":"FE3D","mappings":{"default":{"default":"Präsentationsformular für vertikale linke Doppelwinkelhalterung"}}},{"category":"Pe","key":"FE3E","mappings":{"default":{"default":"Präsentationsformular für vertikale rechtwinklige, rechtwinklige Halterung"}}},{"category":"Ps","key":"FE3F","mappings":{"default":{"default":"Präsentationsformular für vertikale linke Winkelhalterung"}}},{"category":"Pe","key":"FE40","mappings":{"default":{"default":"Präsentationsformular für vertikale rechtwinklige Halterung"}}},{"category":"Ps","key":"FE41","mappings":{"default":{"default":"Präsentationsformular für vertikale linke Eckhalterung"}}},{"category":"Pe","key":"FE42","mappings":{"default":{"default":"Präsentationsformular für vertikale rechte Eckhalterung"}}},{"category":"Ps","key":"FE43","mappings":{"default":{"default":"Präsentationsformular für vertikale linke weiße Eckhalterung"}}},{"category":"Pe","key":"FE44","mappings":{"default":{"default":"Präsentationsformular für vertikale rechte weiße Eckhalterung"}}},{"category":"Ps","key":"FE47","mappings":{"default":{"default":"Präsentationsformular für vertikale linke eckige Klammer"}}},{"category":"Pe","key":"FE48","mappings":{"default":{"default":"Präsentationsformular für vertikale eckige Klammer"}}},{"category":"Ps","key":"FE59","mappings":{"default":{"default":"Kleine linke Klammer"}}},{"category":"Pe","key":"FE5A","mappings":{"default":{"default":"Kleine rechte Klammer"}}},{"category":"Ps","key":"FE5B","mappings":{"default":{"default":"Kleine linke geschweifte Klammer"}}},{"category":"Pe","key":"FE5C","mappings":{"default":{"default":"Kleine rechte geschweifte Klammer"}}},{"category":"Ps","key":"FE5D","mappings":{"default":{"default":"Kleine linke Schildpatt-Klammer"}}},{"category":"Pe","key":"FE5E","mappings":{"default":{"default":"Kleine rechte Schildpatt-Klammer"}}},{"category":"Ps","key":"FF08","mappings":{"default":{"default":"Linke Klammer in voller Breite"}}},{"category":"Pe","key":"FF09","mappings":{"default":{"default":"Rechte Klammer mit voller Breite"}}},{"category":"Ps","key":"FF3B","mappings":{"default":{"default":"Linke eckige Klammer mit voller Breite"}}},{"category":"Pe","key":"FF3D","mappings":{"default":{"default":"Rechteckige Klammer mit voller Breite"}}},{"category":"Ps","key":"FF5B","mappings":{"default":{"default":"Linke geschweifte Klammer mit voller Breite"}}},{"category":"Pe","key":"FF5D","mappings":{"default":{"default":"Rechtwinklige Klammer mit voller Breite"}}},{"category":"Ps","key":"FF5F","mappings":{"default":{"default":"Linke weiße Klammer mit voller Breite"}}},{"category":"Pe","key":"FF60","mappings":{"default":{"default":"Rechte breite weiße Klammer"}}},{"category":"Ps","key":"FF62","mappings":{"default":{"default":"Halbe breite linke eckklammer"}}},{"category":"Pe","key":"FF63","mappings":{"default":{"default":"Halbe breite rechte eckklammer"}}}]
,
"de/symbols/math_geometry.js": 
[{"locale":"de"},{"category":"So","key":"2500","mappings":{"default":{"default":"Box Drawings Light Horizontal"}}},{"category":"So","key":"2501","mappings":{"default":{"default":"Box Drawings Schwere Horizontale"}}},{"category":"So","key":"2502","mappings":{"default":{"default":"Box Drawings Light Vertical"}}},{"category":"So","key":"2503","mappings":{"default":{"default":"Box Drawings Schwere Vertikale"}}},{"category":"So","key":"2504","mappings":{"default":{"default":"Box-Zeichnungen Licht Triple Dash Horizontal"}}},{"category":"So","key":"2505","mappings":{"default":{"default":"Box Drawings Heavy Triple Dash Horizontal"}}},{"category":"So","key":"2506","mappings":{"default":{"default":"Box Drawings Light Triple Dash Vertical"}}},{"category":"So","key":"2507","mappings":{"default":{"default":"Box Drawings Heavy Triple Dash Vertical"}}},{"category":"So","key":"2508","mappings":{"default":{"default":"Box Drawings Light Quadruple Dash Horizontal"}}},{"category":"So","key":"2509","mappings":{"default":{"default":"Box Drawings Heavy Quadruple Dash Horizontal"}}},{"category":"So","key":"250A","mappings":{"default":{"default":"Box Drawings Light Quadruple Dash Vertical"}}},{"category":"So","key":"250B","mappings":{"default":{"default":"Box Drawings Heavy Quadruple Dash Vertical"}}},{"category":"So","key":"250C","mappings":{"default":{"default":"Box Drawings Light Down und Right"}}},{"category":"So","key":"250D","mappings":{"default":{"default":"Box Drawings Down Light und Right Heavy"}}},{"category":"So","key":"250E","mappings":{"default":{"default":"Box Drawings Down Heavy und Right Light"}}},{"category":"So","key":"250F","mappings":{"default":{"default":"Box Drawings Heavy Down und Right"}}},{"category":"So","key":"2510","mappings":{"default":{"default":"Box Drawings Light Down und Links"}}},{"category":"So","key":"2511","mappings":{"default":{"default":"Box Drawings Down Light und Left Heavy"}}},{"category":"So","key":"2512","mappings":{"default":{"default":"Box Drawings Down Heavy und Left Light"}}},{"category":"So","key":"2513","mappings":{"default":{"default":"Box Drawings Heavy Down und Left"}}},{"category":"So","key":"2514","mappings":{"default":{"default":"Box-Zeichnungen leuchten auf und richtig"}}},{"category":"So","key":"2515","mappings":{"default":{"default":"Box Drawings Up Light und Right Heavy"}}},{"category":"So","key":"2516","mappings":{"default":{"default":"Box Drawings Up Heavy und Right Light"}}},{"category":"So","key":"2517","mappings":{"default":{"default":"Box Drawings Heavy Up und Right"}}},{"category":"So","key":"2518","mappings":{"default":{"default":"Box-Zeichnungen leuchten und links"}}},{"category":"So","key":"2519","mappings":{"default":{"default":"Box Drawings Up Light und Left Heavy"}}},{"category":"So","key":"251A","mappings":{"default":{"default":"Box Drawings Up Heavy und Left Light"}}},{"category":"So","key":"251B","mappings":{"default":{"default":"Box Drawings Heavy Up und Links"}}},{"category":"So","key":"251C","mappings":{"default":{"default":"Box Drawings Light Vertical und Right"}}},{"category":"So","key":"251D","mappings":{"default":{"default":"Box Drawings Vertical Light und Right Heavy"}}},{"category":"So","key":"251E","mappings":{"default":{"default":"Box Drawings Up Heavy und Right Down Light"}}},{"category":"So","key":"251F","mappings":{"default":{"default":"Box Drawings Down Heavy und Right Up Light"}}},{"category":"So","key":"2520","mappings":{"default":{"default":"Box Drawings Vertical Heavy und Right Light"}}},{"category":"So","key":"2521","mappings":{"default":{"default":"Box Drawings Down Light und Right Up Heavy"}}},{"category":"So","key":"2522","mappings":{"default":{"default":"Box Drawings Up Light und Right Down Heavy"}}},{"category":"So","key":"2523","mappings":{"default":{"default":"Box Drawings Heavy Vertical und Right"}}},{"category":"So","key":"2524","mappings":{"default":{"default":"Box Drawings Light Vertical und Left"}}},{"category":"So","key":"2525","mappings":{"default":{"default":"Box Drawings Vertical Light und Left Heavy"}}},{"category":"So","key":"2526","mappings":{"default":{"default":"Box Drawings Up Heavy und Left Down Light"}}},{"category":"So","key":"2527","mappings":{"default":{"default":"Box Drawings Down Heavy und Left Up Light"}}},{"category":"So","key":"2528","mappings":{"default":{"default":"Box Drawings Vertical Heavy und Left Light"}}},{"category":"So","key":"2529","mappings":{"default":{"default":"Box Drawings Down Light und Left Up Heavy"}}},{"category":"So","key":"252A","mappings":{"default":{"default":"Box Drawings Up Light und Left Down Heavy"}}},{"category":"So","key":"252B","mappings":{"default":{"default":"Box Drawings Heavy Vertical und Left"}}},{"category":"So","key":"252C","mappings":{"default":{"default":"Box-Zeichnungen leicht und horizontal"}}},{"category":"So","key":"252D","mappings":{"default":{"default":"Box Drawings Left Heavy und Right Down Light"}}},{"category":"So","key":"252E","mappings":{"default":{"default":"Box Drawings Right Heavy und Left Down Light"}}},{"category":"So","key":"252F","mappings":{"default":{"default":"Box Drawings Down Light und Horizontal Heavy"}}},{"category":"So","key":"2530","mappings":{"default":{"default":"Box-Zeichnungen für schweres und horizontales Licht"}}},{"category":"So","key":"2531","mappings":{"default":{"default":"Box Drawings Right Light und Left Down Heavy"}}},{"category":"So","key":"2532","mappings":{"default":{"default":"Box Drawings Left Light und Right Down Heavy"}}},{"category":"So","key":"2533","mappings":{"default":{"default":"Box-Zeichnungen nach unten und horizontal"}}},{"category":"So","key":"2534","mappings":{"default":{"default":"Box-Zeichnungen leuchten und horizontal"}}},{"category":"So","key":"2535","mappings":{"default":{"default":"Box Drawings Left Heavy und Right Up Light"}}},{"category":"So","key":"2536","mappings":{"default":{"default":"Box Drawings Right Heavy und Left Up Light"}}},{"category":"So","key":"2537","mappings":{"default":{"default":"Box Drawings Up Light und Horizontal Heavy"}}},{"category":"So","key":"2538","mappings":{"default":{"default":"Box-Zeichnungen für schweres und horizontales Licht"}}},{"category":"So","key":"2539","mappings":{"default":{"default":"Box Drawings Right Light und Left Up Heavy"}}},{"category":"So","key":"253A","mappings":{"default":{"default":"Box Drawings Left Light und Right Up Heavy"}}},{"category":"So","key":"253B","mappings":{"default":{"default":"Box-Zeichnungen schwer und horizontal"}}},{"category":"So","key":"253C","mappings":{"default":{"default":"Box Drawings Light vertikal und horizontal"}}},{"category":"So","key":"253D","mappings":{"default":{"default":"Kastenzeichnungen links schweres und rechtes vertikales Licht"}}},{"category":"So","key":"253E","mappings":{"default":{"default":"Box Drawings Right Heavy und Left Vertical Light"}}},{"category":"So","key":"253F","mappings":{"default":{"default":"Box Drawings Vertical Light und Horizontal Heavy"}}},{"category":"So","key":"2540","mappings":{"default":{"default":"Box Drawings Up Heavy und Down Horizontal Light"}}},{"category":"So","key":"2541","mappings":{"default":{"default":"Box Drawings Down Heavy und Up Horizontal Light"}}},{"category":"So","key":"2542","mappings":{"default":{"default":"Kastenzeichnungen Vertikales schweres und horizontales Licht"}}},{"category":"So","key":"2543","mappings":{"default":{"default":"Box Drawings Left Up Heavy und Right Down Light"}}},{"category":"So","key":"2544","mappings":{"default":{"default":"Box Drawings Right Up Heavy und Left Down Light"}}},{"category":"So","key":"2545","mappings":{"default":{"default":"Box Drawings Left Down Heavy und Right Up Light"}}},{"category":"So","key":"2546","mappings":{"default":{"default":"Box Drawings Right Down Heavy und Left Up Light"}}},{"category":"So","key":"2547","mappings":{"default":{"default":"Box Drawings Down Light und Up Horizontal Heavy"}}},{"category":"So","key":"2548","mappings":{"default":{"default":"Box Drawings Up Light und Down Horizontal Heavy"}}},{"category":"So","key":"2549","mappings":{"default":{"default":"Box Drawings Right Light und Left Vertical Heavy"}}},{"category":"So","key":"254A","mappings":{"default":{"default":"Box Drawings Left Light und Right Vertical Heavy"}}},{"category":"So","key":"254B","mappings":{"default":{"default":"Box Drawings Heavy vertikal und horizontal"}}},{"category":"So","key":"254C","mappings":{"default":{"default":"Box Drawings Light Double Dash Horizontal"}}},{"category":"So","key":"254D","mappings":{"default":{"default":"Box Drawings Heavy Double Dash Horizontal"}}},{"category":"So","key":"254E","mappings":{"default":{"default":"Box Drawings Light Double Dash Vertical"}}},{"category":"So","key":"254F","mappings":{"default":{"default":"Box Drawings Heavy Double Dash Vertical"}}},{"category":"So","key":"2550","mappings":{"default":{"default":"Box-Zeichnungen doppelt horizontal"}}},{"category":"So","key":"2551","mappings":{"default":{"default":"Box Drawings Double Vertical"}}},{"category":"So","key":"2552","mappings":{"default":{"default":"Box Drawings Down Single und Right Double"}}},{"category":"So","key":"2553","mappings":{"default":{"default":"Box Drawings Down Double und Right Single"}}},{"category":"So","key":"2554","mappings":{"default":{"default":"Box Drawings Double Down und Right"}}},{"category":"So","key":"2555","mappings":{"default":{"default":"Box Drawings Down Single und Left Double"}}},{"category":"So","key":"2556","mappings":{"default":{"default":"Box Drawings Down Double und Left Single"}}},{"category":"So","key":"2557","mappings":{"default":{"default":"Box Drawings Double Down und Links"}}},{"category":"So","key":"2558","mappings":{"default":{"default":"Box Drawings Up Single und Right Double"}}},{"category":"So","key":"2559","mappings":{"default":{"default":"Box Drawings Up Double und Right Single"}}},{"category":"So","key":"255A","mappings":{"default":{"default":"Box-Zeichnungen verdoppeln und rechts"}}},{"category":"So","key":"255B","mappings":{"default":{"default":"Box Drawings Up Single und Left Double"}}},{"category":"So","key":"255C","mappings":{"default":{"default":"Box Drawings Up Double und Left Single"}}},{"category":"So","key":"255D","mappings":{"default":{"default":"Box-Zeichnungen verdoppeln und links"}}},{"category":"So","key":"255E","mappings":{"default":{"default":"Box Drawings Vertical Single und Right Double"}}},{"category":"So","key":"255F","mappings":{"default":{"default":"Box Drawings Vertical Double und Right Single"}}},{"category":"So","key":"2560","mappings":{"default":{"default":"Box Drawings Double Vertical und Right"}}},{"category":"So","key":"2561","mappings":{"default":{"default":"Box Drawings Vertical Single und Left Double"}}},{"category":"So","key":"2562","mappings":{"default":{"default":"Box Drawings Vertical Double und Left Single"}}},{"category":"So","key":"2563","mappings":{"default":{"default":"Box Drawings Double Vertical und Left"}}},{"category":"So","key":"2564","mappings":{"default":{"default":"Box Drawings Down Single und Horizontal Double"}}},{"category":"So","key":"2565","mappings":{"default":{"default":"Box Drawings Down Double und Horizontal Single"}}},{"category":"So","key":"2566","mappings":{"default":{"default":"Box Drawings Double Down und Horizontal"}}},{"category":"So","key":"2567","mappings":{"default":{"default":"Box Drawings Up Single und Horizontal Double"}}},{"category":"So","key":"2568","mappings":{"default":{"default":"Box Drawings Up Double und Horizontal Single"}}},{"category":"So","key":"2569","mappings":{"default":{"default":"Box-Zeichnungen verdoppeln und horizontal"}}},{"category":"So","key":"256A","mappings":{"default":{"default":"Box Drawings Vertical Single und Horizontal Double"}}},{"category":"So","key":"256B","mappings":{"default":{"default":"Box Drawings Vertical Double und Horizontal Single"}}},{"category":"So","key":"256C","mappings":{"default":{"default":"Box-Zeichnungen doppelt vertikal und horizontal"}}},{"category":"So","key":"256D","mappings":{"default":{"default":"Box Drawings Light Arc Down und Right"}}},{"category":"So","key":"256E","mappings":{"default":{"default":"Box Drawings Light Arc Down und Links"}}},{"category":"So","key":"256F","mappings":{"default":{"default":"Box Drawings Light Arc Up und links"}}},{"category":"So","key":"2570","mappings":{"default":{"default":"Box Drawings Light Arc Up und rechts"}}},{"category":"So","key":"2571","mappings":{"default":{"default":"Box Drawings Light Diagonal Oben Rechts Nach Unten Links"}}},{"category":"So","key":"2572","mappings":{"default":{"default":"Box Drawings Light Diagonal von oben links nach rechts unten"}}},{"category":"So","key":"2573","mappings":{"default":{"default":"Box Zeichnungen Light Diagonal Cross"}}},{"category":"So","key":"2574","mappings":{"default":{"default":"Box-Zeichnungen Light Left"}}},{"category":"So","key":"2575","mappings":{"default":{"default":"Box-Zeichnungen leuchten auf"}}},{"category":"So","key":"2576","mappings":{"default":{"default":"Box-Zeichnungen Light Right"}}},{"category":"So","key":"2577","mappings":{"default":{"default":"Box Drawings Light Down"}}},{"category":"So","key":"2578","mappings":{"default":{"default":"Box Drawings Heavy Left"}}},{"category":"So","key":"2579","mappings":{"default":{"default":"Box Drawings Heavy Up"}}},{"category":"So","key":"257A","mappings":{"default":{"default":"Box Drawings Heavy Right"}}},{"category":"So","key":"257B","mappings":{"default":{"default":"Box Drawings Heavy Down"}}},{"category":"So","key":"257C","mappings":{"default":{"default":"Box Drawings Light Left und Heavy Right"}}},{"category":"So","key":"257D","mappings":{"default":{"default":"Box Drawings Light Up und Heavy Down"}}},{"category":"So","key":"257E","mappings":{"default":{"default":"Box Drawings Heavy Left und Light Right"}}},{"category":"So","key":"257F","mappings":{"default":{"default":"Box Drawings Heavy Up und Light Down"}}},{"category":"So","key":"2580","mappings":{"default":{"default":"Block der oberen Hälfte"}}},{"category":"So","key":"2581","mappings":{"default":{"default":"Unterer achter Block"}}},{"category":"So","key":"2582","mappings":{"default":{"default":"Unterer Viertelblock"}}},{"category":"So","key":"2583","mappings":{"default":{"default":"Drei Achtel-Block senken"}}},{"category":"So","key":"2584","mappings":{"default":{"default":"Block der unteren Hälfte"}}},{"category":"So","key":"2585","mappings":{"default":{"default":"Fünf Achtel Block"}}},{"category":"So","key":"2586","mappings":{"default":{"default":"Unterer Dreiviertelblock"}}},{"category":"So","key":"2587","mappings":{"default":{"default":"Unterer Sieben Achtel-Block"}}},{"category":"So","key":"2588","mappings":{"default":{"default":"Voller Block"}}},{"category":"So","key":"2589","mappings":{"default":{"default":"Linker Sieben Achtel Block"}}},{"category":"So","key":"258A","mappings":{"default":{"default":"Linker Dreiviertelblock"}}},{"category":"So","key":"258B","mappings":{"default":{"default":"Linker Block mit fünf Achteln"}}},{"category":"So","key":"258C","mappings":{"default":{"default":"Linke halbe Block"}}},{"category":"So","key":"258D","mappings":{"default":{"default":"Block drei Achtel"}}},{"category":"So","key":"258E","mappings":{"default":{"default":"Linke einen Viertelblock"}}},{"category":"So","key":"258F","mappings":{"default":{"default":"Linker achter Block"}}},{"category":"So","key":"2590","mappings":{"default":{"default":"Rechte halbe Block"}}},{"category":"So","key":"2591","mappings":{"default":{"default":"Lichtschatten"}}},{"category":"So","key":"2592","mappings":{"default":{"default":"Mittlere Schatten"}}},{"category":"So","key":"2593","mappings":{"default":{"default":"Dunkler Schatten"}}},{"category":"So","key":"2594","mappings":{"default":{"default":"Oberer achter Block"}}},{"category":"So","key":"2595","mappings":{"default":{"default":"rechter achter Block"}}},{"category":"So","key":"2596","mappings":{"default":{"default":"Quadrant unten links"}}},{"category":"So","key":"2597","mappings":{"default":{"default":"Quadrant rechts unten"}}},{"category":"So","key":"2598","mappings":{"default":{"default":"Quadrant oben links"}}},{"category":"So","key":"2599","mappings":{"default":{"default":"Quadrant oben links und unten links und rechts unten"}}},{"category":"So","key":"259A","mappings":{"default":{"default":"Quadrant oben links und rechts unten"}}},{"category":"So","key":"259B","mappings":{"default":{"default":"Quadrant oben links und oben rechts und unten links"}}},{"category":"So","key":"259C","mappings":{"default":{"default":"Quadrant oben links und oben rechts und unten rechts"}}},{"category":"So","key":"259D","mappings":{"default":{"default":"Quadrant oben rechts"}}},{"category":"So","key":"259E","mappings":{"default":{"default":"Quadrant oben rechts und unten links"}}},{"category":"So","key":"259F","mappings":{"default":{"default":"Quadrant oben rechts und unten links und unten rechts"}}},{"category":"So","key":"25A0","mappings":{"default":{"default":"Schwarzes Quadrat"}}},{"category":"So","key":"25A1","mappings":{"default":{"default":"Weißes Quadrat"}}},{"category":"So","key":"25A2","mappings":{"default":{"default":"Weißes Quadrat mit abgerundeten Ecken"}}},{"category":"So","key":"25A3","mappings":{"default":{"default":"Weißes Quadrat, das schwarzes kleines Quadrat enthält"}}},{"category":"So","key":"25A4","mappings":{"default":{"default":"Quadrat mit horizontaler Füllung"}}},{"category":"So","key":"25A5","mappings":{"default":{"default":"Quadrat mit vertikaler Füllung"}}},{"category":"So","key":"25A6","mappings":{"default":{"default":"Quadrat mit orthogonaler Schraffurfüllung"}}},{"category":"So","key":"25A7","mappings":{"default":{"default":"Quadrat mit Oben links nach rechts unten füllen"}}},{"category":"So","key":"25A8","mappings":{"default":{"default":"Quadrat mit oberer rechter bis unterer linker Füllung"}}},{"category":"So","key":"25A9","mappings":{"default":{"default":"Quadrat mit diagonaler Schraffurfüllung"}}},{"category":"So","key":"25AA","mappings":{"default":{"default":"Schwarzes kleines Quadrat"}}},{"category":"So","key":"25AB","mappings":{"default":{"default":"Weißes kleines Quadrat"}}},{"category":"So","key":"25AC","mappings":{"default":{"default":"Schwarzes Rechteck"}}},{"category":"So","key":"25AD","mappings":{"default":{"default":"Weißes Rechteck"}}},{"category":"So","key":"25AE","mappings":{"default":{"default":"Schwarzes vertikales Rechteck"}}},{"category":"So","key":"25AF","mappings":{"default":{"default":"Weißes vertikales Rechteck"}}},{"category":"So","key":"25B0","mappings":{"default":{"default":"Schwarzes Parallelogramm"}}},{"category":"So","key":"25B1","mappings":{"default":{"default":"Weißes Parallelogramm"}}},{"category":"So","key":"25B2","mappings":{"default":{"default":"Schwarzes nach oben zeigendes Dreieck"}}},{"category":"So","key":"25B3","mappings":{"default":{"default":"Weißes nach oben zeigendes Dreieck"}}},{"category":"So","key":"25B4","mappings":{"default":{"default":"Schwarzes nach oben zeigendes kleines Dreieck"}}},{"category":"So","key":"25B5","mappings":{"default":{"default":"Weißes nach oben zeigendes kleines Dreieck"}}},{"category":"So","key":"25B6","mappings":{"default":{"default":"Schwarzes rechtwinkliges Dreieck"}}},{"category":"Sm","key":"25B7","mappings":{"default":{"default":"Weißes rechtwinkliges Dreieck"}}},{"category":"So","key":"25B8","mappings":{"default":{"default":"Schwarzes nach rechts zeigendes kleines Dreieck"}}},{"category":"So","key":"25B9","mappings":{"default":{"default":"Weißes nach rechts zeigendes kleines Dreieck"}}},{"category":"So","key":"25BA","mappings":{"default":{"default":"Schwarzer nach rechts zeigender Zeiger"}}},{"category":"So","key":"25BB","mappings":{"default":{"default":"Weißer nach rechts zeigender Zeiger"}}},{"category":"So","key":"25BC","mappings":{"default":{"default":"Schwarzes Abwärtsdreieck"}}},{"category":"So","key":"25BD","mappings":{"default":{"default":"Weißes nach unten zeigendes Dreieck"}}},{"category":"So","key":"25BE","mappings":{"default":{"default":"Schwarzes, nach unten zeigendes kleines Dreieck"}}},{"category":"So","key":"25BF","mappings":{"default":{"default":"Weißes kleines Dreieck"}}},{"category":"So","key":"25C0","mappings":{"default":{"default":"Schwarzes nach links zeigendes Dreieck"}}},{"category":"Sm","key":"25C1","mappings":{"default":{"default":"Weißes nach links zeigendes Dreieck"}}},{"category":"So","key":"25C2","mappings":{"default":{"default":"Schwarzes nach links zeigendes kleines Dreieck"}}},{"category":"So","key":"25C3","mappings":{"default":{"default":"Weißes nach links zeigendes kleines Dreieck"}}},{"category":"So","key":"25C4","mappings":{"default":{"default":"Schwarzer nach links zeigender Zeiger"}}},{"category":"So","key":"25C5","mappings":{"default":{"default":"Weißer nach links zeigender Zeiger"}}},{"category":"So","key":"25C6","mappings":{"default":{"default":"Schwarzer Diamant"}}},{"category":"So","key":"25C7","mappings":{"default":{"default":"Weißer Diamant"}}},{"category":"So","key":"25C8","mappings":{"default":{"default":"Weißer Diamant mit schwarzem kleinem Diamant"}}},{"category":"So","key":"25C9","mappings":{"default":{"default":"Fischauge"}}},{"category":"So","key":"25CA","mappings":{"default":{"default":"Raute"}}},{"category":"So","key":"25CB","mappings":{"default":{"default":"Weißer Kreis"}}},{"category":"So","key":"25CC","mappings":{"default":{"default":"Gepunkteter Kreis"}}},{"category":"So","key":"25CD","mappings":{"default":{"default":"Kreis mit vertikaler Füllung"}}},{"category":"So","key":"25CE","mappings":{"default":{"default":"Bullseye"}}},{"category":"So","key":"25CF","mappings":{"default":{"default":"Schwarzer Kreis"}}},{"category":"So","key":"25D0","mappings":{"default":{"default":"Kreis mit linker Hälfte schwarz"}}},{"category":"So","key":"25D1","mappings":{"default":{"default":"Kreis mit rechter Hälfte schwarz"}}},{"category":"So","key":"25D2","mappings":{"default":{"default":"Kreis mit unterer Hälfte Schwarz"}}},{"category":"So","key":"25D3","mappings":{"default":{"default":"Kreis mit oberer Hälfte schwarz"}}},{"category":"So","key":"25D4","mappings":{"default":{"default":"Kreis mit oberem rechten Quadranten schwarz"}}},{"category":"So","key":"25D5","mappings":{"default":{"default":"Kreis mit ganzem oberen linken Quadranten schwarz"}}},{"category":"So","key":"25D6","mappings":{"default":{"default":"Linker halber schwarzer Kreis"}}},{"category":"So","key":"25D7","mappings":{"default":{"default":"rechter halber schwarzer Kreis"}}},{"category":"So","key":"25D8","mappings":{"default":{"default":"Inverse Kugel"}}},{"category":"So","key":"25D9","mappings":{"default":{"default":"Inverse weißer Kreis"}}},{"category":"So","key":"25DA","mappings":{"default":{"default":"Umgekehrter weißer Kreis der oberen Hälfte"}}},{"category":"So","key":"25DB","mappings":{"default":{"default":"Inverser weißer Kreis der unteren Hälfte"}}},{"category":"So","key":"25DC","mappings":{"default":{"default":"Kreisbogen des oberen linken Quadranten"}}},{"category":"So","key":"25DD","mappings":{"default":{"default":"Kreisbogen im oberen rechten Quadranten"}}},{"category":"So","key":"25DE","mappings":{"default":{"default":"Kreisbogen des rechten unteren Quadranten"}}},{"category":"So","key":"25DF","mappings":{"default":{"default":"Kreisbogen des unteren linken Quadranten"}}},{"category":"So","key":"25E0","mappings":{"default":{"default":"Oberer Halbkreis"}}},{"category":"So","key":"25E1","mappings":{"default":{"default":"Unterer Halbkreis"}}},{"category":"So","key":"25E2","mappings":{"default":{"default":"Schwarzes rechtes Dreieck"}}},{"category":"So","key":"25E3","mappings":{"default":{"default":"Schwarzes unteres linkes Dreieck"}}},{"category":"So","key":"25E4","mappings":{"default":{"default":"Schwarzes oberes linkes Dreieck"}}},{"category":"So","key":"25E5","mappings":{"default":{"default":"Schwarzes rechtes Dreieck"}}},{"category":"So","key":"25E6","mappings":{"default":{"default":"Weiße Kugel"}}},{"category":"So","key":"25E7","mappings":{"default":{"default":"Quadrat mit linker Hälfte schwarz"}}},{"category":"So","key":"25E8","mappings":{"default":{"default":"Quadrat mit rechter Hälfte schwarz"}}},{"category":"So","key":"25E9","mappings":{"default":{"default":"Quadrat mit der oberen linken Diagonalenhälfte schwarz"}}},{"category":"So","key":"25EA","mappings":{"default":{"default":"Quadrat mit unterer rechter diagonaler Hälfte schwarz"}}},{"category":"So","key":"25EB","mappings":{"default":{"default":"Weißes Quadrat mit senkrechter Linie"}}},{"category":"So","key":"25EC","mappings":{"default":{"default":"Weißes nach oben zeigendes Dreieck mit Punkt"}}},{"category":"So","key":"25ED","mappings":{"default":{"default":"Aufwärtszeigendes Dreieck mit linker Hälfte schwarz"}}},{"category":"So","key":"25EE","mappings":{"default":{"default":"Aufwärtszeigendes Dreieck mit rechter Hälfte schwarz"}}},{"category":"So","key":"25EF","mappings":{"default":{"default":"Großer Kreis"}}},{"category":"So","key":"25F0","mappings":{"default":{"default":"Weißes Quadrat mit oberem linken Quadranten"}}},{"category":"So","key":"25F1","mappings":{"default":{"default":"Weißes Quadrat mit unterem linken Quadranten"}}},{"category":"So","key":"25F2","mappings":{"default":{"default":"Weißes Quadrat mit unterem rechten Quadranten"}}},{"category":"So","key":"25F3","mappings":{"default":{"default":"Weißes Quadrat mit rechtem oberen Quadranten"}}},{"category":"So","key":"25F4","mappings":{"default":{"default":"Weißer Kreis mit oberem linkem Quadranten"}}},{"category":"So","key":"25F5","mappings":{"default":{"default":"Weißer Kreis mit unterem linken Quadranten"}}},{"category":"So","key":"25F6","mappings":{"default":{"default":"Weißer Kreis mit unterem rechten Quadranten"}}},{"category":"So","key":"25F7","mappings":{"default":{"default":"Weißer Kreis mit rechtem oberen Quadranten"}}},{"category":"Sm","key":"25F8","mappings":{"default":{"default":"Oberes linkes Dreieck"}}},{"category":"Sm","key":"25F9","mappings":{"default":{"default":"Oberes rechtes Dreieck"}}},{"category":"Sm","key":"25FA","mappings":{"default":{"default":"Unteres linkes Dreieck"}}},{"category":"Sm","key":"25FB","mappings":{"default":{"default":"Weißes mittleres Quadrat"}}},{"category":"Sm","key":"25FC","mappings":{"default":{"default":"Schwarzes mittleres Quadrat"}}},{"category":"Sm","key":"25FD","mappings":{"default":{"default":"Weißes mittleres kleines Quadrat"}}},{"category":"Sm","key":"25FE","mappings":{"default":{"default":"Schwarzes mittleres kleines Quadrat"}}},{"category":"Sm","key":"25FF","mappings":{"default":{"default":"Unteres rechtes Dreieck"}}},{"category":"So","key":"2B12","mappings":{"default":{"default":"Quadrat mit oberer Hälfte schwarz"}}},{"category":"So","key":"2B13","mappings":{"default":{"default":"Quadrat mit unterer Hälfte schwarz"}}},{"category":"So","key":"2B14","mappings":{"default":{"default":"Quadrat mit oberer rechter diagonaler Hälfte schwarz"}}},{"category":"So","key":"2B15","mappings":{"default":{"default":"Quadrat mit unterer linker diagonaler Hälfte schwarz"}}},{"category":"So","key":"2B16","mappings":{"default":{"default":"Diamant mit linker Hälfte schwarz"}}},{"category":"So","key":"2B17","mappings":{"default":{"default":"Diamant mit rechter Hälfte schwarz"}}},{"category":"So","key":"2B18","mappings":{"default":{"default":"Diamant mit oberer Hälfte schwarz"}}},{"category":"So","key":"2B19","mappings":{"default":{"default":"Diamant mit unterer Hälfte schwarz"}}},{"category":"So","key":"2B1A","mappings":{"default":{"default":"Gepunktetes Quadrat"}}},{"category":"So","key":"2B1B","mappings":{"default":{"default":"Schwarzes großes Quadrat"}}},{"category":"So","key":"2B1C","mappings":{"default":{"default":"Weißes großes Quadrat"}}},{"category":"So","key":"2B1D","mappings":{"default":{"default":"Schwarzes sehr kleines Quadrat"}}},{"category":"So","key":"2B1E","mappings":{"default":{"default":"Weißes sehr kleines Quadrat"}}},{"category":"So","key":"2B1F","mappings":{"default":{"default":"Schwarzes Pentagon"}}},{"category":"So","key":"2B20","mappings":{"default":{"default":"Weißes Pentagon"}}},{"category":"So","key":"2B21","mappings":{"default":{"default":"Weißes Sechseck"}}},{"category":"So","key":"2B22","mappings":{"default":{"default":"Schwarzes Sechseck"}}},{"category":"So","key":"2B23","mappings":{"default":{"default":"Horizontales schwarzes Sechseck"}}},{"category":"So","key":"2B24","mappings":{"default":{"default":"Schwarzer großer Kreis"}}},{"category":"So","key":"2B25","mappings":{"default":{"default":"Schwarzer mittlerer Diamant"}}},{"category":"So","key":"2B26","mappings":{"default":{"default":"Weißer mittlerer Diamant"}}},{"category":"So","key":"2B27","mappings":{"default":{"default":"Schwarze mittlere Raute"}}},{"category":"So","key":"2B28","mappings":{"default":{"default":"Weiße mittlere Raute"}}},{"category":"So","key":"2B29","mappings":{"default":{"default":"Schwarzer kleiner Diamant"}}},{"category":"So","key":"2B2A","mappings":{"default":{"default":"Schwarze kleine Raute"}}},{"category":"So","key":"2B2B","mappings":{"default":{"default":"Weiße kleine Raute"}}},{"category":"So","key":"2B2C","mappings":{"default":{"default":"Schwarze horizontale Ellipse"}}},{"category":"So","key":"2B2D","mappings":{"default":{"default":"Weiße horizontale Ellipse"}}},{"category":"So","key":"2B2E","mappings":{"default":{"default":"Schwarze vertikale Ellipse"}}},{"category":"So","key":"2B2F","mappings":{"default":{"default":"Weiße vertikale Ellipse"}}},{"category":"So","key":"2B50","mappings":{"default":{"default":"Weißer mittlerer Stern"}}},{"category":"So","key":"2B51","mappings":{"default":{"default":"Schwarzer kleiner Stern"}}},{"category":"So","key":"2B52","mappings":{"default":{"default":"Weißer kleiner Stern"}}},{"category":"So","key":"2B53","mappings":{"default":{"default":"Schwarzes nach rechts zeigendes Pentagon"}}},{"category":"So","key":"2B54","mappings":{"default":{"default":"Weißes nach rechts zeigendes Pentagon"}}},{"category":"So","key":"2B55","mappings":{"default":{"default":"Schwerer großer Kreis"}}},{"category":"So","key":"2B56","mappings":{"default":{"default":"Schweres Oval mit innenliegendem Oval"}}},{"category":"So","key":"2B57","mappings":{"default":{"default":"Schwerer Kreis mit Kreis nach innen"}}},{"category":"So","key":"2B58","mappings":{"default":{"default":"Schwerer Kreis"}}},{"category":"So","key":"2B59","mappings":{"default":{"default":"Schwerer eingekreister Saltire"}}}]
,
"de/symbols/math_harpoons.js": 
[{"locale":"de"},{"category":"So","key":"21BC","mappings":{"default":{"default":"Linke Harpune mit Barb nach oben"}}},{"category":"So","key":"21BD","mappings":{"default":{"default":"Harpune nach links mit Barb nach unten"}}},{"category":"So","key":"21BE","mappings":{"default":{"default":"Harpune nach oben mit Barb nach rechts"}}},{"category":"So","key":"21BF","mappings":{"default":{"default":"Harpune nach oben mit Barb nach links"}}},{"category":"So","key":"21C0","mappings":{"default":{"default":"Harpune nach rechts mit Barb nach oben"}}},{"category":"So","key":"21C1","mappings":{"default":{"default":"Harpune nach rechts mit Barb nach unten"}}},{"category":"So","key":"21C2","mappings":{"default":{"default":"Harpune nach unten mit Barb nach rechts"}}},{"category":"So","key":"21C3","mappings":{"default":{"default":"Harpune nach unten mit Barb nach links"}}},{"category":"So","key":"21CB","mappings":{"default":{"default":"Harpune nach links über Harpune nach rechts"}}},{"category":"So","key":"21CC","mappings":{"default":{"default":"Harpune nach rechts über Harpune nach links"}}},{"category":"Sm","key":"294A","mappings":{"default":{"default":"Linke Barb Up Rechte Barb Down Harpoon"}}},{"category":"Sm","key":"294B","mappings":{"default":{"default":"Linke Barb Down Rechte Barb Up Harpoon"}}},{"category":"Sm","key":"294C","mappings":{"default":{"default":"Nach oben Barb Rechts Nach links Barb Harpoon"}}},{"category":"Sm","key":"294D","mappings":{"default":{"default":"Up Barb Left Abwärts Barb Right Harpoon"}}},{"category":"Sm","key":"294E","mappings":{"default":{"default":"Linke Barb Up Rechte Barb Up Harpoon"}}},{"category":"Sm","key":"294F","mappings":{"default":{"default":"Barb nach rechts Barb nach rechts Harpoon"}}},{"category":"Sm","key":"2950","mappings":{"default":{"default":"Linke Barb Down Rechte Barb Down Harpoon"}}},{"category":"Sm","key":"2951","mappings":{"default":{"default":"Up Barb Left Abwärts Barb Left Harpoon"}}},{"category":"Sm","key":"2952","mappings":{"default":{"default":"Linke Harpune mit Barb Up To Bar"}}},{"category":"Sm","key":"2953","mappings":{"default":{"default":"Harpune nach rechts mit Barb bis zur Bar"}}},{"category":"Sm","key":"2954","mappings":{"default":{"default":"Harpune nach oben mit Widerhaken rechts"}}},{"category":"Sm","key":"2955","mappings":{"default":{"default":"Harpune nach unten mit Barb Right to Bar"}}},{"category":"Sm","key":"2956","mappings":{"default":{"default":"Linke Harpune mit Widerhaken nach unten"}}},{"category":"Sm","key":"2957","mappings":{"default":{"default":"Harpune nach rechts mit Widerhaken nach unten"}}},{"category":"Sm","key":"2958","mappings":{"default":{"default":"Harpune nach oben mit Barb nach links"}}},{"category":"Sm","key":"2959","mappings":{"default":{"default":"Harpune nach unten mit Barb nach links"}}},{"category":"Sm","key":"295A","mappings":{"default":{"default":"Linke Harpune mit Barb Up von der Bar"}}},{"category":"Sm","key":"295B","mappings":{"default":{"default":"Harpune nach rechts mit Barb Up von der Bar"}}},{"category":"Sm","key":"295C","mappings":{"default":{"default":"Harpune nach oben mit Barb rechts von der Bar"}}},{"category":"Sm","key":"295D","mappings":{"default":{"default":"Harpune nach unten mit Barb rechts von der Bar"}}},{"category":"Sm","key":"295E","mappings":{"default":{"default":"Linke Harpune mit Barb Down von der Bar"}}},{"category":"Sm","key":"295F","mappings":{"default":{"default":"Rechts Harpoon mit Barb Down von der Bar"}}},{"category":"Sm","key":"2960","mappings":{"default":{"default":"Harpune nach oben mit Barb links von der Bar"}}},{"category":"Sm","key":"2961","mappings":{"default":{"default":"Harpune nach unten mit Barb links von der Bar"}}},{"category":"Sm","key":"2962","mappings":{"default":{"default":"Harpune nach links mit Barb nach oben Harpune nach links mit Barb nach unten"}}},{"category":"Sm","key":"2963","mappings":{"default":{"default":"Harpune nach oben mit Barb links neben Harpune nach oben mit Barb rechts"}}},{"category":"Sm","key":"2964","mappings":{"default":{"default":"Harpune nach rechts mit Barb nach oben Harpoon nach rechts mit Barb nach unten"}}},{"category":"Sm","key":"2965","mappings":{"default":{"default":"Harpune nach unten mit Barb nach unten Harpune nach unten mit Barb nach rechts"}}},{"category":"Sm","key":"2966","mappings":{"default":{"default":"Linke Harpune mit Barb Up Oben Rechts Harpoon mit Barb Up"}}},{"category":"Sm","key":"2967","mappings":{"default":{"default":"Linke Harpune mit Widerhaken nach unten Rechts Harpune mit Widerhaken nach unten"}}},{"category":"Sm","key":"2968","mappings":{"default":{"default":"Harpune nach rechts mit Barb nach oben Harpune nach links mit Barb nach oben"}}},{"category":"Sm","key":"2969","mappings":{"default":{"default":"Harpune nach rechts mit Barb nach unten Harpune nach links mit Barb nach unten"}}},{"category":"Sm","key":"296A","mappings":{"default":{"default":"Linke Harpune mit Barb Up Above Long Dash"}}},{"category":"Sm","key":"296B","mappings":{"default":{"default":"Linke Harpune mit Barb Down Under Long Dash"}}},{"category":"Sm","key":"296C","mappings":{"default":{"default":"Harpune nach rechts mit Barb Up Long Long Dash"}}},{"category":"Sm","key":"296D","mappings":{"default":{"default":"Harpune nach rechts mit Widerhaken nach unten"}}},{"category":"Sm","key":"296E","mappings":{"default":{"default":"Harpune nach oben mit Barb links neben Harpune nach unten mit Barb rechts"}}},{"category":"Sm","key":"296F","mappings":{"default":{"default":"Harpune nach unten mit Barb links neben Harpune nach oben mit Barb rechts"}}},{"category":"Sm","key":"297C","mappings":{"default":{"default":"linker Fischschwanz"}}},{"category":"Sm","key":"297D","mappings":{"default":{"default":"rechter Fischschwanz"}}},{"category":"Sm","key":"297E","mappings":{"default":{"default":"Fischschwanz"}}},{"category":"Sm","key":"297F","mappings":{"default":{"default":"Fischschwanz"}}}]
,
"de/symbols/math_non_characters.js": 
[{"locale":"de"},{"category":"Ll","key":"210F","mappings":{"default":{"default":"Planck-Konstante über zwei Pi"}}},{"category":"So","key":"2114","mappings":{"default":{"default":"L B Balkensymbol"}}},{"category":"So","key":"2116","mappings":{"default":{"default":"Numero-Zeichen"}}},{"category":"So","key":"2117","mappings":{"default":{"default":"Tonaufnahme Copyright"}}},{"category":"So","key":"211E","mappings":{"default":{"default":"Rezept nehmen"}}},{"category":"So","key":"211F","mappings":{"default":{"default":"Antwort"}}},{"category":"So","key":"2120","mappings":{"default":{"default":"Dienstleistungsmarke"}}},{"category":"So","key":"2121","mappings":{"default":{"default":"Telefon Zeichen"}}},{"category":"So","key":"2122","mappings":{"default":{"default":"Markenzeichen"}}},{"category":"So","key":"2123","mappings":{"default":{"default":"Versicle"}}},{"category":"So","key":"2125","mappings":{"default":{"default":"Unze-Zeichen"}}},{"category":"Lu","key":"2126","mappings":{"default":{"default":"Ohm-Zeichen"}}},{"category":"So","key":"2127","mappings":{"default":{"default":"Umgekehrtes Ohm-Zeichen"}}},{"category":"Lu","key":"212A","mappings":{"default":{"default":"Kelvin-Zeichen"}}},{"category":"Lu","key":"212B","mappings":{"default":{"default":"Angstrom-Zeichen"}}},{"category":"So","key":"212E","mappings":{"default":{"default":"Geschätztes Symbol"}}},{"category":"Lu","key":"2132","mappings":{"default":{"default":"Umgeschlagenes Kapital F"}}},{"category":"Ll","key":"2139","mappings":{"default":{"default":"Informationsquelle"}}},{"category":"So","key":"213A","mappings":{"default":{"default":"Gedrehtes Kapital Q"}}},{"category":"So","key":"213B","mappings":{"default":{"default":"Faksimile-Zeichen"}}},{"category":"Sm","key":"2141","mappings":{"default":{"default":"Sans-Serif Capital G"}}},{"category":"Sm","key":"2142","mappings":{"default":{"default":"Sans-Serif Capital L"}}},{"category":"Sm","key":"2143","mappings":{"default":{"default":"Aufgehobenes Sans-Serif-Kapital L"}}},{"category":"Sm","key":"2144","mappings":{"default":{"default":"Sans-Serif Capital Y"}}}]
,
"de/symbols/math_symbols.js": 
[{"locale":"de"},{"key":"0021","mappings":{"default":{"default":"Fakultät"}},"category":"Po"},{"key":"0022","mappings":{"default":{"default":"Anführungszeichen"}},"category":"Po"},{"key":"0023","mappings":{"default":{"default":"Nummernzeichen"}},"category":"Po"},{"key":"0024","mappings":{"default":{"default":"Dollar"}},"category":"Sc"},{"key":"0025","mappings":{"default":{"default":"Prozent"}},"category":"Po"},{"key":"0026","mappings":{"default":{"default":"und"}},"category":"Po"},{"key":"0027","mappings":{"default":{"default":"Strich"}},"category":"Po"},{"key":"002A","mappings":{"default":{"default":"times"}},"category":"Po"},{"key":"002B","mappings":{"default":{"default":"plus"}},"category":"Sm"},{"key":"002C","mappings":{"default":{"default":"Komma"}},"category":"Po"},{"key":"002D","mappings":{"default":{"default":"minus"}},"category":"Pd"},{"category":"Po","key":"002E","mappings":{"default":{"default":"Punkt"}}},{"key":"002F","mappings":{"default":{"default":"Schrägstrich"}},"category":"Po"},{"key":"003A","mappings":{"default":{"default":"Doppelpunkt"}},"category":"Po"},{"key":"003B","mappings":{"default":{"default":"Strichpunkt"}},"category":"Po"},{"key":"003C","mappings":{"default":{"default":"kleiner als"}},"category":"Sm"},{"key":"003D","mappings":{"default":{"default":"ist gleich"}},"category":"Sm"},{"key":"003E","mappings":{"default":{"default":"größer als"}},"category":"Sm"},{"key":"003F","mappings":{"default":{"default":"Fragezeichen"}},"category":"Po"},{"key":"0040","mappings":{"default":{"default":"Klammeraffe"}},"category":"Po"},{"key":"005C","mappings":{"default":{"default":"Umgekehrter Schrägstrich"}},"category":"Po"},{"key":"005E","mappings":{"default":{"default":"circumflex"}},"category":"Sk"},{"key":"005F","mappings":{"default":{"default":"Unterstrich"}},"category":"Pc"},{"key":"0060","mappings":{"default":{"default":"Gravis Akzent"}},"category":"Sk"},{"key":"007C","mappings":{"default":{"default":"senkrechter Strich"}},"category":"Sm"},{"key":"007E","mappings":{"default":{"default":"tilde"}},"category":"Sm"},{"key":"00A1","mappings":{"default":{"default":"Umgekehrtes Ausrufezeichen"}},"category":"Po"},{"key":"00A2","mappings":{"default":{"default":"Cent"}},"category":"Sc"},{"key":"00A3","mappings":{"default":{"default":"Pfund"}},"category":"Sc"},{"key":"00A4","mappings":{"default":{"default":"währungszeichen"}},"category":"Sc"},{"key":"00A5","mappings":{"default":{"default":"yen"}},"category":"Sc"},{"key":"00A6","mappings":{"default":{"default":"gebrochene Linie"}},"category":"So"},{"key":"00A7","mappings":{"default":{"default":"Abschnitt"}},"category":"Po"},{"key":"00A8","mappings":{"default":{"default":"diaeresis"}},"category":"Sk"},{"key":"00A9","mappings":{"default":{"default":"Copyright"}},"category":"So"},{"key":"00AA","mappings":{"default":{"default":"weibliches Ordnungszeichen"}},"category":"Lo"},{"category":"Pi","key":"00AB","mappings":{"default":{"default":"Nach links zeigendes doppeltes spitzes Anführungszeichen"}}},{"key":"00AC","mappings":{"default":{"default":"nicht"}},"category":"Sm"},{"key":"00AE","mappings":{"default":{"default":"registered sign"}},"category":"So"},{"key":"00AF","mappings":{"default":{"default":"Überstrich","alternative":"Makron"}},"category":"Sk"},{"key":"00B0","mappings":{"default":{"default":"Grad"}},"category":"So"},{"key":"00B1","mappings":{"default":{"default":"plus minus"}},"category":"Sm"},{"key":"00B4","mappings":{"default":{"default":"Acuteakzent"}},"category":"Sk"},{"key":"00B5","mappings":{"default":{"default":"Mikrozeichen"}},"category":"Ll"},{"key":"00B6","mappings":{"default":{"default":"Paragraf"}},"category":"Po"},{"key":"00B7","mappings":{"default":{"default":"mal","alternative":"Punkt mittig"}},"category":"Po"},{"key":"00B8","mappings":{"default":{"default":"Cedille"}},"category":"Sk"},{"key":"00BA","mappings":{"default":{"default":"Männliches Ordnungszeichen"}},"category":"Lo"},{"category":"Pf","key":"00BB","mappings":{"default":{"default":"Nach rechts zeigendes doppeltes spitzes Anführungszeichen"}}},{"key":"00BF","mappings":{"default":{"default":"umgekehrtes Fragezeichen"}},"category":"Po"},{"category":"Sm","key":"00D7","mappings":{"default":{"default":"mal"}}},{"category":"Sm","key":"00F7","mappings":{"default":{"default":"geteilt durch"}}},{"key":"02B9","mappings":{"default":{"default":"Strich"}},"category":"Lm"},{"key":"02BA","mappings":{"default":{"default":"zwei Strich"}},"category":"Lm"},{"key":"02D8","mappings":{"default":{"default":"Breve"}},"category":"Sk"},{"key":"02D9","mappings":{"default":{"default":"Überpunkt"}},"category":"Sk"},{"key":"02DA","mappings":{"default":{"default":"Überring"}},"category":"Sk"},{"key":"02DB","mappings":{"default":{"default":"Ogonek"}},"category":"Sk"},{"key":"02DC","mappings":{"default":{"default":"Tilde"}},"category":"Sk"},{"key":"02DD","mappings":{"default":{"default":"Doppleakutakzent"}},"category":"Sk"},{"category":"Pd","key":"2010","mappings":{"default":{"default":"Bindestrich"}}},{"category":"Pd","key":"2011","mappings":{"default":{"default":"Nicht brechender Bindestrich"}}},{"category":"Pd","key":"2012","mappings":{"default":{"default":"Abbildung Bindestrich"}}},{"category":"Pd","key":"2013","mappings":{"default":{"default":"En Dash"}}},{"category":"Pd","key":"2014","mappings":{"default":{"default":"EM Dash"}}},{"category":"Pd","key":"2015","mappings":{"default":{"default":"Horizontale Linie"}}},{"category":"Po","key":"2016","mappings":{"default":{"default":"Doppelte vertikale Linie"}}},{"category":"Po","key":"2017","mappings":{"default":{"default":"Doppelter Unterstrich"}}},{"category":"Pi","key":"2018","mappings":{"default":{"default":"Linkes einfaches Anführungszeichen"}}},{"category":"Pf","key":"2019","mappings":{"default":{"default":"Rechtes einfaches Anführungszeichen"}}},{"category":"Ps","key":"201A","mappings":{"default":{"default":"Einzelnes rechtes Anführungszeichen"}}},{"category":"Pi","key":"201B","mappings":{"default":{"default":"Einzelnes linkes Anführungszeichen"}}},{"category":"Pi","key":"201C","mappings":{"default":{"default":"Linkes doppeltes Anführungszeichen"}}},{"category":"Pf","key":"201D","mappings":{"default":{"default":"Rechtes doppeltes Anführungszeichen"}}},{"category":"Ps","key":"201E","mappings":{"default":{"default":"Doppeltes rechtes Anführungszeichen"}}},{"category":"Pi","key":"201F","mappings":{"default":{"default":"Doppeltes linkes Anführungszeichen"}}},{"category":"Po","key":"2020","mappings":{"default":{"default":"Dolch"}}},{"category":"Po","key":"2021","mappings":{"default":{"default":"Doppelter Dolch"}}},{"category":"Po","key":"2022","mappings":{"default":{"default":"Kugel"}}},{"category":"Po","key":"2023","mappings":{"default":{"default":"Dreieckige Kugel"}}},{"category":"Po","key":"2024","mappings":{"default":{"default":"Einzelpunkt"}}},{"category":"Po","key":"2025","mappings":{"default":{"default":"zwei Punkte"}}},{"category":"Po","key":"2026","mappings":{"default":{"default":"horizontale Ellipsis"}}},{"category":"Po","key":"2027","mappings":{"default":{"default":"Trennungspunkt"}}},{"category":"Po","key":"2030","mappings":{"default":{"default":"promille"}}},{"category":"Po","key":"2031","mappings":{"default":{"default":"pro Zehntausend"}}},{"key":"2032","mappings":{"default":{"default":"Strich"}},"category":"Po"},{"key":"2033","mappings":{"default":{"default":"zwei Strich"}},"category":"Po"},{"key":"2034","mappings":{"default":{"default":"drei Strich"}},"category":"Po"},{"category":"Po","key":"2035","mappings":{"default":{"default":"strich invertiert"}}},{"category":"Po","key":"2036","mappings":{"default":{"default":"zwei Strich invertiert"}}},{"category":"Po","key":"2037","mappings":{"default":{"default":"drei Strich invertiert"}}},{"category":"Po","key":"2038","mappings":{"default":{"default":"Caret"}}},{"category":"Pi","key":"2039","mappings":{"default":{"default":"Einfacher nach links zeigender Winkel Anführungszeichen"}}},{"category":"Pf","key":"203A","mappings":{"default":{"default":"Einzelne Nach-rechts-Winkel-Anführungszeichen"}}},{"category":"Po","key":"203B","mappings":{"default":{"default":"Referenzmarke"}}},{"category":"Po","key":"203C","mappings":{"default":{"default":"Doppelausrufezeichen"}}},{"category":"Po","key":"203D","mappings":{"default":{"default":"Interrobang"}}},{"category":"Po","key":"203E","mappings":{"default":{"default":"Überschrift"}}},{"category":"Pc","key":"203F","mappings":{"default":{"default":"Undertie"}}},{"category":"Pc","key":"2040","mappings":{"default":{"default":"Zeichen Krawatte"}}},{"category":"Po","key":"2041","mappings":{"default":{"default":"Caret-Einfügungspunkt"}}},{"category":"Po","key":"2042","mappings":{"default":{"default":"Asterism"}}},{"category":"Po","key":"2043","mappings":{"default":{"default":"Bindestrich Kugel"}}},{"category":"Sm","key":"2044","mappings":{"default":{"default":"Fraktions-Schrägstrich"}}},{"category":"Po","key":"2047","mappings":{"default":{"default":"Doppel-Fragezeichen"}}},{"category":"Po","key":"2048","mappings":{"default":{"default":"Fragezeichen für Ausrufezeichen"}}},{"category":"Po","key":"2049","mappings":{"default":{"default":"Ausrufezeichen"}}},{"category":"Po","key":"204B","mappings":{"default":{"default":"Umgekehrtes Pilcrow-Zeichen"}}},{"category":"Po","key":"204C","mappings":{"default":{"default":"Schwarze Kugel nach links"}}},{"category":"Po","key":"204D","mappings":{"default":{"default":"Schwarze Kugel nach rechts"}}},{"category":"Po","key":"204E","mappings":{"default":{"default":"Niedriges Sternchen"}}},{"category":"Po","key":"204F","mappings":{"default":{"default":"Umgekehrtes Semikolon"}}},{"category":"Po","key":"2050","mappings":{"default":{"default":"Nahansicht"}}},{"category":"Po","key":"2051","mappings":{"default":{"default":"Zwei Sternchen vertikal ausgerichtet"}}},{"category":"Sm","key":"2052","mappings":{"default":{"default":"Kommerzielles Minuszeichen"}}},{"category":"Po","key":"2053","mappings":{"default":{"default":"Swing Dash"}}},{"category":"Pc","key":"2054","mappings":{"default":{"default":"Inverted Undertie"}}},{"category":"Po","key":"2055","mappings":{"default":{"default":"Blume Interpunktionszeichen"}}},{"category":"Po","key":"2056","mappings":{"default":{"default":"Drei Punkt-Interpunktion"}}},{"key":"2057","mappings":{"default":{"default":"vier Strich"}},"category":"Po"},{"category":"Po","key":"2058","mappings":{"default":{"default":"Vierpunkt-Interpunktion"}}},{"category":"Po","key":"2059","mappings":{"default":{"default":"Fünf Punkt-Interpunktion"}}},{"category":"Po","key":"205A","mappings":{"default":{"default":"Zwei Punkt-Interpunktion"}}},{"category":"Po","key":"205B","mappings":{"default":{"default":"Vier-Punkt-Markierung"}}},{"category":"Po","key":"205C","mappings":{"default":{"default":"Gepunktetes Kreuz"}}},{"category":"Po","key":"205D","mappings":{"default":{"default":"Tricolon"}}},{"category":"Po","key":"205E","mappings":{"default":{"default":"Vertikale vier Punkte"}}},{"category":"Sm","key":"207A","mappings":{"default":{"default":"Hochgestelltes Pluszeichen"}}},{"category":"Sm","key":"207B","mappings":{"default":{"default":"Hochgestelltes Minus"}}},{"category":"Sm","key":"207C","mappings":{"default":{"default":"Hochgestelltes Gleichheitszeichen"}}},{"category":"Ps","key":"207D","mappings":{"default":{"default":"Hochgestellte linke Klammer"}}},{"category":"Pe","key":"207E","mappings":{"default":{"default":"Hochgestellte rechte Klammer"}}},{"category":"Sm","key":"208A","mappings":{"default":{"default":"Index Pluszeichen"}}},{"category":"Sm","key":"208B","mappings":{"default":{"default":"Tiefgestelltes Minus"}}},{"category":"Sm","key":"208C","mappings":{"default":{"default":"Tiefgestelltes Gleichheitszeichen"}}},{"category":"Ps","key":"208D","mappings":{"default":{"default":"Tiefgestellte linke Klammer"}}},{"category":"Pe","key":"208E","mappings":{"default":{"default":"Tiefgestellte rechte Klammer"}}},{"category":"So","key":"214A","mappings":{"default":{"default":"Grundstücksgrenze"}}},{"category":"Sm","key":"214B","mappings":{"default":{"default":"Ampersand gedreht"}}},{"category":"So","key":"214C","mappings":{"default":{"default":"Pro Zeichen"}}},{"category":"So","key":"214D","mappings":{"default":{"default":"Aktieselskab"}}},{"category":"Ll","key":"214E","mappings":{"default":{"default":"Gedreht klein F"}}},{"key":"2200","mappings":{"default":{"default":"für alle"}},"category":"Sm"},{"key":"2201","mappings":{"default":{"default":"Komplement"}},"category":"Sm"},{"key":"2203","mappings":{"default":{"default":"es gibt"}},"category":"Sm"},{"key":"2204","mappings":{"default":{"default":"es gibt nicht"}},"category":"Sm"},{"key":"2205","mappings":{"default":{"default":"leere Menge"}},"category":"Sm"},{"key":"2206","mappings":{"default":{"default":"Differenz"}},"category":"Sm"},{"key":"2208","mappings":{"default":{"default":"Element von"}},"category":"Sm"},{"key":"2209","mappings":{"default":{"default":"nicht Element von"}},"category":"Sm"},{"key":"220A","mappings":{"default":{"default":"Element von"}},"category":"Sm"},{"key":"220B","mappings":{"default":{"default":"hat als Element"}},"category":"Sm"},{"key":"220C","mappings":{"default":{"default":"hat nicht als Element"}},"category":"Sm"},{"key":"220D","mappings":{"default":{"default":"hat als Element"}},"category":"Sm"},{"key":"220E","mappings":{"default":{"default":"Ende Beweis"}},"category":"Sm"},{"key":"220F","mappings":{"default":{"default":"Produkt"}},"category":"Sm"},{"key":"2210","mappings":{"default":{"default":"Koprodukt"}},"category":"Sm"},{"key":"2211","mappings":{"default":{"default":"Summe"}},"category":"Sm"},{"key":"2212","mappings":{"default":{"default":"minus"}},"category":"Sm"},{"key":"2213","mappings":{"default":{"default":"minus plus"}},"category":"Sm"},{"category":"Sm","key":"2214","mappings":{"default":{"default":"Plus mit Punkt"}}},{"key":"2215","mappings":{"default":{"default":"Division"}},"category":"Sm"},{"key":"2216","mappings":{"default":{"default":"Mengendifferenz"}},"category":"Sm"},{"key":"2217","mappings":{"default":{"default":"Sternoperator"}},"category":"Sm"},{"key":"2218","mappings":{"default":{"default":"verknüpft mit"}},"category":"Sm"},{"key":"2219","mappings":{"default":{"default":"Kreisoperator"}},"category":"Sm"},{"key":"221A","mappings":{"default":{"default":"Quadratwurzel"}},"category":"Sm"},{"key":"221B","mappings":{"default":{"default":"Kubikwurzel"}},"category":"Sm"},{"key":"221C","mappings":{"default":{"default":"vierte Wurzel"}},"category":"Sm"},{"key":"221D","mappings":{"default":{"default":"proportional zu"}},"category":"Sm"},{"key":"221E","mappings":{"default":{"default":"unendlich"}},"category":"Sm"},{"key":"221F","mappings":{"default":{"default":"rechter Winkel"}},"category":"Sm"},{"key":"2220","mappings":{"default":{"default":"Winkel"}},"category":"Sm"},{"category":"Sm","key":"2221","mappings":{"default":{"default":"gerichteter Winkel"}}},{"category":"Sm","key":"2222","mappings":{"default":{"default":"Raumwinkel"}}},{"key":"2223","mappings":{"default":{"default":"teilt"}},"category":"Sm"},{"key":"2224","mappings":{"default":{"default":"teilt nicht"}},"category":"Sm"},{"key":"2225","mappings":{"default":{"default":"parallel zu"}},"category":"Sm"},{"key":"2226","mappings":{"default":{"default":"nicht parallel zu"}},"category":"Sm"},{"key":"2227","mappings":{"default":{"default":"und"}},"category":"Sm"},{"key":"2228","mappings":{"default":{"default":"oder"}},"category":"Sm"},{"key":"2229","mappings":{"default":{"default":"Durchschnitt"}},"category":"Sm"},{"key":"222A","mappings":{"default":{"default":"Vereinigung"}},"category":"Sm"},{"key":"222B","mappings":{"default":{"default":"Integral"}},"category":"Sm"},{"key":"222C","mappings":{"default":{"default":"Doppelintegral"}},"category":"Sm"},{"key":"222D","mappings":{"default":{"default":"Dreifachintegral"}},"category":"Sm"},{"key":"222E","mappings":{"default":{"default":"Randintegral"}},"category":"Sm"},{"key":"222F","mappings":{"default":{"default":"Oberflächenintegral"}},"category":"Sm"},{"key":"2230","mappings":{"default":{"default":"Volumenintegral"}},"category":"Sm"},{"key":"2231","mappings":{"default":{"default":"Integral im Uhrzeigersinn"}},"category":"Sm"},{"key":"2232","mappings":{"default":{"default":"Kurvenintegral im Uhrzeigersinn"}},"category":"Sm"},{"key":"2233","mappings":{"default":{"default":"Kurvenintegral im Gegenuhrzeigersinn"}},"category":"Sm"},{"key":"2234","mappings":{"default":{"default":"folglich"}},"category":"Sm"},{"key":"2235","mappings":{"default":{"default":"weil"}},"category":"Sm"},{"key":"2236","mappings":{"default":{"default":"Verhältnis"}},"category":"Sm"},{"key":"2237","mappings":{"default":{"default":"Proportion"}},"category":"Sm"},{"category":"Sm","key":"2238","mappings":{"default":{"default":"Minus mit Punkt"}}},{"category":"Sm","key":"2239","mappings":{"default":{"default":"Überschuss"}}},{"key":"223A","mappings":{"default":{"default":"geometrische Proportion"}},"category":"Sm"},{"category":"Sm","key":"223B","mappings":{"default":{"default":"homothetisch"}}},{"category":"Sm","key":"223C","mappings":{"default":{"default":"Tildeoperator"}}},{"category":"Sm","key":"223D","mappings":{"default":{"default":"Umgekehrte Tilde"}}},{"category":"Sm","key":"223E","mappings":{"default":{"default":"Umgekehrtes stummes S"}}},{"category":"Sm","key":"223F","mappings":{"default":{"default":"Sinuswelle"}}},{"category":"Sm","key":"2240","mappings":{"default":{"default":"Kranzprodukt"}}},{"category":"Sm","key":"2241","mappings":{"default":{"default":"durchgestrichene Tilde"}}},{"category":"Sm","key":"2242","mappings":{"default":{"default":"Minus über Tilde"}}},{"key":"2243","mappings":{"default":{"default":"asymptotisch gleich"}},"category":"Sm"},{"key":"2244","mappings":{"default":{"default":"nicht asymptotisch gleich"}},"category":"Sm"},{"key":"2245","mappings":{"default":{"default":"ungefähr gleich"}},"category":"Sm"},{"key":"2246","mappings":{"default":{"default":"ungefähr, aber nicht ganz gleich"}},"category":"Sm"},{"key":"2247","mappings":{"default":{"default":"weder ungefähr noch ganz gleich"}},"category":"Sm"},{"key":"2248","mappings":{"default":{"default":"beinahe gleich"}},"category":"Sm"},{"key":"2249","mappings":{"default":{"default":"nicht beinahe gleich"}},"category":"Sm"},{"key":"224A","mappings":{"default":{"default":"beinahe gleich oder gleich"}},"category":"Sm"},{"category":"Sm","key":"224B","mappings":{"default":{"default":"dreifache Tilde"}}},{"category":"Sm","key":"224C","mappings":{"default":{"default":"alles gleich"}}},{"key":"224D","mappings":{"default":{"default":"äquivalent zu"}},"category":"Sm"},{"key":"224E","mappings":{"default":{"default":"geometrisch äquivalent zu"}},"category":"Sm"},{"key":"224F","mappings":{"default":{"default":"Differenz zwischen"}},"category":"Sm"},{"key":"2250","mappings":{"default":{"default":"nähert sich der Grenze"}},"category":"Sm"},{"key":"2251","mappings":{"default":{"default":"geometrisch gleich"}},"category":"Sm"},{"category":"Sm","key":"2252","mappings":{"default":{"default":"Ungefähr gleich oder das Bild von"}}},{"category":"Sm","key":"2253","mappings":{"default":{"default":"Bild von oder ungefähr gleich"}}},{"key":"2254","mappings":{"default":{"default":"definiert als"}},"category":"Sm"},{"category":"Sm","key":"2255","mappings":{"default":{"default":"definiert als von rechts"}}},{"category":"Sm","key":"2256","mappings":{"default":{"default":"ungefähr gleich"}}},{"category":"Sm","key":"2257","mappings":{"default":{"default":"ungefähr gleich"}}},{"key":"2258","mappings":{"default":{"default":"entspricht"}},"category":"Sm"},{"category":"Sm","key":"2259","mappings":{"default":{"default":"entspricht"}}},{"category":"Sm","key":"225A","mappings":{"default":{"default":"gleichwinklig zu"}}},{"category":"Sm","key":"225B","mappings":{"default":{"default":"gleich mit Stern"}}},{"category":"Sm","key":"225C","mappings":{"default":{"default":"gleich mit Delta"}}},{"key":"225D","mappings":{"default":{"default":"definitionsgemäß gleich"}},"category":"Sm"},{"category":"Sm","key":"225E","mappings":{"default":{"default":"gemessen mit"}}},{"category":"Sm","key":"225F","mappings":{"default":{"default":"vielleicht gleich"}}},{"key":"2260","mappings":{"default":{"default":"ungleich"}},"category":"Sm"},{"key":"2261","mappings":{"default":{"default":"kongruent mit"}},"category":"Sm"},{"key":"2262","mappings":{"default":{"default":"nicht kongruent mit"}},"category":"Sm"},{"key":"2263","mappings":{"default":{"default":"genau äquivalent mit"}},"category":"Sm"},{"key":"2264","mappings":{"default":{"default":"kleiner oder gleich"}},"category":"Sm"},{"key":"2265","mappings":{"default":{"default":"größer oder gleich"}},"category":"Sm"},{"key":"2266","mappings":{"default":{"default":"kleiner als über gleich"}},"category":"Sm"},{"key":"2267","mappings":{"default":{"default":"größer als über gleich"}},"category":"Sm"},{"key":"2268","mappings":{"default":{"default":"kleiner als, aber nicht gleich"}},"category":"Sm"},{"key":"2269","mappings":{"default":{"default":"größer als, aber nicht gleich"}},"category":"Sm"},{"key":"226A","mappings":{"default":{"default":"viel kleiner als"}},"category":"Sm"},{"key":"226B","mappings":{"default":{"default":"viel größer als"}},"category":"Sm"},{"key":"226C","mappings":{"default":{"default":"zwischen"}},"category":"Sm"},{"key":"226D","mappings":{"default":{"default":"nicht äquivalent mit"}},"category":"Sm"},{"key":"226E","mappings":{"default":{"default":"nicht kleiner als"}},"category":"Sm"},{"key":"226F","mappings":{"default":{"default":"nicht größer als"}},"category":"Sm"},{"key":"2270","mappings":{"default":{"default":"weder kleiner als noch gleich"}},"category":"Sm"},{"key":"2271","mappings":{"default":{"default":"weder größer als noch gleich"}},"category":"Sm"},{"key":"2272","mappings":{"default":{"default":"kleiner als oder äquivalent"}},"category":"Sm"},{"key":"2273","mappings":{"default":{"default":"größer als oder äquivalent"}},"category":"Sm"},{"key":"2274","mappings":{"default":{"default":"weder kleiner als noch äquivalent"}},"category":"Sm"},{"key":"2275","mappings":{"default":{"default":"weder größer als noch äquivalent"}},"category":"Sm"},{"key":"2276","mappings":{"default":{"default":"kleiner oder größer als"}},"category":"Sm"},{"key":"2277","mappings":{"default":{"default":"größer oder kleiner als"}},"category":"Sm"},{"key":"2278","mappings":{"default":{"default":"weder kleiner noch größer als"}},"category":"Sm"},{"key":"2279","mappings":{"default":{"default":"weder größer noch kleiner als"}},"category":"Sm"},{"category":"Sm","key":"227A","mappings":{"default":{"default":"vorangehend"}}},{"category":"Sm","key":"227B","mappings":{"default":{"default":"nachfolgend"}}},{"category":"Sm","key":"227C","mappings":{"default":{"default":"vorangehend oder gleich"}}},{"category":"Sm","key":"227D","mappings":{"default":{"default":"nachfolgend oder gleich"}}},{"category":"Sm","key":"227E","mappings":{"default":{"default":"vorangehend oder äquivalent"}}},{"category":"Sm","key":"227F","mappings":{"default":{"default":"nachfolgend oder äquivalent"}}},{"category":"Sm","key":"2280","mappings":{"default":{"default":"nicht vorangehend"}}},{"category":"Sm","key":"2281","mappings":{"default":{"default":"nicht nachfolgend"}}},{"category":"Sm","key":"2282","mappings":{"default":{"default":"echte Teilmenge von"}}},{"category":"Sm","key":"2283","mappings":{"default":{"default":"echte Obermenge von"}}},{"category":"Sm","key":"2284","mappings":{"default":{"default":"keine echte Teilmenge von"}}},{"category":"Sm","key":"2285","mappings":{"default":{"default":"kein echte Obermenge von"}}},{"category":"Sm","key":"2286","mappings":{"default":{"default":"Teilmenge oder gleich"}}},{"category":"Sm","key":"2287","mappings":{"default":{"default":"Obermenge oder gleich"}}},{"category":"Sm","key":"2288","mappings":{"default":{"default":"weder Teilmenge noch gleich"}}},{"category":"Sm","key":"2289","mappings":{"default":{"default":"weder Obermenge noch gleich"}}},{"category":"Sm","key":"228A","mappings":{"default":{"default":"Teilmenge aber nicht gleich"}}},{"category":"Sm","key":"228B","mappings":{"default":{"default":"Obermenge aber nicht gleich"}}},{"category":"Sm","key":"228C","mappings":{"default":{"default":"Multimenge"}}},{"category":"Sm","key":"228D","mappings":{"default":{"default":"Multimengenmultiplikation"}}},{"category":"Sm","key":"228E","mappings":{"default":{"default":"Multimengenvereinigung"}}},{"category":"Sm","key":"228F","mappings":{"default":{"default":"quadratisches Bild von"}}},{"category":"Sm","key":"2290","mappings":{"default":{"default":"quadratisches Original von"}}},{"category":"Sm","key":"2291","mappings":{"default":{"default":"quadratisches Bild oder gleich"}}},{"category":"Sm","key":"2292","mappings":{"default":{"default":"quadratisches Original oder gleich"}}},{"category":"Sm","key":"2293","mappings":{"default":{"default":"quadratische Schnittmenge"}}},{"category":"Sm","key":"2294","mappings":{"default":{"default":"quadratische VereinigungTasse"}}},{"category":"Sm","key":"2295","mappings":{"default":{"default":"eingekreistes Plus"}}},{"category":"Sm","key":"2296","mappings":{"default":{"default":"eingekreistes Minus"}}},{"category":"Sm","key":"2297","mappings":{"default":{"default":"eingekreiste Multiplikation"}}},{"category":"Sm","key":"2298","mappings":{"default":{"default":"eingekreiste Division"}}},{"category":"Sm","key":"2299","mappings":{"default":{"default":"eingekreister Punktoperator"}}},{"category":"Sm","key":"229A","mappings":{"default":{"default":"eingekreister Ringoperator"}}},{"category":"Sm","key":"229B","mappings":{"default":{"default":"eingekreister Sternoperator"}}},{"category":"Sm","key":"229C","mappings":{"default":{"default":"eingekreistes Gleich"}}},{"category":"Sm","key":"229D","mappings":{"default":{"default":"eingekreister Gedankenstrich"}}},{"category":"Sm","key":"229E","mappings":{"default":{"default":"eingerahmtes Plus"}}},{"category":"Sm","key":"229F","mappings":{"default":{"default":"eingerahmtes Minus"}}},{"category":"Sm","key":"22A0","mappings":{"default":{"default":"eingerahmte Multiplikation"}}},{"category":"Sm","key":"22A1","mappings":{"default":{"default":"eingerahmter Punktoperator"}}},{"category":"Sm","key":"22A2","mappings":{"default":{"default":"ergibt"}}},{"category":"Sm","key":"22A3","mappings":{"default":{"default":"ergibt nicht"}}},{"category":"Sm","key":"22A4","mappings":{"default":{"default":"senkrecht von"}}},{"category":"Sm","key":"22A5","mappings":{"default":{"default":"senkrecht auf"}}},{"category":"Sm","key":"22A6","mappings":{"default":{"default":"Behauptung"}}},{"category":"Sm","key":"22A7","mappings":{"default":{"default":"Model"}}},{"category":"Sm","key":"22A8","mappings":{"default":{"default":"wahr"}}},{"category":"Sm","key":"22A9","mappings":{"default":{"default":"erzwingen"}}},{"category":"Sm","key":"22AA","mappings":{"default":{"default":"dreifache vertikale Leiste mit rechtem Drehkreuz"}}},{"category":"Sm","key":"22AB","mappings":{"default":{"default":"doppelte vertikale Leiste mit doppeltem rechtem Drehkreuz"}}},{"category":"Sm","key":"22AC","mappings":{"default":{"default":"beweist nicht"}}},{"category":"Sm","key":"22AD","mappings":{"default":{"default":"nicht wahr"}}},{"category":"Sm","key":"22AE","mappings":{"default":{"default":"nicht erzwingen"}}},{"category":"Sm","key":"22AF","mappings":{"default":{"default":"negierte doppelte vertikale Leiste mit doppeltem rechten Drehkreuz"}}},{"category":"Sm","key":"22B0","mappings":{"default":{"default":"vorangehend in Relation"}}},{"category":"Sm","key":"22B1","mappings":{"default":{"default":"nachfolgend in Relation"}}},{"category":"Sm","key":"22B2","mappings":{"default":{"default":"normale Untergruppe"}}},{"category":"Sm","key":"22B3","mappings":{"default":{"default":"enthält normale Untergruppe"}}},{"category":"Sm","key":"22B4","mappings":{"default":{"default":"normale Untergruppe von oder gleich"}}},{"category":"Sm","key":"22B5","mappings":{"default":{"default":"enthält normale Untergruppe oder gleich"}}},{"category":"Sm","key":"22B6","mappings":{"default":{"default":"Original von"}}},{"category":"Sm","key":"22B7","mappings":{"default":{"default":"Bild von"}}},{"category":"Sm","key":"22B8","mappings":{"default":{"default":"Mehrfachzuordnung"}}},{"category":"Sm","key":"22B9","mappings":{"default":{"default":"hermitesch konjugierte Matrix"}}},{"category":"Sm","key":"22BA","mappings":{"default":{"default":"einschalten"}}},{"category":"Sm","key":"22BB","mappings":{"default":{"default":"Ausschließendes Oder"}}},{"category":"Sm","key":"22BC","mappings":{"default":{"default":"Nand verknüpft mit"}}},{"category":"Sm","key":"22BD","mappings":{"default":{"default":"Nor verknüpft mit"}}},{"category":"Sm","key":"22BF","mappings":{"default":{"default":"rechtwinkliges Dreieck"}}},{"category":"Sm","key":"22C0","mappings":{"default":{"default":"N-stufiges logisches Und"}}},{"category":"Sm","key":"22C1","mappings":{"default":{"default":"N-stufiges logisches Oder"}}},{"category":"Sm","key":"22C2","mappings":{"default":{"default":"N-stufiger Durchschnitt"}}},{"category":"Sm","key":"22C3","mappings":{"default":{"default":"N-stufige Vereinigung"}}},{"category":"Sm","key":"22C4","mappings":{"default":{"default":"Rautenoperator"}}},{"category":"Sm","key":"22C5","mappings":{"default":{"default":"mal","alternative":"Multiplikationspunkt"}}},{"category":"Sm","key":"22C6","mappings":{"default":{"default":"Sternoperator"}}},{"category":"Sm","key":"22C7","mappings":{"default":{"default":"Divisionsanzahl"}}},{"category":"Sm","key":"22C8","mappings":{"default":{"default":"Schleife"}}},{"category":"Sm","key":"22C9","mappings":{"default":{"default":"linkes halbdirektes Produkt"}}},{"category":"Sm","key":"22CA","mappings":{"default":{"default":"rechtes halbdirektes Produkt"}}},{"category":"Sm","key":"22CB","mappings":{"default":{"default":"linkes halbdirektes Produkt"}}},{"category":"Sm","key":"22CC","mappings":{"default":{"default":"rechtes halbdirektes Produkt"}}},{"category":"Sm","key":"22CD","mappings":{"default":{"default":"umgekehrte Tilde gleich"}}},{"category":"Sm","key":"22CE","mappings":{"default":{"default":"geschweiftes logisches Oder"}}},{"category":"Sm","key":"22CF","mappings":{"default":{"default":"geschweiftes logisches Und"}}},{"category":"Sm","key":"22D0","mappings":{"default":{"default":"doppelte Teilmenge"}}},{"category":"Sm","key":"22D1","mappings":{"default":{"default":"doppelte Obermenge"}}},{"category":"Sm","key":"22D2","mappings":{"default":{"default":"doppelter Durchschnitt"}}},{"category":"Sm","key":"22D3","mappings":{"default":{"default":"doppelte Vereinigung"}}},{"category":"Sm","key":"22D4","mappings":{"default":{"default":"echter Durchschnitt"}}},{"category":"Sm","key":"22D5","mappings":{"default":{"default":"gleich und parallel"}}},{"category":"Sm","key":"22D6","mappings":{"default":{"default":"kleiner als mit Punkt"}}},{"category":"Sm","key":"22D7","mappings":{"default":{"default":"größer als mit Punkt"}}},{"category":"Sm","key":"22D8","mappings":{"default":{"default":"sehr viel kleiner als"}}},{"category":"Sm","key":"22D9","mappings":{"default":{"default":"sehr viel größer als"}}},{"category":"Sm","key":"22DA","mappings":{"default":{"default":"kleiner als, gleich oder größer als"}}},{"category":"Sm","key":"22DB","mappings":{"default":{"default":"größer als, gleich oder kleiner als"}}},{"category":"Sm","key":"22DC","mappings":{"default":{"default":"gleich oder kleiner als"}}},{"category":"Sm","key":"22DD","mappings":{"default":{"default":"gleich oder größer als"}}},{"category":"Sm","key":"22DE","mappings":{"default":{"default":"gleich oder vorangehend"}}},{"category":"Sm","key":"22DF","mappings":{"default":{"default":"gleich oder nachfolgend"}}},{"category":"Sm","key":"22E0","mappings":{"default":{"default":"weder vorangehend oder gleich"}}},{"category":"Sm","key":"22E1","mappings":{"default":{"default":"weder nachfolgend oder gleich"}}},{"category":"Sm","key":"22E2","mappings":{"default":{"default":"kein quadratisches Bild oder gleich"}}},{"category":"Sm","key":"22E3","mappings":{"default":{"default":"kein quadratisches Original oder gleich"}}},{"category":"Sm","key":"22E4","mappings":{"default":{"default":"quadratisches Bild oder nicht gleich"}}},{"category":"Sm","key":"22E5","mappings":{"default":{"default":"quadratisches Original oder nicht gleich"}}},{"category":"Sm","key":"22E6","mappings":{"default":{"default":"kleiner als, aber nicht äquivalent"}}},{"category":"Sm","key":"22E7","mappings":{"default":{"default":"größer als, aber nicht äquivalent"}}},{"category":"Sm","key":"22E8","mappings":{"default":{"default":"vorangehend, aber nicht äquivalent"}}},{"category":"Sm","key":"22E9","mappings":{"default":{"default":"nachfolgend, aber nicht äquivalent"}}},{"category":"Sm","key":"22EA","mappings":{"default":{"default":"nicht normale Untergruppe von"}}},{"category":"Sm","key":"22EB","mappings":{"default":{"default":"enthält keine normale Untergruppe"}}},{"category":"Sm","key":"22EC","mappings":{"default":{"default":"keine normale Untergruppe von oder gleich"}}},{"category":"Sm","key":"22ED","mappings":{"default":{"default":"enthält nicht als normale Untergruppe oder gleich"}}},{"category":"Sm","key":"22EE","mappings":{"default":{"default":"Vertikale Ellipsis"}}},{"category":"Sm","key":"22EF","mappings":{"default":{"default":"Zentrierte horizontale Ellipse"}}},{"category":"Sm","key":"22F0","mappings":{"default":{"default":"Diagonale Ellipse unten links nach oben rechts"}}},{"category":"Sm","key":"22F1","mappings":{"default":{"default":"Diagonale Ellipse, oben links nach unten rechts"}}},{"category":"Sm","key":"22F2","mappings":{"default":{"default":"Element mit langem horizontalen Strich"}}},{"category":"Sm","key":"22F3","mappings":{"default":{"default":"Element mit vertikalem Strich am Ende des horizontalen Strichs"}}},{"category":"Sm","key":"22F4","mappings":{"default":{"default":"kleines Element mit vertikalem Strich am Ende des horizontalen Strichs"}}},{"category":"Sm","key":"22F5","mappings":{"default":{"default":"Element mit Punkt"}}},{"category":"Sm","key":"22F6","mappings":{"default":{"default":"Element mit Überstrich"}}},{"key":"22F7","mappings":{"default":{"default":"kleines Element mit Überstrich"}},"category":"Sm"},{"category":"Sm","key":"22F8","mappings":{"default":{"default":"Element mit Unterstrich"}}},{"category":"Sm","key":"22F9","mappings":{"default":{"default":"Element mit 2 horizontalen Strichen"}}},{"category":"Sm","key":"22FA","mappings":{"default":{"default":"umgekehrtes Elementzeichen mit langem horizontalen Strich"}}},{"category":"Sm","key":"22FB","mappings":{"default":{"default":"umgekehrtes Elementzeichen mit vertikalem Strich am Ende des horizontalen Strichs"}}},{"category":"Sm","key":"22FC","mappings":{"default":{"default":"kleines umgekehrtes Elementzeichen mit vertikalem Strich am Ende des horizontalen Strichs"}}},{"key":"22FD","mappings":{"default":{"default":"umgekehrtes Elementzeichen mit Überstrich"}},"category":"Sm"},{"category":"Sm","key":"22FE","mappings":{"default":{"default":"kleines umgekehrtes Elementzeichen mit Überstrich"}}},{"category":"Sm","key":"22FF","mappings":{"default":{"default":"Z-Notation-Bag-Mitgliedschaft"}}},{"category":"So","key":"2300","mappings":{"default":{"default":"Durchmesser-Zeichen"}}},{"category":"So","key":"2302","mappings":{"default":{"default":"Haus"}}},{"category":"So","key":"2305","mappings":{"default":{"default":"Projektiv"}}},{"category":"So","key":"2306","mappings":{"default":{"default":"Perspektive"}}},{"category":"So","key":"2307","mappings":{"default":{"default":"Schlangenlinie"}}},{"category":"So","key":"2310","mappings":{"default":{"default":"Umgekehrtes Nichtzeichen"}}},{"category":"So","key":"2311","mappings":{"default":{"default":"Quadratische Raute"}}},{"category":"So","key":"2312","mappings":{"default":{"default":"Bogen"}}},{"category":"So","key":"2313","mappings":{"default":{"default":"Segment"}}},{"category":"So","key":"2314","mappings":{"default":{"default":"Sektor"}}},{"category":"So","key":"2795","mappings":{"default":{"default":"Schweres Pluszeichen"}}},{"category":"So","key":"2796","mappings":{"default":{"default":"Schweres Minuszeichen"}}},{"category":"So","key":"2797","mappings":{"default":{"default":"Heavy Division Sign"}}},{"category":"So","key":"27B0","mappings":{"default":{"default":"Curly Loop"}}},{"category":"So","key":"27BF","mappings":{"default":{"default":"Doppelte geschweifte Schleife"}}},{"category":"Sm","key":"27C1","mappings":{"default":{"default":"Weißes Dreieck, das kleines weißes Dreieck enthält"}}},{"category":"Sm","key":"27C2","mappings":{"default":{"default":"Aufrecht"}}},{"category":"Sm","key":"27C3","mappings":{"default":{"default":"Subset öffnen"}}},{"category":"Sm","key":"27C4","mappings":{"default":{"default":"Superset öffnen"}}},{"category":"Sm","key":"27C7","mappings":{"default":{"default":"Oder mit innerem Punkt"}}},{"category":"Sm","key":"27C8","mappings":{"default":{"default":"Umgekehrter Schrägstrich Vorhergehender Teilsatz"}}},{"category":"Sm","key":"27C9","mappings":{"default":{"default":"Superset Vorgänger Schrägstrich"}}},{"category":"Sm","key":"27CA","mappings":{"default":{"default":"Vertikaler Balken mit horizontalem Hub"}}},{"category":"Sm","key":"27CB","mappings":{"default":{"default":"Mathematische aufsteigende Diagonale"}}},{"category":"Sm","key":"27CC","mappings":{"default":{"default":"Lange Abteilung"}}},{"category":"Sm","key":"27CD","mappings":{"default":{"default":"Mathematische fallende Diagonale"}}},{"category":"Sm","key":"27CE","mappings":{"default":{"default":"quadratisches und"}}},{"category":"Sm","key":"27CF","mappings":{"default":{"default":"quadratisches oder"}}},{"category":"Sm","key":"27D0","mappings":{"default":{"default":"Weißer Diamant mit zentriertem Punkt"}}},{"category":"Sm","key":"27D1","mappings":{"default":{"default":"Und mit Dot"}}},{"category":"Sm","key":"27D2","mappings":{"default":{"default":"Element der Öffnung nach oben"}}},{"category":"Sm","key":"27D3","mappings":{"default":{"default":"Untere rechte Ecke mit Punkt"}}},{"category":"Sm","key":"27D4","mappings":{"default":{"default":"Obere linke Ecke mit Punkt"}}},{"category":"Sm","key":"27D5","mappings":{"default":{"default":"Linke äußere Verbindung"}}},{"category":"Sm","key":"27D6","mappings":{"default":{"default":"rechter äußerer Join"}}},{"category":"Sm","key":"27D7","mappings":{"default":{"default":"Volle äußere Verbindung"}}},{"category":"Sm","key":"27D8","mappings":{"default":{"default":"Große Up Tack"}}},{"category":"Sm","key":"27D9","mappings":{"default":{"default":"Großes Down Tack"}}},{"category":"Sm","key":"27DA","mappings":{"default":{"default":"Linkes und rechtes doppeltes Drehkreuz"}}},{"category":"Sm","key":"27DB","mappings":{"default":{"default":"Linker und rechter Tack"}}},{"category":"Sm","key":"27DC","mappings":{"default":{"default":"Linke Multimap"}}},{"category":"Sm","key":"27DD","mappings":{"default":{"default":"Long Right Tack"}}},{"category":"Sm","key":"27DE","mappings":{"default":{"default":"Langer linker Tack"}}},{"category":"Sm","key":"27DF","mappings":{"default":{"default":"Up Tack mit Kreis oben"}}},{"category":"Sm","key":"27E0","mappings":{"default":{"default":"Raute durch horizontale Regel geteilt"}}},{"category":"Sm","key":"27E1","mappings":{"default":{"default":"Weißer konkavseitiger Diamant"}}},{"category":"Sm","key":"27E2","mappings":{"default":{"default":"Weißer konkavseitiger Diamant mit Zecke nach links"}}},{"category":"Sm","key":"27E3","mappings":{"default":{"default":"Weißer konkavseitiger Diamant mit Zecke nach rechts"}}},{"category":"Sm","key":"27E4","mappings":{"default":{"default":"Weißes Quadrat mit Zecke nach links"}}},{"category":"Sm","key":"27E5","mappings":{"default":{"default":"Weißes Quadrat mit Tick nach rechts"}}},{"category":"Sm","key":"292B","mappings":{"default":{"default":"Steigende diagonale Kreuzung fallende Diagonale"}}},{"category":"Sm","key":"292C","mappings":{"default":{"default":"Fallende diagonale Kreuzung steigende Diagonale"}}},{"category":"Sm","key":"2980","mappings":{"default":{"default":"Dreifacher vertikaler Balkenbegrenzer"}}},{"category":"Sm","key":"2981","mappings":{"default":{"default":"Z Notationspunkt"}}},{"category":"Sm","key":"2982","mappings":{"default":{"default":"Z Notationstyp Doppelpunkt"}}},{"category":"Sm","key":"2999","mappings":{"default":{"default":"Gepunkteter Zaun"}}},{"category":"Sm","key":"299A","mappings":{"default":{"default":"Vertikale Zickzacklinie"}}},{"category":"Sm","key":"29B0","mappings":{"default":{"default":"Umgekehrter leerer Satz"}}},{"category":"Sm","key":"29B1","mappings":{"default":{"default":"Leeres Set mit Overbar"}}},{"category":"Sm","key":"29B2","mappings":{"default":{"default":"Leeres Set mit kleinem Kreis oben"}}},{"category":"Sm","key":"29B5","mappings":{"default":{"default":"Kreis mit horizontaler Leiste"}}},{"key":"29B6","mappings":{"default":{"default":"circled vertical bar"}},"category":"Sm"},{"category":"Sm","key":"29B7","mappings":{"default":{"default":"eingekreiste Parallele"}}},{"category":"Sm","key":"29B8","mappings":{"default":{"default":"eingekreister umgekehrter Schrägstrich"}}},{"category":"Sm","key":"29B9","mappings":{"default":{"default":"eingekreistes Senkrecht"}}},{"category":"Sm","key":"29BA","mappings":{"default":{"default":"Kreis geteilt durch horizontale Leiste und obere Hälfte geteilt durch vertikale Leiste"}}},{"category":"Sm","key":"29BB","mappings":{"default":{"default":"Kreis mit überlagertem X"}}},{"category":"Sm","key":"29BC","mappings":{"default":{"default":"eingekreistes, gegen den Uhrzeigersinn gedrehtes Divisionszeichen"}}},{"category":"Sm","key":"29BE","mappings":{"default":{"default":"eingekreiste weiße Kugel"}}},{"category":"Sm","key":"29BF","mappings":{"default":{"default":"eingekreiste Kugel"}}},{"category":"Sm","key":"29C0","mappings":{"default":{"default":"eingekreist weniger als"}}},{"category":"Sm","key":"29C1","mappings":{"default":{"default":"eingekreist größer als"}}},{"category":"Sm","key":"29C2","mappings":{"default":{"default":"Kreis mit kleinem Kreis nach rechts"}}},{"category":"Sm","key":"29C3","mappings":{"default":{"default":"Kreis mit zwei horizontalen Strichen nach rechts"}}},{"category":"Sm","key":"29C4","mappings":{"default":{"default":"Quadrat mit Diagonale links unten nach rechts oben"}}},{"category":"Sm","key":"29C5","mappings":{"default":{"default":"Quadrat mit Diagonale links oben nach rechts unten"}}},{"category":"Sm","key":"29C6","mappings":{"default":{"default":"Quadrat Sternchen"}}},{"category":"Sm","key":"29C7","mappings":{"default":{"default":"Quadratischer kleiner Kreis"}}},{"category":"Sm","key":"29C8","mappings":{"default":{"default":"Doppelquadrat"}}},{"category":"Sm","key":"29C9","mappings":{"default":{"default":"Zwei verbundene Quadrate"}}},{"category":"Sm","key":"29CA","mappings":{"default":{"default":"Dreieck mit Punkt oben"}}},{"category":"Sm","key":"29CB","mappings":{"default":{"default":"Dreieck mit Unterleiste"}}},{"category":"Sm","key":"29CC","mappings":{"default":{"default":"S im Dreieck"}}},{"category":"Sm","key":"29CD","mappings":{"default":{"default":"Dreieck mit Serifen unten"}}},{"category":"Sm","key":"29CE","mappings":{"default":{"default":"Rechtes Dreieck über linkem Dreieck"}}},{"category":"Sm","key":"29CF","mappings":{"default":{"default":"Linkes Dreieck neben vertikaler Leiste"}}},{"category":"Sm","key":"29D0","mappings":{"default":{"default":"Vertikaler Balken neben dem rechten Dreieck"}}},{"category":"Sm","key":"29D1","mappings":{"default":{"default":"Fliege mit linker Hälfte schwarz"}}},{"category":"Sm","key":"29D2","mappings":{"default":{"default":"Fliege mit rechter Hälfte schwarz"}}},{"category":"Sm","key":"29D3","mappings":{"default":{"default":"Schwarze Fliege"}}},{"category":"Sm","key":"29D4","mappings":{"default":{"default":"Zeiten mit linker Hälfte schwarz"}}},{"category":"Sm","key":"29D5","mappings":{"default":{"default":"Mal mit rechter Hälfte Schwarz"}}},{"category":"Sm","key":"29D6","mappings":{"default":{"default":"Weiße Sanduhr"}}},{"category":"Sm","key":"29D7","mappings":{"default":{"default":"Schwarze Sanduhr"}}},{"category":"Sm","key":"29DC","mappings":{"default":{"default":"Unvollständige Unendlichkeit"}}},{"category":"Sm","key":"29DD","mappings":{"default":{"default":"Krawatte über Unendlichkeit"}}},{"category":"Sm","key":"29DE","mappings":{"default":{"default":"Unendlichkeit mit vertikaler Leiste negiert"}}},{"category":"Sm","key":"29DF","mappings":{"default":{"default":"Doppelter Multimap"}}},{"category":"Sm","key":"29E0","mappings":{"default":{"default":"Quadrat mit konturierter Kontur"}}},{"category":"Sm","key":"29E1","mappings":{"default":{"default":"Erhöht als"}}},{"category":"Sm","key":"29E2","mappings":{"default":{"default":"Produkt mischen"}}},{"category":"Sm","key":"29E3","mappings":{"default":{"default":"Gleichheitszeichen und geneigte Parallele"}}},{"category":"Sm","key":"29E4","mappings":{"default":{"default":"Gleichheitszeichen und schräge Parallele zu Tilde oben"}}},{"category":"Sm","key":"29E5","mappings":{"default":{"default":"Identisch und schräg parallel"}}},{"category":"Sm","key":"29E6","mappings":{"default":{"default":"Gleich Stark"}}},{"category":"Sm","key":"29E7","mappings":{"default":{"default":"Thermodynamisch"}}},{"category":"Sm","key":"29E8","mappings":{"default":{"default":"Abwärtszeigendes Dreieck mit linker Hälfte schwarz"}}},{"category":"Sm","key":"29E9","mappings":{"default":{"default":"Abwärtsdreieck mit rechter Hälfte schwarz"}}},{"category":"Sm","key":"29EB","mappings":{"default":{"default":"Schwarze Raute"}}},{"category":"Sm","key":"29EE","mappings":{"default":{"default":"Fehlergesperrtes weißes Quadrat"}}},{"category":"Sm","key":"29EF","mappings":{"default":{"default":"Fehlerfreies schwarzes Quadrat"}}},{"category":"Sm","key":"29F0","mappings":{"default":{"default":"Fehlergesperrter weißer Diamant"}}},{"category":"Sm","key":"29F1","mappings":{"default":{"default":"Error-Barred Black Diamond"}}},{"category":"Sm","key":"29F2","mappings":{"default":{"default":"Fehlergesperrter weißer Kreis"}}},{"category":"Sm","key":"29F3","mappings":{"default":{"default":"Fehlergesperrter schwarzer Kreis"}}},{"key":"29F4","mappings":{"default":{"default":"rule delayed"}},"category":"Sm"},{"category":"Sm","key":"29F5","mappings":{"default":{"default":"Umgekehrter Schrägstrich Operator"}}},{"category":"Sm","key":"29F6","mappings":{"default":{"default":"Schrägstrich mit Overbar"}}},{"category":"Sm","key":"29F7","mappings":{"default":{"default":"Umgekehrter Schrägstrich mit horizontalem Hub"}}},{"category":"Sm","key":"29F8","mappings":{"default":{"default":"Großer Schrägstrich"}}},{"category":"Sm","key":"29F9","mappings":{"default":{"default":"Großer umgekehrter Schrägstrich"}}},{"category":"Sm","key":"29FA","mappings":{"default":{"default":"Doppel plus"}}},{"category":"Sm","key":"29FB","mappings":{"default":{"default":"Triple Plus"}}},{"category":"Sm","key":"29FE","mappings":{"default":{"default":"Winzig"}}},{"category":"Sm","key":"29FF","mappings":{"default":{"default":"Miny"}}},{"category":"Sm","key":"2A00","mappings":{"default":{"default":"N-Ary Circled-Punkt-Operator"}}},{"category":"Sm","key":"2A01","mappings":{"default":{"default":"N-Ary Circled Plus Operator"}}},{"category":"Sm","key":"2A02","mappings":{"default":{"default":"N-Ary Circled-Times-Operator"}}},{"category":"Sm","key":"2A03","mappings":{"default":{"default":"N-Ary Union-Operator mit Punkt"}}},{"category":"Sm","key":"2A04","mappings":{"default":{"default":"N-Ary Union-Operator mit Plus"}}},{"category":"Sm","key":"2A05","mappings":{"default":{"default":"N-Ary Square-Kreuzungsoperator"}}},{"category":"Sm","key":"2A06","mappings":{"default":{"default":"N-Ary Square Union-Betreiber"}}},{"category":"Sm","key":"2A07","mappings":{"default":{"default":"Zwei logische und Operator"}}},{"category":"Sm","key":"2A08","mappings":{"default":{"default":"Zwei logisch oder Operator"}}},{"category":"Sm","key":"2A09","mappings":{"default":{"default":"N-Ary Times-Operator"}}},{"category":"Sm","key":"2A0A","mappings":{"default":{"default":"Modulo Zwei Summe"}}},{"category":"Sm","key":"2A0B","mappings":{"default":{"default":"Summation mit Integral"}}},{"category":"Sm","key":"2A0C","mappings":{"default":{"default":"Vierfach-Integral-Operator"}}},{"category":"Sm","key":"2A0D","mappings":{"default":{"default":"Finite Part Integral"}}},{"category":"Sm","key":"2A0E","mappings":{"default":{"default":"Integral mit Doppelhub"}}},{"category":"Sm","key":"2A0F","mappings":{"default":{"default":"Integraler Durchschnitt mit Schrägstrich"}}},{"category":"Sm","key":"2A10","mappings":{"default":{"default":"Zirkulationsfunktion"}}},{"category":"Sm","key":"2A11","mappings":{"default":{"default":"Integration gegen den Uhrzeigersinn"}}},{"category":"Sm","key":"2A12","mappings":{"default":{"default":"Linienintegration mit rechteckigem Pfad um die Pole"}}},{"category":"Sm","key":"2A13","mappings":{"default":{"default":"Linienintegration mit halbkreisförmigem Pfad um den Pol"}}},{"category":"Sm","key":"2A14","mappings":{"default":{"default":"Leitungsintegration ohne Pole"}}},{"category":"Sm","key":"2A15","mappings":{"default":{"default":"Integral um einen Punktoperator"}}},{"category":"Sm","key":"2A16","mappings":{"default":{"default":"Quaternion Integral Operator"}}},{"category":"Sm","key":"2A18","mappings":{"default":{"default":"Integriert mit Times Sign"}}},{"category":"Sm","key":"2A19","mappings":{"default":{"default":"Integral mit der Kreuzung"}}},{"category":"Sm","key":"2A1A","mappings":{"default":{"default":"Integral mit Union"}}},{"category":"Sm","key":"2A1B","mappings":{"default":{"default":"Integral mit Overbar"}}},{"category":"Sm","key":"2A1C","mappings":{"default":{"default":"Integral mit Underbar"}}},{"category":"Sm","key":"2A1D","mappings":{"default":{"default":"Beitreten"}}},{"category":"Sm","key":"2A1E","mappings":{"default":{"default":"Großes linkes Dreieckbediener"}}},{"category":"Sm","key":"2A1F","mappings":{"default":{"default":"Z-Notationsschema-Zusammensetzung"}}},{"category":"Sm","key":"2A20","mappings":{"default":{"default":"Z-Notationsschema-Piping"}}},{"category":"Sm","key":"2A21","mappings":{"default":{"default":"Z-Notationsschema-Projektion"}}},{"category":"Sm","key":"2A22","mappings":{"default":{"default":"Pluszeichen mit kleinem Kreis oben"}}},{"category":"Sm","key":"2A23","mappings":{"default":{"default":"Pluszeichen mit Zirkumflex-Akzent oben"}}},{"category":"Sm","key":"2A24","mappings":{"default":{"default":"Pluszeichen mit Tilde oben"}}},{"category":"Sm","key":"2A25","mappings":{"default":{"default":"Pluszeichen mit Punkt unten"}}},{"category":"Sm","key":"2A26","mappings":{"default":{"default":"Pluszeichen mit Tilde unter"}}},{"category":"Sm","key":"2A27","mappings":{"default":{"default":"Pluszeichen mit Index Zwei"}}},{"category":"Sm","key":"2A28","mappings":{"default":{"default":"Pluszeichen mit schwarzem Dreieck"}}},{"category":"Sm","key":"2A29","mappings":{"default":{"default":"Minuszeichen mit Komma oben"}}},{"category":"Sm","key":"2A2A","mappings":{"default":{"default":"Minuszeichen mit Punkt unten"}}},{"category":"Sm","key":"2A2B","mappings":{"default":{"default":"Minuszeichen mit fallenden Punkten"}}},{"category":"Sm","key":"2A2C","mappings":{"default":{"default":"Minuszeichen mit steigenden Punkten"}}},{"category":"Sm","key":"2A2D","mappings":{"default":{"default":"Pluszeichen Im linken Halbkreis"}}},{"category":"Sm","key":"2A2E","mappings":{"default":{"default":"Pluszeichen Im rechten Halbkreis"}}},{"category":"Sm","key":"2A2F","mappings":{"default":{"default":"Vektor- oder Kreuzprodukt"}}},{"category":"Sm","key":"2A30","mappings":{"default":{"default":"Multiplikationszeichen mit Punkt oben"}}},{"category":"Sm","key":"2A31","mappings":{"default":{"default":"Multiplikationszeichen mit Unterleiste"}}},{"category":"Sm","key":"2A32","mappings":{"default":{"default":"Semidirektprodukt mit geschlossenem Boden"}}},{"key":"2A33","mappings":{"default":{"default":"smash product"}},"category":"Sm"},{"category":"Sm","key":"2A34","mappings":{"default":{"default":"Multiplikationszeichen im linken Halbkreis"}}},{"category":"Sm","key":"2A35","mappings":{"default":{"default":"Multiplikationszeichen im rechten Halbkreis"}}},{"category":"Sm","key":"2A36","mappings":{"default":{"default":"eingekreistes Multiplikationszeichen mit Circumflex-Akzent"}}},{"category":"Sm","key":"2A37","mappings":{"default":{"default":"Multiplikationszeichen im doppelten Kreis"}}},{"key":"2A38","mappings":{"default":{"default":"circled division sign"}},"category":"Sm"},{"category":"Sm","key":"2A39","mappings":{"default":{"default":"Pluszeichen im Dreieck"}}},{"category":"Sm","key":"2A3A","mappings":{"default":{"default":"Minuszeichen im Dreieck"}}},{"category":"Sm","key":"2A3B","mappings":{"default":{"default":"Multiplikationszeichen im Dreieck"}}},{"category":"Sm","key":"2A3C","mappings":{"default":{"default":"Innenprodukt"}}},{"category":"Sm","key":"2A3D","mappings":{"default":{"default":"Produkt für den rechten Innenraum"}}},{"category":"Sm","key":"2A3E","mappings":{"default":{"default":"Z-Notation relationaler Aufbau"}}},{"category":"Sm","key":"2A3F","mappings":{"default":{"default":"Verschmelzung oder Koprodukt"}}},{"category":"Sm","key":"2A40","mappings":{"default":{"default":"Schnittpunkt mit Punkt"}}},{"category":"Sm","key":"2A41","mappings":{"default":{"default":"Union mit Minuszeichen"}}},{"category":"Sm","key":"2A42","mappings":{"default":{"default":"Union mit Overbar"}}},{"category":"Sm","key":"2A43","mappings":{"default":{"default":"Kreuzung mit Overbar"}}},{"category":"Sm","key":"2A44","mappings":{"default":{"default":"Überschneidung mit logischem und"}}},{"category":"Sm","key":"2A45","mappings":{"default":{"default":"Vereinigung mit logischem Or"}}},{"category":"Sm","key":"2A46","mappings":{"default":{"default":"Vereinigung über dem Schnittpunkt"}}},{"category":"Sm","key":"2A47","mappings":{"default":{"default":"Schnittpunkt über Union"}}},{"category":"Sm","key":"2A48","mappings":{"default":{"default":"Vereinigung über Stab über Kreuzung"}}},{"category":"Sm","key":"2A49","mappings":{"default":{"default":"Schnittpunkt oberhalb der Stange oberhalb der Union"}}},{"category":"Sm","key":"2A4A","mappings":{"default":{"default":"Union daneben und mit Union verbunden"}}},{"category":"Sm","key":"2A4B","mappings":{"default":{"default":"Kreuzung neben und mit Kreuzung verbunden"}}},{"category":"Sm","key":"2A4C","mappings":{"default":{"default":"Geschlossene Vereinigung mit Serifen"}}},{"category":"Sm","key":"2A4D","mappings":{"default":{"default":"Geschlossener Schnittpunkt mit Serifen"}}},{"category":"Sm","key":"2A4E","mappings":{"default":{"default":"Doppelter Quadratschnitt"}}},{"category":"Sm","key":"2A4F","mappings":{"default":{"default":"Double Square Union"}}},{"category":"Sm","key":"2A50","mappings":{"default":{"default":"Closed Union mit Serifen und Smash-Produkten"}}},{"category":"Sm","key":"2A51","mappings":{"default":{"default":"Logisch und mit Punkt oben"}}},{"category":"Sm","key":"2A52","mappings":{"default":{"default":"Logisch oder mit Punkt oben"}}},{"key":"2A53","mappings":{"default":{"default":"double logical and"}},"category":"Sm"},{"key":"2A54","mappings":{"default":{"default":"double logical or"}},"category":"Sm"},{"category":"Sm","key":"2A55","mappings":{"default":{"default":"Zwei durchschneiden logisch und"}}},{"category":"Sm","key":"2A56","mappings":{"default":{"default":"Zwei sich überschneidende logische Oder"}}},{"category":"Sm","key":"2A57","mappings":{"default":{"default":"Abfallend groß oder"}}},{"category":"Sm","key":"2A58","mappings":{"default":{"default":"Abfallend groß und"}}},{"category":"Sm","key":"2A59","mappings":{"default":{"default":"Logisch oder Überlappend Logisch Und"}}},{"category":"Sm","key":"2A5A","mappings":{"default":{"default":"Logisch und mit Middle Stem"}}},{"category":"Sm","key":"2A5B","mappings":{"default":{"default":"Logisch oder mit Middle Stem"}}},{"category":"Sm","key":"2A5C","mappings":{"default":{"default":"Logisch und mit Horizontalstrich"}}},{"category":"Sm","key":"2A5D","mappings":{"default":{"default":"Logisch oder mit Horizontalstrich"}}},{"category":"Sm","key":"2A5E","mappings":{"default":{"default":"Logisch und mit Double Overbar"}}},{"category":"Sm","key":"2A5F","mappings":{"default":{"default":"Logisch und mit Underbar"}}},{"category":"Sm","key":"2A60","mappings":{"default":{"default":"Logisch und mit Double Underbar"}}},{"category":"Sm","key":"2A61","mappings":{"default":{"default":"kleines Vee mit Underbar"}}},{"category":"Sm","key":"2A62","mappings":{"default":{"default":"Logisch oder mit Double Overbar"}}},{"category":"Sm","key":"2A63","mappings":{"default":{"default":"Logisch oder mit doppelter Unterleiste"}}},{"category":"Sm","key":"2A64","mappings":{"default":{"default":"Z-Notation Domain Antirestriction"}}},{"category":"Sm","key":"2A65","mappings":{"default":{"default":"Z Notationsbereich Antirestriction"}}},{"category":"Sm","key":"2A66","mappings":{"default":{"default":"Gleichheitszeichen mit Punkt unten"}}},{"category":"Sm","key":"2A67","mappings":{"default":{"default":"Identisch mit dem obigen Punkt"}}},{"category":"Sm","key":"2A68","mappings":{"default":{"default":"Dreifacher Horizontalbalken mit doppeltem vertikalem Hub"}}},{"category":"Sm","key":"2A69","mappings":{"default":{"default":"Dreifacher Horizontalbalken mit dreifachem vertikalem Hub"}}},{"category":"Sm","key":"2A6A","mappings":{"default":{"default":"Tilde-Operator mit Punkt oben"}}},{"category":"Sm","key":"2A6B","mappings":{"default":{"default":"Tilde-Operator mit steigenden Punkten"}}},{"category":"Sm","key":"2A6C","mappings":{"default":{"default":"Ähnlich Minus Ähnlich"}}},{"category":"Sm","key":"2A6D","mappings":{"default":{"default":"Kongruent mit Punkt oben"}}},{"category":"Sm","key":"2A6E","mappings":{"default":{"default":"Entspricht Asterisk"}}},{"category":"Sm","key":"2A6F","mappings":{"default":{"default":"Fast gleichwertig mit dem Circumflex Accent"}}},{"category":"Sm","key":"2A70","mappings":{"default":{"default":"Ungefähr gleich oder gleich zu"}}},{"category":"Sm","key":"2A71","mappings":{"default":{"default":"Gleichheitszeichen über Pluszeichen"}}},{"category":"Sm","key":"2A72","mappings":{"default":{"default":"Pluszeichen über Gleichheitszeichen"}}},{"category":"Sm","key":"2A73","mappings":{"default":{"default":"Gleichheitszeichen über Tilde-Operator"}}},{"category":"Sm","key":"2A74","mappings":{"default":{"default":"Doppelpunkt gleich"}}},{"key":"2A75","mappings":{"default":{"default":"double equal"}},"category":"Sm"},{"category":"Sm","key":"2A76","mappings":{"default":{"default":"Drei aufeinander folgende Gleichheitszeichen"}}},{"category":"Sm","key":"2A77","mappings":{"default":{"default":"Gleichheitszeichen mit zwei Punkten oben und zwei Punkten unten"}}},{"category":"Sm","key":"2A78","mappings":{"default":{"default":"Gleichwertig mit vier Punkten oben"}}},{"category":"Sm","key":"2A79","mappings":{"default":{"default":"Weniger als mit Circle Inside"}}},{"category":"Sm","key":"2A7A","mappings":{"default":{"default":"Größer als mit Kreis nach innen"}}},{"category":"Sm","key":"2A7B","mappings":{"default":{"default":"Weniger als mit Fragezeichen oben"}}},{"category":"Sm","key":"2A7C","mappings":{"default":{"default":"Größer als mit Fragezeichen oben"}}},{"category":"Sm","key":"2A7D","mappings":{"default":{"default":"Weniger als oder schräg gleich"}}},{"category":"Sm","key":"2A7E","mappings":{"default":{"default":"Größer als oder schräg gleich"}}},{"category":"Sm","key":"2A7F","mappings":{"default":{"default":"Weniger als oder schräg gleich mit Punkt nach innen"}}},{"category":"Sm","key":"2A80","mappings":{"default":{"default":"Größer als oder schräg gleich mit Punkt nach innen"}}},{"category":"Sm","key":"2A81","mappings":{"default":{"default":"Weniger als oder schräg gleich mit Punkt oben"}}},{"category":"Sm","key":"2A82","mappings":{"default":{"default":"Größer als oder schräg gleich mit Punkt oben"}}},{"category":"Sm","key":"2A83","mappings":{"default":{"default":"Weniger als oder schräg gleich mit Punkt oben rechts"}}},{"category":"Sm","key":"2A84","mappings":{"default":{"default":"Größer als oder schräg gleich mit Punkt oben links"}}},{"category":"Sm","key":"2A85","mappings":{"default":{"default":"Weniger als oder ungefähr"}}},{"category":"Sm","key":"2A86","mappings":{"default":{"default":"Größer als oder ungefähr"}}},{"category":"Sm","key":"2A87","mappings":{"default":{"default":"Kleiner als aber nicht gleich"}}},{"category":"Sm","key":"2A88","mappings":{"default":{"default":"Größer als aber nicht gleich"}}},{"key":"2A89","mappings":{"default":{"default":"Kleiner als aber nicht ungefähr gleich"}},"category":"Sm"},{"key":"2A8A","mappings":{"default":{"default":"Größer als aber nicht ungefähr gleich"}},"category":"Sm"},{"category":"Sm","key":"2A8B","mappings":{"default":{"default":"Weniger als aber größer als oben"}}},{"category":"Sm","key":"2A8C","mappings":{"default":{"default":"Größer als aber weniger als unten und größer als oben"}}},{"category":"Sm","key":"2A8D","mappings":{"default":{"default":"Weniger als oben ähnlich oder gleich"}}},{"category":"Sm","key":"2A8E","mappings":{"default":{"default":"Größer als oben ähnlich oder gleich"}}},{"category":"Sm","key":"2A8F","mappings":{"default":{"default":"Weniger als oben ähnlich oben größer als"}}},{"category":"Sm","key":"2A90","mappings":{"default":{"default":"Größer als oben Ähnlich oben"}}},{"key":"2A91","mappings":{"default":{"default":"less than greater than or equal to"}},"category":"Sm"},{"key":"2A92","mappings":{"default":{"default":"greater than less than or equal to"}},"category":"Sm"},{"category":"Sm","key":"2A93","mappings":{"default":{"default":"Weniger als oben schräg gleich oben Größer als oben schräg gleich"}}},{"category":"Sm","key":"2A94","mappings":{"default":{"default":"Größer als oben schräg gleich oben kleiner als oben schräg gleich groß"}}},{"category":"Sm","key":"2A95","mappings":{"default":{"default":"Schräg gleich oder kleiner als"}}},{"category":"Sm","key":"2A96","mappings":{"default":{"default":"Schräg gleich oder größer als"}}},{"category":"Sm","key":"2A97","mappings":{"default":{"default":"Schräg gleich oder kleiner als mit innenliegendem Punkt"}}},{"category":"Sm","key":"2A98","mappings":{"default":{"default":"Schräg gleich oder größer als mit Punkt nach innen"}}},{"key":"2A99","mappings":{"default":{"default":"Gleich oder kleiner als"}},"category":"Sm"},{"key":"2A9A","mappings":{"default":{"default":"Gleich oder größer als"}},"category":"Sm"},{"category":"Sm","key":"2A9B","mappings":{"default":{"default":"Doppelte Linie schräg gleich oder kleiner als"}}},{"category":"Sm","key":"2A9C","mappings":{"default":{"default":"Doppelte Linie schräg gleich oder größer als"}}},{"key":"2A9D","mappings":{"default":{"default":"Äquivalent oder kleiner als"}},"category":"Sm"},{"category":"Sm","key":"2A9E","mappings":{"default":{"default":"Ähnlich oder größer als"}}},{"category":"Sm","key":"2A9F","mappings":{"default":{"default":"Ähnlich oben Weniger als Gleiches Gleichheitszeichen"}}},{"category":"Sm","key":"2AA0","mappings":{"default":{"default":"Ähnlich oben größer als oben Gleichheitszeichen"}}},{"key":"2AA1","mappings":{"default":{"default":"Verschachteltes kleiner als"}},"category":"Sm"},{"key":"2AA2","mappings":{"default":{"default":"Verschachteltes größer als"}},"category":"Sm"},{"category":"Sm","key":"2AA3","mappings":{"default":{"default":"Doppelt verschachtelt, weniger als mit Unterleiste"}}},{"key":"2AA4","mappings":{"default":{"default":"less than greater than overlay"}},"category":"Sm"},{"category":"Sm","key":"2AA5","mappings":{"default":{"default":"Größer als weniger als"}}},{"category":"Sm","key":"2AA6","mappings":{"default":{"default":"Weniger als durch Kurve geschlossen"}}},{"category":"Sm","key":"2AA7","mappings":{"default":{"default":"Größer als durch Kurve geschlossen"}}},{"category":"Sm","key":"2AA8","mappings":{"default":{"default":"Weniger als dann geschlossen durch Kurve oberhalb der gleichen Neigung"}}},{"category":"Sm","key":"2AA9","mappings":{"default":{"default":"Größer als geschlossen durch Kurve oberhalb der gleichen Neigung"}}},{"category":"Sm","key":"2AAA","mappings":{"default":{"default":"kleiner als"}}},{"category":"Sm","key":"2AAB","mappings":{"default":{"default":"Größer als"}}},{"category":"Sm","key":"2AAC","mappings":{"default":{"default":"kleiner als oder gleich"}}},{"category":"Sm","key":"2AAD","mappings":{"default":{"default":"Größer als oder gleich"}}},{"key":"2AAE","mappings":{"default":{"default":"difference between (variant"}},"category":"Sm"},{"category":"Sm","key":"2AAF","mappings":{"default":{"default":"Stellt über dem einzeiligen Gleichheitszeichen ein"}}},{"category":"Sm","key":"2AB0","mappings":{"default":{"default":"Erfolg über dem einzeiligen Gleichheitszeichen"}}},{"category":"Sm","key":"2AB1","mappings":{"default":{"default":"Vorangehende Single-Line ist nicht gleich"}}},{"category":"Sm","key":"2AB2","mappings":{"default":{"default":"Erfolg über Single-Line nicht gleich"}}},{"category":"Sm","key":"2AB3","mappings":{"default":{"default":"Vorangegangenes Gleichheitszeichen"}}},{"category":"Sm","key":"2AB4","mappings":{"default":{"default":"Erfolgreich über Gleichheitszeichen"}}},{"key":"2AB5","mappings":{"default":{"default":"precedes but not equal to"}},"category":"Sm"},{"key":"2AB6","mappings":{"default":{"default":"succeeds but not equal to"}},"category":"Sm"},{"category":"Sm","key":"2AB7","mappings":{"default":{"default":"Vorangegangenes fast gleich zu"}}},{"category":"Sm","key":"2AB8","mappings":{"default":{"default":"Erreicht über fast gleich viel"}}},{"category":"Sm","key":"2AB9","mappings":{"default":{"default":"Vorangegangene nicht annähernd gleich"}}},{"category":"Sm","key":"2ABA","mappings":{"default":{"default":"Erreicht oben nicht annähernd gleich"}}},{"category":"Sm","key":"2ABB","mappings":{"default":{"default":"Doppelter Vorgänger"}}},{"category":"Sm","key":"2ABC","mappings":{"default":{"default":"Doppelter Erfolg"}}},{"category":"Sm","key":"2ABD","mappings":{"default":{"default":"Teilmenge mit Punkt"}}},{"category":"Sm","key":"2ABE","mappings":{"default":{"default":"Superset mit Punkt"}}},{"category":"Sm","key":"2ABF","mappings":{"default":{"default":"Teilmenge mit Pluszeichen unten"}}},{"category":"Sm","key":"2AC0","mappings":{"default":{"default":"Superset mit Pluszeichen unten"}}},{"category":"Sm","key":"2AC1","mappings":{"default":{"default":"Teilmenge mit Multiplikationszeichen unten"}}},{"category":"Sm","key":"2AC2","mappings":{"default":{"default":"Superset mit Multiplikationszeichen unten"}}},{"category":"Sm","key":"2AC3","mappings":{"default":{"default":"Teilmenge oder gleich mit dem obigen Punkt"}}},{"category":"Sm","key":"2AC4","mappings":{"default":{"default":"Superset von oder gleich mit Punkt oben"}}},{"category":"Sm","key":"2AC5","mappings":{"default":{"default":"Teilmenge des obigen Gleichheitszeichens"}}},{"category":"Sm","key":"2AC6","mappings":{"default":{"default":"Superset des obigen Gleichheitszeichens"}}},{"key":"2AC7","mappings":{"default":{"default":"approximate subset of"}},"category":"Sm"},{"key":"2AC8","mappings":{"default":{"default":"approximate superset of"}},"category":"Sm"},{"category":"Sm","key":"2AC9","mappings":{"default":{"default":"Teilmenge von oben fast gleich"}}},{"category":"Sm","key":"2ACA","mappings":{"default":{"default":"Superset von oben fast gleichwertig"}}},{"category":"Sm","key":"2ACB","mappings":{"default":{"default":"Teilmenge von oben nicht gleich"}}},{"category":"Sm","key":"2ACC","mappings":{"default":{"default":"Superset von oben nicht gleich"}}},{"category":"Sm","key":"2ACD","mappings":{"default":{"default":"Platz links Feldbetreiber geöffnet"}}},{"category":"Sm","key":"2ACE","mappings":{"default":{"default":"Rechteckige Box mit offener Box"}}},{"category":"Sm","key":"2ACF","mappings":{"default":{"default":"Geschlossener Teilsatz"}}},{"category":"Sm","key":"2AD0","mappings":{"default":{"default":"Superset geschlossen"}}},{"category":"Sm","key":"2AD1","mappings":{"default":{"default":"Geschlossene Untermenge oder gleich"}}},{"category":"Sm","key":"2AD2","mappings":{"default":{"default":"Superset geschlossen oder gleich"}}},{"key":"2AD3","mappings":{"default":{"default":"subset over superset"}},"category":"Sm"},{"key":"2AD4","mappings":{"default":{"default":"superset over subset"}},"category":"Sm"},{"key":"2AD5","mappings":{"default":{"default":"subset over subset"}},"category":"Sm"},{"key":"2AD6","mappings":{"default":{"default":"superset over superset"}},"category":"Sm"},{"category":"Sm","key":"2AD7","mappings":{"default":{"default":"Superset neben dem Subset"}}},{"category":"Sm","key":"2AD8","mappings":{"default":{"default":"Superset Neben und Joined von Dash mit Subset"}}},{"category":"Sm","key":"2AD9","mappings":{"default":{"default":"Element der Öffnung nach unten"}}},{"category":"Sm","key":"2ADA","mappings":{"default":{"default":"Heugabel mit T-Stück"}}},{"category":"Sm","key":"2ADB","mappings":{"default":{"default":"Transversalschnitt"}}},{"category":"Sm","key":"2ADC","mappings":{"default":{"default":"Gabelung"}}},{"category":"Sm","key":"2ADD","mappings":{"default":{"default":"Nicht Gabeln"}}},{"category":"Sm","key":"2ADE","mappings":{"default":{"default":"Kurzer Linker Tack"}}},{"category":"Sm","key":"2ADF","mappings":{"default":{"default":"Kurzer Down Tack"}}},{"category":"Sm","key":"2AE0","mappings":{"default":{"default":"Short Up Tack"}}},{"category":"Sm","key":"2AE1","mappings":{"default":{"default":"Rechtwinklig zu S"}}},{"category":"Sm","key":"2AE2","mappings":{"default":{"default":"Vertikales Dreifach-Drehkreuz"}}},{"category":"Sm","key":"2AE3","mappings":{"default":{"default":"Doppelter vertikaler Balken linkes Drehkreuz"}}},{"key":"2AE4","mappings":{"default":{"default":"double left turnstile vertical bar"}},"category":"Sm"},{"category":"Sm","key":"2AE5","mappings":{"default":{"default":"Doppelter vertikaler Balken Doppelter linker Drehkreuz"}}},{"category":"Sm","key":"2AE6","mappings":{"default":{"default":"Long Dash von Left Member von Double Vertical"}}},{"category":"Sm","key":"2AE7","mappings":{"default":{"default":"Short Down Tack mit Overbar"}}},{"key":"2AE8","mappings":{"default":{"default":"perpendicular over bar"}},"category":"Sm"},{"category":"Sm","key":"2AE9","mappings":{"default":{"default":"Short Up Tack Über Short Down Tack"}}},{"category":"Sm","key":"2AEA","mappings":{"default":{"default":"Double Down Tack"}}},{"category":"Sm","key":"2AEB","mappings":{"default":{"default":"Verdoppeln Sie Tack"}}},{"category":"Sm","key":"2AEC","mappings":{"default":{"default":"Doppelstrich nicht unterschreiben"}}},{"category":"Sm","key":"2AED","mappings":{"default":{"default":"Umgekehrter Doppelstrich nicht unterschrieben"}}},{"category":"Sm","key":"2AEE","mappings":{"default":{"default":"Teilt sich nicht mit umgekehrten Negationsschrägstrich"}}},{"category":"Sm","key":"2AEF","mappings":{"default":{"default":"Vertikale Linie mit Kreis oben"}}},{"category":"Sm","key":"2AF0","mappings":{"default":{"default":"Vertikale Linie mit Kreis unten"}}},{"category":"Sm","key":"2AF1","mappings":{"default":{"default":"Down Tack mit Kreis unten"}}},{"category":"Sm","key":"2AF2","mappings":{"default":{"default":"Parallel zum horizontalen Anschlag"}}},{"category":"Sm","key":"2AF3","mappings":{"default":{"default":"Parallel zu Tilde Operator"}}},{"category":"Sm","key":"2AF4","mappings":{"default":{"default":"Dreifache vertikale Balken-Binärbeziehung"}}},{"category":"Sm","key":"2AF5","mappings":{"default":{"default":"Dreifacher vertikaler Strich mit horizontalem Hub"}}},{"category":"Sm","key":"2AF6","mappings":{"default":{"default":"Triple Colon Operator"}}},{"category":"Sm","key":"2AF7","mappings":{"default":{"default":"Dreifach verschachtelt weniger als"}}},{"category":"Sm","key":"2AF8","mappings":{"default":{"default":"Dreifach verschachteltes Größeres als"}}},{"category":"Sm","key":"2AF9","mappings":{"default":{"default":"Doppellinie schräg, weniger als oder gleich"}}},{"category":"Sm","key":"2AFA","mappings":{"default":{"default":"Doppelte Linie schräg größer als oder gleich"}}},{"category":"Sm","key":"2AFB","mappings":{"default":{"default":"Dreifache Schrägstrich-Binärbeziehung"}}},{"category":"Sm","key":"2AFC","mappings":{"default":{"default":"Großer dreifacher vertikaler Barbediener"}}},{"category":"Sm","key":"2AFD","mappings":{"default":{"default":"Doppelter Schrägstrich-Operator"}}},{"category":"Sm","key":"2AFE","mappings":{"default":{"default":"Weiße vertikale Leiste"}}},{"category":"Sm","key":"2AFF","mappings":{"default":{"default":"N-Ary White Vertical Bar"}}},{"category":"Pd","key":"301C","mappings":{"default":{"default":"Wave Dash"}}},{"category":"Po","key":"FE10","mappings":{"default":{"default":"Präsentationsformular für vertikales Komma"}}},{"category":"Po","key":"FE13","mappings":{"default":{"default":"Präsentationsformular für vertikalen Doppelpunkt"}}},{"category":"Po","key":"FE14","mappings":{"default":{"default":"Präsentationsformular für vertikales Semikolon"}}},{"category":"Po","key":"FE15","mappings":{"default":{"default":"Präsentationsformular für vertikales Ausrufezeichen"}}},{"category":"Po","key":"FE16","mappings":{"default":{"default":"Präsentationsformular für vertikales Fragezeichen"}}},{"category":"Po","key":"FE19","mappings":{"default":{"default":"Präsentationsformular für vertikale horizontale Ellipsen"}}},{"category":"Po","key":"FE30","mappings":{"default":{"default":"Präsentationsformular für vertikalen Zwei-Punkt-Leader"}}},{"category":"Pd","key":"FE31","mappings":{"default":{"default":"Präsentationsformular für Vertical Em Dash"}}},{"category":"Pd","key":"FE32","mappings":{"default":{"default":"Präsentationsformular für Vertical En Dash"}}},{"category":"Pc","key":"FE33","mappings":{"default":{"default":"Präsentationsformular für Vertical Low Line"}}},{"category":"Pc","key":"FE34","mappings":{"default":{"default":"Präsentationsformular für Vertical Wavy Low Line"}}},{"category":"Po","key":"FE45","mappings":{"default":{"default":"Sesampunkt"}}},{"category":"Po","key":"FE46","mappings":{"default":{"default":"Weißer Sesam Punkt"}}},{"category":"Po","key":"FE49","mappings":{"default":{"default":"Gestrichelte Überlänge"}}},{"category":"Po","key":"FE4A","mappings":{"default":{"default":"Centreline Overline"}}},{"category":"Po","key":"FE4B","mappings":{"default":{"default":"Wellenförmige Überlänge"}}},{"category":"Po","key":"FE4C","mappings":{"default":{"default":"Doppelte Wellenlinie"}}},{"category":"Pc","key":"FE4D","mappings":{"default":{"default":"Gestrichelte Low Line"}}},{"category":"Pc","key":"FE4E","mappings":{"default":{"default":"Mittellinie Low Line"}}},{"category":"Pc","key":"FE4F","mappings":{"default":{"default":"Wellenförmige Low Line"}}},{"category":"Po","key":"FE50","mappings":{"default":{"default":"kleines Komma"}}},{"category":"Po","key":"FE52","mappings":{"default":{"default":"kleiner Punkt"}}},{"category":"Po","key":"FE54","mappings":{"default":{"default":"kleines Semikolon"}}},{"category":"Po","key":"FE55","mappings":{"default":{"default":"kleiner Doppelpunkt"}}},{"category":"Po","key":"FE56","mappings":{"default":{"default":"kleines Fragezeichen"}}},{"category":"Po","key":"FE57","mappings":{"default":{"default":"kleines Ausrufezeichen"}}},{"category":"Pd","key":"FE58","mappings":{"default":{"default":"kleiner Em Dash"}}},{"category":"Po","key":"FE5F","mappings":{"default":{"default":"kleines Nummernzeichen"}}},{"category":"Po","key":"FE60","mappings":{"default":{"default":"kleines kaufmännisches"}}},{"category":"Po","key":"FE61","mappings":{"default":{"default":"kleines Sternchen"}}},{"category":"Sm","key":"FE62","mappings":{"default":{"default":"kleines Pluszeichen"}}},{"category":"Pd","key":"FE63","mappings":{"default":{"default":"kleiner Bindestrich-Minus"}}},{"category":"Sm","key":"FE64","mappings":{"default":{"default":"kleines weniger als Zeichen"}}},{"category":"Sm","key":"FE65","mappings":{"default":{"default":"kleines Größeres als Zeichen"}}},{"category":"Sm","key":"FE66","mappings":{"default":{"default":"kleines Gleichheitszeichen"}}},{"category":"Po","key":"FE68","mappings":{"default":{"default":"kleiner umgekehrter Schrägstrich"}}},{"category":"Sc","key":"FE69","mappings":{"default":{"default":"kleines Dollarzeichen"}}},{"category":"Po","key":"FE6A","mappings":{"default":{"default":"kleines Prozentzeichen"}}},{"category":"Po","key":"FE6B","mappings":{"default":{"default":"kleine kommerzielle bei"}}},{"category":"Po","key":"FF01","mappings":{"default":{"default":"vollbreites Ausrufezeichen"}}},{"category":"Po","key":"FF02","mappings":{"default":{"default":"vollbreites Anführungszeichen"}}},{"category":"Po","key":"FF03","mappings":{"default":{"default":"vollbreites Nummernzeichen"}}},{"category":"Sc","key":"FF04","mappings":{"default":{"default":"vollbreites Dollarzeichen"}}},{"category":"Po","key":"FF05","mappings":{"default":{"default":"vollbreites Prozentzeichen"}}},{"category":"Po","key":"FF06","mappings":{"default":{"default":"vollbreites kaufmännisches und"}}},{"category":"Po","key":"FF07","mappings":{"default":{"default":"vollbreites Apostroph"}}},{"category":"Po","key":"FF0A","mappings":{"default":{"default":"vollbreiter Stern"}}},{"category":"Sm","key":"FF0B","mappings":{"default":{"default":"vollbreites Plus"}}},{"category":"Po","key":"FF0C","mappings":{"default":{"default":"vollbreites Komma"}}},{"category":"Pd","key":"FF0D","mappings":{"default":{"default":"vollbreites Minus"}}},{"category":"Po","key":"FF0E","mappings":{"default":{"default":"vollbreiter Punkt"}}},{"category":"Po","key":"FF0F","mappings":{"default":{"default":"Schrägstrich mit voller Breite"}}},{"category":"Po","key":"FF1A","mappings":{"default":{"default":"vollbreiter Doppelpunkt"}}},{"category":"Po","key":"FF1B","mappings":{"default":{"default":"vollbreites Semikolon"}}},{"category":"Sm","key":"FF1C","mappings":{"default":{"default":"Weniger als das gesamte Zeichen"}}},{"category":"Sm","key":"FF1D","mappings":{"default":{"default":"vollbreites Gleichspannungszeichen"}}},{"category":"Sm","key":"FF1E","mappings":{"default":{"default":"Fullwidth Mehr als Zeichen"}}},{"category":"Po","key":"FF1F","mappings":{"default":{"default":"vollbreites Fragezeichen"}}},{"category":"Po","key":"FF20","mappings":{"default":{"default":"Fullwidth Commercial at"}}},{"category":"Po","key":"FF3C","mappings":{"default":{"default":"Umgekehrter Schrägstrich mit voller Breite"}}},{"category":"Sk","key":"FF3E","mappings":{"default":{"default":"Circumflex-Akzent mit voller Breite"}}},{"category":"Pc","key":"FF3F","mappings":{"default":{"default":"vollbreites Low Line"}}},{"category":"Sk","key":"FF40","mappings":{"default":{"default":"vollbreiter Gravis Akzent"}}},{"category":"Sm","key":"FF5C","mappings":{"default":{"default":"vollbreites Vertikale Linie"}}},{"category":"Sm","key":"FF5E","mappings":{"default":{"default":"vollbreite Tilde"}}},{"category":"Sc","key":"FFE0","mappings":{"default":{"default":"vollbreites Cent-Zeichen"}}},{"category":"Sc","key":"FFE1","mappings":{"default":{"default":"vollbreites Pfundzeichen"}}},{"category":"Sm","key":"FFE2","mappings":{"default":{"default":"vollbreite nicht Symbol"}}},{"category":"Sk","key":"FFE3","mappings":{"default":{"default":"vollbreiter Überstrich","alternative":"vollbreites Makron"}}},{"category":"So","key":"FFE4","mappings":{"default":{"default":"vollbreite gestrichelte Linie"}}},{"category":"Sc","key":"FFE5","mappings":{"default":{"default":"vollbreites Yen-Zeichen"}}},{"category":"Sc","key":"FFE6","mappings":{"default":{"default":"vollbreites Won-Zeichen"}}},{"category":"So","key":"FFE8","mappings":{"default":{"default":"Halbbreite bildet leicht vertikal"}}},{"category":"So","key":"FFED","mappings":{"default":{"default":"Schwarzes Quadrat mit halber Breite"}}},{"category":"So","key":"FFEE","mappings":{"default":{"default":"Weißer Kreis mit halber Breite"}}}]
,
"de/symbols/math_whitespace.js": 
[{"locale":"de"},{"category":"Zs","key":"0020","mappings":{"default":{"default":"Leerzeichen"}}},{"key":"00A0","mappings":{"default":{"default":" "}},"category":"Zs"},{"key":"00AD","mappings":{"default":{"default":"weicher Trennstrich"}},"category":"Cf"},{"category":"Zs","key":"2000","mappings":{"default":{"default":"En Quad"}}},{"category":"Zs","key":"2001","mappings":{"default":{"default":"Em Quad"}}},{"category":"Zs","key":"2002","mappings":{"default":{"default":"En Space"}}},{"key":"2003","mappings":{"default":{"default":""}},"category":"Zs"},{"category":"Zs","key":"2004","mappings":{"default":{"default":"Raum für drei Per-Em"}}},{"category":"Zs","key":"2005","mappings":{"default":{"default":"Raum für vier pro Em"}}},{"category":"Zs","key":"2006","mappings":{"default":{"default":"Six-Per-Em Space"}}},{"category":"Zs","key":"2007","mappings":{"default":{"default":"Raum darstellen"}}},{"category":"Zs","key":"2008","mappings":{"default":{"default":"Interpunktionsraum"}}},{"key":"2009","mappings":{"default":{"default":""}},"category":"Zs"},{"key":"200A","mappings":{"default":{"default":""}},"category":"Zs"},{"key":"200B","mappings":{"default":{"default":""}},"category":"Cf"},{"category":"Cf","key":"200C","mappings":{"default":{"default":"Nullbreite ohne Joiner"}}},{"category":"Cf","key":"200D","mappings":{"default":{"default":"Zero Width Joiner"}}},{"category":"Cf","key":"200E","mappings":{"default":{"default":"Markierung von links nach rechts"}}},{"category":"Cf","key":"200F","mappings":{"default":{"default":"Markierung von rechts nach links"}}},{"category":"Zl","key":"2028","mappings":{"default":{"default":"Trennzeichen"}}},{"category":"Zp","key":"2029","mappings":{"default":{"default":"Absatztrennzeichen"}}},{"category":"Cf","key":"202A","mappings":{"default":{"default":"Einbetten von links nach rechts"}}},{"category":"Cf","key":"202B","mappings":{"default":{"default":"Einbetten von rechts nach links"}}},{"category":"Cf","key":"202C","mappings":{"default":{"default":"Pop Richtungsformatierung"}}},{"category":"Cf","key":"202D","mappings":{"default":{"default":"Von links nach rechts überschreiben"}}},{"category":"Cf","key":"202E","mappings":{"default":{"default":"Von rechts nach links überschreiben"}}},{"category":"Zs","key":"202F","mappings":{"default":{"default":"Schmaler No-Break-Bereich"}}},{"key":"205F","mappings":{"default":{"default":""}},"category":"Zs"},{"category":"Cf","key":"2060","mappings":{"default":{"default":"Word Joiner"}}},{"key":"2061","mappings":{"default":{"default":"von"}},"category":"Cf"},{"key":"2062","mappings":{"default":{"default":" "}},"category":"Cf"},{"key":"2063","mappings":{"default":{"default":""}},"category":"Cf"},{"category":"Cf","key":"2064","mappings":{"default":{"default":"Unsichtbares Plus"}}},{"category":"Cf","key":"206A","mappings":{"default":{"default":"Symmetrisches Austauschen verhindern"}}},{"category":"Cf","key":"206B","mappings":{"default":{"default":"Aktivieren Sie das symmetrische Austauschen"}}},{"category":"Cf","key":"206E","mappings":{"default":{"default":"Nationale Ziffernformen"}}},{"category":"Cf","key":"206F","mappings":{"default":{"default":"Nominale Ziffernformen"}}},{"category":"Cf","key":"FEFF","mappings":{"default":{"default":"Nullbreite Leerzeichen"}}},{"category":"Cf","key":"FFF9","mappings":{"default":{"default":"Interlinearer Anmerkungsanker"}}},{"category":"Cf","key":"FFFA","mappings":{"default":{"default":"Separator für interlineare Anmerkungen"}}},{"category":"Cf","key":"FFFB","mappings":{"default":{"default":"Interlinear Annotation Terminator"}}}]
,
"de/symbols/other_stars.js": 
[{"locale":"de"},{"category":"So","key":"23E8","mappings":{"default":{"default":"Dezimalexponentensymbol"}}},{"category":"So","key":"2605","mappings":{"default":{"default":"Schwarzer Stern"}}},{"category":"So","key":"2606","mappings":{"default":{"default":"weißer Stern"}}},{"category":"So","key":"26AA","mappings":{"default":{"default":"Mittlerer weißer Kreis"}}},{"category":"So","key":"26AB","mappings":{"default":{"default":"Mittlerer schwarzer Kreis"}}},{"category":"So","key":"2705","mappings":{"default":{"default":"Weißes schweres Häkchen"}}},{"category":"So","key":"2713","mappings":{"default":{"default":"Häkchen"}}},{"category":"So","key":"2714","mappings":{"default":{"default":"Schweres Häkchen"}}},{"category":"So","key":"2715","mappings":{"default":{"default":"Multiplikation X"}}},{"category":"So","key":"2716","mappings":{"default":{"default":"Schwere Multiplikation X"}}},{"category":"So","key":"2717","mappings":{"default":{"default":"Stimmzettel X"}}},{"category":"So","key":"2718","mappings":{"default":{"default":"Schwerer Stimmzettel X"}}},{"category":"So","key":"271B","mappings":{"default":{"default":"Center Cross öffnen"}}},{"category":"So","key":"271C","mappings":{"default":{"default":"Schweres offenes Mittelkreuz"}}},{"category":"So","key":"2720","mappings":{"default":{"default":"Malteserkreuz"}}},{"category":"So","key":"2721","mappings":{"default":{"default":"Davidstern"}}},{"category":"So","key":"2722","mappings":{"default":{"default":"Vier Teardrop-Spoked Asterisk"}}},{"category":"So","key":"2723","mappings":{"default":{"default":"Vier Ballon-Speichen-Sternchen"}}},{"category":"So","key":"2724","mappings":{"default":{"default":"Schwere Vier-Ballon-Speichen-Sternchen"}}},{"category":"So","key":"2725","mappings":{"default":{"default":"Vier Club-Spoked Asterisk"}}},{"category":"So","key":"2726","mappings":{"default":{"default":"Schwarzer vierzackiger Stern"}}},{"category":"So","key":"2727","mappings":{"default":{"default":"Weißer vierzackiger Stern"}}},{"category":"So","key":"2728","mappings":{"default":{"default":"Funkelt"}}},{"category":"So","key":"2729","mappings":{"default":{"default":"Stress umrissener weißer Stern"}}},{"category":"So","key":"272A","mappings":{"default":{"default":"Eingekreister weißer Stern"}}},{"category":"So","key":"272B","mappings":{"default":{"default":"Open Center Black Star"}}},{"category":"So","key":"272C","mappings":{"default":{"default":"Schwarzer Center White Star"}}},{"category":"So","key":"272D","mappings":{"default":{"default":"Schwarzer Stern umrissen"}}},{"category":"So","key":"272E","mappings":{"default":{"default":"Schwerer umrissener schwarzer Stern"}}},{"category":"So","key":"272F","mappings":{"default":{"default":"Sternrad Stern"}}},{"category":"So","key":"2730","mappings":{"default":{"default":"Beschatteter weißer Stern"}}},{"category":"So","key":"2731","mappings":{"default":{"default":"Schwerer Stern"}}},{"category":"So","key":"2732","mappings":{"default":{"default":"Öffnen Sie Center Asterisk"}}},{"category":"So","key":"2733","mappings":{"default":{"default":"Acht Speichen-Sternchen"}}},{"category":"So","key":"2734","mappings":{"default":{"default":"Achtzackiger schwarzer Stern"}}},{"category":"So","key":"2735","mappings":{"default":{"default":"Achtzackiger Sternradstern"}}},{"category":"So","key":"2736","mappings":{"default":{"default":"Sechszackiger schwarzer Stern"}}},{"category":"So","key":"2739","mappings":{"default":{"default":"Zwölf schwarzer Stern"}}},{"category":"So","key":"273A","mappings":{"default":{"default":"Sechzehn Spitzen Sternchen"}}},{"category":"So","key":"273B","mappings":{"default":{"default":"Teardrop-Spoked Sternchen"}}},{"category":"So","key":"273C","mappings":{"default":{"default":"Open Center Teardrop-Spoked Sternchen"}}},{"category":"So","key":"273D","mappings":{"default":{"default":"Schwerer Tropfen-Speichen-Sternchen"}}},{"category":"So","key":"273E","mappings":{"default":{"default":"Sechs Petalled Schwarzweiss-Florette"}}},{"category":"So","key":"273F","mappings":{"default":{"default":"Schwarze Florette"}}},{"category":"So","key":"2740","mappings":{"default":{"default":"Weiße Florette"}}},{"category":"So","key":"2741","mappings":{"default":{"default":"Acht Petalled umrissene schwarze Florette"}}},{"category":"So","key":"2742","mappings":{"default":{"default":"Eingekreistes offenes Zentrum Achtstern"}}},{"category":"So","key":"2743","mappings":{"default":{"default":"Schweres, tropfenbesetztes Nadelrad-Sternchen"}}},{"category":"So","key":"2744","mappings":{"default":{"default":"Schneeflocke"}}},{"category":"So","key":"2745","mappings":{"default":{"default":"Feste Trifoliate-Schneeflocke"}}},{"category":"So","key":"2746","mappings":{"default":{"default":"Schwere Chevron-Schneeflocke"}}},{"category":"So","key":"2747","mappings":{"default":{"default":"Funkeln"}}},{"category":"So","key":"2748","mappings":{"default":{"default":"Schweres Funkeln"}}},{"category":"So","key":"2749","mappings":{"default":{"default":"Ballon-Speichen-Sternchen"}}},{"category":"So","key":"274A","mappings":{"default":{"default":"Acht Teardrop-Spoked Propeller Sternchen"}}},{"category":"So","key":"274B","mappings":{"default":{"default":"Schwere Acht Teardrop-Spoked Propeller Sternchen"}}},{"category":"So","key":"274C","mappings":{"default":{"default":"Cross Mark"}}},{"category":"So","key":"274D","mappings":{"default":{"default":"Beschatteter weißer Kreis"}}}]
,
"de/units/area.js": 
[{"locale":"de"},{"category":"area","mappings":{"default":{"default":"Quadrat"}},"key":"sq","names":["sq","sq."]},{"category":"area","mappings":{"default":{"default":"Quadratzoll"}},"key":"sq inch","names":["sq in","sq. in.","sq inch","sq. inch"]},{"category":"area","mappings":{"default":{"default":"Quadratrute","plural":"Quadratruten"}},"key":"sq rd","names":["sq rd","sq. rd."]},{"category":"area","mappings":{"default":{"default":"Quadratfuß"}},"key":"sq ft","names":["sq ft","sq. ft."]},{"category":"area","mappings":{"default":{"default":"Quadratyard","plural":"Quadratyards"}},"key":"sq yd","names":["sq yd","sq. yd."]},{"category":"area","mappings":{"default":{"default":"Quadratmeile","plural":"Quadratmeilen"}},"key":"sq mi","names":["sq mi","sq. mi."]},{"category":"area","mappings":{"default":{"default":"Morgen"}},"key":"acr","names":["ac","ac.","acr","acr."]},{"key":"ha","mappings":{"default":{"default":"Hektar"}},"category":"area","names":["ha"]}]
,
"de/units/currency.js": 
[{"locale":"de"},{"locale":"de"},{"key":"$","mappings":{"default":{"default":"Dollar"}},"category":"currency","names":["$","💲","＄","﹩","USD"]},{"key":"£","mappings":{"default":{"default":"Pfund"}},"category":"currency","names":["£","￡","GBP"]},{"key":"¥","mappings":{"default":{"default":"Yen"}},"category":"currency","names":["¥","￥","JPY"]},{"key":"€","mappings":{"default":{"default":"Euro"}},"category":"currency","names":["€","EUR"]},{"key":"₡","mappings":{"default":{"default":"Colon"}},"category":"currency","names":["₡","CRC"]},{"key":"₢","mappings":{"default":{"default":"Cruzeiro"}},"category":"currency","names":["₢"]},{"key":"₣","mappings":{"default":{"default":"Franc"}},"category":"currency","names":["₣"]},{"key":"₤","mappings":{"default":{"default":"Lira","plural":"Lire"}},"category":"currency","names":["₤"]},{"key":"₥","mappings":{"default":{"default":"Mill"}},"category":"currency","names":["₥"]},{"key":"₦","mappings":{"default":{"default":"Naira"}},"category":"currency","names":["₦","NGN"]},{"key":"₧","mappings":{"default":{"default":"Pesete","plural":"Peseten"}},"category":"currency","names":["₧"]},{"key":"₨","mappings":{"default":{"plural":"Rupies","default":"Rupie"}},"category":"currency","names":["₨","₹","INR","NPR","PKR","LKR"]},{"key":"₩","mappings":{"default":{"default":"Won"}},"category":"currency","names":["₩","￦","KRW"]},{"key":"₪","mappings":{"default":{"default":"Schekel"}},"category":"currency","names":["₪"]},{"key":"₫","mappings":{"default":{"default":"Dong"}},"category":"currency","names":["₫"]},{"key":"₭","mappings":{"default":{"default":"Kip"}},"category":"currency","names":["₭"]},{"key":"₮","mappings":{"default":{"default":"Tögrög"}},"category":"currency","names":["₮"]},{"key":"₯","mappings":{"default":{"plural":"Drachmen","default":"Drachme"}},"category":"currency","names":["₯"]},{"key":"₰","mappings":{"default":{"default":"Pfennig"}},"category":"currency","names":["₰"]},{"key":"₱","mappings":{"default":{"default":"Peso"}},"category":"currency","names":["₱"]},{"key":"₲","mappings":{"default":{"default":"Guarani"}},"category":"currency","names":["₲"]},{"key":"₳","mappings":{"default":{"default":"Austral"}},"category":"currency","names":["₳"]},{"key":"₴","mappings":{"default":{"plural":"Hrywen","default":"Hrywnja"}},"category":"currency","names":["₴","UAH"]},{"key":"₵","mappings":{"default":{"default":"Cedi"}},"category":"currency","names":["₵","GHS"]},{"key":"元","mappings":{"default":{"default":"Yuan"}},"category":"currency","names":["元"]},{"category":"currency","key":"¢","mappings":{"default":{"default":"Cent"}},"names":["￠","¢"]}]
,
"de/units/energy.js": 
[{"locale":"de"},{"key":"W","si":true,"mappings":{"default":{"default":"Watt"}},"category":"energy","names":["W","w"]},{"category":"energy","mappings":{"default":{"default":"Kilowattstunde","plural":"Kilowattstunden"}},"key":"kwh","names":["kwh","kWh"]},{"key":"J","si":true,"mappings":{"default":{"default":"Joule"}},"category":"energy","names":["J"]},{"key":"N","si":true,"mappings":{"default":{"default":"Newton"}},"category":"energy","names":["N"]},{"key":"A","si":true,"mappings":{"default":{"default":"Ampere"}},"category":"energy","names":["A"]},{"key":"V","si":true,"mappings":{"default":{"default":"Volt"}},"category":"energy","names":["V"]},{"key":"Ohm","mappings":{"default":{"default":"Ohm"}},"category":"energy","names":["Ohm","ohm"]},{"key":"Ω","si":true,"mappings":{"default":{"default":"Ohm"}},"category":"energy","names":["Ω"]}]
,
"de/units/length.js": 
[{"locale":"de"},{"key":"m","si":true,"mappings":{"default":{"default":"Meter"}},"category":"length","names":["m"]},{"key":"ft","mappings":{"default":{"default":"Fuß"}},"category":"length","names":["ft","ft."]},{"key":"in","mappings":{"default":{"default":"Zoll"}},"category":"length","names":["in","in."]},{"key":"mi","mappings":{"default":{"plural":"Meilen","default":"Meile"}},"category":"length","names":["mi","mi."]},{"key":"yd","mappings":{"default":{"plural":"Yards","default":"Yard"}},"category":"length","names":["yd","yd."]},{"key":"n.m.","names":["n.m."],"mappings":{"default":{"default":"nautische Meile","plural":"nautische Meilen"}},"category":"length"},{"category":"length","mappings":{"default":{"default":"Kettenglied","plural":"Kettenglieder"}},"key":"link","names":["li","li."]},{"category":"length","mappings":{"default":{"default":"Rute","plural":"Ruten"}},"key":"rod","names":["rd","rd."]},{"category":"length","mappings":{"default":{"default":"Kette","plural":"Ketten"}},"key":"chain","names":["ch","ch."]},{"category":"length","mappings":{"default":{"default":"Furchenlänge","plural":"Furchenlängen"}},"key":"furlong","names":["fur","fur."]}]
,
"de/units/memory.js": 
[{"locale":"de"},{"key":"b","mappings":{"default":{"plural":"Bits","default":"Bit"}},"category":"memory","names":["b"]},{"key":"B","si":true,"mappings":{"default":{"plural":"Bytes","default":"Byte"}},"category":"memory","names":["B"]},{"key":"KB","mappings":{"default":{"plural":"Kilobytes","default":"Kilobyte"}},"category":"memory","names":["KB"]}]
,
"de/units/other.js": 
[{"locale":"de"},{"category":"other","mappings":{"default":{"default":"Dutzend"}},"key":"doz","names":["doz","doz.","dz","dz."]}]
,
"de/units/speed.js": 
[{"locale":"de"},{"category":"speed","mappings":{"default":{"default":"Knoten"}},"key":"kt","names":["kt","kt."]},{"key":"mph","mappings":{"default":{"plural":"Meilen pro Stunde","default":"Meile pro Stunde"}},"category":"speed","names":["mph"]},{"key":"kmh","mappings":{"default":{"default":"Kilometer pro Stunde"}},"category":"speed","names":["kmh"]},{"key":"rpm","mappings":{"default":{"plural":"Umdrehungen pro Minute","default":"Umdrehung pro Minute"}},"category":"speed","names":["rpm"]}]
,
"de/units/temperature.js": 
[{"locale":"de"},{"key":"F","mappings":{"default":{"default":"Grad Fahrenheit"}},"category":"temperature","names":["F","F.","°F"]},{"key":"C","mappings":{"default":{"default":"Grad Celsius"}},"category":"temperature","names":["C","°C"]},{"key":"K","mappings":{"default":{"default":"Kelvin"}},"category":"temperature","names":["K","°K"]}]
,
"de/units/time.js": 
[{"locale":"de"},{"category":"time","si":true,"mappings":{"default":{"default":"Sekunde","plural":"Sekunden"}},"key":"s","names":["s"]},{"category":"time","mappings":{"default":{"default":"Sekunde","plural":"Sekunden"}},"key":"″","names":["″"]},{"category":"time","mappings":{"default":{"default":"Minute","plural":"Minuten"}},"key":"min","names":["min","′"]},{"category":"time","mappings":{"default":{"default":"Grad"}},"key":"°","names":["°"]},{"category":"time","mappings":{"default":{"default":"Stunde","plural":"Stunden"}},"key":"h","names":["h","hr"]}]
,
"de/units/volume.js": 
[{"locale":"de"},{"category":"volume","mappings":{"default":{"default":"Kubik"}},"key":"cu","names":["cu","cu."]},{"category":"volume","mappings":{"default":{"default":"Kubikinch"}},"key":"cu inch","names":["cu in","cu. in."]},{"category":"volume","mappings":{"default":{"default":"Kubikfuß"}},"key":"cu ft","names":["cu ft","cu. ft."]},{"category":"volume","mappings":{"default":{"default":"Kubikyard","plural":"Kubikyards"}},"key":"cu yd","names":["cu yd","cu. yd."]},{"key":"bbl","mappings":{"default":{"default":"Fass"}},"category":"volume","names":["bbl.","bbl"]},{"category":"volume","mappings":{"default":{"default":"Fluid ounce","plural":"Fluid ounces"}},"key":"fl. oz.","names":["fl. oz.","fl oz"]},{"key":"gal","mappings":{"default":{"plural":"Gallonen","default":"Gallone"}},"category":"volume","names":["gal","gal."]},{"key":"pt","mappings":{"default":{"default":"Pint"}},"category":"volume","names":["pt","pt."]},{"key":"qt","mappings":{"default":{"plural":"Quarts","default":"Quart"}},"category":"volume","names":["qt","qt."]},{"key":"tbsp","mappings":{"default":{"default":"Esslöffel"}},"category":"volume","names":["tbsp","tbsp.","Tbsp","Tbsp."]},{"key":"tsp","mappings":{"default":{"default":"Teelöffel"}},"category":"volume","names":["tsp","tsp."]},{"category":"volume","mappings":{"default":{"plural":"Flüssigdrachmen","default":"Flüssigdrachme"}},"key":"fluid dram","names":["fl dr","fl. dr."]},{"category":"volume","mappings":{"default":{"plural":"Tassen","default":"Tasse"}},"key":"cup","names":["cp","cp."]},{"category":"volume","mappings":{"default":{"default":"Kubikzentimeter"}},"key":"cc","names":["cc","ccm"]},{"category":"volume","si":true,"mappings":{"default":{"default":"Liter"}},"key":"l","names":["l"]}]
,
"de/units/weight.js": 
[{"locale":"de"},{"key":"lb","mappings":{"default":{"default":"Pfund"}},"category":"weight","names":["lb","lb."]},{"key":"oz","mappings":{"default":{"plural":"Unzen","default":"Unze"}},"category":"weight","names":["oz","oz."]},{"category":"weight","mappings":{"default":{"default":"Long ton"}},"key":"LT","names":["LT","L.T."]},{"key":"gr","mappings":{"default":{"default":"Gramm"}},"category":"weight","names":["gr"]},{"key":"g","si":true,"mappings":{"default":{"default":"Gramm"}},"category":"weight","names":["g"]},{"category":"weight","mappings":{"default":{"default":"Mikrogramm"}},"key":"mcg","names":["mcg"]},{"key":"t","mappings":{"default":{"plural":"Tonnen","default":"Tonne"}},"category":"weight","names":["t","T"]},{"category":"weight","mappings":{"default":{"plural":"Drachmen","default":"Drachme"}},"key":"dram","names":["dr","dr."]},{"category":"weight","mappings":{"default":{"default":"Stone"}},"key":"st","names":["st","st."]},{"category":"weight","mappings":{"default":{"default":"Viertelzentner"}},"key":"qtr","names":["qtr","qtr."]},{"category":"weight","mappings":{"default":{"default":"Zentner"}},"key":"cwt","names":["cwt","cwt.","Ztr.","Ctr."]}]
,
"de/rules/clearspeak_german.js": 
{"locale":"de","domain":"clearspeak","modality":"speech","rules":[["Rule","collapsed","default","[n] . (engine:modality=summary,grammar:collapsed); [t] \"kollabiert\"","self::*[@alternative]","not(contains(@grammar, \"collapsed\"))"],["Rule","direct-speech","default","[t] @ext-speech","self::*[@ext-speech]"],["Rule","stree","default","[n] ./*[1]","self::stree"],["Rule","unknown","default","[n] text()","self::unknown"],["Rule","protected","default","[n] text() (grammar:ignoreCaps=\"großes\")","self::number","contains(@grammar, \"protected\")"],["Rule","omit-empty","default","[p] (pause:\"short\")","self::empty"],["Rule","font","default","[t] @font (grammar:localFont); [n] self::* (grammar:ignoreFont=@font,pause:\"short\")","self::*","name(self::*)!=\"number\"","@font","not(contains(@grammar, \"ignoreFont\"))","@font!=\"normal\""],["SpecializedRule","font","default","Caps_SayCaps"],["Rule","font-number","default","[t] @font (grammar:localFontNumber); [n] . (grammar:ignoreFont=@font)","self::number","@font","not(contains(@grammar, \"ignoreFont\"))","@font!=\"normal\""],["Rule","font-identifier","default","[t] @font (grammar:localFont); [n] self::* (grammar:ignoreFont=@font,pause:\"short\")","self::identifier","string-length(text())=1 or string-length(text())=2","@font","@font=\"normal\"","not(contains(@grammar, \"ignoreFont\"))","@role!=\"unit\""],["SpecializedRule","font-identifier","default","Caps_SayCaps"],["Rule","omit-font","default","[n] self::* (grammar:ignoreFont=@font)","self::identifier","string-length(text())=1 or string-length(text())=2","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"italic\"","self::*"],["SpecializedRule","omit-font","default","Caps_SayCaps"],["Rule","font-double-struck","default","[n] . (grammar:ignoreFont=@font); [t] @font (grammar:localFont)","self::*","name(self::*)!=\"number\"","string-length(text())=1 or string-length(text())=2","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"double-struck\""],["SpecializedRule","font-double-struck","default","Caps_SayCaps"],["Rule","font-number-double-struck","default","[n] . (grammar:ignoreFont=@font); [t] @font (grammar:localFontNumber)","self::number","string-length(text())=1 or string-length(text())=2","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"double-struck\""],["Rule","text","default","[n] text()","self::text"],["Rule","capital","default","[n] text() (pitch:0.6,grammar:ignoreCaps=\"großes\")","self::identifier","@role=\"latinletter\" or @role=\"greekletter\" or @role=\"simple function\"","CQFisCapital"],["Rule","capital","Caps_SayCaps","[n] text()","self::identifier","@role=\"latinletter\" or @role=\"greekletter\"","CQFisCapital"],["Rule","capital","Caps_SayCaps","[p] (pause:\"short\"); [n] text()","self::identifier","@role=\"latinletter\" or @role=\"greekletter\"","CQFisCapital","preceding-sibling::*[1]","not(name(preceding-sibling::*[1])=\"function\")","not(contains(@grammar, \"angle\"))"],["Rule","capital","Caps_SayCaps","[n] text() (pause:\"short\")","self::identifier","@role=\"latinletter\" or @role=\"greekletter\"","CQFisCapital","following-sibling::*[1]"],["Rule","capital","Caps_SayCaps","[p] (pause:\"short\"); [n] text() (pause:\"short\")","self::identifier","@role=\"latinletter\" or @role=\"greekletter\"","CQFisCapital","preceding-sibling::*[1]","following-sibling::*[1]","not(name(preceding-sibling::*[1])=\"function\")","not(contains(@grammar, \"angle\"))"],["Rule","punctuation-lr","default","[p] (pause:\"short\"); [n] text() (pause:\"short\")","self::punctuation","@role=\"comma\""],["Rule","punctuation","default","[n] text()","self::punctuation","@role=\"comma\"","not(preceding-sibling::*[1]/children)","not(following-sibling::*[1]/children)"],["Rule","punctuation-l","default","[p] (pause:\"short\"); [n] text()","self::punctuation","@role=\"comma\"","not(following-sibling::*[1]/children)"],["Rule","punctuation-r","default","[n] text() (pause:\"short\")","self::punctuation","@role=\"comma\"","not(preceding-sibling::*[1]/children)"],["Rule","ellipsis","Ellipses_AndSoOn","[t] \"und so weiter\"","self::punctuation","@role=\"ellipsis\"","not(following-sibling::*[1])","not(preceding-sibling::*[last()][@role=\"ellipsis\"])"],["Rule","ellipsis","Ellipses_AndSoOn","[t] \"und so weiter bis\"","self::punctuation","@role=\"ellipsis\"","preceding-sibling::*[1]","following-sibling::*[1]"],["Rule","vbar-evaluated","default","[n] children/*[1] (pause:\"short\"); [t] \"ausgewertet für\"; [n] content/*[1]/children/*[2] (pause:\"short\")","self::punctuated","@role=\"endpunct\"","content/*[1][@role=\"vbar\"]","content/*[1][@embellished]","name(content/*[1])=\"subscript\""],["Rule","vbar-evaluated","default","[n] children/*[1] (pause:\"short\"); [t] \"ausgewertet für\"; [n] content/*[1]/children/*[2] (pause:\"short\"); [t] \"minus des gleichen Ausdrucks ausgewertet für\"; [n] content/*[1]/children/*[1]/children/*[2] (pause:\"short\")","self::punctuated","@role=\"endpunct\"","content/*[1][@role=\"vbar\"]","content/*[1][@embellished]","name(content/*[1])=\"superscript\"","name(content/*[1]/children/*[1])=\"subscript\""],["Rule","vbar-such-that","VerticalLine_SuchThat","[t] \"so dass\"","self::punctuation","@role=\"vbar\"","not(parent::*/parent::*[@embellished=\"punctuation\"])"],["Rule","vbar-divides","default","[t] \"teilt\"","self::punctuation","@role=\"vbar\"","not(parent::*/parent::*[@embellished=\"punctuation\"])","parent::*/parent::*[@role=\"sequence\"]"],["Rule","vbar-divides","VerticalLine_Divides","[t] \"teilt\"","self::punctuation","@role=\"vbar\"","not(parent::*/parent::*[@embellished=\"punctuation\"])"],["Rule","vbar-given","VerticalLine_Given","[t] \"für die gilt\"","self::punctuation","@role=\"vbar\"","not(parent::*/parent::*[@embellished=\"punctuation\"])"],["Rule","set-member","default","[t] \"in\"","self::operator","@role=\"set extended\"","text()=\"∈\" or text()=\"∊\""],["SpecializedRule","set-member","default","SetMemberSymbol_Element","[t] \"Element von\""],["SpecializedRule","set-member","default","SetMemberSymbol_Belongs","[t] \"gehört zu\""],["Rule","set-not-member","default","[t] \"nicht in\"","self::operator","@role=\"set extended\"","text()=\"∉\""],["SpecializedRule","set-not-member","default","SetMemberSymbol_Element","[t] \"kein Element von\""],["SpecializedRule","set-not-member","default","SetMemberSymbol_Belongs","[t] \"gehört nicht zu\""],["Rule","prime","default","[n] children/*[1]; [n] children/*[2]","self::superscript","children/*[2]","children/*[2][@role=\"prime\"]","self::*"],["Rule","degrees","default","[m] children/* (grammar:degree)","self::punctuated[@role=\"sequence\"]","content/*[1][@role=\"degree\"]"],["Rule","feet","default","[n] children/*[1]; [t] \"ft\" (grammar:annotation=\"unit\":translate:plural)","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"number\"","children/*[2][text()=\"′\"]","not(contains(@grammar, \"degree\"))"],["Rule","foot","default","[n] children/*[1]; [t] \"ft\" (grammar:annotation=\"unit\":translate)","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"number\"","children/*[2][text()=\"′\"]","children/*[1][text()=\"1\"]","not(contains(@grammar, \"degree\"))"],["Rule","inches","default","[n] children/*[1]; [t] \"in\" (grammar:annotation=\"unit\":translate:plural)","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"number\"","children/*[2][text()=\"″\"]","not(contains(@grammar, \"degree\"))"],["Rule","inch","default","[n] children/*[1]; [t] \"in\" (grammar:annotation=\"unit\":translate)","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"number\"","children/*[2][text()=\"″\"]","children/*[1][text()=\"1\"]","not(contains(@grammar, \"degree\"))"],["Rule","minutes","default","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate:plural)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"′\"]","contains(@grammar, \"degree\")"],["Rule","minute","default","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"′\"]","contains(@grammar, \"degree\")","children/*[1][text()=\"1\"]"],["Rule","seconds","default","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate:plural)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"″\"]","contains(@grammar, \"degree\")"],["Rule","second","default","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"″\"]","contains(@grammar, \"degree\")","children/*[1][text()=\"1\"]"],["Rule","degrees-angle","default","[t] text() (grammar:annotation=\"unit\":translate:plural, pause:short)","self::punctuation","@role=\"degree\""],["Rule","degree-angle","default","[t] text() (grammar:annotation=\"unit\":translate, pause:short)","self::punctuation","@role=\"degree\"","preceding-sibling::*[text()=\"1\"]"],["Rule","minutes-angle","Prime_Angle","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate:plural)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"′\"]","not(contains(@grammar, \"degree\"))","name(children/*[1])=\"number\" or (children/*[1][@role=\"latinletter\"] and \"\"=translate(children/*[1]/text(),\"abcdefghijklmnopqrstuvwxyz\", \"\"))"],["Rule","minute-angle","Prime_Angle","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"′\"]","not(contains(@grammar, \"degree\"))","children/*[1][text()=\"1\"]"],["Rule","seconds-angle","Prime_Angle","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate:plural)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"″\"]","not(contains(@grammar, \"degree\"))","name(children/*[1])=\"number\" or (children/*[1][@role=\"latinletter\"] and \"\"=translate(children/*[1]/text(),\"abcdefghijklmnopqrstuvwxyz\", \"\"))"],["Rule","second-angle","Prime_Angle","[n] children/*[1]; [t] children/*[2]/text() (grammar:annotation=\"unit\":translate)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"″\"]","not(contains(@grammar, \"degree\"))","children/*[1][text()=\"1\"]"],["Rule","feet-length","Prime_Length","[n] children/*[1]; [t] \"ft\" (grammar:annotation=\"unit\":translate:plural, pause:short)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"′\"]","not(contains(@grammar, \"degree\"))","name(children/*[1])=\"number\" or (children/*[1][@role=\"latinletter\"] and \"\"=translate(children/*[1]/text(),\"abcdefghijklmnopqrstuvwxyz\", \"\"))"],["Rule","foot-length","Prime_Length","[n] children/*[1]; [t] \"ft\" (grammar:annotation=\"unit\":translate, pause:short)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"′\"]","not(contains(@grammar, \"degree\"))","children/*[1][text()=\"1\"]"],["Rule","inches-length","Prime_Length","[n] children/*[1]; ; [t] \"in\" (grammar:annotation=\"unit\":translate:plural, pause:short)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"″\"]","not(contains(@grammar, \"degree\"))","name(children/*[1])=\"number\" or (children/*[1][@role=\"latinletter\"] and \"\"=translate(children/*[1]/text(),\"abcdefghijklmnopqrstuvwxyz\", \"\"))"],["Rule","inch-length","Prime_Length","[n] children/*[1]; ; [t] \"in\" (grammar:annotation=\"unit\":translate, pause:short)","self::superscript","children/*[2][@role=\"prime\"]","children/*[2][text()=\"″\"]","not(contains(@grammar, \"degree\"))","children/*[1][text()=\"1\"]"],["Rule","punctuated","default","[m] children/*","self::punctuated"],["Rule","function","default","[n] text()","self::function"],["Rule","function-article","default","[t] \"der\" (grammar:article); [n] text()","self::function","@role=\"prefix function\"","contains(@grammar, \"addArticle\")"],["Rule","function-article-fem","default","[t] \"die\" (grammar:article); [n] text()","self::function","@role=\"prefix function\"","contains(@grammar, \"addArticle\")","text()=\"det\" or text()=\"dim\" or text()=\"tr\""],["Rule","appl","default","[n] children/*[1]; [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl"],["Rule","appl-simple","default","[n] children/*[1]; [t] \"von\" (pause:\"short\"); [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"simple function\"","name(children/*[2])=\"appl\""],["Rule","appl-simple","default","[n] children/*[1]; [t] \"von\" (pause:\"short\"); [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"simple function\"","name(children/*[2])=\"fenced\"","name(children/*[2]/children/*[1])=\"appl\""],["Rule","appl","Functions_None","[p] (pause:\"short\"); [n] children/*[1]; [t] \"mal\"; [n] children/*[2] (pause:\"short\")","self::appl"],["Rule","function-prefix","default","[n] children/*[1]; [n] children/*[2]","self::appl","@role=\"prefix function\""],["Rule","binary-operation","ImpliedTimes_MoreImpliedTimes","[n] . (grammar:impliedTimes, pause:\"short\")","self::appl","@role=\"prefix function\"","parent::*/parent::infixop[@role=\"implicit\"]","following-sibling::*","not(contains(@grammar, \"impliedTimes\"))"],["Rule","function-prefix-simple-arg","default","[n] children/*[1]; [n] children/*[2]","self::appl","@role=\"prefix function\"","name(children/*[2])=\"fenced\"","contains(children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\")","name(children/*[2]/children/*[1])!=\"number\"","name(children/*[2]/children/*[1])!=\"identifier\"","name(children/*[2]/children/*[1])!=\"appl\""],["Rule","function-prefix-embell","default","[p] (pause:\"short\"); [n] children/*[1]; [n] children/*[2] (pause:\"short\"); ","self::appl","@role=\"prefix function\"","name(children/*[1])!=\"function\""],["Rule","function-prefix-fenced-or-frac-arg","default","[p] (pause:\"short\"); [n] children/*[1] (grammar:addArticle); [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","(name(children/*[2])=\"fenced\" and not(contains(children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\"))) or name(children/*[2])=\"fraction\" or (name(children/*[2])!=\"fenced\" and not(contains(children/*[2]/@annotation, \"clearspeak:simple\")))","self::*"],["Rule","function-prefix-subscript","default","[p] (pause:\"short\"); [n] children/*[1] (grammar:addArticle); [t] \"von\" (pause:\"short\"); [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","name(children/*[1])=\"subscript\"","self::*"],["Rule","function-ln","default","[n] children/*[1]; [n] children/*[2]","self::appl","@role=\"prefix function\"","content/*[2][text()=\"ln\"]","not(following-sibling::*)","not(contains(@grammar, \"NatLog\"))"],["Rule","function-ln","default","[n] children/*[1]; [n] children/*[2] (pause:\"short\")","self::appl","@role=\"prefix function\"","content/*[2][text()=\"ln\"]","not(contains(@grammar, \"NatLog\"))"],["Rule","function-ln","default","[n] children/*[1]; [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","content/*[2][text()=\"ln\"]","name(children/*[2])=\"fenced\"","not(contains(@grammar, \"NatLog\"))"],["Rule","function-ln","Log_LnAsNaturalLog","[n] . (grammar:NatLog)","self::appl","@role=\"prefix function\"","content/*[2][text()=\"ln\"]","not(following-sibling::*)","not(contains(@grammar, \"NatLog\"))"],["Rule","function-ln","Log_LnAsNaturalLog","[n] . (grammar:NatLog, pause:\"short\")","self::appl","@role=\"prefix function\"","content/*[2][text()=\"ln\"]","not(contains(@grammar, \"NatLog\"))"],["Rule","function-prefix-as-exp","default","[n] children/*[1]; [t] \"von\" (pause:\"short\"); [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","name(parent::*/parent::*)=\"superscript\"","not(following-sibling::*)","(name(children/*[2])=\"fenced\" and not(contains(children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\"))) or name(children/*[2])=\"fraction\" or (name(children/*[2])!=\"fenced\" and not(contains(children/*[2]/@annotation, \"clearspeak:simple\")))"],["Rule","function-prefix-subscript-as-exp","default","[n] children/*[1]; [t] \"von\" (pause:\"short\"); [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","name(parent::*/parent::*)=\"superscript\"","not(following-sibling::*)","name(children/*[1])=\"subscript\""],["Rule","function-prefix-hyper","default","[p] (pause:\"short\"); [n] children/*[1]; [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","CQFisHyperbolic"],["Rule","function-prefix-inverse","default","[p] (pause:\"short\"); [t] \"der\" (grammar:article); [t] \"inverse\" (grammar:masculine); [n] children/*[1]/children/*[1]; [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","name(children/*[1])=\"superscript\"","name(children/*[1]/children/*[2])=\"prefixop\"","children/*[1]/children/*[2][@role=\"negative\"]","children/*[1]/children/*[2]/children/*[1][text()=\"1\"]","not(contains(@grammar, \"functions_none\"))"],["Rule","appl-triginverse","Trig_TrigInverse","[p] (pause:\"short\"); [n] children/*[1]; [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","name(children/*[1])=\"superscript\"","name(children/*[1]/children/*[2])=\"prefixop\"","children/*[1]/children/*[2][@role=\"negative\"]","children/*[1]/children/*[2]/children/*[1][text()=\"1\"]"],["Rule","function-prefix-arc-simple","Trig_ArcTrig","[p] (pause:\"short\"); [t] \"Arkus\" (join:\"\"); [n] children/*[1]/children/*[1] (grammar:lowercase); [n] children/*[2] (pause:\"short\")","self::appl","@role=\"prefix function\"","name(children/*[1])=\"superscript\"","name(children/*[1]/children/*[2])=\"prefixop\"","children/*[1]/children/*[2][@role=\"negative\"]","children/*[1]/children/*[2]/children/*[1][text()=\"1\"]","not(contains(@grammar, \"functions_none\"))"],["Rule","function-prefix-arc-simple","Trig_ArcTrig","[p] (pause:\"short\"); [t] \"Arkus\" (join:\"\"); [n] children/*[1]/children/*[1] (grammar:lowercase, pause:\"short\"); [n] children/*[2] (pause:\"short\")","self::appl","@role=\"prefix function\"","name(children/*[1])=\"superscript\"","name(children/*[1]/children/*[2])=\"prefixop\"","children/*[1]/children/*[2][@role=\"negative\"]","children/*[1]/children/*[2]/children/*[1][text()=\"1\"]","name(children/*[2])=\"fenced\"","children/*[2]/children/*[1][@role=\"prefix function\"]","contains(children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\")","not(contains(@grammar, \"functions_none\"))"],["Rule","function-prefix-arc","Trig_ArcTrig","[p] (pause:\"short\"); [t] \"Arkus\" (join:\"\"); [n] children/*[1]/children/*[1] (grammar:lowercase); [t] \"von\"; [n] children/*[2] (grammar:case=\"dative\", pause:\"short\")","self::appl","@role=\"prefix function\"","name(children/*[1])=\"superscript\"","name(children/*[1]/children/*[2])=\"prefixop\"","children/*[1]/children/*[2][@role=\"negative\"]","children/*[1]/children/*[2]/children/*[1][text()=\"1\"]","not(contains(@grammar, \"functions_none\"))","(name(children/*[2])=\"fenced\" and not(contains(children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\"))) or (name(children/*[2])=\"fraction\" and children/*[2][@role!=\"vulgar\"])"],["Rule","function-inverse","default","[n] children/*[1]; [t] \"invers\"","self::superscript","@role=\"prefix function\" or @role=\"simple function\"","name(children/*[2])=\"prefixop\"","children/*[2][@role=\"negative\"]","children/*[2]/children/*[1][text()=\"1\"]","not(contains(@grammar, \"functions_none\"))"],["Rule","superscript-prefix-function","default","[t] \"die\" (grammar:article); [n] children/*[2] (grammar:ordinal); [t] \"Potenz von\"; [n] children/*[1]","self::superscript","@role=\"prefix function\"","name(children/*[2])=\"number\"","children/*[2][@role=\"integer\"]"],["Rule","superscript-prefix-function","default","[t] \"die\" (grammar:article); [n] children/*[2] (grammar:ordinal); [t] \"Potenz von\"; [n] children/*[1]","self::superscript","@role=\"prefix function\"","name(children/*[2])=\"identifier\""],["Rule","function-inverse","Functions_None","[n] . (grammar:functions_none)","self::superscript","@role=\"prefix function\" or @role=\"simple function\"","name(children/*[2])=\"prefixop\"","children/*[2][@role=\"negative\"]","children/*[2]/children/*[1][text()=\"1\"]","not(contains(@grammar, \"functions_none\"))"],["Rule","superscript","default","[n] children/*[1]; [t] \"mit Exponent\" (pause:\"short\"); [n] children/*[2] (pause:\"short\"); [t] \"Ende Exponent\" (pause:\"short\")","self::superscript"],["Rule","superscript-ordinal","default","[n] children/*[1]; [t] \"hoch\"; [n] children/*[2] (pause:\"short\")","self::superscript","name(children/*[2])=\"number\"","children/*[2][@role=\"integer\"]"],["Rule","superscript-ordinal","default","[n] children/*[1]; [t] \"hoch\"; [n] children/*[2] (pause:\"short\")","self::superscript","name(children/*[2])=\"prefixop\"","children/*[2][@role=\"negative\"]","name(children/*[2]/children/*[1])=\"number\"","children/*[2]/children/*[1][@role=\"integer\"]"],["Rule","superscript-ordinal","default","[n] children/*[1]; [t] \"hoch\"; [n] children/*[2] (pause:\"short\")","self::superscript","name(children/*[2])=\"identifier\"","children/*[2][@role=\"latinletter\" or @role=\"greekletter\" or @role=\"otherletter\"]"],["Rule","superscript-ordinal-default","default","[n] children/*[1]; [t] \"mit Exponent\" (pause:\"short\"); [n] children/*[2] (pause:\"short\"); [t] \"Ende Exponent\" (pause:\"short\")","self::superscript","children//superscript"],["Rule","superscript-simple-exponent","default","[n] children/*[1]; [t] \"hoch\"; [n] children/*[2] (pause:\"short\")","self::superscript","not(descendant::superscript)"],["Rule","superscript-simple-exponent-end","default","[n] children/*[1]; [t] \"hoch\"; [n] children/*[2]; ","self::superscript","not(descendant::superscript)","not(following-sibling::*)"],["Aliases","superscript-simple-exponent","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"number\"","contains(children/superscript/children/*[1]/@annotation, \"clearspeak:simple\")"],["Aliases","superscript-simple-exponent","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"fraction\"","contains(children/superscript/children/*[1]/@annotation, \"clearspeak:simple\")"],["Aliases","superscript-simple-exponent","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"identifier\""],["Aliases","superscript-simple-exponent","self::superscript","children/*[2][@role=\"implicit\"]","count(children/*[2]/children/*)=2","contains(children/*[2]/children/*[1]/@annotation, \"simple\")","name(children/*[2]/children/*[2])=\"superscript\"","(name(children/*[2]/children/*[2]/children/*[1])=\"number\" and contains(children/*[2]/children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\")) or name(children/*[2]/children/*[2]/children/*[1])=\"identifier\"","children/*[2]/children/*[2]/children/*[2][text()=\"2\"] or children/*[2]/children/*[2]/children/*[2][text()=\"3\"]"],["Rule","superscript-simple-power","Exponent_OrdinalPower","[n] children/*[1]; [t] \"potenziert mit\"; [n] children/*[2] (pause:\"short\")","self::superscript","not(descendant::superscript)"],["Rule","superscript-simple-power-end","Exponent_OrdinalPower","[n] children/*[1]; [t] \"potenziert mit\"; [n] children/*[2]; ","self::superscript","not(descendant::superscript)","not(following-sibling::*)"],["Aliases","superscript-simple-power","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"number\"","contains(children/superscript/children/*[1]/@annotation, \"clearspeak:simple\")"],["Aliases","superscript-simple-power","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"fraction\"","contains(children/superscript/children/*[1]/@annotation, \"clearspeak:simple\")"],["Aliases","superscript-simple-power","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"identifier\""],["Aliases","superscript-simple-power","self::superscript","children/*[2][@role=\"implicit\"]","count(children/*[2]/children/*)=2","contains(children/*[2]/children/*[1]/@annotation, \"simple\")","name(children/*[2]/children/*[2])=\"superscript\"","(name(children/*[2]/children/*[2]/children/*[1])=\"number\" and contains(children/*[2]/children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\")) or name(children/*[2]/children/*[2]/children/*[1])=\"identifier\"","children/*[2]/children/*[2]/children/*[2][text()=\"2\"] or children/*[2]/children/*[2]/children/*[2][text()=\"3\"]"],["Rule","superscript-ordinal-power","Exponent_OrdinalPower","[n] children/*[1]; [t] \"zur\"; [n] children/*[2] (grammar:ordinal,join:\"\"); [t] \"n Potenz\" (pause:\"short\")","self::superscript","name(children/*[2])=\"number\"","children/*[2][@role=\"integer\"]"],["Aliases","superscript-ordinal-power","self::superscript","name(children/*[2])=\"identifier\"","children/*[2][@role=\"latinletter\" or @role=\"greekletter\" or @role=\"otherletter\"]"],["Rule","superscript-non-ordinal","Exponent_OrdinalPower","[n] children/*[1]; [t] \"zur negativ\"; [n] children/*[2]/children/*[1] (grammar:ordinal, join:\"\"); [t] \"n Potenz\" (pause:\"short\")","self::superscript","children/*[2][@role=\"negative\"]","name(children/*[2]/children/*[1])=\"number\"","children/*[2]/children/*[1][@role=\"integer\"]"],["Rule","superscript-simple-function","Exponent_OrdinalPower","[t] \"die\" (grammar:article); [n] children/*[2] (grammar:ordinal); [t] \"Potenz von\" (pause:\"short\"); [n] children/*[1]","self::superscript","name(children/*[1])=\"identifier\"","children/*[1][@role=\"simple function\"]","children/*[2][@role!=\"prime\"]","not(contains(@grammar, \"functions_none\"))"],["Rule","exponent","default","[n] text() (join:\"\"); [t] \"te\"","self::identifier","contains(@grammar, \"ordinal\")"],["Rule","exponent","default","[t] CSFwordOrdinal","self::number","@role=\"integer\"","contains(@grammar, \"ordinal\")","text()!=\"0\""],["Rule","exponent","default","[t] \"nullte\"","self::number","@role=\"integer\"","contains(@grammar, \"ordinal\")","text()=\"0\""],["Rule","superscript-simple-exp","Exponent_Exponent","[n] children/*[1]; [t] \"mit Exponent\"; [n] children/*[2] (pause:\"short\")","self::superscript","not(descendant::superscript)"],["Rule","superscript-simple-exp-end","Exponent_Exponent","[n] children/*[1]; [t] \"mit Exponent\"; [n] children/*[2]; ","self::superscript","not(descendant::superscript)","not(following-sibling::*)"],["Aliases","superscript-simple-exp","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"number\"","contains(children/superscript/children/*[1]/@annotation, \"clearspeak:simple\")"],["Aliases","superscript-simple-exp","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"fraction\"","contains(children/superscript/children/*[1]/@annotation, \"clearspeak:simple\")"],["Aliases","superscript-simple-exp","self::superscript","children/superscript/children/*[2][text()=\"2\"] or children/superscript/children/*[2][text()=\"3\"]","name(children/superscript/children/*[1])=\"identifier\""],["Aliases","superscript-simple-exp","self::superscript","children/*[2][@role=\"implicit\"]","count(children/*[2]/children/*)=2","contains(children/*[2]/children/*[1]/@annotation, \"simple\")","name(children/*[2]/children/*[2])=\"superscript\"","(name(children/*[2]/children/*[2]/children/*[1])=\"number\" and contains(children/*[2]/children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\")) or name(children/*[2]/children/*[2]/children/*[1])=\"identifier\"","children/*[2]/children/*[2]/children/*[2][text()=\"2\"] or children/*[2]/children/*[2]/children/*[2][text()=\"3\"]"],["Rule","superscript-simple-function","Functions_None","[n] . (grammar:functions_none)","self::superscript","name(children/*[1])=\"identifier\"","children/*[1][@role=\"simple function\"]","not(contains(@grammar, \"functions_none\"))"],["Rule","square","default","[n] children/*[1]; [t] \"Quadrat\"","self::superscript","children/*[2][text()=\"2\"]","name(children/*[1])!=\"text\" or not(name(children/*[1])=\"text\" and (name(../../../punctuated[@role=\"text\"]/..)=\"stree\" or name(..)=\"stree\"))","self::*","self::*"],["Rule","cube","default","[n] children/*[1]; [t] \"Kubik\"","self::superscript","children/*[2][text()=\"3\"]","name(children/*[1])!=\"text\" or not(name(children/*[1])=\"text\" and (name(../../../punctuated[@role=\"text\"]/..)=\"stree\" or name(..)=\"stree\"))","self::*","self::*"],["Rule","paren-simple","default","[n] children/*[1]","self::fenced","@role=\"leftright\"","contains(children/*[1]/@annotation, \"clearspeak:simple\")","name(../..)!=\"superscript\" and name(../..)!=\"subscript\""],["Rule","paren-simple-exp","default","[n] children/*[1]","self::fenced","@role=\"leftright\"","name(../..)=\"superscript\"","children/*[1][@role=\"integer\"] or children/*[1][@role=\"float\"] or (children/*[1][@role=\"vulgar\"] and contains(children/*[1]/@annotation, \"clearspeak:simple\")) or children/*[1][@role=\"latinletter\"] or children/*[1][@role=\"greekletter\"] or children/*[1][@role=\"otherletter\"]"],["Rule","paren-simple-nested-func","default","[n] children/*[1]","self::fenced","@role=\"leftright\"","name(../*[1])=\"identifier\" or name(../*[1])=\"function\"","parent::*/parent::*[@role=\"simple function\" or @role=\"prefix function\"]","children/*[1][@role=\"simple function\" or @role=\"prefix function\"]","contains(children/*[1]/children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\") or name(children/*[1]/children/*[2]/children/*[1])=\"subscript\" or name(children/*[1]/children/*[2]/children/*[1])=\"superscript\" or children/*[1]/children/*[2]/children/*[1][@role=\"vulgar\"] "],["Rule","paren-simple-nested-func-no-bracket","Functions_None","[n] children/*[1];","self::fenced","@role=\"leftright\"","name(../*[1])=\"identifier\" or name(../*[1])=\"function\"","parent::*/parent::*[@role=\"simple function\" or @role=\"prefix function\"]","children/*[1][@role=\"simple function\" or @role=\"prefix function\"]","name(children/*[1]/children/*[1])=\"identifier\" or name(children/*[1]/children/*[1])=\"function\"","contains(children/*[1]/children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\")","name(children/*[1]/children/*[2]/children/*[1])=\"identifier\" or name(children/*[1]/children/*[2]/children/*[1])=\"number\""],["Rule","fences-open-close","default","[p] (pause:\"short\"); [n] content/*[1] (grammar:spokenFence, pause:\"short\"); [n] children/*[1] (pause:\"short\"); [n] content/*[2] (grammar:spokenFence, pause:\"short\")","self::fenced","@role=\"leftright\""],["Rule","paren-simple-nested-func","default","[p] (pause:\"short\"); [n] content/*[1] (pause:\"short\"); [n] children/*[1] (pause:\"short\"); [n] content/*[2] (pause:\"short\")","self::fenced","@role=\"leftright\"","name(../*[1])=\"identifier\" or name(../*[1])=\"function\"","parent::*/parent::*[@role=\"simple function\" or @role=\"prefix function\"]","not(contains(children/*[1]/@annotation, \"clearspeak:simple\"))"],["Rule","paren-simple-nested-func","Functions_None","[p] (pause:\"short\"); [n] content/*[1] (grammar:spokenFence, pause:\"short\"); [n] children/*[1] (pause:\"short\"); [n] content/*[2] (grammar:spokenFence, pause:\"short\")","self::fenced","@role=\"leftright\"","name(../*[1])=\"identifier\" or name(../*[1])=\"function\"","parent::*/parent::*[@role=\"simple function\" or @role=\"prefix function\"]","children/*[1][@role=\"simple function\" or @role=\"prefix function\"]","contains(children/*[1]/children/*[2]/children/*[1]/@annotation, \"clearspeak:simple\") or name(children/*[1]/children/*[2]/children/*[1])=\"subscript\" or name(children/*[1]/children/*[2]/children/*[1])=\"superscript\" or children/*[1]/children/*[2]/children/*[1][@role=\"vulgar\"] "],["SpecializedRule","fences-open-close","default","Paren_Speak"],["Aliases","fences-open-close","self::fenced","@role=\"composed function\""],["Rule","fence-silent","Paren_Silent","[p] (pause:\"short\"); [n] children/*[1] (pause:\"short\")","self::fenced"],["Rule","fences-open-close","ImpliedTimes_None","[p] (pause:\"short\"); [n] content/*[1] (grammar:spokenFence, pause:\"short\"); [n] children/*[1] (pause:\"short\"); [n] content/*[2] (grammar:spokenFence, pause:\"short\")","self::fenced","@role=\"leftright\"","parent::*/parent::*[@role!=\"simple function\"]","parent::*/parent::*[@role!=\"prefix function\"]"],["Rule","fence-nesting","Paren_SpeakNestingLevel","[n] text() (grammar:insertNesting=CSFnestingDepth)","self::fence","contains(@grammar, \"spokenFence\")","CQFmatchingFences"],["Rule","fence-no-nesting","Paren_SpeakNestingLevel","[n] text()","self::fence"],["Rule","fences-points","Paren_CoordPoint","[t] \"der Punkt mit Koordinaten\"; [n] children/*[1]","self::fenced","name(children/*[1])=\"punctuated\"","children/*[1][@role=\"sequence\"]"],["Rule","fences-interval","Paren_Interval","[t] \"das Interval von\"; [n] children/*[1]/children/*[1]; [t] \"bis\"; [n] children/*[1]/children/*[3] (pause:\"short\"); [n] . (grammar:interval)","self::fenced","not(contains(@grammar, \"interval\"))","name(children/*[1])=\"punctuated\"","children/*[1][@role=\"sequence\"]","count(./children/*[1]/content/*)=1","children/*[1]/content/*[1][@role=\"comma\"]"],["Rule","interval-open","Paren_Interval","[t] \"ohne\"; [n] children/*[1]/children/*[1]; [t] \"und\"; [n] children/*[1]/children/*[3]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"(\"","content/*[2]/text()=\")\""],["Rule","interval-closed-open","Paren_Interval","[t] \"einschließlich\"; [n] children/*[1]/children/*[1] (pause:\"short\"); [t] \"aber ohne\"; [n] children/*[1]/children/*[3]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"[\"","content/*[2]/text()=\")\""],["Rule","interval-open-closed","Paren_Interval","[t] \"ohne\"; [n] children/*[1]/children/*[1] (pause:\"short\"); [t] \"aber einschließlich\"; [n] children/*[1]/children/*[3]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"(\"","content/*[2]/text()=\"]\""],["Rule","interval-closed","Paren_Interval","[t] \"einschließlich\"; [n] children/*[1]/children/*[1]; [t] \"und\"; [n] children/*[1]/children/*[3]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"[\"","content/*[2]/text()=\"]\""],["Rule","interval-open-inf-r","Paren_Interval","[t] \"ohne\"; [n] children/*[1]/children/*[1]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"(\"","content/*[2]/text()=\")\"","children/*[1]/children/*[3]/text()=\"∞\" or (name(children/*[1]/children/*[3])=\"prefixop\" and children/*[1]/children/*[3]/children/*[1]/text()=\"∞\")"],["Rule","interval-open-inf-l","Paren_Interval","[t] \"ohne\"; [n] children/*[1]/children/*[3]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"(\"","content/*[2]/text()=\")\"","children/*[1]/children/*[1]/text()=\"∞\" or (name(children/*[1]/children/*[1])=\"prefixop\" and children/*[1]/children/*[1]/children/*[1]/text()=\"∞\")"],["Rule","interval-open-inf-lr","Paren_Interval","","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"(\"","content/*[2]/text()=\")\"","children/*[1]/children/*[3]/text()=\"∞\" or (name(children/*[1]/children/*[3])=\"prefixop\" and children/*[1]/children/*[3]/children/*[1]/text()=\"∞\")","children/*[1]/children/*[1]/text()=\"∞\" or (name(children/*[1]/children/*[1])=\"prefixop\" and children/*[1]/children/*[1]/children/*[1]/text()=\"∞\")"],["Rule","interval-closed-open-inf","Paren_Interval","[t] \"einschließlich\"; [n] children/*[1]/children/*[1]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"[\"","content/*[2]/text()=\")\"","children/*[1]/children/*[3]/text()=\"∞\" or (name(children/*[1]/children/*[3])=\"prefixop\" and children/*[1]/children/*[3]/children/*[1]/text()=\"∞\")"],["Rule","interval-open-closed-inf","Paren_Interval","[t] \"einschließlich\"; [n] children/*[1]/children/*[3]","self::fenced","contains(@grammar, \"interval\")","content/*[1]/text()=\"(\"","content/*[2]/text()=\"]\"","children/*[1]/children/*[1]/text()=\"∞\" or (name(children/*[1]/children/*[1])=\"prefixop\" and children/*[1]/children/*[1]/children/*[1]/text()=\"∞\")"],["Rule","paren-nested-embellished-funcs","Functions_None","[p] (pause:\"short\"); [n] content/*[1] (pause:\"short\"); [n] children/*[1] (pause:\"short\"); [n] content/*[2] (pause:\"short\")","self::fenced","@role=\"leftright\"","name(../..)=\"appl\"","name(children/*[1]) = \"appl\"","preceding-sibling::*/descendant-or-self::*[@role=\"subsup\"] or children/*[1]/descendant-or-self::*[@role=\"subsup\"]"],["Rule","set-empty","default","[t]  \"die leere Menge\"","self::fenced","@role=\"set empty\""],["Rule","set-extended","default","[t] \"die Menge aller\"; [n] children/*[1]/children/*[1]; [t] \"mit\"; [n] children/*[1]/children/*[3]","self::fenced","@role=\"set extended\""],["Rule","set-collection","default","[t] \"die Menge\"; [n] children/*[1]","self::fenced","@role=\"set collection\""],["Aliases","set-collection","self::fenced","@role=\"set singleton\""],["Rule","set-extended","Sets_woAll","[t] \"die Menge von\"; [n] children/*[1]/children/*[1]; [t] \"mit\"; [n] children/*[1]/children/*[3]","self::fenced","@role=\"set extended\""],["Rule","set-collection","Sets_SilentBracket","[n] children/*[1]","self::fenced","@role=\"set collection\""],["Rule","subscript","default","[p] (pause:short); [n] children/*[1]; [t] \"Index\"; [n] children/*[2] (pause:short)","self::subscript"],["Rule","subscript-base","default","[n] children/*[1]; [t] \"Basis\"; [n] children/*[2]","self::subscript","CQFisLogarithm"],["Rule","subscript-index","default","[n] children/*[1]; [t] \"Index\"; [n] children/*[2]","self::subscript","contains(@grammar, \"simpleDet\")"],["Rule","fraction","default","[p] (pause:short); [t] \"Bruch mit Zähler\"; [n] children/*[1] (pause:short); [t] \"und Nenner\"; [n] children/*[2] (pause:short)","self::fraction"],["Rule","fraction","Functions_None","[p] (pause:short); [t] \"Bruch mit Zähler\"; [n] children/*[1] (pause:short); [t] \"und Nenner\"; [n] children/*[2] (pause:short)","self::fraction","name(children/*[1])=\"appl\" or name(children/*[2])=\"appl\""],["Rule","simple-fraction","default","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction","contains(children/*[1]/@annotation, \"clearspeak:simple\") or contains(children/*[1]/@annotation, \"clearspeak:unit\")","contains(children/*[2]/@annotation, \"clearspeak:simple\") or contains(children/*[2]/@annotation, \"clearspeak:unit\")"],["Rule","simple-vulgar-fraction","default","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction","@role=\"vulgar\""],["Rule","simple-text-fraction","default","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction","name(children/*[1])=\"text\"","name(children/*[2])=\"text\""],["Rule","simple-text-fraction","default","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction","name(children/*[1])=\"infixop\"","children/*[1][@role=\"unit\"]","name(children/*[2])=\"text\""],["Rule","vulgar-fraction","default","[t] CSFvulgarFraction (grammar:correctOne)","self::fraction","@role=\"vulgar\"","CQFvulgarFractionSmall"],["Rule","fraction","Fraction_Over","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction"],["Rule","fraction","Fraction_OverEndFrac","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short); [t] \"Ende Bruch\" (pause:short)","self::fraction"],["Rule","fraction","Fraction_FracOver","[p] (pause:short); [t] \"Bruch\"; [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction"],["Rule","fraction","Fraction_Per","[p] (pause:short); [n] children/*[1]; [t] \"per\"; [n] children/*[2] (pause:short)","self::fraction"],["Rule","fraction","Fraction_GeneralEndFrac","[p] (pause:short); [t] \"Bruch mit Zähler\"; [n] children/*[1] (pause:short); [t] \"und Nenner\"; [n] children/*[2] (pause:short); [t] \"Ende Bruch\" (pause:short)","self::fraction"],["Rule","fraction","Fraction_General","[p] (pause:short); [t] \"Bruch mit Zähler\"; [n] children/*[1] (pause:short); [t] \"und Nenner\"; [n] children/*[2] (pause:short)","self::fraction"],["Rule","simple-vulgar-fraction","Fraction_Ordinal","[t] CSFvulgarFraction (grammar:correctOne)","self::fraction","@role=\"vulgar\""],["Rule","fraction","Fraction_EndFrac","[p] (pause:short); [n] . (grammar:endfrac); [t] \"Ende Bruch\" (pause:short)","self::fraction","not(contains(@grammar, \"endfrac\"))","not(contains(children/*[1]/@annotation, \"clearspeak:unit\"))","not(contains(children/*[2]/@annotation, \"clearspeak:unit\"))"],["Rule","vulgar-fraction","Fraction_EndFrac","[p] (pause:short); [n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2] (pause:short)","self::fraction","name(children/*[1])=\"fraction\"","name(children/*[2])=\"fraction\"","contains(children/*[1]/@annotation, \"clearspeak:simple\")","contains(children/*[2]/@annotation, \"clearspeak:simple\")"],["Rule","simple-vulgar-fraction","Fraction_EndFrac","[t] CSFvulgarFraction (grammar:correctOne)","self::fraction","@role=\"vulgar\"","contains(@annotation, \"clearspeak:simple\")","self::*"],["Rule","sqrt","default","[t] \"Quadratwurzel aus\"; [n] children/*[1] (grammar:EndRoot=false, pause:short)","self::sqrt"],["Rule","sqrt-nested","default","[p] (pause: \"short\"); [t] \"Quadratwurzel aus\"; [n] children/*[1] (grammar:EndRoot=false, pause:short)","self::sqrt","not(preceding-sibling::*)","ancestor::sqrt|ancestor::root"],["Rule","negative-sqrt","default","[t] \"negative Quadratwurzel aus\"; [n] children/*[1]/children/*[1] (grammar:EndRoot=false, pause:short)","self::prefixop","@role=\"negative\"","name(children/*[1])=\"sqrt\""],["Rule","negative-sqrt","default","[p] (pause: \"short\"); [t] \"negative Quadratwurzel aus\"; [n] children/*[1]/children/*[1] (grammar:EndRoot=false, pause:short)","self::prefixop","@role=\"negative\"","name(children/*[1])=\"sqrt\"","not(preceding-sibling::*)","ancestor::sqrt|ancestor::root"],["Rule","sqrt-plus-minus","Roots_PosNegSqRoot","[t] \"positive Quadratwurzel aus\"; [n] children/*[1] (grammar:EndRoot=false, pause:short)","self::sqrt","parent::stree or not(parent::*/parent::infixop[@role=\"addition\"]) or (parent::*/parent::*[1]/text()!=\"±\" and parent::*/parent::*/text()!=\"∓\")"],["Rule","sqrt-nested-plus-minus","Roots_PosNegSqRoot","[p] (pause: \"short\"); [t] \"positive Quadratwurzel aus\"; [n] children/*[1] (grammar:EndRoot=false, pause:short)","self::sqrt","not(preceding-sibling::*)","ancestor::sqrt|ancestor::root","parent::stree or not(parent::*/parent::infixop[@role=\"addition\"]) or (parent::*/parent::*[1]/text()!=\"±\" and parent::*/parent::*/text()!=\"∓\")"],["Rule","sqrt-plus-minus","Roots_PosNegSqRootEnd","[t] \"positive Quadratwurzel aus\"; [n] children/*[1] (grammar:EndRoot=false, pause:short)","self::sqrt","parent::stree or not(parent::*/parent::infixop[@role=\"addition\"]) or (parent::*/parent::*[1]/text()!=\"±\" and parent::*/parent::*/text()!=\"∓\")"],["Rule","sqrt-nested-plus-minus","Roots_PosNegSqRootEnd","[p] (pause: \"short\"); [t] \"positive Quadratwurzel aus\"; [n] children/*[1] (grammar:EndRoot=false, pause:short)","self::sqrt","not(preceding-sibling::*)","ancestor::sqrt|ancestor::root","parent::stree or not(parent::*/parent::infixop[@role=\"addition\"]) or (parent::*/parent::*[1]/text()!=\"±\" and parent::*/parent::*/text()!=\"∓\")"],["Rule","sqrt-endroot","Roots_RootEnd","[n] . (grammar:EndRoot); [t] \"Wurzel Ende\" (pause:short)","self::sqrt","not(contains(@grammar, \"EndRoot\"))"],["Rule","negative-sqrt-endroot","Roots_RootEnd","[n] . (grammar:EndRoot); [t] \"Wurzel Ende\" (pause:short)","self::prefixop","@role=\"negative\"","name(children/*[1])=\"sqrt\"","not(contains(@grammar, \"EndRoot\"))"],["Rule","sqrt-endroot","Roots_PosNegSqRootEnd","[n] . (grammar:EndRoot); [t] \"Wurzel Ende\" (pause:short)","self::sqrt","not(contains(@grammar, \"EndRoot\"))"],["Rule","negative-sqrt-endroot","Roots_PosNegSqRootEnd","[n] . (grammar:EndRoot); [t] \"Wurzel Ende\" (pause:short)","self::prefixop","@role=\"negative\"","name(children/*[1])=\"sqrt\"","not(contains(@grammar, \"EndRoot\"))"],["Rule","cube","default","[t] \"Kubikwurzel aus\"; [n] children/*[2] (grammar:EndRoot=false, pause:short)","self::root","children/*[1][text()=\"3\"]"],["Rule","cube-nested","default","[p] (pause:short); [t] \"Kubikwurzel aus\"; [n] children/*[2] (grammar:EndRoot=false, pause:short)","self::root","children/*[1][text()=\"3\"]","not(preceding-sibling::*)","ancestor::sqrt|ancestor::root"],["Rule","root","default","[t] \"die\" (grammar:article); [n] children/*[1] (grammar:ordinal); [t] \"Wurzel aus\"; [n] children/*[2] (grammar:EndRoot=false, pause:short)","self::root"],["Rule","root-nested","default","[p] (pause:short); [t] \"die\" (grammar:article); [n] children/*[1] (grammar:ordinal); [t] \"Wurzel aus\"; [n] children/*[2] (grammar:EndRoot=false, pause:short)","self::root","not(preceding-sibling::*)","ancestor::sqrt|ancestor::root"],["Rule","root-endroot","Roots_RootEnd","[n] . (grammar:EndRoot); [t] \"Wurzel Ende\" (pause:short)","self::root","not(contains(@grammar, \"EndRoot\"))"],["Rule","root-endroot","Roots_PosNegSqRootEnd","[n] . (grammar:EndRoot); [t] \"Wurzel Ende\" (pause:short)","self::root","not(contains(@grammar, \"EndRoot\"))"],["Rule","negative","default","[t] \"minus\"; [n] children/*[1]","self::prefixop","@role=\"negative\""],["Rule","positive","default","[t] \"plus\"; [n] children/*[1]","self::prefixop","@role=\"positive\""],["Rule","angle-measure","default","[t] \"das Maß des Winkels\"; [n] children/*[2] (grammar:angle)","self::infixop","content/*[1]/text()=\"∠\"","children/*[1][text()=\"m\"]"],["Rule","prefix","default","[m] content/* (grammar:prefix); [n] children/*[1]","self::prefixop"],["Rule","postfix","default","[n] children/*[1]; [m] content/* (grammar:postfix)","self::postfixop"],["Rule","set-prefix-operators","default","[t] \"der\" (grammar:article); [n] self::* (grammar:!prefix); [t] \"von\"","self::*","contains(@grammar,\"prefix\")","descendant-or-self::*/text()=\"∩\"","self::*","self::*","self::*"],["Rule","set-prefix-operators","default","[t] \"die\" (grammar:article); [n] self::* (grammar:!prefix); [t] \"von\"","self::*","contains(@grammar,\"prefix\")","descendant-or-self::*/text()=\"∪\"","self::*","self::*","self::*"],["Rule","binary-operation","default","[m] children/* (sepFunc:CTFcontentIterator);","self::infixop"],["Rule","division","default","[n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2]","self::infixop","@role=\"division\"","count(children/*)=2"],["Rule","binary-operation","ImpliedTimes_MoreImpliedTimes","[m] children/* (sepFunc:CTFcontentIterator);","self::infixop","@role=\"implicit\""],["Rule","binary-operation-pause","default","[p] (pause:short); [m] children/* (sepFunc:CTFcontentIterator);","self::infixop","@role=\"implicit\"","name(children/*[1])=\"appl\""],["Rule","binary-operation-pause","default","[m] children/* (sepFunc:CTFcontentIterator, pause:short)","self::infixop","@role=\"implicit\"","name(children/*[last()])=\"appl\""],["Rule","binary-operation-pause","default","[p] (pause:short); [m] children/* (sepFunc:CTFcontentIterator, pause:short)","self::infixop","@role=\"implicit\"","name(children/*[1])=\"appl\"","name(children/*[last()])=\"appl\""],["Rule","implicit-times","default","[p] (pause:short)","self::operator","@role=\"multiplication\"","text()=\"⁢\""],["Rule","implicit-times","default","","self::operator","@role=\"multiplication\"","text()=\"⁢\"","CQFsimpleArguments"],["Rule","implicit-times","default","[n] text()","self::operator","@role=\"multiplication\"","text()=\"⁢\"","CQFfencedArguments"],["Rule","implicit-times","ImpliedTimes_MoreImpliedTimes","[n] text()","self::operator","@role=\"multiplication\"","text()=\"⁢\""],["Rule","implicit-times","ImpliedTimes_None","","self::operator","@role=\"multiplication\"","text()=\"⁢\""],["Rule","binary-operation-simple","default","[m] children/* (rate:\"0.5\", pause:short)","self::infixop","@role=\"implicit\"","contains(@annotation, \"clearspeak:simple\")","not(contains(@grammar, \"inFrac\"))"],["Rule","simple-in-fraction","default","[n] . (rate:\"0.5\",grammar:inFrac)","self::*","contains(@annotation, \"clearspeak:simple\")","not(contains(@grammar, \"inFrac\"))","name(.)!=\"identifier\"","name(.)!=\"function\"","name(.)!=\"number\"","name(parent::*/parent::*)=\"fraction\"","not(preceding-sibling::*)"],["Rule","operators-after-power","Exponent_AfterPower","[m] children/* (rate:\"0.5\")","self::infixop","@role=\"implicit\"","contains(@grammar, \"afterPower\")"],["Rule","relseq","default","[m] children/* (sepFunc:CTFcontentIterator)","self::relseq"],["Rule","multrel","default","[m] children/* (sepFunc:CTFcontentIterator)","self::multirel"],["Rule","natural-numbers","default","[t] \"die natürlichen Zahlen\"","self::identifier","text()=\"ℕ\" or (text()=\"N\" and @font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","integers","default","[t] \"die ganzen Zahlen\"","self::identifier","text()=\"ℤ\" or (text()=\"Z\" and @font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","rational-numbers","default","[t] \"die rationalen Zahlen\"","self::identifier","text()=\"ℚ\" or (text()=\"Q\" and @font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","real-numbers","default","[t] \"die reellen Zahlen\"","self::identifier","text()=\"ℝ\" or (text()=\"R\" and @font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","complex-numbers","default","[t] \"die komplexen Zahlen\"","self::identifier","text()=\"ℂ\" or (text()=\"C\" and @font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","natural-numbers-super","default","[t] \"n\" (join: \"-\"); [n] children/*[2] (grammar:numbers2alpha)","self::superscript","children/*[1]/text()=\"ℕ\" or (children/*[1]/text()=\"N\" and children/*[1]/@font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","integers-super","default","[t] \"z\" (join: \"-\"); [n] children/*[2] (grammar:numbers2alpha)","self::superscript","children/*[1]/text()=\"ℤ\" or (children/*[1]/text()=\"Z\" and children/*[1]/@font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","rational-numbers-super","default","[t] \"q\" (join: \"-\"); [n] children/*[2] (grammar:numbers2alpha)","self::superscript","children/*[1]/text()=\"ℚ\" or (children/*[1]/text()=\"Q\" and children/*[1]/@font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","real-numbers-super","default","[t] \"r\" (join:\"-\"); [n] children/*[2] (grammar:numbers2alpha)","self::superscript","children/*[1]/text()=\"ℝ\" or (children/*[1]/text()=\"R\" and children/*[1]/@font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","complex-numbers-super","default","[t] \"c\" (join:\"-\"); [n] children/*[2] (grammar:numbers2alpha)","self::superscript","children/*[1]/text()=\"ℂ\" or (children/*[1]/text()=\"C\" and children/*[1]/@font=\"double-struck\")","self::*","self::*","self::*","self::*","self::*"],["Rule","natural-numbers-with-zero","default","[t] \"die natürlichen Zahlen mit Null\"","self::subscript","children/*[1]/text()=\"ℕ\" or (children/*[1]/text()=\"N\" and children/*[1]/@font=\"double-struck\")","children/*[2]/text()=\"0\"","self::*","self::*","self::*","self::*","self::*"],["Rule","positive-integers","default","[t] \"die positiven ganzen Zahlen\"","self::superscript","children/*[1]/text()=\"ℤ\" or (children/*[1]/text()=\"Z\" and children/*[1]/@font=\"double-struck\")","children/*[2]/text()=\"+\"","self::*","self::*","self::*","self::*","self::*"],["Rule","positive-integers","default","[t] \"die negativen ganzen Zahlen\"","self::superscript","children/*[1]/text()=\"ℤ\" or (children/*[1]/text()=\"Z\" and children/*[1]/@font=\"double-struck\")","children/*[2]/text()=\"-\"","self::*","self::*","self::*","self::*","self::*"],["Rule","positive-rational-numbers","default","[t] \"die positiven rationalen Zahlen\"","self::superscript","children/*[1]/text()=\"ℚ\" or (children/*[1]/text()=\"Q\" and children/*[1]/@font=\"double-struck\")","children/*[2]/text()=\"+\"","self::*","self::*","self::*","self::*","self::*"],["Rule","negative-rational-numbers","default","[t] \"die negativen rationalen Zahlen\"","self::superscript","children/*[1]/text()=\"ℚ\" or (children/*[1]/text()=\"Q\" and children/*[1]/@font=\"double-struck\")","children/*[2]/text()=\"-\"","self::*","self::*","self::*","self::*","self::*"],["Rule","fences-neutral","default","[p] (pause:short); [t] \"der Betrag von\"; [n] children/*[1] (pause: short)","self::fenced","@role=\"neutral\"","content/*[1][text()]=\"|\" or content/*[1][text()]=\"❘\" or content/*[1][text()]=\"｜\" or content/*[1][text()]=\"∣\""],["Rule","fences-neutral","AbsoluteValue_AbsEnd","[p] (pause:short); [t] \"der Betrag von\"; [n] children/*[1] (pause: short); [t] \"Ende Betrag\" (pause: short)","self::fenced","@role=\"neutral\"","content/*[1][text()]=\"|\" or content/*[1][text()]=\"❘\" or content/*[1][text()]=\"｜\" or content/*[1][text()]=\"∣\""],["Rule","fences-neutral","AbsoluteValue_Cardinality","[p] (pause:short); [t] \"die Mächtigkeit der Menge\"; [n] children/*[1] (pause: short)","self::fenced","@role=\"neutral\"","content/*[1][text()]=\"|\" or content/*[1][text()]=\"❘\" or content/*[1][text()]=\"｜\" or content/*[1][text()]=\"∣\""],["Rule","fences-neutral","AbsoluteValue_Determinant","[p] (pause:short); [t] \"die Determinante von\"; [n] children/*[1] (pause: short)","self::fenced","@role=\"neutral\"","content/*[1][text()]=\"|\" or content/*[1][text()]=\"❘\" or content/*[1][text()]=\"｜\" or content/*[1][text()]=\"∣\""],["Rule","matrix","default","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Matrize\" (pause:long); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\", pause:long)","self::matrix"],["Rule","matrix-simple","default","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Matrize\" (pause:long); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\",grammar:simpleDet, pause:long)","self::matrix","count(children/*)<4","count(children/*[1]/children/*)<4","CQFcellsSimple"],["Rule","matrix-trivial","default","[t] \"die 1 mal 1 Matrize mit Element\"; [n] children/*[1] (pause:long)","self::vector","@role=\"squarematrix\""],["Rule","determinant","default","[t] \"die\" (grammar:article); [t] \"Determinante der\"; [t] count(children/*); [t] \"mal\"; [t] count(children/*[1]/children/*); [t] \"Matrize\" (pause:long); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\",grammar:simpleDet, pause:long)","self::matrix","@role=\"determinant\"","count(children/*)<4","CQFcellsSimple"],["Rule","determinant-simple","default","[t] \"die\" (grammar:article); [t] \"Determinante der\"; [t] count(children/*); [t] \"mal\"; [t] count(children/*[1]/children/*); [t] \"Matrize\" (pause:long); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\", pause:long)","self::matrix","@role=\"determinant\""],["Rule","matrix-vector","default","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Spaltenmatrize\" (pause:long); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\",grammar:simpleDet, pause:long)","self::vector"],["SpecializedRule","matrix-vector","default","Matrix_SpeakColNum"],["Rule","matrix-vector-simple","default","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Spaltenmatrize\" (pause:long); [m] children/* (sepFunc:CTFpauseSeparator,separator:\"short\",grammar:simpleDet, pause:long)","self::vector","count(children/*)<4","CQFcellsSimple","@role!=\"squarematrix\""],["Rule","matrix-vector-simple","Matrix_SilentColNum","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Spaltenmatrize\" (pause:long); [m] children/* (sepFunc:CTFpauseSeparator,separator:\"short\",grammar:simpleDet, pause:long)","self::vector"],["Rule","matrix-row-vector","default","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Zeilenmatrize\" (pause:long); [m] children/*[1]/children/* (ctxtFunc:CTFnodeCounter,context:\"Spalte-:\",grammar:simpleDet, pause:long)","self::matrix","@role=\"rowvector\""],["SpecializedRule","matrix-row-vector","default","Matrix_SpeakColNum"],["Rule","matrix-row-vector-simple","default","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Zeilenmatrize\" (pause:long); [m] children/*[1]/children/* (sepFunc:CTFpauseSeparator,separator:\"short\",grammar:simpleDet, pause:long)","self::matrix","@role=\"rowvector\"","count(children/*[1]/children/*)<4","CQFcellsSimple"],["Rule","matrix-row-vector-simple","Matrix_SilentColNum","[t] \"die\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Zeilenmatrize\" (pause:long); [m] children/*[1]/children/* (sepFunc:CTFpauseSeparator,separator:\"short\",grammar:simpleDet, pause:long)","self::matrix","@role=\"rowvector\""],["Rule","matrix-row-simple","default","[m] children/* (sepFunc:CTFpauseSeparator,separator:\"short\")","self::row","contains(@grammar, \"simpleDet\")"],["Rule","matrix-row-simple","Matrix_SilentColNum","[m] children/* (sepFunc:CTFpauseSeparator,separator:\"short\")","self::row"],["Rule","line-simple","default","[n] children/*[1]","self::line","contains(@grammar, \"simpleDet\")"],["Rule","matrix-row","default","[m] children/* (ctxtFunc:CTFnodeCounter,context:\"Spalte-,- \",sepFunc:CTFpauseSeparator,separator:\"medium\", pause:long)","self::row"],["SpecializedRule","matrix-row","default","Matrix_SpeakColNum"],["Rule","matrix-cell","default","[n] children/*[1]","self::cell"],["Rule","matrix-end-matrix","Matrix_EndMatrix","[n] . (grammar:EndMatrix); [t] \"Ende Matrize\"","self::matrix","not(contains(@grammar, \"EndMatrix\"))"],["Rule","matrix-end-vector","Matrix_EndMatrix","[n] . (grammar:EndMatrix); [t] \"Ende Matrize\"","self::vector","not(contains(@grammar, \"EndMatrix\"))"],["Rule","matrix-end-determinant","Matrix_EndMatrix","[n] . (grammar:EndMatrix); [t] \"Ende Determinante\"","self::matrix","@role=\"determinant\"","not(contains(@grammar, \"EndMatrix\"))"],["Rule","vector","Matrix_Vector","[t] \"der\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Spaltenvektor\" (pause:long); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\",grammar:simpleDet, pause:long)","self::vector"],["SpecializedRule","vector","Matrix_Vector","Matrix_EndVector"],["Rule","vector-simple","Matrix_Vector","[t] \"der\" (grammar:article); [t] count(children/*);  [t] \"mal\"; [t] count(children/*[1]/children/*); [t] \"Spaltenvektor\" (pause:long); [m] children/* (sepFunc:CTFpauseSeparator,separator:\"short\",grammar:simpleDet, pause:long)","self::vector","count(children/*)<4","CQFcellsSimple"],["SpecializedRule","vector-simple","Matrix_Vector","Matrix_EndVector"],["Rule","row-vector","Matrix_Vector","[t] \"der\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Zeilenvektor\" (pause:long); [m] children/*[1]/children/* (ctxtFunc:CTFnodeCounter,context:\"Spalte-:\",grammar:simpleDet, pause:long)","self::matrix","@role=\"rowvector\""],["SpecializedRule","row-vector","Matrix_Vector","Matrix_EndVector"],["Rule","row-vector-simple","Matrix_Vector","[t] \"der\" (grammar:article); [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Zeilenvektor\" (pause:long); [m] children/*[1]/children/* (sepFunc:CTFpauseSeparator,separator:\"short\",grammar:simpleDet, pause:long)","self::matrix","@role=\"rowvector\"","count(children/*[1]/children/*)<4","CQFcellsSimple"],["SpecializedRule","row-vector-simple","Matrix_Vector","Matrix_EndVector"],["Rule","vector-end-matrix","Matrix_EndVector","[n] . (grammar:EndMatrix); [t] \"Ende Matrize\"","self::matrix","not(contains(@grammar, \"EndMatrix\"))","self::*"],["Rule","vector-end-vector","Matrix_EndVector","[n] . (grammar:EndMatrix); [t] \"Ende Vektor\"","self::vector","not(contains(@grammar, \"EndMatrix\"))","self::*"],["Rule","vector-end-vector","Matrix_EndVector","[n] . (grammar:EndMatrix); [t] \"Ende Vektor\"","self::matrix","@role=\"rowvector\"","not(contains(@grammar, \"EndMatrix\"))","self::*"],["Rule","vector-end-determinant","Matrix_EndVector","[n] . (grammar:EndMatrix); [t] \"Ende Determinante\"","self::matrix","@role=\"determinant\"","not(contains(@grammar, \"EndMatrix\"))","self::*"],["Rule","binomial","Matrix_Combinatoric","[n] children/*[1]/children/*[1]; [t] \"über\"; [n] children/*[2]/children/*[1]; ","self::vector","@role=\"binomial\""],["Rule","lines-summary","default","[p] (pause:short); [t] count(children/*); [t] \"Zeilen\";  [n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))","self::*"],["Rule","lines-summary","MultiLineOverview_None","[n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))","self::*"],["Aliases","lines-summary","self::table","not(contains(@grammar, \"layoutSummary\"))"],["Rule","cases-summary","default","[p] (pause:short); [t] count(children/*); [t] \"Fälle\";  [n] . (grammar:layoutSummary)","self::cases","not(contains(@grammar, \"layoutSummary\"))"],["Rule","cases-summary","MultiLineOverview_None","[n] . (grammar:layoutSummary)","self::cases","not(contains(@grammar, \"layoutSummary\"))","self::*"],["Rule","lines","default","[p] (pause:short); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table"],["Aliases","lines","self::multiline"],["Rule","line","default","[n] children/*[1]","self::line"],["Rule","row-medium","default","[m] children/* (sepFunc:CTFpauseSeparator,separator:\"medium\")","self::row","@role=\"table\""],["Aliases","row-medium","self::row","@role=\"cases\""],["Rule","row-long","MultiLinePausesBetweenColumns_Long","[m] children/* (sepFunc:CTFpauseSeparator,separator:\"long\")","self::row","@role=\"table\""],["Aliases","row-long","self::row","@role=\"cases\""],["Rule","row-short","MultiLinePausesBetweenColumns_Short","[m] children/* (sepFunc:CTFpauseSeparator,separator:\"short\")","self::row","@role=\"table\""],["Aliases","row-short","self::row","@role=\"cases\""],["Rule","blank-cell","default","[t] \"leer\"","self::cell","count(children/*)=0"],["Rule","blank-line","default","[t] \"leer\"","self::line","count(children/*)=0"],["Rule","blank-cell-empty","default","[t] \"leer\"","self::empty","count(../*)=1","name(../..)=\"cell\""],["Rule","blank-line-empty","default","[t] \"leer\"","self::empty","count(../*)=1","name(../..)=\"line\""],["Rule","cases","default","[p] (pause:short);  [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Fall-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::cases"],["Rule","lines-cases-summary","MultiLineLabel_Case","[p] (pause:short); [t] count(children/*); [t] \"Fälle\";  [n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))"],["Aliases","lines-cases-summary","self::table","not(contains(@grammar, \"layoutSummary\"))"],["Rule","lines-cases","MultiLineLabel_Case","[p] (pause:short); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Fall-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table"],["Aliases","lines-cases","self::multiline"],["Rule","lines-equations-summary","MultiLineLabel_Equation","[p] (pause:short); [t] count(children/*); [t] \"Gleichungen\";  [n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))"],["Aliases","lines-equations-summary","self::table","not(contains(@grammar, \"layoutSummary\"))"],["Rule","lines-equations","MultiLineLabel_Equation","[p] (pause:short); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Gleichung-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table"],["Aliases","lines-equations","self::multiline"],["Rule","lines-steps-summary","MultiLineLabel_Step","[p] (pause:short); [t] count(children/*); [t] \"Rechenschritte\";  [n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))"],["Aliases","lines-steps-summary","self::table","not(contains(@grammar, \"layoutSummary\"))"],["Rule","lines-steps","MultiLineLabel_Step","[p] (pause:short); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Schritt-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table"],["Aliases","lines-steps","self::multiline"],["Rule","lines-rows-summary","MultiLineLabel_Row","[p] (pause:short); [t] count(children/*); [t] \"Zeilen\";  [n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))"],["Aliases","lines-rows-summary","self::table","not(contains(@grammar, \"layoutSummary\"))"],["Rule","lines-rows","MultiLineLabel_Row","[p] (pause:short); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Zeile-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table"],["Aliases","lines-rows","self::multiline"],["Rule","lines-constraints-summary","MultiLineLabel_Constraint","[p] (pause:short); [t] count(children/*); [t] \"Bedingungen\";  [n] . (grammar:layoutSummary)","self::multiline","not(contains(@grammar, \"layoutSummary\"))"],["Aliases","lines-constraints-summary","self::table","not(contains(@grammar, \"layoutSummary\"))"],["Rule","lines-constraints","MultiLineLabel_Constraint","[p] (pause:short); [m] children/* (ctxtFunc:CTFnodeCounter,context:\"Bedingung-:\",sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table"],["Aliases","lines-constraints","self::multiline"],["Rule","lines-none","MultiLineLabel_None","[p] (pause:short); [m] children/* (sepFunc:CTFpauseSeparator,separator:\"long\", pause:long)","self::table","contains(@grammar, \"layoutSummary\")"],["Aliases","lines-none","self::multiline","contains(@grammar, \"layoutSummary\")"],["Aliases","lines-none","self::cases","contains(@grammar, \"layoutSummary\")"],["Rule","bigop","default","[n] children/*[1]; [t] \"über\"; [n] children/*[2] (pause:short)","self::bigop"],["Rule","limboth","default","[n] children/*[1]; [t] \"von\"; [n] children/*[2];[t] \"bis\"; [n] children/*[3];","self::limboth"],["Rule","limlower","default","[n] children/*[1]; [t] \"über\"; [n] children/*[2] (pause:short)","self::limlower"],["Rule","limupper","default","[n] children/*[1]; [t] \"unter\"; [n] children/*[2] (pause:short)","self::limupper"],["Rule","integral","default","[t] \"das\"; [n] children/*[1]; [t] \"über\"; [n] children/*[2] (pause:short)","self::integral"],["Rule","overscript","default","[n] children/*[1]; [t] \"unter\"; [n] children/*[2] (pause:short)","self::overscore"],["Rule","overscript","default","[n] children/*[1]; [n] children/*[2];","self::overscore","children/*[2][@role=\"overaccent\"]"],["Rule","overscript-limits","default","[n] children/*[1]; [t] \"bis\"; [n] children/*[2]","self::overscore","children/*[2][@role!=\"overaccent\"]","name(children/*[1])=\"underscore\"","children/*[1]/children/*[2][@role!=\"underaccent\"]"],["Rule","underscript","default","[n] children/*[1]; [t] \"über\"; [n] children/*[2] (pause:short)","self::underscore"],["Rule","underscript-limits","default","[n] children/*[1]; [t] \"von\"; [n] children/*[2]","self::underscore","@role=\"underover\"","children/*[2][@role!=\"underaccent\"]"],["Rule","number","default","[n] text()","self::number"],["Rule","mixed-number","default","[n] children/*[1]; [n] children/*[2]; ","self::number","@role=\"mixed\""],["Rule","number-with-chars","default","[t] \"Zahl\"; [m] CQFspaceoutNumber (grammar:protected)","self::number","@role=\"othernumber\"","\"\" != translate(text(), \"0123456789.,\", \"\")","not(contains(@grammar, \"protected\"))"],["Rule","decimal-period","default","[t] \"Dezimalbruch\"; [n] children/*[1] (grammar:spaceout); [t] \"Komma mit Periode\";  [n] children/*[3]/children/*[1] (grammar:spaceout)","self::punctuated","@role=\"sequence\"","count(./content/*)=1","./content/*[1][@role=\"fullstop\"]","name(children/*[1])=\"number\"","children/*[1][@role=\"integer\"]","name(children/*[3])=\"overscore\"","children/*[3][@role=\"integer\"]","children/*[3]/children/*[2][@role=\"overaccent\"]","children/*[3]/children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]"],["Rule","decimal-period","default","[t] \"Dezimalbruch\"; [n] children/*[1] (grammar:spaceout); [t] \"mit Periode\";  [n] children/*[2]/children/*[1] (grammar:spaceout);","self::infixop","@role=\"implicit\"","count(./children/*)=2","name(children/*[1])=\"number\"","children/*[1][@role=\"float\"]","name(children/*[2])=\"overscore\"","children/*[2][@role=\"integer\"]","children/*[2]/children/*[2][@role=\"overaccent\"]","children/*[2]/children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]"],["Rule","decimal-period-singular","default","[t] \"Dezimalbruch\"; [n] children/*[1] (grammar:spaceout); [t] \"Komma mit Periode\";  [n] children/*[3]/children/*[1] (grammar:spaceout)","self::punctuated","@role=\"sequence\"","count(./content/*)=1","./content/*[1][@role=\"fullstop\"]","name(children/*[1])=\"number\"","children/*[1][@role=\"integer\"]","name(children/*[3])=\"overscore\"","children/*[3][@role=\"integer\"]","children/*[3]/children/*[2][@role=\"overaccent\"]","children/*[3]/children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]","string-length(./children/*[3]/children/*[1]/text())=1"],["Rule","decimal-period-singular","default","[t] \"Dezimalbruch\"; [n] children/*[1] (grammar:spaceout); [t] \"mit Periode\";  [n] children/*[2]/children/*[1] (grammar:spaceout);","self::infixop","@role=\"implicit\"","count(./children/*)=2","name(children/*[1])=\"number\"","children/*[1][@role=\"float\"]","name(children/*[2])=\"overscore\"","children/*[2][@role=\"integer\"]","children/*[2]/children/*[2][@role=\"overaccent\"]","children/*[2]/children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]","string-length(./children/*[2]/children/*[1]/text())=1"],["Rule","number-with-spaces","default","[m] CQFspaceoutNumber (grammar:!spaceout:number)","self::number","contains(@grammar, \"spaceout\")"],["Rule","decimal-point","default","[t] \"Komma\"","self::punctuation","@role=\"fullstop\"","contains(@grammar,\"number\")"],["Rule","line-segment","default","[t] \"die Strecke\"; [n] children/*[1]/children/*[1]; [n] children/*[1]/children/*[2] (pause:short)","self::overscore","@role=\"implicit\"","children/*[2][@role=\"overaccent\"]","children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]","name(children/*[1])=\"infixop\"","count(./children/*[1]/children/*)=2"],["Rule","conjugate","Bar_Conjugate","[t] \"die komplexe Konjugation von\"; [n] children/*[1]","self::overscore","children/*[2][@role=\"overaccent\"]","children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]"],["Rule","defined-by","default","[t] \"ist definiert als\" (pause:short)","self::overscore","@role=\"equality\"","@embellished=\"relation\"","name(children/*[2])=\"text\"","children/*[2][text()]=\"def\""],["Rule","adorned-sign","default","[n] children/*[1] ; [t] \"Zeichen mit darüberstehendem\"; [n] children/*[2]","self::overscore","@embellished","name(children/*[1])=\"operator\" or name(children/*[1])=\"relation\""],["Rule","factorial","default","[t] \"Fakultät\"","self::punctuation","text()=\"!\"","name(preceding-sibling::*[1])!=\"text\""],["Rule","tensor-base","default","[n] children/*[2]; [n] children/*[3]; [n] children/*[1]; [n] children/*[4]; [n] children/*[5]","self::tensor"],["Rule","left-super","default","[t] \"linker oberer Index\"; [n] text()","self::*[@role=\"leftsuper\"]","not(contains(@grammar,\"combinatorics\"))"],["Rule","left-super","default","[t] \"linker oberer Index\"; [m] children/*","self::punctuated","@role=\"leftsuper\"","not(contains(@grammar,\"combinatorics\"))"],["Rule","left-sub","default","[t] \"linker unterer Index\"; [n] text()","self::*[@role=\"leftsub\"]","not(contains(@grammar,\"combinatorics\"))"],["Rule","left-sub","default","[t] \"linker unterer Index\"; [m] children/*","self::punctuated","@role=\"leftsub\"","not(contains(@grammar,\"combinatorics\"))"],["Rule","right-super","default","[t] \"rechter oberer Index\"; [n] text()","self::*[@role=\"rightsuper\"]","not(contains(@grammar,\"combinatorics\"))"],["Rule","right-super","default","[t] \"rechter oberer Index\"; [m] children/*","self::punctuated","@role=\"rightsuper\"","not(contains(@grammar,\"combinatorics\"))"],["Rule","right-sub","default","[t] \"rechter unterer Index\"; [n] text()","self::*[@role=\"rightsub\"]","not(contains(@grammar,\"combinatorics\"))"],["Rule","right-sub","default","[t] \"rechter unterer Index\"; [m] children/*","self::punctuated","@role=\"rightsub\"","not(contains(@grammar,\"combinatorics\"))"],["Rule","empty-index","default","[p] (pause:medium)","self::empty","@role=\"rightsub\" or @role=\"rightsuper\" or @role=\"leftsub\" or @role=\"leftsuper\""],["Rule","combinatorics","default","[n] children/*[2] (grammar:combinatorics); [n] children/*[1]; [n] children/*[4] (grammar:combinatorics)","self::tensor","name(children/*[3])=\"empty\"","name(children/*[5])=\"empty\"","children/*[1][text()=\"P\" or text()=\"C\"]"],["Rule","choose","CombinationPermutation_ChoosePermute","[n] children/*[4] (grammar:combinatorics); [t] \"aus\"; [n] children/*[2] (grammar:combinatorics)","self::tensor","name(children/*[3])=\"empty\"","name(children/*[5])=\"empty\"","children/*[1][text()=\"C\"]"],["Rule","permute","CombinationPermutation_ChoosePermute","[n] children/*[4] (grammar:combinatorics); [t] \"Permutionen von\"; [n] children/*[2] (grammar:combinatorics)","self::tensor","name(children/*[3])=\"empty\"","name(children/*[5])=\"empty\"","children/*[1][text()=\"P\"]"],["Rule","unit-singular","default","[t] text() (grammar:annotation=\"unit\":translate)","self::identifier[@role=\"unit\"]"],["Rule","unit-plural","default","[t] text() (grammar:annotation=\"unit\":translate:plural)","self::identifier[@role=\"unit\"]","not(contains(@grammar, \"singularUnit\"))"],["Rule","unit-square","default","[t] \"Quadrat\" (join:\"\"); [n] children/*[1] (grammar:lowercase)","self::superscript[@role=\"unit\"]","children/*[2][text()=2]","name(children/*[1])=\"identifier\""],["Rule","unit-cubic","default","[t] \"Kubik\" (join:\"\"); [n] children/*[1] (grammar:lowercase)","self::superscript[@role=\"unit\"]","children/*[2][text()=3]","name(children/*[1])=\"identifier\""],["Rule","reciprocal","default","[n] children/*[1]; [t] \"invers\"","self::superscript[@role=\"unit\"]","name(children/*[1])=\"identifier\"","name(children/*[2])=\"prefixop\"","children/*[2][@role=\"negative\"]","children/*[2]/children/*[1][text()=1]","count(preceding-sibling::*)=0 or preceding-sibling::*[@role!=\"unit\"]"],["Rule","reciprocal","default","[t] \"pro\"; [n] children/*[1]","self::superscript[@role=\"unit\"]","name(children/*[1])=\"identifier\"","name(children/*[2])=\"prefixop\"","children/*[2][@role=\"negative\"]","children/*[2]/children/*[1][text()=1]","preceding-sibling::*[@role=\"unit\"]"],["Rule","unit-combine","default","[m] children/*","self::infixop[@role=\"unit\"]"],["Rule","unit-combine-singular","default","[n] children/*[1]; [n] children/*[2] (grammar:singularUnit); [m] children/*[position()>2]","self::infixop[@role=\"unit\"]","name(children/*[1])=\"number\"","children/*[1][text()=1]"],["Rule","unit-divide","default","[n] children/*[1]; [t] \"pro\"; [n] children/*[2]","self::fraction[@role=\"unit\"]"],["Rule","currency","default","[m] children/*[position()>1]; [n] children/*[1];","self::infixop","contains(@annotation, \"clearspeak:unit\")","children/*[1][@role=\"unit\"]","CQFfirstCurrency"],["Rule","currency","Currency_Position","[m] children/*","self::infixop","contains(@annotation, \"clearspeak:unit\")"],["SpecializedRule","currency","Currency_Position","Currency_Prefix"],["Rule","currency","Currency_Prefix","[n] children/*[last()]; [m] children/*[position()<last()]; ","self::infixop","contains(@annotation, \"clearspeak:unit\")","children/*[last()][@role=\"unit\"]","CQFlastCurrency"]],"annotators":["simple","unit"]}
,
"de/rules/mathspeak_german.js": 
{"locale":"de","domain":"mathspeak","modality":"speech","rules":[["Rule","collapsed","default","[n] . (engine:modality=summary,grammar:collapsed); [t] \"kollabiert\"","self::*[@alternative]","not(contains(@grammar, \"collapsed\"))"],["SpecializedRule","collapsed","default","brief"],["SpecializedRule","collapsed","brief","sbrief"],["Rule","direct-speech","default","[t] @ext-speech","self::*[@ext-speech]"],["Rule","stree","default","[n] ./*[1]","self::stree","CQFresetNesting"],["Rule","unknown","default","[n] text()","self::unknown"],["Rule","protected","default","[n] text() (grammar:ignoreCaps=\"großes\")","self::number","contains(@grammar, \"protected\")"],["Rule","omit-empty","default","[p] (pause:100)","self::empty"],["Rule","blank-cell-empty","default","[t] \"leer\"","self::empty","count(../*)=1","name(../..)=\"cell\""],["Rule","blank-line-empty","default","[t] \"leer\"","self::empty","count(../*)=1","name(../..)=\"line\""],["Rule","font","default","[t] @font (grammar:localFont); [n] . (grammar:ignoreFont=@font)","self::*","name(self::*)!=\"number\"","@font","not(contains(@grammar, \"ignoreFont\"))","@font!=\"normal\""],["Rule","font-number","default","[t] @font (grammar:localFontNumber); [n] . (grammar:ignoreFont=@font)","self::number","@font","not(contains(@grammar, \"ignoreFont\"))","@font!=\"normal\""],["Rule","font-identifier-short","default","[t] @font (grammar:localFont); [n] . (grammar:ignoreFont=@font)","self::identifier","string-length(text())=1","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"normal\"","\"\"=translate(text(), \"abcdefghijklmnopqrstuvwxyzαβγδεζηθικλμνξοπρςστυφχψωABCDEFGHIJKLMNOPQRSTUVWXYZΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΣΤΥΦΧΨΩ\", \"\")","@role!=\"unit\""],["Rule","font-identifier","default","[t] @font (grammar:localFont); [n] . (grammar:ignoreFont=@font)","self::identifier","string-length(text())=1 or string-length(text())=2","@font","@font=\"normal\"","not(contains(@grammar, \"ignoreFont\"))","@role!=\"unit\""],["Rule","omit-font","default","[n] . (grammar:ignoreFont=@font)","self::identifier","string-length(text())=1 or string-length(text())=2","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"italic\"","self::*"],["Rule","font-double-struck","default","[n] . (grammar:ignoreFont=@font); [t] @font (grammar:localFont)","self::*","name(self::*)!=\"number\"","string-length(text())=1 or string-length(text())=2","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"double-struck\""],["Rule","font-number-double-struck","default","[n] . (grammar:ignoreFont=@font); [t] @font (grammar:localFontNumber)","self::number","string-length(text())=1 or string-length(text())=2","@font","not(contains(@grammar, \"ignoreFont\"))","@font=\"double-struck\""],["Rule","number","default","[n] text()","self::number"],["Rule","mixed-number","default","[n] children/*[1]; [n] children/*[2]; ","self::number","@role=\"mixed\""],["Rule","number-with-chars","default","[t] \"Zahl\"; [m] CQFspaceoutNumber (grammar:protected)","self::number","@role=\"othernumber\"","\"\" != translate(text(), \"0123456789.,\", \"\")","not(contains(@grammar, \"protected\"))"],["Rule","number-as-upper-word","default","[t] \"Wort groß\"; [t] CSFspaceoutText","self::number","string-length(text())>1","text()=translate(text(), \"abcdefghijklmnopqrstuvwxyzαβγδεζηθικλμνξοπρςστυφχψω\", \"ABCDEFGHIJKLMNOPQRSTUVWXYZΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΣΤΥΦΧΨΩ\")","\"\"=translate(text(), \"ABCDEFGHIJKLMNOPQRSTUVWXYZΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΣΤΥΦΧΨΩ\",\"\")"],["SpecializedRule","number-as-upper-word","default","brief"],["SpecializedRule","number-as-upper-word","default","sbrief"],["Rule","number-baseline","default","[t] \"Grundlinie\"; [n] . (grammar:baseline)","self::number","not(contains(@grammar, \"ignoreFont\"))","preceding-sibling::identifier","not(contains(@grammar, \"baseline\"))","preceding-sibling::*[1][contains(@role,\"letter\")]","parent::*/parent::infixop[@role=\"implicit\"]"],["SpecializedRule","number-baseline","default","brief","[t] \"Grund\"; [n] . (grammar:baseline)"],["SpecializedRule","number-baseline","brief","sbrief"],["Rule","number-baseline-font","default","[t] \"Grundlinie\"; [t] @font (grammar:localFont); [n] . (grammar:ignoreFont=@font)","self::number","@font","not(contains(@grammar, \"ignoreFont\"))","@font!=\"normal\"","preceding-sibling::identifier","preceding-sibling::*[contains(@role,\"letter\")]","parent::*/parent::infixop[@role=\"implicit\"]"],["SpecializedRule","number-baseline-font","default","brief","[t] \"Grund\"; [t] @font (grammar:localFont); [n] . (grammar:ignoreFont=@font)"],["SpecializedRule","number-baseline-font","brief","sbrief"],["Rule","identifier","default","[m] CQFspaceoutIdentifier","self::identifier","string-length(text())>1","@role!=\"unit\"","not(@font) or @font=\"normal\" or contains(@grammar, \"ignoreFont\")","text()!=translate(text(), \"abcdefghijklmnopqrstuvwxyzαβγδεζηθικλμνξοπρςστυφχψωABCDEFGHIJKLMNOPQRSTUVWXYZΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΣΤΥΦΧΨΩ\", \"\")"],["Rule","identifier","default","[n] text()","self::identifier"],["Rule","negative","default","[t] \"minus\"; [n] children/*[1]","self::prefixop","@role=\"negative\"","children/identifier"],["Aliases","negative","self::prefixop","@role=\"negative\"","children/number"],["Aliases","negative","self::prefixop","@role=\"negative\"","children/fraction[@role=\"vulgar\"]"],["Rule","negative","default","[t] \"minus\"; [n] children/*[1]","self::prefixop","@role=\"negative\""],["Rule","prefix","default","[m] content/*; [n] children/*[1]","self::prefixop"],["Rule","postfix","default","[n] children/*[1]; [m] content/*","self::postfixop"],["Rule","binary-operation","default","[m] children/* (sepFunc:CTFcontentIterator);","self::infixop"],["Rule","division","default","[n] children/*[1]; [t] \"geteilt durch\"; [n] children/*[2]","self::infixop","@role=\"division\"","count(children/*)=2"],["Rule","implicit","default","[m] children/*","self::infixop","@role=\"implicit\""],["Aliases","implicit","self::infixop","@role=\"leftsuper\" or @role=\"leftsub\" or @role=\"rightsuper\" or @role=\"rightsub\""],["Rule","subtraction","default","[m] children/* (separator:\"minus\");","self::infixop","@role=\"subtraction\""],["Rule","function-unknown","default","[n] children/*[1]; [n] children/*[2]","self::appl"],["Rule","function-prefix","default","[n] children/*[1]; [n] children/*[2]","self::appl","children/*[1][@role=\"prefix function\"]"],["Rule","fences-open-close","default","[n] content/*[1]; [n] children/*[1]; [n] content/*[2]","self::fenced","@role=\"leftright\""],["Rule","fences-neutral","default","[t] \"Anfang Betrag\"; [n] children/*[1]; [t] \"Ende Betrag\"","self::fenced","@role=\"neutral\"","content/*[1][text()]=\"|\" or content/*[1][text()]=\"❘\" or content/*[1][text()]=\"｜\" or content/*[1][text()]=\"∣\""],["SpecializedRule","fences-neutral","default","sbrief","[t] \"Betrag\"; [n] children/*[1]; [t] \"Ende Betrag\""],["Rule","fences-neutral","default","[n] content/*[1]; [n] children/*[1]; [n] content/*[2]","self::fenced","@role=\"neutral\""],["Rule","empty-set","default","[t] \"leere Menge\"","self::fenced[@role=\"set empty\"]","not(name(../..)=\"appl\")"],["SpecializedRule","empty-set","default","sbrief"],["Rule","fences-set","default","[t] \"Anfang Menge\"; [n] children/*[1]; [t] \"Ende Menge\"","self::fenced","@role=\"set empty\" or @role=\"set extended\" or @role=\"set singleton\" or @role=\"set collection\"","not(name(../..)=\"appl\")"],["SpecializedRule","fences-set","default","sbrief","[t] \"Menge\"; [n] children/*[1]; [t] \"Ende Menge\""],["Rule","text","default","[n] text()","self::text"],["Rule","factorial","default","[t] \"Fakultät\"","self::punctuation","text()=\"!\"","name(preceding-sibling::*[1])!=\"text\""],["Rule","minus","default","[t] \"minus\"","self::operator","text()=\"-\""],["Rule","fraction","default","[t] CSFopenFracVerbose; [n] children/*[1]; [t] CSFoverFracVerbose; [n] children/*[2]; [t] CSFcloseFracVerbose","self::fraction"],["Rule","fraction","brief","[t] CSFopenFracBrief; [n] children/*[1]; [t] CSFoverFracVerbose; [n] children/*[2]; [t] CSFcloseFracBrief","self::fraction"],["Rule","fraction","sbrief","[t] CSFopenFracSbrief; [n] children/*[1]; [t] CSFoverFracSbrief; [n] children/*[2]; [t] CSFcloseFracSbrief","self::fraction"],["Rule","vulgar-fraction","default","[t] CSFvulgarFraction (grammar:correctOne)","self::fraction","@role=\"vulgar\"","CQFvulgarFractionSmall"],["SpecializedRule","vulgar-fraction","default","brief"],["SpecializedRule","vulgar-fraction","default","sbrief"],["Rule","continued-fraction-outer","default","[t] \"Kettenbruch\"; [n] children/*[1];[t] \"durch\"; [n] children/*[2]","self::fraction","not(ancestor::fraction)","children/*[2]/descendant-or-self::*[@role=\"ellipsis\" and not(following-sibling::*)]"],["Rule","continued-fraction-inner","default","[t] \"Anfang Bruch\"; [n] children/*[1];[t] \"durch\"; [n] children/*[2]","self::fraction","ancestor::fraction","children/*[2]/descendant-or-self::*[@role=\"ellipsis\" and not(following-sibling::*)]"],["SpecializedRule","continued-fraction-inner","default","sbrief","[t] \"Bruch\"; [n] children/*[1];[t] \"durch\"; [n] children/*[2]"],["Rule","sqrt","default","[t] CSFopenRadicalVerbose; [n] children/*[1]; [t] CSFcloseRadicalVerbose","self::sqrt"],["Rule","sqrt","brief","[t] CSFopenRadicalBrief; [n] children/*[1]; [t] CSFcloseRadicalBrief","self::sqrt"],["Rule","sqrt","sbrief","[t] CSFopenRadicalSbrief; [n] children/*[1]; [t] CSFcloseRadicalBrief","self::sqrt"],["Rule","root-small","default","[t] CSFopenRadicalVerbose; [n] children/*[2]; [t] CSFcloseRadicalVerbose","self::root","children/*[1][text()=3 or text()=2]"],["Rule","root-small","brief","[t] CSFopenRadicalBrief; [n] children/*[2]; [t] CSFcloseRadicalBrief","self::root","children/*[1][text()=3 or text()=2]"],["Rule","root-small","sbrief","[t] CSFopenRadicalSbrief; [n] children/*[2]; [t] CSFcloseRadicalBrief","self::root","children/*[1][text()=3 or text()=2]"],["Rule","root","default","[t] CSFindexRadicalVerbose; [n] children/*[1];[t] CSFopenRadicalVerbose; [n] children/*[2]; [t] CSFcloseRadicalVerbose","self::root"],["Rule","root","brief","[t] CSFindexRadicalBrief; [n] children/*[1];[t] CSFopenRadicalBrief; [n] children/*[2]; [t] CSFcloseRadicalBrief","self::root"],["Rule","root","sbrief","[t] CSFindexRadicalSbrief; [n] children/*[1];[t] CSFopenRadicalSbrief; [n] children/*[2]; [t] CSFcloseRadicalBrief","self::root"],["Rule","limboth","default","[n] children/*[1]; [t] CSFunderscript; [n] children/*[2];[t] CSFoverscript; [n] children/*[3]","self::limboth","name(../..)=\"underscore\" or name(../..)=\"overscore\"","following-sibling::*[@role!=\"underaccent\" and @role!=\"overaccent\"]"],["Rule","limlower","default","[n] children/*[1]; [t] CSFunderscript; [n] children/*[2];","self::limlower","name(../..)=\"underscore\" or name(../..)=\"overscore\"","following-sibling::*[@role!=\"underaccent\" and @role!=\"overaccent\"]"],["Rule","limupper","default","[n] children/*[1]; [t] CSFoverscript; [n] children/*[2];","self::limupper","name(../..)=\"underscore\" or name(../..)=\"overscore\"","following-sibling::*[@role!=\"underaccent\" and @role!=\"overaccent\"]"],["Aliases","limlower","self::underscore","@role=\"limit function\"","name(../..)=\"underscore\" or name(../..)=\"overscore\"","following-sibling::*[@role!=\"underaccent\" and @role!=\"overaccent\"]"],["Aliases","limlower","self::underscore","children/*[2][@role!=\"underaccent\"]","name(../..)=\"underscore\" or name(../..)=\"overscore\"","following-sibling::*[@role!=\"underaccent\" and @role!=\"overaccent\"]"],["Aliases","limupper","self::overscore","children/*[2][@role!=\"overaccent\"]","name(../..)=\"underscore\" or name(../..)=\"overscore\"","following-sibling::*[@role!=\"underaccent\" and @role!=\"overaccent\"]"],["Rule","limboth-end","default","[n] children/*[1]; [t] CSFunderscript; [n] children/*[2];[t] CSFoverscript; [n] children/*[3]; [t] \"Ende Überschrift\"","self::limboth"],["Rule","limlower-end","default","[n] children/*[1]; [t] CSFunderscript; [n] children/*[2]; [t] \"Ende Unterschrift\"","self::limlower"],["Rule","limupper-end","default","[n] children/*[1]; [t] CSFoverscript; [n] children/*[2]; [t] \"Ende Überschrift\"","self::limupper"],["Aliases","limlower-end","self::underscore","@role=\"limit function\""],["Aliases","limlower-end","self::underscore"],["Aliases","limupper-end","self::overscore"],["Rule","integral","default","[n] children/*[1]; [n] children/*[2]; [n] children/*[3];","self::integral"],["Rule","integral","default","[n] children/*[1]; [t] \"Index\"; [n] children/*[2];[t] \"Hoch\"; [n] children/*[3]; [t] \"Grundlinie\";","self::limboth","@role=\"integral\""],["SpecializedRule","integral","default","brief","[n] children/*[1]; [t] \"Index\"; [n] children/*[2];[t] \"Hoch\"; [n] children/*[3]; [t] \"Base\";"],["SpecializedRule","integral","brief","sbrief"],["Rule","bigop","default","[n] children/*[1]; [n] children/*[2];","self::bigop"],["Rule","relseq","default","[m] children/* (sepFunc:CTFcontentIterator)","self::relseq"],["Rule","equality","default","[n] children/*[1]; [n] content/*[1]; [n] children/*[2]","self::relseq","@role=\"equality\"","count(./children/*)=2"],["Rule","multi-equality","default","[m] children/* (sepFunc:CTFcontentIterator)","self::relseq","@role=\"equality\"","count(./children/*)>2"],["Rule","multrel","default","[m] children/* (sepFunc:CTFcontentIterator)","self::multirel"],["Rule","subscript","default","[n] children/*[1]; [t] CSFsubscriptVerbose; [n] children/*[2]","self::subscript"],["Rule","subscript","brief","[n] children/*[1]; [t] CSFsubscriptBrief; [n] children/*[2]","self::subscript"],["SpecializedRule","subscript","brief","sbrief"],["Rule","subscript-simple","default","[n] children/*[1]; [n] children/*[2]","self::subscript","name(./children/*[1])=\"identifier\"","name(./children/*[2])=\"number\"","./children/*[2][@role!=\"mixed\"]","./children/*[2][@role!=\"othernumber\"]"],["SpecializedRule","subscript-simple","default","brief"],["SpecializedRule","subscript-simple","default","sbrief"],["Rule","subscript-baseline","default","[n] children/*[1]; [t] CSFsubscriptVerbose; [n] children/*[2]; [t] CSFbaselineVerbose","self::subscript","following-sibling::*","not(name(following-sibling::subscript/children/*[1])=\"empty\" or (name(following-sibling::infixop[@role=\"implicit\"]/children/*[1])=\"subscript\" and name(following-sibling::*/children/*[1]/children/*[1])=\"empty\")) and @role!=\"subsup\"","not(following-sibling::*[@role=\"rightsuper\" or @role=\"rightsub\" or @role=\"leftsub\" or @role=\"leftsub\"])"],["SpecializedRule","subscript-baseline","default","brief","[n] children/*[1]; [t] CSFsubscriptBrief; [n] children/*[2]; [t] CSFbaselineBrief"],["SpecializedRule","subscript-baseline","brief","sbrief"],["Aliases","subscript-baseline","self::subscript","not(following-sibling::*)","ancestor::fenced|ancestor::root|ancestor::sqrt|ancestor::punctuated|ancestor::fraction","not(ancestor::punctuated[@role=\"leftsuper\" or @role=\"rightsub\" or @role=\"rightsuper\" or @role=\"rightsub\"])"],["Aliases","subscript-baseline","self::subscript","not(following-sibling::*)","ancestor::relseq|ancestor::multirel","CGFbaselineConstraint"],["Aliases","subscript-baseline","self::subscript","not(following-sibling::*)","@embellished"],["Rule","subscript-empty-sup","default","[n] children/*[1]; [n] children/*[2]","self::subscript","name(children/*[2])=\"infixop\"","name(children/*[2][@role=\"implicit\"]/children/*[1])=\"superscript\"","name(children/*[2]/children/*[1]/children/*[1])=\"empty\""],["SpecializedRule","subscript-empty-sup","default","brief"],["SpecializedRule","subscript-empty-sup","brief","sbrief"],["Aliases","subscript-empty-sup","self::subscript","name(children/*[2])=\"superscript\"","name(children/*[2]/children/*[1])=\"empty\""],["Rule","superscript","default","[n] children/*[1]; [t] CSFsuperscriptVerbose; [n] children/*[2]","self::superscript"],["SpecializedRule","superscript","default","brief","[n] children/*[1]; [t] CSFsuperscriptBrief; [n] children/*[2]"],["SpecializedRule","superscript","brief","sbrief"],["Rule","superscript-baseline","default","[n] children/*[1]; [t] CSFsuperscriptVerbose; [n] children/*[2];[t] CSFbaselineVerbose","self::superscript","following-sibling::*","not(name(following-sibling::superscript/children/*[1])=\"empty\" or (name(following-sibling::infixop[@role=\"implicit\"]/children/*[1])=\"superscript\" and name(following-sibling::*/children/*[1]/children/*[1])=\"empty\")) and not(following-sibling::*[@role=\"rightsuper\" or @role=\"rightsub\" or @role=\"leftsub\" or @role=\"leftsub\"])"],["SpecializedRule","superscript-baseline","default","brief","[n] children/*[1]; [t] CSFsuperscriptBrief; [n] children/*[2];[t] CSFbaselineBrief"],["SpecializedRule","superscript-baseline","brief","sbrief"],["Aliases","superscript-baseline","self::superscript","not(following-sibling::*)","ancestor::punctuated","ancestor::*/following-sibling::* and not(ancestor::punctuated[@role=\"leftsuper\" or @role=\"rightsub\" or @role=\"rightsuper\" or @role=\"rightsub\"])"],["Aliases","superscript-baseline","self::superscript","not(following-sibling::*)","ancestor::fraction|ancestor::fenced|ancestor::root|ancestor::sqrt"],["Aliases","superscript-baseline","self::superscript","not(following-sibling::*)","ancestor::relseq|ancestor::multirel","not(@embellished)","CGFbaselineConstraint"],["Aliases","superscript-baseline","self::superscript","not(following-sibling::*)","@embellished","not(children/*[2][@role=\"prime\"])"],["Rule","superscript-empty-sub","default","[n] children/*[1]; [n] children/*[2]","self::superscript","name(children/*[2])=\"infixop\"","name(children/*[2][@role=\"implicit\"]/children/*[1])=\"subscript\"","name(children/*[2]/children/*[1]/children/*[1])=\"empty\""],["SpecializedRule","superscript-empty-sub","default","brief"],["SpecializedRule","superscript-empty-sub","brief","sbrief"],["Aliases","superscript-empty-sub","self::superscript","name(children/*[2])=\"subscript\"","name(children/*[2]/children/*[1])=\"empty\""],["Rule","square","default","[n] children/*[1]; [t] \"Quadrat\"","self::superscript","children/*[2]","children/*[2][text()=2]","name(children/*[1])!=\"text\" or not(name(children/*[1])=\"text\" and (name(../../../punctuated[@role=\"text\"]/..)=\"stree\" or name(..)=\"stree\"))","name(children/*[1])!=\"subscript\" or (name(children/*[1])=\"subscript\" and name(children/*[1]/children/*[1])=\"identifier\" and name(children/*[1]/children/*[2])=\"number\" and children/*[1]/children/*[2][@role!=\"mixed\"] and children/*[1]/children/*[2][@role!=\"othernumber\"])","not(@embellished)"],["SpecializedRule","square","default","brief"],["SpecializedRule","square","default","sbrief"],["Aliases","square","self::superscript","children/*[2]","children/*[2][text()=2]","@embellished","children/*[1][@role=\"prefix operator\"]"],["Rule","cube","default","[n] children/*[1]; [t] \"Kubik\"","self::superscript","children/*[2]","children/*[2][text()=3]","name(children/*[1])!=\"text\" or not(name(children/*[1])=\"text\" and (name(../../../punctuated[@role=\"text\"]/..)=\"stree\" or name(..)=\"stree\"))","name(children/*[1])!=\"subscript\" or (name(children/*[1])=\"subscript\" and name(children/*[1]/children/*[1])=\"identifier\" and name(children/*[1]/children/*[2])=\"number\" and children/*[1]/children/*[2][@role!=\"mixed\"] and children/*[1]/children/*[2][@role!=\"othernumber\"])","not(@embellished)"],["SpecializedRule","cube","default","brief"],["SpecializedRule","cube","default","sbrief"],["Aliases","cube","self::superscript","children/*[2]","children/*[2][text()=3]","@embellished","children/*[1][@role=\"prefix operator\"]"],["Rule","prime","default","[n] children/*[1]; [n] children/*[2]","self::superscript","children/*[2]","children/*[2][@role=\"prime\"]"],["SpecializedRule","prime","default","brief"],["SpecializedRule","prime","default","sbrief"],["Rule","double-prime","default","[t] \"zwei Strich\"","self::punctuated","@role=\"prime\"","count(children/*)=2"],["Aliases","double-prime","self::operator","@role=\"prime\"","string-length(text())=2"],["Rule","triple-prime","default","[t] \"drei Strich\"","self::punctuated","@role=\"prime\"","count(children/*)=3"],["Aliases","triple-prime","self::operator","@role=\"prime\"","string-length(text())=3"],["Rule","quadruple-prime","default","[t] \"vier Strich\"","self::punctuated","@role=\"prime\"","count(children/*)=4"],["Aliases","quadruple-prime","self::operator","@role=\"prime\"","string-length(text())=4"],["Rule","counted-prime","default","[t] count(children/*) (grammar:numbers2alpha); [t] \"Strich\"","self::punctuated","@role=\"prime\""],["Rule","counted-prime","default","[t] string-length(text()) (grammar:numbers2alpha); [t] \"Strich\"","self::operator","@role=\"prime\"","string-length(text())>4"],["Rule","prime-subscript","default","[n] children/*[1]/children/*[1]; [n] children/*[2]; [t] CSFsubscriptVerbose; [n] children/*[1]/children/*[2]","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"subscript\"","not(following-sibling::*)"],["SpecializedRule","prime-subscript","default","brief","[n] children/*[1]/children/*[1]; [n] children/*[2]; [t] CSFsubscriptBrief; [n] children/*[1]/children/*[2]"],["SpecializedRule","prime-subscript","brief","sbrief"],["Rule","prime-subscript-baseline","default","[n] children/*[1]/children/*[1]; [n] children/*[2]; [t] CSFsubscriptVerbose; [n] children/*[1]/children/*[2]; [t] CSFbaselineVerbose","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"subscript\"","following-sibling::*"],["SpecializedRule","prime-subscript-baseline","default","brief","[n] children/*[1]/children/*[1]; [n] children/*[2]; [t] CSFsubscriptBrief; [n] children/*[1]/children/*[2]; [t] CSFbaselineBrief"],["SpecializedRule","prime-subscript-baseline","brief","sbrief"],["Aliases","prime-subscript-baseline","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"subscript\"","not(following-sibling::*)","@embellished"],["Rule","prime-subscript-simple","default","[n] children/*[1]/children/*[1]; [n] children/*[2];[n] children/*[1]/children/*[2]","self::superscript","children/*[2][@role=\"prime\"]","name(children/*[1])=\"subscript\"","name(children/*[1]/children/*[1])=\"identifier\"","name(children/*[1]/children/*[2])=\"number\"","children/*[1]/children/*[2][@role!=\"mixed\"]","children/*[1]/children/*[2][@role!=\"othernumber\"]"],["SpecializedRule","prime-subscript-simple","default","brief"],["SpecializedRule","prime-subscript-simple","default","sbrief"],["Rule","overscore","default","[t] \"modifiziert oben\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]","self::overscore","children/*[2][@role=\"overaccent\"]"],["SpecializedRule","overscore","default","brief","[t] \"mod oben\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]"],["SpecializedRule","overscore","brief","sbrief"],["Rule","double-overscore","default","[t] \"modifiziert oben oben\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]","self::overscore","children/*[2][@role=\"overaccent\"]","name(children/*[1])=\"overscore\"","children/*[1]/children/*[2][@role=\"overaccent\"]"],["SpecializedRule","double-overscore","default","brief","[t] \"mod oben oben\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]"],["SpecializedRule","double-overscore","brief","sbrief"],["Rule","underscore","default","[t] \"modifiziert unten\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]","self::underscore","children/*[2][@role=\"underaccent\"]"],["SpecializedRule","underscore","default","brief","[t] \"mod unten\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]"],["SpecializedRule","underscore","brief","sbrief"],["Rule","double-underscore","default","[t] \"modifiziert unten unten\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]","self::underscore","children/*[2][@role=\"underaccent\"]","name(children/*[1])=\"underscore\"","children/*[1]/children/*[2][@role=\"underaccent\"]"],["SpecializedRule","double-underscore","default","brief","[t] \"mod unten unten\"; [n] children/*[1]; [t] \"mit\"; [n] children/*[2]"],["SpecializedRule","double-underscore","brief","sbrief"],["Rule","overbar","default","[n] children/*[1]; [t] \"Überstrich\"","self::overscore","contains(@role,\"letter\")","children/*[2][@role=\"overaccent\"]","children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]"],["Rule","underbar","default","[n] children/*[1]; [t] \"Unterstrich\"","self::underscore","contains(@role,\"letter\")","children/*[2][@role=\"underaccent\"]","children/*[2][text()=\"¯\" or text()=\"￣\" or text()=\"＿\" or text()=\"_\" or text()=\"‾\"]"],["Rule","overtilde","default","[n] children/*[1]; [t] \"Tilde oben\"","self::overscore","children/*[2][@role=\"overaccent\"]","contains(@role,\"letter\")","children/*[2][text()=\"~\" or text()=\"˜\" or text()=\"∼\" or text()=\"～\"]"],["Rule","undertilde","default","[n] children/*[1]; [t] \"Tilde unten\"","self::underscore","contains(@role,\"letter\")","children/*[2][@role=\"underaccent\"]","children/*[2][text()=\"~\" or text()=\"˜\" or text()=\"∼\" or text()=\"～\"]"],["Rule","matrix","default","[t] \"Anfang\"; [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Matrize\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Matrize\"","self::matrix"],["Rule","matrix","sbrief","[t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Matrize\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Matrize\"","self::matrix"],["Aliases","matrix","self::vector"],["Rule","matrix-row","default","[m] children/* (ctxtFunc:CTFordinalCounter,context:\"Spalte\", pause: 200)","self::row"],["Rule","row-with-label","default","[t] \"mit Bezeichner\"; [n] content/*[1]; [t] \"Ende Bezeichner\" (pause: 200); [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Spalte\")","self::row","content"],["Rule","row-with-label","brief","[t] \"Bezeichner\"; [n] content/*[1]; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Spalte\")","self::row","content"],["SpecializedRule","row-with-label","brief","sbrief"],["Rule","row-with-text-label","sbrief","[t] \"Bezeichner\"; [t] CSFRemoveParens;[m] children/* (ctxtFunc:CTFordinalCounter,context:\"Spalte\")","self::row","content","name(content/cell/children/*[1])=\"text\""],["Rule","empty-row","default","[t] \"leer\"","self::row","count(children/*)=0"],["Rule","matrix-cell","default","[n] children/*[1] (pause: 300)","self::cell"],["Rule","empty-cell","default","[t] \"leer\" (pause: 300)","self::cell","count(children/*)=0"],["Rule","determinant","default","[t] \"Anfang\"; [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Determinante\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Determinante\"","self::matrix","@role=\"determinant\""],["SpecializedRule","determinant","default","sbrief","[t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Determinante\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Determinante\""],["Rule","determinant-simple","default","[t] \"Anfang\"; [t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Determinante\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile\",grammar:simpleDet); [t] \"Ende Determinante\"","self::matrix","@role=\"determinant\"","CQFdetIsSimple"],["SpecializedRule","determinant-simple","default","sbrief","[t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Determinante\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile\",grammar:simpleDet); [t] \"Ende Determinante\""],["Rule","row-simple","default","[m] children/*;","self::row","@role=\"determinant\"","contains(@grammar, \"simpleDet\")"],["Rule","layout","default","[t] \"Anfang Anordnung\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Anordnung\"","self::table"],["Rule","layout","sbrief","[t] \"Anordnung\"; [m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Anordnung\"","self::table"],["Rule","binomial","default","[t] \"Anfang Binomialkoeffizient\"; [n] children/*[2]/children/*[1]; [t] \"aus\";  [n] children/*[1]/children/*[1]; [t] \"Ende Binomialkoeffizient\"","self::vector","@role=\"binomial\""],["Rule","binomial","brief","[t] \"Anfang Binomial\"; [n] children/*[2]/children/*[1]; [t] \"aus\";  [n] children/*[1]/children/*[1]; [t] \"Ende Binomial\"","self::vector","@role=\"binomial\""],["Rule","binomial","sbrief","[t] \"Binomial\"; [n] children/*[2]/children/*[1]; [t] \"aus\";  [n] children/*[1]/children/*[1]; [t] \"Ende Binomial\"","self::vector","@role=\"binomial\""],["Rule","cases","default","[t] \"Anfang Fallunterscheidung\"; [t] \"große\"; [n] content/*[1];[m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Fallunterscheidung\"","self::cases"],["Rule","cases","brief","[t] \"Anfang Fälle\"; [t] \"große\"; [n] content/*[1];[m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Fälle\"","self::cases"],["Rule","cases","sbrief","[t] \"Fälle\"; [t] \"große\"; [n] content/*[1];[m] children/* (ctxtFunc:CTFordinalCounter,context:\"Zeile \"); [t] \"Ende Fälle\"","self::cases"],["Aliases","layout","self::multiline"],["Rule","line","default","[m] children/*","self::line"],["Rule","line-with-label","default","[t] \"mit Bezeichner\"; [n] content/*[1]; [t] \"Ende Bezeichner\" (pause: 200); [m] children/*","self::line","content"],["SpecializedRule","line-with-label","default","brief","[t] \"Bezeichner\"; [n] content/*[1] (pause: 200); [m] children/*"],["SpecializedRule","line-with-label","brief","sbrief"],["Rule","line-with-text-label","sbrief","[t] \"Bezeichner\"; [t] CSFRemoveParens; [m] children/*","self::line","content","name(content/cell/children/*[1])=\"text\""],["Rule","empty-line","default","[t] \"leer\"","self::line","count(children/*)=0","not(content)"],["SpecializedRule","empty-line","default","brief"],["SpecializedRule","empty-line","brief","sbrief"],["Rule","empty-line-with-label","default","[t] \"mit Bezeichner\"; [n] content/*[1]; [t] \"Ende Bezeichner\" (pause: 200); [t] \"leer\"","self::line","count(children/*)=0","content"],["SpecializedRule","empty-line-with-label","default","brief","[t] \"Bezeichner\"; [n] content/*[1] (pause: 200); [t] \"leer\""],["SpecializedRule","empty-line-with-label","brief","sbrief"],["Rule","enclose","default","[t] \"Anfang Umschließung\"; [t] @role (grammar:localEnclose); [n] children/*[1]; [t] \"Ende Umschließung\"","self::enclose"],["Aliases","overbar","self::enclose","@role=\"top\""],["Aliases","underbar","self::enclose","@role=\"bottom\""],["Rule","leftbar","default","[t] \"senkrechter Strich\"; [n] children/*[1]","self::enclose","@role=\"left\""],["Rule","rightbar","default","[n] children/*[1]; [t] \"senkrechter Strich\"","self::enclose","@role=\"right\""],["Rule","crossout","default","[t] \"durchgestrichen\"; [n] children/*[1]; [t] \"Ende duchgestrichen\"","self::enclose","@role=\"updiagonalstrike\" or @role=\"downdiagonalstrike\" or @role=\"horizontalstrike\""],["Rule","cancel","default","[t] \"durchgestrichen\"; [n] children/*[1]/children/*[1]; [t] \"mit\"; [n] children/*[2]; [t] \"Ende duchgestrichen\"","self::overscore","@role=\"updiagonalstrike\" or @role=\"downdiagonalstrike\" or @role=\"horizontalstrike\""],["SpecializedRule","cancel","default","brief"],["SpecializedRule","cancel","default","sbrief"],["Aliases","cancel","self::underscore","@role=\"updiagonalstrike\" or @role=\"downdiagonalstrike\" or @role=\"horizontalstrike\""],["Rule","cancel-reverse","default","[t] \"durchgestrichen\"; [n] children/*[2]/children/*[1]; [t] \"mit\"; [n] children/*[1]; [t] \"Ende duchgestrichen\"","self::overscore","name(children/*[2])=\"enclose\"","children/*[2][@role=\"updiagonalstrike\" or @role=\"downdiagonalstrike\" or @role=\"horizontalstrike\"]"],["SpecializedRule","cancel-reverse","default","brief"],["SpecializedRule","cancel-reverse","default","sbrief"],["Aliases","cancel-reverse","self::underscore","name(children/*[2])=\"enclose\"","children/*[2][@role=\"updiagonalstrike\" or @role=\"downdiagonalstrike\" or @role=\"horizontalstrike\"]"],["Rule","end-punct","default","[m] children/*","self::punctuated","@role=\"endpunct\""],["Rule","start-punct","default","[n] content/*[1]; [m] children/*[position()>1]","self::punctuated","@role=\"startpunct\""],["Rule","punctuated","default","[m] children/*","self::punctuated"],["Rule","unit","default","[t] text() (grammar:annotation=\"unit\":translate:plural)","self::identifier[@role=\"unit\"]"],["Rule","unit-combine","default","[m] children/*","self::infixop[@role=\"unit\"]"],["Rule","inference","default","[t] \"Schlussregel\"; [m] content/*; [t] \"mit Folgerung\"; [n] children/*[1]; [t] \"aus\"; [t] count(children/*[2]/children/*); [t] \"Prämissen\"","self::inference"],["Rule","inference","default","[t] \"Schlussregel\"; ; [m] content/*; [t] \"mit Folgerung\"; [n] children/*[1]; [t] \"aus\"; [t] count(children/*[2]/children/*); [t] \"Prämisse\"","self::inference","count(children/*[2]/children/*)<2"],["Rule","premise","default","[m] children/* (ctxtFunc:CTFordinalCounter,context:\"Prämisse \");","self::premises"],["Rule","conclusion","default","[n] children/*[1]","self::conclusion"],["Rule","label","default","[t] \"Regel\"; [n] children/*[1]","self::rulelabel"],["Rule","axiom","default","[t] \"Axiom\"; [m] children/*[1];","self::inference","@role=\"axiom\""],["Rule","axiom","default","[t] \"leeres Axiom\";","self::empty","@role=\"axiom\""],["Generator","CGFtensorRules"]]}
,
"de/rules/prefix_german.js": 
{"modality":"prefix","locale":"de","domain":"default","rules":[["Rule","numerator","default","[t] \"Zähler\" (pause:200)","self::*","name(../..)=\"fraction\"","count(preceding-sibling::*)=0"],["Rule","denominator","default","[t] \"Nenner\" (pause:200)","self::*","name(../..)=\"fraction\"","count(preceding-sibling::*)=1"],["Rule","base","default","[t] \"Basis\" (pause:200)","self::*","count(preceding-sibling::*)=0","name(../..)=\"superscript\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"subscript\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"overscore\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"underscore\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"tensor\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"limlower\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"limupper\""],["Alias","base","self::*","count(preceding-sibling::*)=0","name(../..)=\"limboth\""],["Rule","exponent","default","[t] \"Exponent\" (pause:200)","self::*","name(../..)=\"superscript\"","count(preceding-sibling::*)=1"],["Rule","subscript","default","[t] \"Index\" (pause:200)","self::*","name(../..)=\"subscript\"","count(preceding-sibling::*)=1"],["Rule","overscript","default","[t] \"Oberer Grenzwert\" (pause:200)","self::*","count(preceding-sibling::*)=1","name(../..)=\"overscore\""],["Alias","overscript","self::*","count(preceding-sibling::*)=1","name(../..)=\"limupper\""],["Alias","overscript","self::*","count(preceding-sibling::*)=2","name(../..)=\"limboth\""],["Rule","underscript","default","[t] \"Unterer Grenzwert\" (pause:200)","self::*","count(preceding-sibling::*)=1","name(../..)=\"underscore\""],["Alias","underscript","self::*","count(preceding-sibling::*)=1","name(../..)=\"limlower\""],["Alias","underscript","self::*","count(preceding-sibling::*)=1","name(../..)=\"limboth\""],["Rule","radicand","default","[t] \"Radikand\" (pause:200)","self::*","name(../..)=\"sqrt\""],["Rule","radicand","default","[t] \"Radikand\" (pause:200)","self::*","name(../..)=\"root\"","count(preceding-sibling::*)=1"],["Rule","index","default","[t] \"Wurzelexponent\" (pause:200)","self::*","name(../..)=\"root\"","count(preceding-sibling::*)=0"],["Rule","leftsub","default","[t] \"linker unterer Index\" (pause:200)","self::*","name(../..)=\"tensor\"","@role=\"leftsub\""],["Rule","leftsub","default","[t] CSFordinalPosition; [t] \"linker unterer Index\" (pause:200)","self::*","name(../..)=\"punctuated\"","name(../../../..)=\"tensor\"","../../@role=\"leftsub\""],["Rule","leftsuper","default","[t] \"linker oberer Index\" (pause:200)","self::*","name(../..)=\"tensor\"","@role=\"leftsuper\""],["Rule","leftsuper","default","[t] CSFordinalPosition; [t] \"linker oberer Index\" (pause:200)","self::*","name(../..)=\"punctuated\"","name(../../../..)=\"tensor\"","../../@role=\"leftsuper\""],["Rule","rightsub","default","[t] \"rechter unterer Index\" (pause:200)","self::*","name(../..)=\"tensor\"","@role=\"rightsub\""],["Rule","rightsub","default","[t] CSFordinalPosition; [t] \"rechter unterer Index\" (pause:200)","self::*","name(../..)=\"punctuated\"","name(../../../..)=\"tensor\"","../../@role=\"rightsub\""],["Rule","rightsuper","default","[t] \"rechter oberer Index\" (pause:200)","self::*","name(../..)=\"tensor\"","@role=\"rightsuper\""],["Rule","rightsuper","default","[t] CSFordinalPosition; [t] \"rechter oberer Index\" (pause:200)","self::*","name(../..)=\"punctuated\"","name(../../../..)=\"tensor\"","../../@role=\"rightsuper\""],["Rule","choice","default","[t] \"Grundgesamtheit\" (pause:200)","self::line","@role=\"binomial\"","parent::*/parent::vector","count(preceding-sibling::*)=0"],["Rule","select","default","[t] \"Stichprobengröße\" (pause:200)","self::line","@role=\"binomial\"","parent::*/parent::vector","count(preceding-sibling::*)=1"],["Rule","row","default","[t] CSFordinalPosition; [t] \"Zeile\" (pause:200)","self::row"],["Aliases","row","self::line"],["Rule","cell","default","[n] ../..; [t] CSFordinalPosition; [t] \"Spalte\" (pause:200)","self::cell","contains(@grammar,\"depth\")"],["Rule","cell","default","[t] CSFordinalPosition; [t] \"Spalte\" (pause:200)","self::cell"]]}
,
"de/rules/summary_german.js": 
{"modality":"summary","locale":"de","rules":[["Rule","abstr-identifier","default.default","[t] \"langer Bezeichner\"","self::identifier","contains(@grammar, \"collapsed\")"],["Rule","abstr-identifier","default.default","[t] \"Bezeichner\"","self::identifier"],["Rule","abstr-number","default.default","[t] \"lange Zahl\"","self::number","contains(@grammar, \"collapsed\")"],["Rule","abstr-number","default.default","[t] \"Zahl\"","self::number"],["Rule","abstr-mixed-number","default.default","[t] \"langer gemischter Bruch\"","self::number","@role=\"mixed\"","contains(@grammar, \"collapsed\")"],["Rule","abstr-mixed-number","default.default","[t] \"gemischter Bruch\"","self::number","@role=\"mixed\""],["Rule","abstr-text","default.default","[t] \"Text\"","self::text"],["Rule","abstr-function","default.default","[t] \"Funktionsausdruck\"","self::function"],["Rule","abstr-function","mathspeak.brief","[t] \"Funktion\"","self::function"],["SpecializedRule","abstr-function","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-lim","default.default","[t] \"Grenzwertfunktion\"","self::function","@role=\"limit function\""],["Rule","abstr-lim","mathspeak.brief","[t] \"Grenzwert\"","self::function","@role=\"limit function\""],["SpecializedRule","abstr-lim","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-fraction","default.default","[t] \"Bruch\"","self::fraction"],["Rule","abstr-fraction","mathspeak.brief","[t] \"Bruch\"","self::fraction"],["SpecializedRule","abstr-fraction","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-continued-fraction","default.default","[t] \"Kettenbruch\"","self::fraction","children/*[2]/descendant-or-self::*[@role=\"ellipsis\"]"],["Rule","abstr-continued-fraction","mathspeak.brief","[t] \"Kettenbruch\"","self::fraction","children/*[2]/descendant-or-self::*[@role=\"ellipsis\"]"],["SpecializedRule","abstr-continued-fraction","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-sqrt","default.default","[t] \"Quadratwurzel\"","self::sqrt"],["Rule","abstr-sqrt-nested","default.default","[t] \"verschachtelte Quadratwurzel\"","self::sqrt","children/*/descendant-or-self::sqrt or children/*/descendant-or-self::root"],["Rule","abstr-root","default.default","[t] \"Wurzel mit Exponent\"; [n] children/*[1] (engine:modality=\"speech\"); [t] \"Ende Exponent\"","self::root","contains(@grammar, \"collapsed\")","following-sibling::* or ancestor::*/following-sibling::*"],["Rule","abstr-root","default.default","[t] \"Wurzel mit Exponent\"; [n] children/*[1] (engine:modality=\"speech\")","self::root"],["Rule","abstr-root","mathspeak.brief","[t] \"Wurzel\"","self::root"],["SpecializedRule","abstr-root","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-root-nested","default.default","[t] \"verschachtelte Wurzel mit Exponent\"; [n] children/*[1] (engine:modality=\"speech\"); [t] \"Ende Exponent\"","self::root","contains(@grammar, \"collapsed\")","children/*/descendant-or-self::sqrt or children/*/descendant-or-self::root","following-sibling::* or ancestor::*/following-sibling::*"],["Rule","abstr-root-nested","default.default","[t] \"verschachtelte Wurzel mit Exponent\"; [n] children/*[1] (engine:modality=\"speech\")","self::root","children/*/descendant-or-self::sqrt or children/*/descendant-or-self::root"],["Rule","abstr-root-nested","mathspeak.brief","[t] \"verschachtelte Wurzel\"","self::root","children/*/descendant-or-self::sqrt or children/*/descendant-or-self::root"],["SpecializedRule","abstr-root-nested","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-superscript","default.default","[t] \"Potenz\"","self::superscript"],["Rule","abstr-subscript","default.default","[t] \"Index\"","self::subscript"],["Rule","abstr-subsup","default.default","[t] \"Potenz mit Index\"","self::superscript","name(children/*[1])=\"subscript\""],["Rule","abstr-infixop","default.default","[t] @role (grammar:localRole); [t] \"mit\"; [t] count(./children/*); [t] \"Elementen\"","self::infixop"],["Rule","abstr-infixop","default.default","[t] @role (grammar:localRole); [t] \"mit veränderlicher Anzahl an Elementen\"","self::infixop","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-infixop","mathspeak.brief","[t] @role (grammar:localRole)","self::infixop"],["SpecializedRule","abstr-infixop","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-addition","default.default","[t] \"Summe mit\"; [t] count(./children/*); [t] \"Summanden\"","self::infixop","@role=\"addition\""],["Rule","abstr-addition","mathspeak.brief","[t] \"Summe\"","self::infixop","@role=\"addition\""],["SpecializedRule","abstr-addition","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-addition","default.default","[t] \"Summe mit veränderlicher Anzahl an Summanden\"","self::infixop","@role=\"addition\"","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-multiplication","default.default","[t] \"Produkt mit\"; [t] count(./children/*); [t] \"Faktoren\"","self::infixop","@role=\"multiplication\""],["Rule","abstr-multiplication","mathspeak.brief","[t] \"Produkt\"","self::infixop","@role=\"multiplication\""],["SpecializedRule","abstr-multiplication","mathspeak.brief","mathspeak.sbrief"],["Aliases","abstr-multiplication","self::infixop","@role=\"implicit\""],["Rule","abstr-var-multiplication","default.default","[t] \"Produkt mit veränderlicher Anzahl an Faktoren\"","self::infixop","@role=\"multiplication\"","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["Aliases","abstr-var-multiplication","self::infixop","@role=\"implicit\"","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-vector","default.default","[t] count(./children/*) ; [t] \"dimensionaler Vektor\"","self::vector"],["Rule","abstr-vector","mathspeak.brief","[t] \"Vektor\"","self::vector"],["SpecializedRule","abstr-vector","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-vector","default.default","[t] \"n dimensionaler Vektor\"","self::vector","./children/*/children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-binomial","default.default","[t] \"Binomialkoeffizient\"","self::vector","@role=\"binomial\""],["SpecializedRule","abstr-binomial","default.default","mathspeak.brief"],["SpecializedRule","abstr-binomial","default.default","mathspeak.sbrief"],["Rule","abstr-determinant","default.default","[t] count(./children/*); [t] \"dimensionale Determinante\"","self::matrix","@role=\"determinant\""],["Rule","abstr-determinant","mathspeak.brief","[t] \"Determinante\"","self::matrix","@role=\"determinant\""],["SpecializedRule","abstr-determinant","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-determinant","default.default","[t] \"n dimensionale Determinante\"","self::matrix","@role=\"determinant\"","./children/*/children/*/children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-squarematrix","default.default","[t] count(./children/*); [t] \"dimensionale quadratische Matrize\"","self::matrix","@role=\"squarematrix\""],["Rule","abstr-squarematrix","mathspeak.brief","[t] \"quadratische Matrize\"","self::matrix","@role=\"squarematrix\""],["SpecializedRule","abstr-squarematrix","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-rowvector","default.default","[t] count(./children/row/children/*); [t] \"dimensionaler Zeilenvektor\"","self::matrix","@role=\"rowvector\""],["Rule","abstr-rowvector","mathspeak.brief","[t] \"Zeilenvektor\"","self::matrix","@role=\"rowvector\""],["SpecializedRule","abstr-rowvector","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-matrix","default.default","[t] \"n dimensionaler Zeilenvektor\"","self::matrix","@role=\"rowvector\"","./children/*/children/*/children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-matrix","default.default","[t] count(children/*);  [t] \"mal\";[t] count(children/*[1]/children/*); [t] \"Matrize\"","self::matrix"],["Rule","abstr-matrix","mathspeak.brief","[t] \"Matrize\"","self::matrix"],["SpecializedRule","abstr-matrix","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-matrix","default.default","[t] \"n mal m dimensionale Matrize\"","self::matrix","./children/*/children/*/children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-cases","default.default","[t] \"Fallunterscheidung\";[t] \"mit\"; [t] count(children/*); [t] \"Fällen\"","self::cases"],["Rule","abstr-cases","mathspeak.brief","[t] \"Fallunterscheidung\"","self::cases"],["SpecializedRule","abstr-cases","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-cases","default.default","[t] \"Fallunterscheidung mit veränderlicher Anzahl an Fällen\"","self::cases","./children/row/children/cell/children/punctuation[@role=\"ellipsis\"]or ./children/line/children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-punctuated","default.default","[t] \"mit\"; [n] content/*[1]; [t] \"getrennte Liste der Länge\"; [t] count(children/*) - count(content/*)","self::punctuated"],["Rule","abstr-punctuated","mathspeak.brief","[t] \"mit\"; [n] content/*[1]; [t] \"getrennte Liste\";","self::punctuated"],["SpecializedRule","abstr-punctuated","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-punctuated","default.default","[t] \"mit\"; [n] content/*[1]; [t] \"getrennte Liste\";[t] \"veränderlicher Länge\"","self::punctuated","./children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-bigop","default.default","[n] content/*[1]","self::bigop"],["Rule","abstr-integral","default.default","[t] \"Integral\"","self::*","@role=\"integral\""],["Rule","abstr-relation","default.default","[t] @role (grammar:localRole);","self::relseq","count(./children/*)=2"],["Rule","abstr-relation-seq","default.default","[t] @role (grammar:localRole, join:\"\"); [t] \"ssequenz\"; [t] \"mit\"; [t] count(./children/*); [t] \"Elementen\"","self::relseq","count(./children/*)>2"],["Rule","abstr-relation-seq","mathspeak.brief","[t] @role (grammar:localRole, join:\"\"); [t] \"ssequenz\"","self::relseq","count(./children/*)>2"],["SpecializedRule","abstr-relation-seq","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-relation","default.default","[t] @role (grammar:localRole, join:\"\"); [t] \"ssequenz\";[t] \"mit veränderlicher Anzahl an Elementen\"","self::relseq","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["UniqueAlias","abstr-relation","default.default","self::multirel","@role!=\"unknown\"","count(./children/*)>2"],["Aliases","abstr-var-relation","self::multirel","@role!=\"unknown\"","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-multirel","default.default","[t] \"Relationsequenz\"; [t] \"mit\"; [t] count(./children/*); [t] \"Elementen\"","self::multirel","count(./children/*)>2"],["Rule","abstr-multirel","mathspeak.brief","[t] \"Relationsequenz\"","self::multirel","count(./children/*)>2"],["SpecializedRule","abstr-multirel","mathspeak.brief","mathspeak.sbrief"],["Rule","abstr-var-multirel","default.default","[t] \"Relationsequenz mit veränderlicher Anzahl an Elementen\"","self::multirel","count(./children/*)>2","./children/punctuation[@role=\"ellipsis\"]"],["Rule","abstr-table","default.default","[t] \"Tabelle mit\"; [t] count(children/*); [t] \"Zeilen und\";[t] count(children/*[1]/children/*); [t] \"Spalten\"","self::table"],["Rule","abstr-line","default.default","[t] \"in\"; [t] @role (grammar:localRole);","self::line"],["Rule","abstr-row","default.default","[t] \"in\"; [t] @role (grammar:localRole);[t] count(preceding-sibling::..); [t] \"mit\";[t] count(children/*); [t] \"Spalten\"","self::row"],["Rule","abstr-cell","default.default","[t] \"in\"; [t] @role (grammar:localRole);","self::cell"]]}
}

